import { defineComponent, computed, isVNode, createVNode, mergeProps } from "vue";
import { isFunction } from "lodash-es";
import { IxButton } from "@idux/components/button";
const footerProps = {
  cancel: Function,
  cancelButton: Object,
  cancelLoading: Boolean,
  cancelText: String,
  cancelVisible: {
    type: Boolean,
    default: true
  },
  disabled: {
    type: Boolean,
    default: void 0
  },
  footer: [Boolean, Array, Object, Function],
  ok: Function,
  okButton: Object,
  okLoading: Boolean,
  okText: String
};
var Footer = /* @__PURE__ */ defineComponent({
  name: "\u0275Footer",
  props: footerProps,
  setup(props, {
    slots
  }) {
    const cancelButtonProps = computed(() => {
      const {
        cancel,
        cancelButton,
        cancelVisible,
        cancelLoading,
        cancelText
      } = props;
      if (!cancelVisible || !cancel && !cancelText && !cancelButton) {
        return void 0;
      }
      return {
        key: "__IDUX_FOOTER_BUTTON_CANCEL",
        text: cancelText,
        onClick: cancel,
        loading: cancelLoading,
        ...cancelButton
      };
    });
    const okButtonProps = computed(() => {
      const {
        ok,
        okButton,
        okLoading,
        okText
      } = props;
      if (!ok && !okText && !okButton) {
        return void 0;
      }
      return {
        key: "__IDUX_FOOTER_BUTTON_OK",
        text: okText,
        onClick: ok,
        loading: okLoading,
        mode: cancelButtonProps.value ? "primary" : "default",
        ...okButton
      };
    });
    return () => {
      const footerProp = props.footer;
      const footerSlot = slots.footer;
      if (!footerProp && !footerSlot) {
        return void 0;
      }
      let children;
      if (footerSlot) {
        children = footerSlot(props);
      } else if (isFunction(footerProp)) {
        children = footerProp();
      } else if (isVNode(footerProp)) {
        children = footerProp;
      } else {
        let buttonProps = footerProp;
        if (!Array.isArray(buttonProps)) {
          const cancelButton = cancelButtonProps.value;
          const okButton = okButtonProps.value;
          buttonProps = [];
          okButton && buttonProps.push(okButton);
          cancelButton && buttonProps.push(cancelButton);
        }
        children = buttonProps.map((item) => {
          const {
            text,
            disabled,
            ...rest
          } = item;
          const _text = isFunction(text) ? text() : text;
          return createVNode(IxButton, mergeProps({
            "disabled": disabled || props.disabled
          }, rest), {
            default: () => [_text]
          });
        });
      }
      return createVNode("div", null, [children]);
    };
  }
});
const \u0275Footer = Footer;
export { \u0275Footer };
