/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { ValidateStatus } from '@idux/cdk/forms';
import type { ExtractInnerPropTypes, ExtractPublicPropTypes } from '@idux/cdk/utils';
import type { FormSize } from '@idux/components/form';
import type { DefineComponent, InputHTMLAttributes, PropType } from 'vue';
export declare const inputProps: {
    readonly addonAfter: StringConstructor;
    readonly addonBefore: StringConstructor;
    readonly borderless: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly clearable: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly clearIcon: StringConstructor;
    readonly clearVisible: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly focused: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly prefix: StringConstructor;
    readonly size: PropType<FormSize>;
    readonly status: PropType<ValidateStatus>;
    readonly suffix: StringConstructor;
    readonly onClear: PropType<(evt: MouseEvent) => void>;
};
export type InputProps = ExtractInnerPropTypes<typeof inputProps>;
export type InputPublicProps = ExtractPublicPropTypes<typeof inputProps>;
export interface InputBindings {
    getInputElement: () => HTMLInputElement | undefined;
}
export type InputComponent = DefineComponent<Omit<InputHTMLAttributes, keyof InputPublicProps> & InputPublicProps, InputBindings>;
export type InputInstance = InstanceType<DefineComponent<InputProps, InputBindings>>;
