import { defineComponent, computed, createVNode, Transition, withDirectives, vShow } from "vue";
import { useGlobalConfig } from "@idux/components/config";
import { useThemeToken } from "@idux/components/theme";
const maskProps = {
  mask: { type: Boolean, default: true },
  transitionName: String,
  visible: { type: Boolean, default: true },
  zIndex: Number
};
var Mask = /* @__PURE__ */ defineComponent({
  name: "\u0275Mask",
  props: maskProps,
  setup(props) {
    const common = useGlobalConfig("common");
    const {
      globalHashId
    } = useThemeToken();
    const mergedPrefixCls = computed(() => `${common.prefixCls}-mask`);
    return () => {
      const {
        mask,
        transitionName,
        visible,
        zIndex
      } = props;
      if (!mask) {
        return null;
      }
      return createVNode(Transition, {
        "appear": true,
        "name": transitionName != null ? transitionName : `${common.prefixCls}-fade`
      }, {
        default: () => [withDirectives(createVNode("div", {
          "class": [mergedPrefixCls.value, globalHashId.value],
          "style": {
            zIndex
          }
        }, null), [[vShow, visible]])]
      });
    };
  }
});
const \u0275Mask = Mask;
export { \u0275Mask };
