/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { ExtractInnerPropTypes, ExtractPublicPropTypes, VKey } from '@idux/cdk/utils';
import type { DefineComponent, HTMLAttributes, PropType } from 'vue';
export interface ItemData {
    key: VKey;
    [prop: string]: any;
}
export declare const overflowItemProps: {
    readonly prefixCls: {
        readonly type: StringConstructor;
        readonly required: true;
    };
    readonly display: {
        readonly type: BooleanConstructor;
        readonly default: true;
    };
    readonly itemKey: {
        readonly type: PropType<VKey>;
        readonly required: true;
    };
    readonly data: PropType<ItemData>;
    readonly onSizeChange: PropType<(entry: ResizeObserverEntry | undefined, key?: VKey) => void>;
};
export declare const overflowProps: {
    readonly maxLabel: {
        readonly type: PropType<number | "responsive">;
        readonly default: number;
    };
    readonly getKey: {
        readonly type: PropType<(item: ItemData) => any>;
        readonly required: true;
    };
    readonly prefixCls: {
        readonly type: StringConstructor;
        readonly required: true;
    };
    readonly dataSource: {
        readonly type: ArrayConstructor;
        readonly default: () => never[];
    };
};
export type OverflowProps = ExtractInnerPropTypes<typeof overflowProps>;
export type OverflowItemProps = ExtractInnerPropTypes<typeof overflowItemProps>;
export type OverflowPublicProps = ExtractPublicPropTypes<typeof overflowProps>;
export type OverflowComponent = DefineComponent<Omit<HTMLAttributes, keyof OverflowPublicProps> & OverflowPublicProps>;
export type OverflowInstance = InstanceType<DefineComponent<OverflowProps>>;
