import { defineComponent, provide, createVNode, computed, watch, inject, onMounted, nextTick, onBeforeUnmount, ref, toRef, Fragment, Transition, mergeProps, withDirectives, vShow, cloneVNode } from "vue";
import { isFunction } from "lodash-es";
import { vClickOutside } from "@idux/cdk/click-outside";
import { usePopper } from "@idux/cdk/popper";
import { CdkPortal } from "@idux/cdk/portal";
import { useState, useControlledProp, convertElement, callEmit, getFirstValidNode, Logger } from "@idux/cdk/utils";
import { useGlobalConfig } from "@idux/components/config";
import { useThemeToken } from "@idux/components/theme";
import { useZIndex } from "@idux/components/utils";
const OverlayVisibleLockToken = Symbol("OverlayVisibleLockToken");
var Content = /* @__PURE__ */ defineComponent({
  props: {
    lock: {
      type: Function,
      required: true
    },
    unlock: {
      type: Function,
      required: true
    }
  },
  setup(props, {
    slots,
    attrs
  }) {
    provide(OverlayVisibleLockToken, {
      lock: props.lock,
      unlock: props.unlock
    });
    return () => {
      var _a;
      return createVNode("div", attrs, [(_a = slots.default) == null ? void 0 : _a.call(slots)]);
    };
  }
});
function useMergedEvents(triggerEvents, popperEvents, stateTriggerEvents, statePopperEvents) {
  const mergedTriggerEvents = computed(() => mergeEvents(triggerEvents.value, stateTriggerEvents));
  const mergedPopperEvents = computed(() => mergeEvents(popperEvents.value, statePopperEvents));
  return {
    mergedTriggerEvents,
    mergedPopperEvents
  };
}
function mergeEvents(events, extendedEvents) {
  const mergedEvents = { ...events };
  Object.entries(extendedEvents).forEach(([key, cb]) => {
    if (key in mergedEvents) {
      const originalCb = mergedEvents[key];
      mergedEvents[key] = (evt) => {
        cb(evt);
        originalCb(evt);
      };
    }
  });
  return mergedEvents;
}
function useOverlayStates() {
  const [isHovered, setIsHovered] = useState(false);
  const [isFocused, setIsFocused] = useState(false);
  const handleMouseenter = () => setIsHovered(true);
  const handleMouseleave = () => setIsHovered(false);
  const handleFocus = () => setIsFocused(true);
  const handleBlur = () => setIsFocused(false);
  const stateTriggerEvents = {
    onMouseenter: handleMouseenter,
    onMouseleave: handleMouseleave,
    onFocus: handleFocus,
    onBlur: handleBlur
  };
  const statePopperEvents = {
    onMouseenter: handleMouseenter,
    onMouseleave: handleMouseleave
  };
  return {
    isHovered,
    isFocused,
    stateTriggerEvents,
    statePopperEvents
  };
}
function usePopperOptions(props, arrowRef) {
  var _a;
  const [allowEnter, setAllowEnter] = useState(props.allowEnter);
  const [autoAdjust, setAutoAdjust] = useState(props.autoAdjust);
  const [delay, setDelay] = useState(props.delay);
  const [disabled, setDisabled] = useState(props.disabled);
  const [offset, setOffset] = useState(props.offset);
  const [placement, setPlacement] = useState(props.placement);
  const [trigger, setTrigger] = useState((_a = props.trigger) != null ? _a : "hover");
  const options = {
    allowEnter,
    autoAdjust,
    delay,
    disabled,
    offset,
    placement,
    trigger
  };
  const updateOptions = () => {
    const { allowEnter: allowEnter2, autoAdjust: autoAdjust2, delay: delay2, disabled: disabled2, offset: offset2, placement: placement2, trigger: trigger2 } = props;
    let _offset;
    if (!arrowRef.value) {
      _offset = offset2;
    } else {
      const { offsetHeight, offsetWidth } = arrowRef.value;
      _offset = offset2 ? [...offset2] : [0, 0];
      _offset[1] += [offsetWidth, offsetHeight][1] / 2;
    }
    setAllowEnter(allowEnter2);
    setAutoAdjust(autoAdjust2);
    setDelay(delay2);
    setDisabled(disabled2);
    setOffset(_offset);
    setPlacement(placement2);
    setTrigger(trigger2 != null ? trigger2 : "hover");
  };
  watch(props, updateOptions, { immediate: true, deep: true });
  watch(arrowRef, updateOptions);
  return {
    options,
    update: updateOptions
  };
}
function useVisible(props, trigger, isHovered, isFocused) {
  const parentLockContext = inject(OverlayVisibleLockToken, null);
  const [visible, setVisible] = useControlledProp(props, "visible", false);
  const [visibleLocked, setVisibleLocked] = useState(false);
  const lock = () => {
    if (visibleLocked.value) {
      return;
    }
    setVisibleLocked(true);
    if (parentLockContext == null ? void 0 : parentLockContext.lock) {
      parentLockContext.lock();
    }
  };
  const unlock = () => {
    if (!visibleLocked.value) {
      return;
    }
    setVisibleLocked(false);
    if (trigger.value === "hover" && !isHovered.value || trigger.value === "focus" && !isFocused.value) {
      setVisible(false);
    }
  };
  const updateVisible = (v) => {
    if (visibleLocked.value) {
      return;
    }
    setVisible(v);
  };
  onMounted(() => {
    watch(visible, (v) => {
      nextTick(() => {
        if (v) {
          parentLockContext == null ? void 0 : parentLockContext.lock();
        } else {
          parentLockContext == null ? void 0 : parentLockContext.unlock();
        }
      });
    });
    if (visible.value) {
      parentLockContext == null ? void 0 : parentLockContext.lock();
    }
  });
  onBeforeUnmount(() => {
    parentLockContext == null ? void 0 : parentLockContext.unlock();
  });
  return {
    visible,
    visibleLocked,
    updateVisible,
    lock,
    unlock
  };
}
const overlayPlacementDef = String;
const overlayTriggerDef = String;
const overlayDelayDef = [Number, Array];
const overlayProps = {
  visible: {
    type: Boolean,
    default: void 0
  },
  placement: overlayPlacementDef,
  allowEnter: {
    type: Boolean,
    default: void 0
  },
  autoAdjust: {
    type: Boolean,
    default: void 0
  },
  clickOutside: {
    type: Boolean,
    default: void 0
  },
  container: {
    type: [String, HTMLElement, Function],
    default: void 0
  },
  containerFallback: {
    type: String,
    required: true
  },
  delay: overlayDelayDef,
  destroyOnHide: {
    type: Boolean,
    default: void 0
  },
  disabled: {
    type: Boolean,
    default: void 0
  },
  lazy: {
    type: Boolean,
    default: true
  },
  offset: Array,
  showArrow: {
    type: Boolean,
    default: void 0
  },
  transitionName: String,
  trigger: overlayTriggerDef,
  triggerId: { type: null, default: void 0 },
  zIndex: Number,
  "onUpdate:visible": [Function, Array],
  "onUpdate:placement": [Function, Array],
  onAfterLeave: [Function, Array]
};
var Overlay = /* @__PURE__ */ defineComponent({
  name: "\u0275Overlay",
  inheritAttrs: false,
  props: overlayProps,
  setup(props, {
    slots,
    attrs,
    expose
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId
    } = useThemeToken();
    const mergedPrefixCls = computed(() => `${common.prefixCls}-overlay`);
    const contentArrowRef = ref();
    const {
      options: popperOptions,
      update: updateOptions
    } = usePopperOptions(props, contentArrowRef);
    const {
      isHovered,
      isFocused,
      statePopperEvents,
      stateTriggerEvents
    } = useOverlayStates();
    const {
      visible,
      visibleLocked,
      updateVisible,
      lock,
      unlock
    } = useVisible(props, popperOptions.trigger, isHovered, isFocused);
    const {
      arrowRef,
      popperRef,
      popperEvents,
      triggerRef,
      triggerEvents,
      visibility,
      placement,
      initialize,
      update,
      hide,
      destroy
    } = usePopper({
      ...popperOptions,
      visible,
      onVisibleChange: updateVisible
    });
    const {
      mergedTriggerEvents,
      mergedPopperEvents
    } = useMergedEvents(triggerEvents, popperEvents, stateTriggerEvents, statePopperEvents);
    const currentZIndex = useZIndex(toRef(props, "zIndex"), toRef(common, "overlayZIndex"), visibility);
    const mergedContainer = computed(() => {
      var _a;
      const {
        container = common.overlayContainer
      } = props;
      return (_a = isFunction(container) ? container(convertElement(triggerRef)) : container) != null ? _a : props.containerFallback;
    });
    onMounted(() => initialize());
    onBeforeUnmount(() => destroy());
    watch(visibility, (value) => {
      if (value) {
        nextTick(updateOptions);
        props.destroyOnHide && initialize();
      }
    });
    watch(placement, (value) => callEmit(props["onUpdate:placement"], value));
    watch(contentArrowRef, () => {
      arrowRef.value = contentArrowRef.value;
    }, {
      immediate: true
    });
    const onAfterLeave = () => {
      if (props.destroyOnHide) {
        destroy();
      }
      callEmit(props.onAfterLeave);
    };
    expose({
      updatePopper: update,
      getPopperElement: () => convertElement(popperRef)
    });
    const handleClickOutside = (evt) => {
      if (visibleLocked.value) {
        return;
      }
      const popperElement = convertElement(popperRef);
      const target = evt.target;
      if (!popperElement || popperElement === target || popperElement.contains(target)) {
        return;
      }
      hide();
    };
    return () => {
      var _a, _b;
      const triggerNode = getFirstValidNode((_a = slots.default) == null ? void 0 : _a.call(slots));
      if (!triggerNode) {
        process.env.NODE_ENV !== "production" && Logger.warn("components/overlay", "Trigger must is single rooted node");
        return null;
      }
      const trigger = renderTrigger(props, triggerNode, {
        ref: triggerRef,
        ...mergedTriggerEvents.value
      }, handleClickOutside);
      const contentNode = (_b = slots.content) == null ? void 0 : _b.call(slots);
      if (!getFirstValidNode(contentNode)) {
        return createVNode(Fragment, null, [trigger, createVNode(CdkPortal, {
          "target": mergedContainer.value,
          "load": false
        }, null)]);
      }
      const content = renderContent(props, globalHashId, mergedPrefixCls, visibility, currentZIndex, contentNode, contentArrowRef, popperRef, mergedPopperEvents, attrs, lock, unlock);
      return createVNode(Fragment, null, [trigger, createVNode(CdkPortal, {
        "target": mergedContainer.value,
        "load": props.lazy ? visibility.value : true
      }, {
        default: () => [createVNode(Transition, {
          "appear": true,
          "name": props.transitionName,
          "onAfterLeave": onAfterLeave
        }, {
          default: () => [content]
        })]
      })]);
    };
  }
});
function renderContent(props, globalHashId, mergedPrefixCls, visibility, currentZIndex, contentNode, arrowRef, popperRef, popperEvents, attrs, lock, unlock) {
  if (props.destroyOnHide && !visibility.value) {
    return null;
  }
  const prefixCls = mergedPrefixCls.value;
  const {
    triggerId
  } = props;
  const overlayId = triggerId != null ? `__IDUX_OVERLAY-${triggerId}` : void 0;
  const style = `z-index: ${currentZIndex.value}`;
  const overlay = createVNode(Content, mergeProps({
    "ref": popperRef,
    "id": overlayId,
    "class": [prefixCls, globalHashId.value],
    "style": style,
    "lock": lock,
    "unlock": unlock
  }, popperEvents.value, attrs), {
    default: () => [contentNode, props.showArrow && createVNode("div", {
      "ref": arrowRef,
      "class": `${prefixCls}-arrow`
    }, null)]
  });
  return props.destroyOnHide ? overlay : withDirectives(overlay, [[vShow, visibility.value]]);
}
function renderTrigger(props, triggerNode, extraProps, handleClickOutside) {
  const element = cloneVNode(triggerNode, extraProps, true);
  if (props.clickOutside) {
    return withDirectives(element, [[vClickOutside, handleClickOutside]]);
  }
  return element;
}
const \u0275Overlay = Overlay;
export { \u0275Overlay, overlayDelayDef as \u0275OverlayDelayDef, overlayPlacementDef as \u0275OverlayPlacementDef, overlayTriggerDef as \u0275OverlayTriggerDef };
