/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { PopperOptions, PopperPlacement, PopperTrigger } from '@idux/cdk/popper';
import type { ExtractInnerPropTypes, ExtractPublicPropTypes, MaybeArray } from '@idux/cdk/utils';
import type { OverlayContainerType } from '@idux/components/utils';
import type { DefineComponent, HTMLAttributes, PropType } from 'vue';
export declare const overlayPlacementDef: PropType<PopperPlacement>;
export declare const overlayTriggerDef: PropType<PopperTrigger>;
export declare const overlayDelayDef: PropType<number | [number | null, number | null]>;
export declare const overlayProps: {
    readonly visible: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly placement: PropType<PopperPlacement>;
    readonly allowEnter: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly autoAdjust: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly clickOutside: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly container: {
        readonly type: PropType<OverlayContainerType>;
        readonly default: undefined;
    };
    readonly containerFallback: {
        readonly type: StringConstructor;
        readonly required: true;
    };
    readonly delay: PropType<number | [number | null, number | null]>;
    readonly destroyOnHide: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly lazy: {
        readonly type: BooleanConstructor;
        readonly default: true;
    };
    readonly offset: PropType<[number, number]>;
    readonly showArrow: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly transitionName: StringConstructor;
    readonly trigger: PropType<PopperTrigger>;
    readonly triggerId: {
        readonly type: null;
        readonly default: undefined;
    };
    readonly zIndex: NumberConstructor;
    readonly 'onUpdate:visible': PropType<MaybeArray<(visible: boolean) => void>>;
    readonly 'onUpdate:placement': PropType<MaybeArray<(placement: PopperPlacement) => void>>;
    readonly onAfterLeave: PropType<MaybeArray<() => void>>;
};
export interface OverlayBindings {
    updatePopper: (options?: Partial<PopperOptions>) => void;
    getPopperElement: () => HTMLElement | undefined | null;
}
export type OverlayProps = ExtractInnerPropTypes<typeof overlayProps>;
export type OverlayPublicProps = ExtractPublicPropTypes<typeof overlayProps>;
export type OverlayComponent = DefineComponent<Omit<HTMLAttributes, keyof OverlayPublicProps> & OverlayPublicProps, OverlayBindings>;
export type OverlayInstance = InstanceType<DefineComponent<OverlayProps, OverlayBindings>>;
