/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { ExtractInnerPropTypes, ExtractPublicPropTypes } from '@idux/cdk/utils';
import type { DefineComponent, HTMLAttributes, PropType } from 'vue';
import { MaybeArray } from '@idux/cdk/utils';
export type AmPm = 'am' | 'pm';
export declare const baseTimePanelProps: {
    readonly disabledHours: {
        readonly type: PropType<(selectedAmPm: AmPm | undefined) => number[]>;
        readonly default: () => never[];
    };
    readonly disabledMinutes: {
        readonly type: PropType<(selectedHour: number | undefined, selectedAmPm: AmPm | undefined) => number[]>;
        readonly default: () => never[];
    };
    readonly disabledSeconds: {
        readonly type: PropType<(selectedHour: number | undefined, selectedMinute: number | undefined, selectedAmPm: AmPm | undefined) => number[]>;
        readonly default: () => never[];
    };
    readonly hideDisabledOptions: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly hourStep: {
        readonly type: NumberConstructor;
        readonly default: 1;
    };
    readonly minuteStep: {
        readonly type: NumberConstructor;
        readonly default: 1;
    };
    readonly secondStep: {
        readonly type: NumberConstructor;
        readonly default: 1;
    };
};
export declare const timePanelProps: {
    readonly value: DateConstructor;
    readonly activeValue: DateConstructor;
    readonly visible: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly hourEnabled: {
        readonly type: BooleanConstructor;
        readonly default: true;
    };
    readonly minuteEnabled: {
        readonly type: BooleanConstructor;
        readonly default: true;
    };
    readonly secondEnabled: {
        readonly type: BooleanConstructor;
        readonly default: true;
    };
    readonly use12Hours: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly 'onUpdate:value': PropType<MaybeArray<(value: Date) => void>>;
    readonly 'onUpdate:activeValue': PropType<MaybeArray<(value: Date) => void>>;
    readonly onChange: PropType<MaybeArray<(value: Date) => void>>;
    readonly disabledHours: {
        readonly type: PropType<(selectedAmPm: AmPm | undefined) => number[]>;
        readonly default: () => never[];
    };
    readonly disabledMinutes: {
        readonly type: PropType<(selectedHour: number | undefined, selectedAmPm: AmPm | undefined) => number[]>;
        readonly default: () => never[];
    };
    readonly disabledSeconds: {
        readonly type: PropType<(selectedHour: number | undefined, selectedMinute: number | undefined, selectedAmPm: AmPm | undefined) => number[]>;
        readonly default: () => never[];
    };
    readonly hideDisabledOptions: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly hourStep: {
        readonly type: NumberConstructor;
        readonly default: 1;
    };
    readonly minuteStep: {
        readonly type: NumberConstructor;
        readonly default: 1;
    };
    readonly secondStep: {
        readonly type: NumberConstructor;
        readonly default: 1;
    };
};
export type TimePanelProps = ExtractInnerPropTypes<typeof timePanelProps>;
export type TimePanelPublicProps = ExtractPublicPropTypes<typeof timePanelProps>;
export type TimePanelComponent = DefineComponent<Omit<HTMLAttributes, keyof TimePanelPublicProps> & TimePanelPublicProps>;
export type TimePanelInstance = InstanceType<DefineComponent<TimePanelProps>>;
export interface TimePanelCell {
    value: number | AmPm;
    disabled: boolean;
}
export declare const timePanelColumnProps: {
    readonly selectedValue: PropType<number | AmPm>;
    readonly activeValue: {
        readonly type: PropType<number | AmPm>;
        readonly required: true;
    };
    readonly options: {
        readonly type: PropType<TimePanelCell[]>;
        readonly require: true;
    };
    readonly visible: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly onActiveChange: PropType<MaybeArray<(cell: TimePanelCell) => void>>;
};
export declare const timePanelCellProps: {
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly selected: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly value: {
        readonly type: PropType<number | AmPm>;
        readonly required: true;
    };
    readonly onActive: PropType<MaybeArray<(cell: TimePanelCell) => void>>;
};
export type BaseTimePanelProps = ExtractInnerPropTypes<typeof baseTimePanelProps>;
export type TimePanelColumnProps = ExtractInnerPropTypes<typeof timePanelColumnProps>;
export type TimePanelCellProps = ExtractInnerPropTypes<typeof timePanelCellProps>;
export type TimePanelColumnType = 'hour' | 'minute' | 'second' | 'AM/PM';
