/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { ValidateStatus } from '@idux/cdk/forms';
import type { ExtractInnerPropTypes, ExtractPublicPropTypes, MaybeArray } from '@idux/cdk/utils';
import type { FormSize } from '@idux/components/form';
import type { DefineComponent, HTMLAttributes, PropType, Slot } from 'vue';
export declare const triggerProps: {
    readonly value: {
        readonly type: null;
        readonly default: undefined;
    };
    readonly raw: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly borderless: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly clearable: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly clearIcon: {
        readonly type: PropType<string>;
        readonly default: "close-circle";
    };
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly focused: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly monitorFocus: {
        readonly type: BooleanConstructor;
        readonly default: true;
    };
    readonly paddingless: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly placeholder: StringConstructor;
    readonly readonly: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly size: {
        readonly type: PropType<FormSize>;
        readonly default: "md";
    };
    readonly status: PropType<ValidateStatus>;
    readonly suffix: PropType<string>;
    readonly suffixRotate: {
        readonly type: PropType<string | number | boolean>;
        readonly default: undefined;
    };
    readonly onClear: PropType<MaybeArray<(evt: MouseEvent) => void>>;
    readonly onFocus: PropType<MaybeArray<(evt: FocusEvent) => void>>;
    readonly onBlur: PropType<MaybeArray<(evt: FocusEvent) => void>>;
};
interface TriggerBindings {
    blur: () => void;
    focus: (options?: FocusOptions) => void;
}
export type TriggerDefaultSlotParams = {
    value: any;
    borderless: boolean;
    disabled: boolean;
    readonly: boolean;
    focused: boolean;
    size: FormSize;
    status: ValidateStatus | undefined;
    suffix: string | undefined;
    suffixRotate: boolean | string | number | undefined;
    clearable: boolean;
    clearIcon: string;
};
export type TriggerSlots = {
    suffix?: Slot;
    clearIcon?: Slot;
    default?: Slot<TriggerDefaultSlotParams>;
    placeholder?: Slot;
};
export type TriggerProps = ExtractInnerPropTypes<typeof triggerProps>;
export type TriggerPublicProps = ExtractPublicPropTypes<typeof triggerProps>;
export type TriggerComponent = DefineComponent<Omit<HTMLAttributes, keyof TriggerPublicProps> & TriggerPublicProps, TriggerBindings>;
export type TriggerInstance = InstanceType<DefineComponent<TriggerProps, TriggerBindings>>;
export {};
