/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { ExtractInnerPropTypes, ExtractPublicPropTypes, MaybeArray } from '@idux/cdk/utils';
import type { DefineComponent, HTMLAttributes, PropType } from 'vue';
export declare const alertProps: {
    readonly banner: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly closable: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly closeIcon: {
        readonly type: StringConstructor;
        readonly default: "close";
    };
    readonly description: StringConstructor;
    readonly icon: StringConstructor;
    readonly title: PropType<string | string[]>;
    readonly pagination: {
        readonly type: PropType<boolean | AlertPagination>;
        readonly default: false;
    };
    readonly type: {
        readonly type: PropType<AlertType>;
        readonly default: "warning";
    };
    readonly onAfterClose: PropType<MaybeArray<() => void>>;
    readonly onBeforeClose: PropType<MaybeArray<() => void | boolean | Promise<boolean>>>;
    readonly onClose: PropType<MaybeArray<() => void>>;
};
export type AlertProps = ExtractInnerPropTypes<typeof alertProps>;
export type AlertPublicProps = ExtractPublicPropTypes<typeof alertProps>;
export type AlertComponent = DefineComponent<Omit<HTMLAttributes, keyof AlertPublicProps> & AlertPublicProps>;
export type AlertInstance = InstanceType<DefineComponent<AlertProps>>;
export type AlertType = 'success' | 'info' | 'warning' | 'error' | 'offline';
export interface AlertPagination {
    pageIndex?: number;
    onChange?: (pageIndex: number) => void;
}
