import { defineComponent, computed, ref, onMounted, nextTick, onUnmounted, createVNode, Transition, withDirectives, vShow } from "vue";
import { throttle } from "lodash-es";
import { getScroll, scrollToTop } from "@idux/cdk/scroll";
import { on, off, callEmit } from "@idux/cdk/utils";
import { useGlobalConfig } from "@idux/components/config";
import { IxIcon } from "@idux/components/icon";
import { useThemeToken } from "@idux/components/theme";
import { convertTarget } from "@idux/components/utils";
const backTopProps = {
  target: [String, HTMLElement, Function],
  duration: Number,
  visibilityHeight: Number,
  onClick: [Function, Array]
};
function getDefaultThemeTokens(tokens, algorithms) {
  const { getColorPalette } = algorithms;
  const { overlayBorderWidth, overlayBorderType, overlayBorderColor, colorPrimary, colorContainerBg } = tokens;
  const primaryColorPalette = getColorPalette(colorPrimary);
  return {
    sizeLg: 64,
    sizeMd: 48,
    sizeSm: 40,
    boxShadow: `0 2px 8px 0 rgba(0, 0, 0, 0.05)`,
    color: primaryColorPalette.l30,
    colorHover: colorPrimary,
    bgColor: colorContainerBg,
    borderWidth: overlayBorderWidth,
    borderType: overlayBorderType,
    borderColor: overlayBorderColor
  };
}
const getThemeTokens = (tokens, presetTheme, algorithms) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens, algorithms) : getDefaultThemeTokens(tokens, algorithms);
};
var BackTop = /* @__PURE__ */ defineComponent({
  name: "IxBackTop",
  props: backTopProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("backTop");
    registerToken(getThemeTokens);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-back-top`);
    const config = useGlobalConfig("backTop");
    const eventType = "scroll";
    const visible = ref(false);
    const container = ref(null);
    const duration = computed(() => {
      var _a;
      return (_a = props.duration) != null ? _a : config.duration;
    });
    const handleScroll = () => {
      var _a;
      const {
        scrollTop
      } = getScroll(container.value);
      visible.value = scrollTop >= ((_a = props.visibilityHeight) != null ? _a : config.visibilityHeight);
    };
    const handleClick = (evt) => {
      evt.stopPropagation();
      scrollToTop({
        top: 0,
        duration: duration.value,
        target: container.value
      });
      callEmit(props.onClick, evt);
    };
    const throttledScrollHandler = throttle(handleScroll, 300);
    onMounted(() => {
      nextTick(() => {
        container.value = convertTarget(props.target);
        on(container.value, eventType, throttledScrollHandler);
        handleScroll();
      });
    });
    onUnmounted(() => {
      off(container.value, eventType, throttledScrollHandler);
    });
    return () => {
      return createVNode(Transition, {
        "name": `${common.prefixCls}-fade`
      }, {
        default: () => [withDirectives(createVNode("div", {
          "class": [mergedPrefixCls.value, globalHashId.value, hashId.value],
          "onClick": handleClick
        }, [slots.default ? slots.default() : createVNode(IxIcon, {
          "name": "arrow-up"
        }, null)]), [[vShow, visible.value]])]
      });
    };
  }
});
const IxBackTop = BackTop;
export { IxBackTop, getThemeTokens as getBackTopThemeTokens };
