import { defineComponent, computed, provide, toRef, createVNode, inject } from "vue";
import { useGlobalConfig } from "@idux/components/config";
import { useThemeToken } from "@idux/components/theme";
import { Logger } from "@idux/cdk/utils";
const breadcrumbToken = Symbol("breadcrumbToken");
const breadcrumbProps = {
  separator: {
    type: String,
    default: "/"
  }
};
const breadcrumbItemProps = {
  separator: String
};
function getDefaultThemeTokens(tokens) {
  const { fontSizeSm, fontSizeMd, colorTextInfo, colorLink, colorTextDisabled, colorTextTitle, marginSizeXs } = tokens;
  return {
    fontSize: fontSizeMd,
    itemColor: colorTextInfo,
    linkColor: colorTextInfo,
    linkColorHover: colorLink,
    separatorColor: colorTextDisabled,
    separatorFontSize: fontSizeSm,
    lastItemColor: colorTextTitle,
    separatorMargin: `0 ${marginSizeXs}px`
  };
}
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens) : getDefaultThemeTokens(tokens);
};
var Breadcrumb = /* @__PURE__ */ defineComponent({
  name: "IxBreadcrumb",
  props: breadcrumbProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("breadcrumb");
    registerToken(getThemeTokens);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-breadcrumb`);
    provide(breadcrumbToken, {
      mergedPrefixCls,
      separatorRef: toRef(props, "separator")
    });
    return () => {
      var _a;
      return createVNode("nav", {
        "class": [mergedPrefixCls.value, globalHashId.value, hashId.value],
        "aria-label": "Breadcrumb"
      }, [createVNode("ol", null, [(_a = slots.default) == null ? void 0 : _a.call(slots)])]);
    };
  }
});
var BreadcrumbItem = /* @__PURE__ */ defineComponent({
  name: "IxBreadcrumbItem",
  props: breadcrumbItemProps,
  setup(props, {
    slots
  }) {
    const breadcrumbInjection = inject(breadcrumbToken, null);
    if (!breadcrumbInjection) {
      if (process.env.NODE_ENV !== "production") {
        Logger.error("components/breadcrumb", "`breadcrumb-item` must be placed inside `breadcrumb`.");
      }
      return () => null;
    }
    const {
      mergedPrefixCls,
      separatorRef
    } = breadcrumbInjection;
    return () => {
      var _a, _b;
      return createVNode("li", {
        "class": `${mergedPrefixCls.value}-item`
      }, [createVNode("span", {
        "class": `${mergedPrefixCls.value}-item-link`
      }, [(_a = slots.default) == null ? void 0 : _a.call(slots)]), createVNode("span", {
        "class": `${mergedPrefixCls.value}-item-separator`,
        "aria-hidden": "true"
      }, [slots.separator ? slots.separator() : (_b = props.separator) != null ? _b : separatorRef.value])]);
    };
  }
});
const IxBreadcrumb = Breadcrumb;
const IxBreadcrumbItem = BreadcrumbItem;
export { IxBreadcrumb, IxBreadcrumbItem, getThemeTokens as getBreadcrumbThemeTokens };
