import { defineComponent, computed, inject, ref, normalizeClass, createVNode, provide } from "vue";
import { callEmit } from "@idux/cdk/utils";
import { \u0275Wave as _Wave } from "@idux/components/_private/wave";
import { useGlobalConfig } from "@idux/components/config";
import { FORM_TOKEN } from "@idux/components/form";
import { IxIcon } from "@idux/components/icon";
import { getSolidColor, getAlphaColor, useThemeToken } from "@idux/components/theme";
import { IxSpace } from "@idux/components/space";
const buttonToken = Symbol("button");
const buttonProps = {
  block: { type: Boolean, default: void 0 },
  danger: { type: Boolean, default: void 0 },
  disabled: { type: Boolean, default: void 0 },
  ghost: { type: Boolean, default: void 0 },
  icon: String,
  loading: { type: Boolean, default: void 0 },
  mode: String,
  size: String,
  shape: String,
  type: { type: String, default: "button" },
  waveless: { type: Boolean, default: void 0 },
  onClick: [Function, Array]
};
const buttonGroupProps = {
  align: { type: String, default: "center" },
  block: { type: Boolean, default: void 0 },
  danger: { type: Boolean, default: void 0 },
  disabled: { type: Boolean, default: void 0 },
  gap: { type: [Number, String], default: 0 },
  ghost: { type: Boolean, default: void 0 },
  mode: String,
  size: String,
  shape: String
};
function getDefaultThemeTokens(tokens) {
  const {
    fontSizeSm,
    fontSizeMd,
    fontSizeLg,
    heightSm,
    heightMd,
    heightLg,
    heightXl,
    paddingSizeSm,
    paddingSizeMd,
    paddingSizeLg,
    paddingSizeXl,
    paddingSize2Xl,
    colorPrimary,
    colorPrimaryHover,
    colorPrimaryActive,
    colorErrorBg,
    colorErrorBgHover,
    colorErrorBgActive,
    colorErrorText,
    colorErrorTextHover,
    colorErrorTextActive,
    colorText,
    colorTextInverse,
    colorTextDisabled,
    colorContainerBg,
    colorContainerBgDisabled,
    colorBorder,
    borderRadiusSm,
    fontSizeIcon
  } = tokens;
  return {
    fontSizeXs: fontSizeSm,
    fontSizeSm,
    fontSizeMd,
    fontSizeLg,
    fontSizeXl: fontSizeLg,
    minWidthXs: 56,
    minWidthSm: 64,
    minWidthMd: 72,
    minWidthLg: 120,
    minWidthXl: 160,
    heightXs: heightSm,
    heightSm: heightMd,
    heightMd,
    heightLg,
    heightXl,
    paddingSizeHorizontalXs: paddingSizeSm,
    paddingSizeHorizontalSm: paddingSizeMd,
    paddingSizeHorizontalMd: paddingSizeLg,
    paddingSizeHorizontalLg: paddingSizeXl,
    paddingSizeHorizontalXl: paddingSize2Xl,
    color: colorText,
    colorDisabled: colorTextDisabled,
    bgColor: colorContainerBg,
    bgColorDisabled: colorContainerBgDisabled,
    borderColor: colorBorder,
    borderRadius: borderRadiusSm,
    primaryColor: colorTextInverse,
    primaryBgColor: colorPrimary,
    primaryBgColorHover: colorPrimaryHover,
    primaryBgColorActive: colorPrimaryActive,
    dangerColor: colorErrorText,
    dangerColorHover: colorErrorTextHover,
    dangerColorActive: colorErrorTextActive,
    dangerBgColor: colorErrorBg,
    dangerBgColorHover: colorErrorBgHover,
    dangerBgColorActive: colorErrorBgActive,
    ghostColor: colorTextInverse,
    ghostColorHover: colorTextInverse,
    ghostColorActive: getSolidColor(colorTextInverse, 65),
    ghostBorderColorHover: colorTextInverse,
    ghostBorderColorActive: colorTextInverse,
    ghostBorderColor: colorTextInverse,
    ghostBgColorHover: getAlphaColor(colorTextInverse, 0.2),
    ghostBgColorActive: colorTextInverse,
    ghostBgColorDisabled: getAlphaColor(colorTextInverse, 0.4),
    iconColor: "inherit",
    iconFontSize: fontSizeIcon
  };
}
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens) : getDefaultThemeTokens(tokens);
};
const aProps = ["href", "target", "rel", "download", "hreflang", "ping"];
var Button = /* @__PURE__ */ defineComponent({
  name: "IxButton",
  props: buttonProps,
  setup(props, {
    slots,
    attrs
  }) {
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("button");
    registerToken(getThemeTokens);
    const common = useGlobalConfig("common");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-button`);
    const config = useGlobalConfig("button");
    const groupProps = inject(buttonToken, {});
    const formContext = inject(FORM_TOKEN, null);
    const waveRef = ref();
    const mode = computed(() => {
      var _a, _b;
      return (_b = (_a = props.mode) != null ? _a : groupProps.mode) != null ? _b : "default";
    });
    const size = computed(() => {
      var _a, _b, _c;
      return (_c = (_b = (_a = props.size) != null ? _a : groupProps.size) != null ? _b : formContext == null ? void 0 : formContext.size.value) != null ? _c : config.size;
    });
    const mergedWaveless = computed(() => {
      var _a;
      return mode.value === "text" || mode.value === "link" || ((_a = props.waveless) != null ? _a : config.waveless);
    });
    const mergedDisabled = computed(() => {
      var _a;
      return (_a = props.disabled) != null ? _a : groupProps.disabled;
    });
    const classes = computed(() => {
      const {
        block = groupProps.block,
        danger = groupProps.danger,
        ghost = groupProps.ghost,
        loading,
        icon,
        shape = groupProps.shape
      } = props;
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [prefixCls]: true,
        [`${prefixCls}-block`]: block,
        [`${prefixCls}-danger`]: danger,
        [`${prefixCls}-disabled`]: mergedDisabled.value,
        [`${prefixCls}-ghost`]: ghost,
        [`${prefixCls}-loading`]: loading,
        [`${prefixCls}-icon-only`]: !slots.default && (icon || loading || slots.icon),
        [`${prefixCls}-${mode.value}`]: mode.value,
        [`${prefixCls}-${shape}`]: !!shape,
        [`${prefixCls}-${size.value}`]: true
      });
    });
    const handleClick = (evt) => {
      if (mergedDisabled.value || props.loading) {
        evt.preventDefault();
        evt.stopImmediatePropagation();
        return;
      }
      if (!mergedWaveless.value && waveRef.value) {
        waveRef.value.play();
      }
      callEmit(props.onClick, evt);
    };
    return () => {
      const {
        loading,
        icon,
        type
      } = props;
      const children = [];
      if (loading) {
        children.push(createVNode(IxIcon, {
          "name": "loading"
        }, null));
      } else if (slots.icon) {
        children.push(slots.icon());
      } else if (icon) {
        children.push(createVNode(IxIcon, {
          "name": icon
        }, null));
      }
      if (slots.default) {
        children.push(createVNode("span", null, [slots.default()]));
      }
      if (mode.value === "link" && Object.keys(attrs).some((attr) => aProps.includes(attr))) {
        return createVNode("a", {
          "class": classes.value,
          "onClick": handleClick
        }, [children]);
      }
      return createVNode("button", {
        "class": classes.value,
        "disabled": mergedDisabled.value || loading,
        "type": type,
        "onClick": handleClick
      }, [children, !mergedWaveless.value && createVNode(_Wave, {
        "ref": waveRef
      }, null)]);
    };
  }
});
var ButtonGroup = /* @__PURE__ */ defineComponent({
  name: "IxButtonGroup",
  props: buttonGroupProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-button-group`);
    const classes = computed(() => {
      const {
        gap
      } = props;
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [prefixCls]: true,
        [`${prefixCls}-compact`]: !gap || gap === "0"
      });
    });
    provide(buttonToken, props);
    return () => createVNode(IxSpace, {
      "class": classes.value,
      "align": props.align,
      "block": props.block,
      "size": props.gap
    }, {
      default: () => [slots.default && slots.default()]
    });
  }
});
const IxButton = Button;
const IxButtonGroup = ButtonGroup;
export { IxButton, IxButtonGroup, getThemeTokens as getButtonThemeTokens };
