/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { ExtractInnerPropTypes, ExtractPublicPropTypes } from '@idux/cdk/utils';
import type { SpaceAlign, SpaceProps } from '@idux/components/space';
import type { AnchorHTMLAttributes, ButtonHTMLAttributes, DefineComponent, HTMLAttributes, PropType } from 'vue';
export type ButtonMode = 'primary' | 'default' | 'dashed' | 'text' | 'link';
export type ButtonShape = 'circle' | 'round' | 'square';
export type ButtonSize = 'lg' | 'xl' | 'md' | 'sm' | 'xs';
export type ButtonType = 'button' | 'submit' | 'reset';
export declare const buttonProps: {
    readonly block: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly danger: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly ghost: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly icon: StringConstructor;
    readonly loading: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly mode: PropType<ButtonMode>;
    readonly size: PropType<ButtonSize>;
    readonly shape: PropType<ButtonShape>;
    readonly type: {
        readonly type: PropType<ButtonType>;
        readonly default: "button";
    };
    readonly waveless: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly onClick: PropType<(evt: MouseEvent) => void>;
};
export type ButtonProps = ExtractInnerPropTypes<typeof buttonProps>;
export type ButtonPublicProps = ExtractPublicPropTypes<typeof buttonProps>;
export type ButtonComponent = DefineComponent<Omit<ButtonHTMLAttributes | AnchorHTMLAttributes, keyof ButtonPublicProps> & ButtonPublicProps>;
export type ButtonInstance = InstanceType<DefineComponent<ButtonProps>>;
export declare const buttonGroupProps: {
    readonly align: {
        readonly type: PropType<SpaceAlign>;
        readonly default: "center";
    };
    readonly block: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly danger: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly gap: {
        readonly type: PropType<string | number>;
        readonly default: 0;
    };
    readonly ghost: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly mode: PropType<ButtonMode>;
    readonly size: PropType<ButtonSize>;
    readonly shape: PropType<ButtonShape>;
};
export type ButtonGroupProps = ExtractInnerPropTypes<typeof buttonGroupProps>;
export type ButtonGroupPublicProps = ExtractPublicPropTypes<typeof buttonGroupProps> & Omit<SpaceProps, 'size'>;
export type ButtonGroupComponent = DefineComponent<Omit<HTMLAttributes, keyof ButtonGroupPublicProps> & ButtonGroupPublicProps>;
export type ButtonGroupInstance = InstanceType<DefineComponent<ButtonGroupProps>>;
