/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { ExtractInnerPropTypes, ExtractPublicPropTypes, MaybeArray } from '@idux/cdk/utils';
import type { ButtonProps } from '@idux/components/button';
import type { HeaderProps } from '@idux/components/header';
import type { DefineComponent, HTMLAttributes, PropType, VNode } from 'vue';
export type CardSize = 'sm' | 'md' | 'lg';
export interface CardCover {
    alt: string;
    src: string;
    srcset: string;
}
export interface CardButtonProps extends ButtonProps {
    key?: string | number;
    text?: string;
    onClick?: (evt: Event) => void;
}
export declare const cardProps: {
    readonly borderless: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly cover: PropType<string | CardCover>;
    readonly header: PropType<string | HeaderProps>;
    readonly hoverable: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly shadow: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly loading: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly size: PropType<CardSize>;
    readonly footer: PropType<(VNode<import("vue").RendererNode, import("vue").RendererElement, {
        [key: string]: any;
    }> | CardButtonProps)[]>;
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly selected: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly selectable: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly 'onUpdate:selected': PropType<MaybeArray<(selected: boolean) => void>>;
    readonly onSelectedChange: PropType<MaybeArray<(selected: boolean) => void>>;
};
export type CardProps = ExtractInnerPropTypes<typeof cardProps>;
export type CardPublicProps = ExtractPublicPropTypes<typeof cardProps>;
export type CardComponent = DefineComponent<Omit<HTMLAttributes, keyof CardPublicProps> & CardPublicProps>;
export type CardInstance = InstanceType<DefineComponent<CardProps>>;
export declare const cardGridProps: {
    readonly hoverable: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
};
export type CardGridProps = ExtractInnerPropTypes<typeof cardGridProps>;
export type CardGridPublicProps = ExtractPublicPropTypes<typeof cardGridProps>;
export type CardGridComponent = DefineComponent<Omit<HTMLAttributes, keyof CardGridPublicProps> & CardGridPublicProps>;
export type CardGridInstance = InstanceType<DefineComponent<CardGridProps>>;
