import { computed, toRaw, defineComponent, inject, normalizeClass, createVNode, mergeProps, ref, provide, toRef, watch } from "vue";
import { useAccessorAndControl } from "@idux/cdk/forms";
import { useControlledProp, callEmit, convertArray, NoopArray, useState } from "@idux/cdk/utils";
import { \u0275Input as _Input } from "@idux/components/_private/input";
import { useGlobalConfig } from "@idux/components/config";
import { IxControlTrigger } from "@idux/components/control-trigger";
import { useFormItemRegister, useFormStatus } from "@idux/components/form";
import { IxSelector } from "@idux/components/selector";
import { useThemeToken } from "@idux/components/theme";
import { convertIconVNode, useGetKey, useGetDisabled, useMergedCommonControlProps } from "@idux/components/utils";
import { isNil, isFunction } from "lodash-es";
import { CdkVirtualScroll } from "@idux/cdk/scroll";
import { \u0275Empty as _Empty } from "@idux/components/_private/empty";
import { IxCheckbox } from "@idux/components/checkbox";
function useDataSource(props, mergedGetKey, mergedChildrenKey, mergedLabelKey, mergedFullPath) {
  const mergedData = computed(() => {
    return convertMergedData(
      props,
      mergedGetKey.value,
      mergedChildrenKey.value,
      mergedLabelKey.value,
      mergedFullPath.value,
      props.dataSource
    );
  });
  const mergedDataMap = computed(() => {
    const map = /* @__PURE__ */ new Map();
    convertMergedDataMap(map, mergedData.value);
    return map;
  });
  return { mergedData, mergedDataMap };
}
function convertMergedData(props, getKey, childrenKey, labelKey, fullPath, data, parentKey, parentLabel) {
  const { loadChildren, separator } = props;
  return data.map(
    (item) => convertMergedItem(item, getKey, childrenKey, labelKey, separator, fullPath, !!loadChildren, parentKey, parentLabel)
  );
}
function convertMergedItem(rawData, getKey, childrenKey, labelKey, separator, fullPath, hasLoad, parentKey, parentLabel) {
  var _a;
  const key = getKey(rawData);
  const label = fullPath && !isNil(parentLabel) ? `${parentLabel}${separator}${rawData[labelKey]}` : rawData[labelKey];
  const subData = rawData[childrenKey];
  const children = subData == null ? void 0 : subData.map(
    (item) => convertMergedItem(item, getKey, childrenKey, labelKey, separator, fullPath, hasLoad, key, label)
  );
  return {
    children,
    key,
    isLeaf: (_a = rawData.isLeaf) != null ? _a : !((children == null ? void 0 : children.length) || hasLoad),
    label,
    parentKey,
    rawData
  };
}
function convertMergedDataMap(map, data) {
  data.forEach((item) => {
    const { key, children } = item;
    map.set(key, item);
    if (children) {
      convertMergedDataMap(map, children);
    }
  });
}
function usePanelProps(props, searchValue, setOverlayOpened) {
  const [expandedKeys, setExpandedKeys] = useControlledProp(props, "expandedKeys");
  const [loadedKeys, setLoadedKeys] = useControlledProp(props, "loadedKeys");
  const onSelect = () => {
    if (!props.multiple) {
      setOverlayOpened(false);
    }
  };
  return computed(() => ({
    expandedKeys: expandedKeys.value,
    loadedKeys: loadedKeys.value,
    childrenKey: props.childrenKey,
    customAdditional: props.customAdditional,
    disableData: props.disableData,
    empty: props.empty,
    expandIcon: props.expandIcon,
    expandTrigger: props.expandTrigger,
    fullPath: props.fullPath,
    getKey: props.getKey,
    labelKey: props.labelKey,
    loadChildren: props.loadChildren,
    maxLabel: props.maxLabel,
    multiple: props.multiple,
    multipleLimit: props.multipleLimit,
    searchable: props.searchable,
    searchFn: props.searchFn,
    searchValue: searchValue.value,
    strategy: props.strategy,
    virtual: props.virtual,
    virtualScrollMode: props.virtualScrollMode,
    virtualItemHeight: props.virtualItemHeight,
    "onUpdate:expandedKeys": setExpandedKeys,
    "onUpdate:loadedKeys": setLoadedKeys,
    onSelect,
    onLoaded: props.onLoaded
  }));
}
function useSelectedData(selectedKeys, mergedDataMap) {
  return computed(() => {
    const dataMap = mergedDataMap.value;
    return selectedKeys.value.map((key) => dataMap.get(key)).filter(Boolean);
  });
}
function callChange(mergedDataMap, newKeys, onChange) {
  if (onChange) {
    const data = [];
    const dataMap = mergedDataMap.value;
    newKeys.forEach((key) => {
      const currData = dataMap.get(key);
      currData && data.push(currData.rawData);
    });
    callEmit(onChange, newKeys, data);
  }
}
function getChildrenKeys(currData, filterDisabled, getDisabledFn) {
  if (!currData || !currData.children) {
    return [];
  }
  const keys = [];
  currData.children.forEach((item) => {
    const { key, rawData } = item;
    if (!getDisabledFn(rawData)) {
      keys.push(key);
      keys.push(...getChildrenKeys(item, filterDisabled, getDisabledFn));
    }
  });
  return keys;
}
function getParentKeys(dataMap, currData, filterDisabled, getDisabledFn) {
  const keys = [];
  while (currData && !isNil(currData.parentKey)) {
    const { parentKey, rawData } = currData;
    if (!filterDisabled || !getDisabledFn(rawData)) {
      keys.unshift(parentKey);
      currData = dataMap.get(parentKey);
    } else {
      currData = void 0;
    }
  }
  return keys;
}
function useSelectedState(mergedDataMap, mergedFullPath, mergedGetDisabled, multiple, strategy, selectedKeys, setSelectedKeys) {
  const resolvedSelectedKeys = computed(() => {
    const tempKeys = convertArray(selectedKeys.value);
    if (!mergedFullPath.value) {
      return tempKeys;
    }
    if (!multiple.value) {
      const lastKey = tempKeys[tempKeys.length - 1];
      return isNil(lastKey) ? [] : [lastKey];
    }
    return tempKeys.map((keys) => keys[keys.length - 1]).filter((key) => !isNil(key));
  });
  const strategyEnabled = computed(() => multiple.value && strategy.value !== "off");
  const selectedWithStrategyKeys = computed(() => {
    return strategyEnabled.value ? getCascadedKeys(mergedDataMap.value, resolvedSelectedKeys.value, mergedGetDisabled.value) : resolvedSelectedKeys.value;
  });
  const indeterminateKeys = computed(() => {
    if (!strategyEnabled.value) {
      return NoopArray;
    }
    const indeterminateKeySet = /* @__PURE__ */ new Set();
    const cascadedKeys = selectedWithStrategyKeys.value;
    const dataMap = mergedDataMap.value;
    const getDisabledFn = mergedGetDisabled.value;
    cascadedKeys.forEach((key) => {
      let currData = dataMap.get(key);
      while (currData && !isNil(currData.parentKey)) {
        const parentKey = currData.parentKey;
        const parent = dataMap.get(parentKey);
        if (parent && !getDisabledFn(parent.rawData) && !cascadedKeys.includes(parentKey)) {
          indeterminateKeySet.add(parentKey);
        }
        currData = parent;
      }
    });
    return [...indeterminateKeySet];
  });
  const getFullPath = (currKey) => {
    const getDisabledFn = mergedGetDisabled.value;
    const dataMap = mergedDataMap.value;
    const fullPath = getParentKeys(dataMap, dataMap.get(currKey), true, getDisabledFn);
    fullPath.push(currKey);
    return fullPath;
  };
  const getCurrValue = (keys) => {
    if (!mergedFullPath.value) {
      return multiple.value ? keys : keys[0];
    }
    if (keys.length === 0) {
      return keys;
    }
    if (multiple.value) {
      return keys.map(getFullPath);
    } else {
      return getFullPath(keys[0]);
    }
  };
  const setValue = (keys) => {
    const currValue = getCurrValue(keys);
    const oldValue = toRaw(selectedKeys.value);
    if (currValue !== oldValue) {
      setSelectedKeys(currValue);
    }
  };
  const handleSelect = (key) => {
    const cascadedKeys = selectedWithStrategyKeys.value;
    const currIndex = cascadedKeys.indexOf(key);
    const isSelected = currIndex > -1;
    if (!multiple.value) {
      !isSelected && setValue([key]);
      return;
    }
    const dataMap = mergedDataMap.value;
    const currData = dataMap.get(key);
    const _strategyEnabled = strategyEnabled.value;
    const getDisabledFn = mergedGetDisabled.value;
    const childrenKeys = _strategyEnabled ? getChildrenKeys(currData, true, getDisabledFn) : [];
    const keySet = new Set(cascadedKeys);
    if (isSelected || childrenKeys.length > 0 && childrenKeys.every((key2) => keySet.has(key2) || indeterminateKeys.value.includes(key2))) {
      keySet.delete(key);
      if (_strategyEnabled) {
        getParentKeys(dataMap, currData, true, getDisabledFn).forEach((key2) => keySet.delete(key2));
        childrenKeys.forEach((key2) => keySet.delete(key2));
      }
    } else {
      keySet.add(key);
      if (_strategyEnabled) {
        setParentChecked(dataMap, currData, keySet, getDisabledFn);
        childrenKeys.forEach((key2) => keySet.add(key2));
      }
    }
    setValue([...getCascadedKeysByStrategy(dataMap, keySet, strategy.value)]);
  };
  return {
    resolvedSelectedKeys,
    selectedWithStrategyKeys,
    strategyEnabled,
    handleSelect,
    setValue
  };
}
function getCascadedKeys(dataMap, checkedKys, getDisabledFn) {
  const keySet = new Set(checkedKys);
  let lastParentKey;
  checkedKys.forEach((checkedKey) => {
    const currData = dataMap.get(checkedKey);
    if (!currData) {
      return;
    }
    const { parentKey } = currData;
    const childrenKeys = getChildrenKeys(currData, true, getDisabledFn);
    childrenKeys.forEach((key) => keySet.add(key));
    if (parentKey && lastParentKey !== parentKey) {
      setParentChecked(dataMap, currData, keySet, getDisabledFn);
      lastParentKey = parentKey;
    }
  });
  return [...keySet];
}
function setParentChecked(dataMap, currData, keySet, getDisabledFn) {
  let parentSelected = true;
  while (parentSelected && currData && !isNil(currData.parentKey)) {
    const parentKey = currData.parentKey;
    const parent = dataMap.get(parentKey);
    if (parent && !getDisabledFn(currData.rawData)) {
      parentSelected = parent.children.every((item) => getDisabledFn(item.rawData) || keySet.has(item.key));
      parentSelected && keySet.add(currData.parentKey);
    }
    currData = parent;
  }
}
function getCascadedKeysByStrategy(dataMap, ketSet, strategy) {
  if (strategy === "parent") {
    const newSet = /* @__PURE__ */ new Set();
    ketSet.forEach((checkedKey) => {
      const currData = dataMap.get(checkedKey);
      if (currData) {
        const { key: selfKey, parentKey } = currData;
        if (!parentKey || !ketSet.has(parentKey)) {
          newSet.add(selfKey);
        }
      }
    });
    return newSet;
  }
  if (strategy === "child") {
    const newSet = /* @__PURE__ */ new Set();
    ketSet.forEach((checkedKey) => {
      const currData = dataMap.get(checkedKey);
      if (currData && currData.isLeaf) {
        newSet.add(currData.key);
      }
    });
    return newSet;
  }
  return ketSet;
}
const CASCADER_PANEL_DATA_TOKEN = Symbol("CASCADER_PANEL_DATA_TOKEN");
const cascaderPanelToken = Symbol("cascaderPanelToken");
var Option = /* @__PURE__ */ defineComponent({
  props: {
    children: {
      type: Array,
      default: void 0
    },
    index: {
      type: Number,
      required: true
    },
    isLeaf: {
      type: Boolean,
      required: true
    },
    label: {
      type: String,
      required: true
    },
    optionKey: {
      type: [String, Number, Symbol],
      required: true
    },
    parentKey: {
      type: [String, Number, Symbol],
      default: void 0
    },
    rawData: {
      type: Object,
      required: true
    }
  },
  setup(props) {
    const {
      props: cascaderPanelProps2,
      slots,
      mergedPrefixCls,
      mergedExpandIcon,
      mergedGetDisabled,
      mergedLabelKey,
      activeKey,
      setActiveKey,
      expandedKeys,
      setExpandedKeys,
      loadingKeys,
      selectedWithStrategyKeys,
      selectedLimit,
      selectedLimitTitle,
      indeterminateKeys,
      handleSelect,
      handleExpand
    } = inject(cascaderPanelToken);
    const isActive = computed(() => props.optionKey === activeKey.value);
    const isDisabled = computed(() => mergedGetDisabled.value(props.rawData));
    const isExpanded = computed(() => expandedKeys.value.includes(props.optionKey));
    const isLoading = computed(() => loadingKeys.value.includes(props.optionKey));
    const isSelected = computed(() => selectedWithStrategyKeys.value.includes(props.optionKey));
    const isIndeterminate = computed(() => indeterminateKeys.value.includes(props.optionKey));
    const classes = computed(() => {
      const prefixCls = `${mergedPrefixCls.value}-option`;
      return normalizeClass({
        [prefixCls]: true,
        [`${prefixCls}-leaf`]: props.isLeaf,
        [`${prefixCls}-active`]: isActive.value,
        [`${prefixCls}-disabled`]: isDisabled.value,
        [`${prefixCls}-expanded`]: isExpanded.value,
        [`${prefixCls}-loading`]: isLoading.value,
        [`${prefixCls}-selected`]: isSelected.value
      });
    });
    const _handleSelect = () => {
      handleSelect(props.optionKey);
      callEmit(cascaderPanelProps2.onSelect, props.rawData, isSelected.value);
    };
    const handleClick = () => {
      if (props.isLeaf) {
        if (!isSelected.value && selectedLimit.value) {
          return;
        }
        _handleSelect();
        isNil(props.parentKey) && setExpandedKeys([]);
      } else {
        cascaderPanelProps2.strategy === "off" && _handleSelect();
        cascaderPanelProps2.expandTrigger === "click" && handleExpand(props.optionKey);
      }
    };
    const handleCheckboxClick = (evt) => {
      evt.stopPropagation();
      _handleSelect();
    };
    const handleMouseEnter = () => {
      setActiveKey(props.optionKey);
      !props.isLeaf && cascaderPanelProps2.expandTrigger === "hover" && handleExpand(props.optionKey);
    };
    return () => {
      const {
        rawData,
        label
      } = props;
      const {
        multiple
      } = cascaderPanelProps2;
      const disabled = isDisabled.value;
      const selected = isSelected.value;
      const prefixCls = `${mergedPrefixCls.value}-option`;
      const searchValue = cascaderPanelProps2.searchValue;
      const mergedLabel = searchValue ? label : rawData[mergedLabelKey.value];
      const title = !(disabled || selected) && selectedLimitTitle.value || mergedLabel;
      const customAdditional = cascaderPanelProps2.customAdditional ? cascaderPanelProps2.customAdditional({
        data: rawData,
        index: props.index
      }) : void 0;
      return createVNode("div", mergeProps({
        "class": classes.value,
        "title": title,
        "onClick": disabled ? void 0 : handleClick,
        "onMouseenter": disabled ? void 0 : handleMouseEnter,
        "aria-label": label,
        "aria-selected": selected
      }, customAdditional), [multiple && createVNode(IxCheckbox, {
        "checked": selected,
        "disabled": disabled || !selected && selectedLimit.value,
        "indeterminate": isIndeterminate.value,
        "onClick": handleCheckboxClick
      }, null), createVNode("span", {
        "key": "__label",
        "class": `${prefixCls}-label`
      }, [renderLabel(slots.optionLabel, mergedLabel, rawData, searchValue, prefixCls)]), !props.isLeaf && createVNode("span", {
        "key": "__expand-icon",
        "class": `${prefixCls}-expand-icon`
      }, [convertIconVNode(slots.expandIcon, isLoading.value ? "loading" : mergedExpandIcon.value, {
        key: props.optionKey,
        expanded: isExpanded.value,
        data: rawData
      })])]);
    };
  }
});
function renderLabel(labelSlot, label, data, searchValue, prefixCls) {
  if (labelSlot) {
    return labelSlot({
      label,
      data,
      searchValue
    });
  }
  if (label && searchValue) {
    const startIndex = label.toUpperCase().indexOf(searchValue.toUpperCase());
    if (startIndex > -1) {
      const endIndex = startIndex + searchValue.length;
      const beforeLabel = label.substring(0, startIndex);
      const afterLabel = label.substring(endIndex);
      const highlightLabel = createVNode("span", {
        "class": `${prefixCls}-label-highlight`
      }, [label.substring(startIndex, endIndex)]);
      return [beforeLabel, highlightLabel, afterLabel];
    }
  }
  return label;
}
var OverlayOptionGroup = /* @__PURE__ */ defineComponent({
  props: {
    dataSource: {
      type: Array,
      required: true
    }
  },
  setup(props) {
    const {
      props: cascaderPanelProps2,
      slots,
      mergedPrefixCls
    } = inject(cascaderPanelToken);
    const classes = computed(() => {
      const prefixCls = `${mergedPrefixCls.value}-option-group`;
      return normalizeClass({
        [prefixCls]: true,
        [`${prefixCls}-empty`]: props.dataSource.length === 0
      });
    });
    return () => {
      const {
        dataSource
      } = props;
      let children;
      if (dataSource.length > 0) {
        const {
          _virtualScrollHeight,
          virtualItemHeight,
          virtualScrollMode,
          virtual
        } = cascaderPanelProps2;
        const rowRender = ({
          item,
          index
        }) => createVNode(Option, mergeProps({
          "index": index,
          "optionKey": item.key
        }, item), null);
        children = createVNode(CdkVirtualScroll, {
          "dataSource": dataSource,
          "getKey": "key",
          "height": _virtualScrollHeight,
          "rowHeight": virtualItemHeight,
          "rowRender": rowRender,
          "virtual": virtual,
          "scrollMode": virtual ? virtualScrollMode : void 0
        }, null);
      } else {
        children = createVNode(_Empty, {
          "empty": cascaderPanelProps2.empty
        }, slots);
      }
      return createVNode("div", {
        "class": classes.value
      }, [children]);
    };
  }
});
function getDefaultThemeTokens(tokens) {
  const { colorPrimaryText, fontSizeSm, heightMd, marginSizeSm, paddingSizeXs, paddingSizeSm, paddingSizeMd } = tokens;
  return {
    optionFontSize: fontSizeSm,
    optionHeight: heightMd,
    optionPadding: `${paddingSizeSm}px ${paddingSizeMd}px`,
    optionLabelHighlightColor: colorPrimaryText,
    optionGroupMinWidth: 120,
    optionGroupMinHeight: 180,
    optionLabelMarginLeft: marginSizeSm,
    optionContainerPadding: `${paddingSizeXs}px 0`,
    overlaySearchWrapperPadding: `${paddingSizeXs}px ${paddingSizeMd}px ${paddingSizeSm}px`
  };
}
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens) : getDefaultThemeTokens(tokens);
};
function useActiveState(mergedDataMap) {
  const [activeKey, setActiveKey] = useState(void 0);
  const activeData = computed(() => {
    const currKey = activeKey.value;
    return currKey !== void 0 ? mergedDataMap.value.get(currKey) : void 0;
  });
  return { activeKey, activeData, setActiveKey };
}
function useExpandable(props, mergedGetKey, mergedGetDisabled, mergedChildrenKey, mergedLabelKey, mergedFullPath, mergedDataMap, selectedKeys) {
  const getDefaultExpandedKeys = (firstSelectedKey) => {
    if (isNil(firstSelectedKey)) {
      return [];
    }
    const dataMap = mergedDataMap.value;
    const currData = dataMap.get(firstSelectedKey);
    return getParentKeys(dataMap, currData, false, mergedGetDisabled.value);
  };
  const [expandedKeys, setExpandedKeys] = useControlledProp(
    props,
    "expandedKeys",
    () => getDefaultExpandedKeys(selectedKeys.value[0])
  );
  const [loadedKeys, setLoadedKeys] = useControlledProp(props, "loadedKeys", () => []);
  const loadingKeys = ref([]);
  const handleExpand = async (key) => {
    const { loadChildren } = props;
    if (loadingKeys.value.includes(key)) {
      return;
    }
    const dataMap = mergedDataMap.value;
    const currData = dataMap.get(key);
    if (currData && (!currData.children || currData.children.length === 0)) {
      if (!loadChildren || loadedKeys.value.includes(key)) {
        return;
      }
      loadingKeys.value.push(key);
      const rawData = currData.rawData;
      const rawChildren = await loadChildren(rawData);
      loadingKeys.value.splice(loadingKeys.value.indexOf(key), 1);
      const dataMap2 = mergedDataMap.value;
      if (rawChildren.length) {
        const childrenKey = mergedChildrenKey.value;
        const mergedChildren = convertMergedData(
          props,
          mergedGetKey.value,
          childrenKey,
          mergedLabelKey.value,
          mergedFullPath.value,
          rawChildren,
          key,
          currData.label
        );
        convertMergedDataMap(dataMap2, mergedChildren);
        currData.rawData[childrenKey] = rawChildren;
        currData.children = mergedChildren;
        const newLoadedKeys = [...loadedKeys.value, key];
        setLoadedKeys(newLoadedKeys);
        callEmit(props.onLoaded, newLoadedKeys, rawData);
      } else {
        return;
      }
    }
    const currIndex = expandedKeys.value.indexOf(key);
    const currExpanded = currIndex >= 0;
    if (!currExpanded) {
      const dataMap2 = mergedDataMap.value;
      const newKeys = getParentKeys(dataMap2, dataMap2.get(key), false, mergedGetDisabled.value);
      newKeys.push(key);
      handleChange(newKeys, !currExpanded, currData.rawData);
    }
  };
  const handleChange = (newKeys, newExpanded, rawData) => {
    setExpandedKeys(newKeys);
    const { onExpand, onExpandedChange } = props;
    callEmit(onExpand, newExpanded, rawData);
    callChange(mergedDataMap, newKeys, onExpandedChange);
  };
  return { expandedKeys, setExpandedKeys, handleExpand, loadingKeys };
}
function useIndeterminateKeys(mergedDataMap, selectedWithStrategyKeys, mergedGetDisabled, strategyEnabled) {
  return computed(() => {
    if (!strategyEnabled.value) {
      return NoopArray;
    }
    const indeterminateKeySet = /* @__PURE__ */ new Set();
    const cascadedKeys = selectedWithStrategyKeys.value;
    const dataMap = mergedDataMap.value;
    const getDisabledFn = mergedGetDisabled.value;
    cascadedKeys.forEach((key) => {
      let currData = dataMap.get(key);
      while (currData && !isNil(currData.parentKey)) {
        const parentKey = currData.parentKey;
        const parent = dataMap.get(parentKey);
        if (parent && !getDisabledFn(parent.rawData) && !cascadedKeys.includes(parentKey)) {
          indeterminateKeySet.add(parentKey);
        }
        currData = parent;
      }
    });
    return [...indeterminateKeySet];
  });
}
function useSearchable(props, mergedData, mergedDataMap, mergedLabelKey, mergedGetDisabled) {
  const mergedSearchFn = useSearchFn(props, mergedLabelKey);
  const parentEnabled = computed(() => props.multiple || props.strategy === "off");
  const searchedKeys = computed(() => {
    const searchValue = props.searchValue;
    const searchFn = mergedSearchFn.value;
    if (!searchValue) {
      return NoopArray;
    }
    const _parentEnabled = parentEnabled.value;
    const getDisabledFn = mergedGetDisabled.value;
    const keySet = /* @__PURE__ */ new Set();
    mergedData.value.forEach((data) => doSearch(keySet, data, searchFn, searchValue, _parentEnabled, getDisabledFn));
    return [...keySet];
  });
  const searchedData = computed(() => {
    const dataMap = mergedDataMap.value;
    return searchedKeys.value.map((key) => dataMap.get(key)).filter(Boolean);
  });
  return { searchedData };
}
function useSearchFn(props, mergedLabelKey) {
  return computed(() => {
    const searchFn = props.searchFn;
    if (isFunction(searchFn)) {
      return searchFn;
    }
    return searchFn ? getDefaultSearchFn(mergedLabelKey.value) : false;
  });
}
function getDefaultSearchFn(labelKey) {
  return (data, searchValue) => {
    const label = data[labelKey];
    return label ? label.toLowerCase().includes(searchValue.toLowerCase()) : false;
  };
}
function doSearch(keySet, data, searchFn, searchValue, _parentEnabled, getDisabledFn) {
  const { key, rawData } = data;
  if (keySet.has(key) || getDisabledFn(rawData)) {
    return;
  }
  if (searchFn === false || searchFn(rawData, searchValue)) {
    if (_parentEnabled || data.isLeaf) {
      keySet.add(key);
    }
    processChildren(keySet, data, _parentEnabled, getDisabledFn);
  } else if (data.children) {
    data.children.forEach((child) => doSearch(keySet, child, searchFn, searchValue, _parentEnabled, getDisabledFn));
  }
}
function processChildren(keySet, data, parentEnabled, getDisabledFn) {
  if (!data || !data.children) {
    return;
  }
  data.children.forEach((child) => {
    if (keySet.has(child.key) || getDisabledFn(child.rawData)) {
      return;
    }
    if (parentEnabled || child.isLeaf) {
      keySet.add(child.key);
    }
    processChildren(keySet, child, parentEnabled, getDisabledFn);
  });
}
function useSelectedLimit(selectedKeys, multipleLimit) {
  const locale = useGlobalConfig("locale");
  const selectedLimit = computed(() => selectedKeys.value.length >= multipleLimit.value);
  const selectedLimitTitle = computed(() => {
    if (!selectedLimit.value) {
      return "";
    }
    return locale.select.limitMessage.replace("${0}", `${multipleLimit.value}`);
  });
  return {
    selectedLimit,
    selectedLimitTitle
  };
}
const cascaderPanelProps = {
  selectedKeys: { type: null, default: void 0 },
  expandedKeys: { type: Array, default: void 0 },
  loadedKeys: { type: Array, default: void 0 },
  childrenKey: { type: String, default: void 0 },
  customAdditional: { type: Function, default: void 0 },
  dataSource: { type: Array, default: () => [] },
  disableData: { type: Function },
  empty: { type: [String, Object], default: "simple" },
  expandIcon: { type: String, default: void 0 },
  expandTrigger: { type: String, default: "click" },
  fullPath: { type: Boolean, default: void 0 },
  getKey: { type: [String, Function], default: void 0 },
  labelKey: { type: String, default: void 0 },
  loadChildren: {
    type: Function,
    default: void 0
  },
  maxLabel: { type: [Number, String], default: Number.MAX_SAFE_INTEGER },
  multiple: { type: Boolean, default: false },
  multipleLimit: { type: Number, default: Number.MAX_SAFE_INTEGER },
  searchable: { type: [Boolean, String], default: false },
  searchFn: { type: [Boolean, Function], default: true },
  searchValue: String,
  separator: { type: String, default: "/" },
  strategy: { type: String, default: "all" },
  virtual: { type: Boolean, default: false },
  virtualScrollMode: { type: String, default: void 0 },
  virtualItemHeight: { type: Number, default: 32 },
  "onUpdate:selectedKeys": [Function, Array],
  "onUpdate:expandedKeys": [Function, Array],
  "onUpdate:loadedKeys": [Function, Array],
  onExpand: [Function, Array],
  onExpandedChange: [Function, Array],
  onLoaded: [Function, Array],
  onSelect: [Function, Array],
  _virtualScrollHeight: { type: Number, default: 256 }
};
const cascaderProps = {
  control: {
    type: [String, Number, Object, Array],
    default: void 0
  },
  value: { type: null, default: void 0 },
  expandedKeys: { type: Array, default: void 0 },
  loadedKeys: { type: Array, default: void 0 },
  open: { type: Boolean, default: void 0 },
  autocomplete: { type: String, default: "off" },
  autofocus: { type: Boolean, default: false },
  borderless: { type: Boolean, default: void 0 },
  childrenKey: { type: String, default: void 0 },
  clearable: { type: Boolean, default: false },
  clearIcon: { type: String, default: void 0 },
  customAdditional: { type: Function, default: void 0 },
  dataSource: { type: Array, default: () => [] },
  disabled: { type: Boolean, default: false },
  disableData: { type: Function },
  empty: { type: [String, Object], default: "simple" },
  expandIcon: { type: String, default: void 0 },
  expandTrigger: { type: String, default: "click" },
  fullPath: { type: Boolean, default: void 0 },
  getKey: { type: [String, Function], default: void 0 },
  labelKey: { type: String, default: void 0 },
  loadChildren: {
    type: Function,
    default: void 0
  },
  maxLabel: { type: [Number, String], default: Number.MAX_SAFE_INTEGER },
  multiple: { type: Boolean, default: false },
  multipleLimit: { type: Number, default: Number.MAX_SAFE_INTEGER },
  overlayClassName: { type: String, default: void 0 },
  overlayContainer: {
    type: [String, HTMLElement, Function],
    default: void 0
  },
  overlayTabindex: { type: Number, default: void 0 },
  overlayMatchWidth: { type: [Boolean, String], default: void 0 },
  overlayRender: { type: Function, default: void 0 },
  placeholder: { type: String, default: void 0 },
  readonly: { type: Boolean, default: false },
  searchable: { type: [Boolean, String], default: false },
  searchFn: { type: [Boolean, Function], default: true },
  searchPlaceholder: { type: String, default: void 0 },
  separator: { type: String, default: "/" },
  size: { type: String, default: void 0 },
  status: String,
  strategy: { type: String, default: "all" },
  suffix: { type: String, default: void 0 },
  virtual: { type: Boolean, default: false },
  virtualItemHeight: { type: Number, default: void 0 },
  virtualScrollMode: { type: String, default: void 0 },
  "onUpdate:value": [Function, Array],
  "onUpdate:expandedKeys": [Function, Array],
  "onUpdate:loadedKeys": [Function, Array],
  "onUpdate:open": [Function, Array],
  onChange: [Function, Array],
  onClear: [Function, Array],
  onFocus: [Function, Array],
  onBlur: [Function, Array],
  onExpand: [Function, Array],
  onExpandedChange: [Function, Array],
  onLoaded: [Function, Array],
  onSearch: [Function, Array]
};
var CascaderPanel = /* @__PURE__ */ defineComponent({
  props: cascaderPanelProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("cascader");
    registerToken(getThemeTokens);
    const config = useGlobalConfig("cascader");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-cascader`);
    const cascaderPanelContext = useCascaderPanelContext(props, config);
    const {
      mergedData,
      mergedDataMap,
      expandedKeys,
      searchedData
    } = cascaderPanelContext;
    const defaultKey = Symbol();
    const contentData = computed(() => {
      const dataSource = [{
        key: defaultKey,
        dataSource: mergedData.value
      }];
      const dataMap = mergedDataMap.value;
      expandedKeys.value.forEach((key) => {
        const currData = dataMap.get(key);
        if (currData && currData.children) {
          dataSource.push({
            key,
            dataSource: currData.children
          });
        }
      });
      return dataSource;
    });
    provide(cascaderPanelToken, {
      ...cascaderPanelContext,
      props,
      config,
      slots,
      mergedPrefixCls
    });
    return () => createVNode("div", {
      "class": [`${mergedPrefixCls.value}-panel`, globalHashId.value, hashId.value]
    }, [props.searchValue ? createVNode(OverlayOptionGroup, {
      "dataSource": searchedData.value
    }, null) : contentData.value.map((item) => createVNode(OverlayOptionGroup, item, null))]);
  }
});
function useCascaderPanelContext(props, config) {
  let dataContext = inject(CASCADER_PANEL_DATA_TOKEN, null);
  const mergedChildrenKey = computed(() => {
    var _a;
    return (_a = props.childrenKey) != null ? _a : config.childrenKey;
  });
  const mergedExpandIcon = computed(() => {
    var _a;
    return (_a = props.expandIcon) != null ? _a : config.expandIcon;
  });
  const mergedFullPath = computed(() => {
    var _a;
    return (_a = props.fullPath) != null ? _a : config.fullPath;
  });
  const mergedGetKey = useGetKey(props, config, "components/cascader");
  const mergedGetDisabled = useGetDisabled(props);
  const mergedLabelKey = computed(() => {
    var _a;
    return (_a = props.labelKey) != null ? _a : config.labelKey;
  });
  if (!dataContext) {
    const [selectedKeys, setSelectedKeys] = useControlledProp(props, "selectedKeys");
    const dataSourceContext = useDataSource(props, mergedGetKey, mergedChildrenKey, mergedLabelKey, mergedFullPath);
    const {
      mergedDataMap: mergedDataMap2
    } = dataSourceContext;
    const selectedStateContext = useSelectedState(mergedDataMap2, mergedFullPath, mergedGetDisabled, toRef(props, "multiple"), toRef(props, "strategy"), selectedKeys, setSelectedKeys);
    dataContext = {
      ...dataSourceContext,
      ...selectedStateContext
    };
  }
  const {
    resolvedSelectedKeys,
    selectedWithStrategyKeys,
    strategyEnabled,
    mergedDataMap,
    mergedData
  } = dataContext;
  const selectedLimitContext = useSelectedLimit(resolvedSelectedKeys, toRef(props, "multipleLimit"));
  const expandableContext = useExpandable(props, mergedGetKey, mergedGetDisabled, mergedChildrenKey, mergedLabelKey, mergedFullPath, mergedDataMap, resolvedSelectedKeys);
  const indeterminateKeys = useIndeterminateKeys(mergedDataMap, selectedWithStrategyKeys, mergedGetDisabled, strategyEnabled);
  const searchableContext = useSearchable(props, mergedData, mergedDataMap, mergedLabelKey, mergedGetDisabled);
  const activeStateContext = useActiveState(mergedDataMap);
  return {
    ...activeStateContext,
    ...dataContext,
    ...expandableContext,
    ...selectedLimitContext,
    ...searchableContext,
    mergedChildrenKey,
    mergedExpandIcon,
    mergedFullPath,
    mergedGetKey,
    mergedGetDisabled,
    mergedLabelKey,
    indeterminateKeys
  };
}
const defaultOffset = [0, 4];
var Cascader = /* @__PURE__ */ defineComponent({
  name: "IxCascader",
  inheritAttrs: false,
  props: cascaderProps,
  setup(props, {
    attrs,
    expose,
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("cascader");
    registerToken(getThemeTokens);
    const config = useGlobalConfig("cascader");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-cascader`);
    const mergedChildrenKey = computed(() => {
      var _a;
      return (_a = props.childrenKey) != null ? _a : config.childrenKey;
    });
    const mergedClearIcon = computed(() => {
      var _a;
      return (_a = props.clearIcon) != null ? _a : config.clearIcon;
    });
    const mergedFullPath = computed(() => {
      var _a;
      return (_a = props.fullPath) != null ? _a : config.fullPath;
    });
    const mergedGetKey = useGetKey(props, config, "components/cascader");
    const mergedGetDisabled = useGetDisabled(props);
    const mergedLabelKey = computed(() => {
      var _a;
      return (_a = props.labelKey) != null ? _a : config.labelKey;
    });
    const triggerRef = ref();
    const [inputValue, _setInputValue] = useState("");
    const setInputValue = (input) => {
      _setInputValue(input);
      props.searchable && callEmit(props.onSearch, input);
    };
    const focus = () => {
      var _a;
      return (_a = triggerRef.value) == null ? void 0 : _a.focus();
    };
    const blur = () => {
      var _a;
      return (_a = triggerRef.value) == null ? void 0 : _a.blur();
    };
    const clearInput = () => {
      var _a;
      props.searchable === "overlay" ? setInputValue("") : (_a = triggerRef.value) == null ? void 0 : _a.clearInput();
    };
    expose({
      focus,
      blur
    });
    const [overlayOpened, setOverlayOpened] = useControlledProp(props, "open");
    const {
      accessor,
      control
    } = useAccessorAndControl();
    const commonControlProps = useMergedCommonControlProps(props, config);
    useFormItemRegister(control);
    const mergedStatus = useFormStatus(props, control);
    const dataSourceContext = useDataSource(props, mergedGetKey, mergedChildrenKey, mergedLabelKey, mergedFullPath);
    const {
      mergedDataMap
    } = dataSourceContext;
    const selectedStateContext = useSelectedState(mergedDataMap, mergedFullPath, mergedGetDisabled, toRef(props, "multiple"), toRef(props, "strategy"), toRef(accessor, "value"), (keys) => {
      const oldKeys = toRaw(accessor.value);
      accessor.setValue(keys);
      callEmit(props.onChange, keys, oldKeys);
    });
    const {
      resolvedSelectedKeys,
      setValue
    } = selectedStateContext;
    const selectedData = useSelectedData(resolvedSelectedKeys, mergedDataMap);
    watch(overlayOpened, (opened) => {
      opened && focus();
      clearInput();
    });
    const onFocus = (evt) => {
      callEmit(props.onFocus, evt);
    };
    const onBlur = (evt) => {
      accessor.markAsBlurred();
      callEmit(props.onBlur, evt);
    };
    const handleItemRemove = (key) => {
      focus();
      selectedStateContext.handleSelect(key);
    };
    const handleClear = (evt) => {
      setValue([]);
      callEmit(props.onClear, evt);
    };
    provide(CASCADER_PANEL_DATA_TOKEN, {
      ...dataSourceContext,
      ...selectedStateContext
    });
    const overlayClasses = computed(() => {
      const {
        overlayClassName,
        multiple
      } = props;
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [`${prefixCls}-overlay`]: true,
        [`${prefixCls}-overlay-multiple`]: multiple,
        [overlayClassName || ""]: !!overlayClassName
      });
    });
    const renderTrigger = ({
      borderless,
      status,
      clearable,
      clearIcon,
      readonly,
      disabled,
      size,
      suffix,
      suffixRotate,
      focused,
      opened
    }) => [createVNode(IxSelector, {
      "ref": triggerRef,
      "class": mergedPrefixCls.value,
      "allowInput": props.searchable === true ? "searchable" : false,
      "autocomplete": props.autocomplete,
      "autofocus": props.autofocus,
      "borderless": borderless,
      "clearable": clearable,
      "clearIcon": clearIcon,
      "dataSource": selectedData.value,
      "disabled": disabled,
      "focused": focused,
      "maxLabel": props.maxLabel,
      "multiple": props.multiple,
      "opened": opened,
      "placeholder": props.placeholder,
      "readonly": readonly,
      "size": size,
      "status": status,
      "suffix": suffix,
      "suffixRotate": suffixRotate,
      "onClear": handleClear,
      "onInputValueChange": setInputValue,
      "onItemRemove": handleItemRemove
    }, slots)];
    const panelProps = usePanelProps(props, inputValue, setOverlayOpened);
    const handleSearchInput = (evt) => {
      const {
        value
      } = evt.target;
      setInputValue(value);
    };
    const handleSearchClear = () => {
      setInputValue("");
    };
    const renderContent = () => {
      const {
        searchable,
        overlayRender,
        searchPlaceholder
      } = props;
      const searchValue = inputValue.value;
      const prefixCls = mergedPrefixCls.value;
      const panelSlots = {
        empty: slots.empty,
        optionLabel: slots.optionLabel
      };
      const children = [createVNode("div", {
        "key": "__content",
        "class": `${prefixCls}-overlay-content`
      }, [createVNode(CascaderPanel, panelProps.value, panelSlots)])];
      if (searchable === "overlay") {
        children.unshift(createVNode("div", {
          "key": "__search-wrapper",
          "class": `${prefixCls}-overlay-search-wrapper`
        }, [createVNode(_Input, {
          "clearable": true,
          "clearIcon": mergedClearIcon.value,
          "clearVisible": !!searchValue,
          "size": "sm",
          "suffix": "search",
          "value": searchValue,
          "placeholder": searchPlaceholder,
          "onClear": handleSearchClear,
          "onInput": handleSearchInput
        }, null)]));
      }
      return createVNode("div", null, [overlayRender ? overlayRender(children) : children]);
    };
    return () => {
      var _a, _b, _c;
      const {
        suffix,
        borderless,
        clearIcon,
        size
      } = commonControlProps.value;
      const controlTriggerProps = {
        autofocus: props.autofocus,
        overlayClassName: overlayClasses.value,
        overlayContainer: (_a = props.overlayContainer) != null ? _a : config.overlayContainer,
        overlayTabindex: (_b = props.overlayTabindex) != null ? _b : config.overlayTabindex,
        overlayContainerFallback: `.${mergedPrefixCls.value}-overlay-container`,
        overlayMatchWidth: (_c = props.overlayMatchWidth) != null ? _c : config.overlayMatchWidth,
        class: mergedPrefixCls.value,
        borderless,
        value: resolvedSelectedKeys.value,
        offset: defaultOffset,
        open: overlayOpened.value,
        readonly: props.readonly,
        size,
        status: mergedStatus.value,
        suffix,
        clearable: props.clearable,
        clearIcon,
        disabled: accessor.disabled,
        onFocus,
        onBlur,
        "onUpdate:open": setOverlayOpened
      };
      const controlTriggerSlots = {
        trigger: renderTrigger,
        overlay: renderContent
      };
      return createVNode(IxControlTrigger, mergeProps(controlTriggerProps, attrs), controlTriggerSlots);
    };
  }
});
const IxCascader = Cascader;
const IxCascaderPanel = CascaderPanel;
export { IxCascader, IxCascaderPanel, getThemeTokens as getCascaderThemeTokens };
