/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { AbstractControl, ValidateStatus } from '@idux/cdk/forms';
import type { VirtualScrollMode } from '@idux/cdk/scroll';
import type { ExtractInnerPropTypes, ExtractPublicPropTypes, MaybeArray, VKey } from '@idux/cdk/utils';
import type { EmptyProps } from '@idux/components/empty';
import type { FormSize } from '@idux/components/form';
import type { OverlayContainerType } from '@idux/components/utils';
import type { DefineComponent, HTMLAttributes, PropType, VNode, VNodeChild } from 'vue';
export declare const cascaderPanelProps: {
    readonly selectedKeys: {
        readonly type: null;
        readonly default: undefined;
    };
    readonly expandedKeys: {
        readonly type: PropType<VKey[]>;
        readonly default: undefined;
    };
    readonly loadedKeys: {
        readonly type: PropType<VKey[]>;
        readonly default: undefined;
    };
    readonly childrenKey: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly customAdditional: {
        readonly type: PropType<CascaderCustomAdditional>;
        readonly default: undefined;
    };
    readonly dataSource: {
        readonly type: PropType<CascaderData<VKey>[]>;
        readonly default: () => never[];
    };
    readonly disableData: {
        readonly type: PropType<(data: CascaderData<any>) => boolean>;
    };
    readonly empty: {
        readonly type: PropType<"default" | "simple" | EmptyProps>;
        readonly default: "simple";
    };
    readonly expandIcon: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly expandTrigger: {
        readonly type: PropType<CascaderExpandTrigger>;
        readonly default: "click";
    };
    readonly fullPath: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly getKey: {
        readonly type: PropType<string | ((data: CascaderData<any>) => any)>;
        readonly default: undefined;
    };
    readonly labelKey: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly loadChildren: {
        readonly type: PropType<(data: CascaderData<any>) => Promise<CascaderData<any>[]>>;
        readonly default: undefined;
    };
    readonly maxLabel: {
        readonly type: PropType<number | "responsive">;
        readonly default: number;
    };
    readonly multiple: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly multipleLimit: {
        readonly type: NumberConstructor;
        readonly default: number;
    };
    readonly searchable: {
        readonly type: PropType<boolean | "overlay">;
        readonly default: false;
    };
    readonly searchFn: {
        readonly type: PropType<boolean | CascaderSearchFn>;
        readonly default: true;
    };
    readonly searchValue: StringConstructor;
    readonly separator: {
        readonly type: StringConstructor;
        readonly default: "/";
    };
    readonly strategy: {
        readonly type: PropType<CascaderStrategy>;
        readonly default: "all";
    };
    readonly virtual: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly virtualScrollMode: {
        readonly type: PropType<VirtualScrollMode>;
        readonly default: undefined;
    };
    readonly virtualItemHeight: {
        readonly type: NumberConstructor;
        readonly default: 32;
    };
    readonly 'onUpdate:selectedKeys': PropType<MaybeArray<(value: any) => void>>;
    readonly 'onUpdate:expandedKeys': PropType<MaybeArray<(keys: any[]) => void>>;
    readonly 'onUpdate:loadedKeys': PropType<MaybeArray<(keys: any[]) => void>>;
    readonly onExpand: PropType<MaybeArray<(expanded: boolean, data: CascaderData) => void>>;
    readonly onExpandedChange: PropType<MaybeArray<(expendedKeys: any[], data: CascaderData[]) => void>>;
    readonly onLoaded: PropType<MaybeArray<(loadedKeys: any[], data: CascaderData) => void>>;
    readonly onSelect: PropType<MaybeArray<(option: CascaderData, isSelected: boolean) => void>>;
    readonly _virtualScrollHeight: {
        readonly type: NumberConstructor;
        readonly default: 256;
    };
};
export declare const cascaderProps: {
    readonly control: {
        readonly type: PropType<string | number | (string | number)[] | AbstractControl<any>>;
        readonly default: undefined;
    };
    readonly value: {
        readonly type: null;
        readonly default: undefined;
    };
    readonly expandedKeys: {
        readonly type: PropType<VKey[]>;
        readonly default: undefined;
    };
    readonly loadedKeys: {
        readonly type: PropType<VKey[]>;
        readonly default: undefined;
    };
    readonly open: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly autocomplete: {
        readonly type: StringConstructor;
        readonly default: "off";
    };
    readonly autofocus: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly borderless: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly childrenKey: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly clearable: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly clearIcon: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly customAdditional: {
        readonly type: PropType<CascaderCustomAdditional>;
        readonly default: undefined;
    };
    readonly dataSource: {
        readonly type: PropType<CascaderData<VKey>[]>;
        readonly default: () => never[];
    };
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly disableData: {
        readonly type: PropType<(data: CascaderData<any>) => boolean>;
    };
    readonly empty: {
        readonly type: PropType<"default" | "simple" | EmptyProps>;
        readonly default: "simple";
    };
    readonly expandIcon: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly expandTrigger: {
        readonly type: PropType<CascaderExpandTrigger>;
        readonly default: "click";
    };
    readonly fullPath: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly getKey: {
        readonly type: PropType<string | ((data: CascaderData<any>) => any)>;
        readonly default: undefined;
    };
    readonly labelKey: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly loadChildren: {
        readonly type: PropType<(data: CascaderData<any>) => Promise<CascaderData<any>[]>>;
        readonly default: undefined;
    };
    readonly maxLabel: {
        readonly type: PropType<number | "responsive">;
        readonly default: number;
    };
    readonly multiple: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly multipleLimit: {
        readonly type: NumberConstructor;
        readonly default: number;
    };
    readonly overlayClassName: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly overlayContainer: {
        readonly type: PropType<OverlayContainerType>;
        readonly default: undefined;
    };
    readonly overlayTabindex: {
        readonly type: NumberConstructor;
        readonly default: undefined;
    };
    readonly overlayMatchWidth: {
        readonly type: PropType<boolean | "minWidth">;
        readonly default: undefined;
    };
    readonly overlayRender: {
        readonly type: PropType<(children: VNode[]) => VNodeChild>;
        readonly default: undefined;
    };
    readonly placeholder: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly readonly: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly searchable: {
        readonly type: PropType<boolean | "overlay">;
        readonly default: false;
    };
    readonly searchFn: {
        readonly type: PropType<boolean | CascaderSearchFn>;
        readonly default: true;
    };
    readonly searchPlaceholder: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly separator: {
        readonly type: StringConstructor;
        readonly default: "/";
    };
    readonly size: {
        readonly type: PropType<FormSize>;
        readonly default: undefined;
    };
    readonly status: PropType<ValidateStatus>;
    readonly strategy: {
        readonly type: PropType<CascaderStrategy>;
        readonly default: "all";
    };
    readonly suffix: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly virtual: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly virtualItemHeight: {
        readonly type: NumberConstructor;
        readonly default: undefined;
    };
    readonly virtualScrollMode: {
        readonly type: PropType<VirtualScrollMode>;
        readonly default: undefined;
    };
    readonly 'onUpdate:value': PropType<MaybeArray<(value: any) => void>>;
    readonly 'onUpdate:expandedKeys': PropType<MaybeArray<(keys: any[]) => void>>;
    readonly 'onUpdate:loadedKeys': PropType<MaybeArray<(keys: any[]) => void>>;
    readonly 'onUpdate:open': PropType<MaybeArray<(opened: boolean) => void>>;
    readonly onChange: PropType<MaybeArray<(value: any, oldValue: any) => void>>;
    readonly onClear: PropType<MaybeArray<(evt: Event) => void>>;
    readonly onFocus: PropType<MaybeArray<(evt: FocusEvent) => void>>;
    readonly onBlur: PropType<MaybeArray<(evt: FocusEvent) => void>>;
    readonly onExpand: PropType<MaybeArray<(expanded: boolean, data: CascaderData) => void>>;
    readonly onExpandedChange: PropType<MaybeArray<(expendedKeys: any[], data: CascaderData[]) => void>>;
    readonly onLoaded: PropType<MaybeArray<(loadedKeys: any[], data: CascaderData) => void>>;
    readonly onSearch: PropType<MaybeArray<(value: string) => void>>;
};
export type CascaderPanelProps = ExtractInnerPropTypes<typeof cascaderPanelProps>;
export type CascaderPanelPublicProps = ExtractPublicPropTypes<typeof cascaderPanelProps>;
export type CascaderPanelComponent = DefineComponent<Omit<HTMLAttributes, keyof CascaderPanelPublicProps> & CascaderPanelPublicProps>;
export type CascaderPanelInstance = InstanceType<DefineComponent<CascaderProps, CascaderBindings>>;
export type CascaderProps = ExtractInnerPropTypes<typeof cascaderProps>;
export type CascaderPublicProps = Omit<ExtractPublicPropTypes<typeof cascaderProps>, 'overlayHeight'>;
export interface CascaderBindings {
    blur: () => void;
    focus: (options?: FocusOptions) => void;
}
export type CascaderComponent = DefineComponent<Omit<HTMLAttributes, keyof CascaderPublicProps> & CascaderPublicProps, CascaderBindings>;
export type CascaderInstance = InstanceType<DefineComponent<CascaderProps, CascaderBindings>>;
export type CascaderStrategy = 'all' | 'parent' | 'child' | 'off';
export type CascaderCustomAdditional = (options: {
    data: CascaderData<any>;
    index: number;
}) => Record<string, any> | undefined;
export interface CascaderData<K = VKey> {
    children?: CascaderData<K>[];
    disabled?: boolean;
    isLeaf?: boolean;
    key?: K;
    label?: string;
    [key: string]: unknown;
}
export type CascaderExpandTrigger = 'click' | 'hover';
export type CascaderSearchFn = (data: CascaderData<any>, searchValue: string) => boolean;
