import { defineComponent, computed, ref, inject, normalizeClass, createVNode, mergeProps, provide } from "vue";
import { isNil } from "lodash-es";
import { useAccessorAndControl } from "@idux/cdk/forms";
import { useKey, callEmit } from "@idux/cdk/utils";
import { \u0275Wave as _Wave } from "@idux/components/_private/wave";
import { getButtonThemeTokens } from "@idux/components/button";
import { useGlobalConfig } from "@idux/components/config";
import { useFormElement, FORM_TOKEN, useFormItemRegister } from "@idux/components/form";
import { useThemeToken } from "@idux/components/theme";
import { convertStringVNode } from "@idux/components/utils";
import { IxSpace } from "@idux/components/space";
const checkboxGroupToken = Symbol("checkboxGroupToken");
const checkboxProps = {
  control: {
    type: [String, Number, Object, Array],
    default: void 0
  },
  checked: { type: [String, Number, Boolean], default: void 0 },
  autofocus: { type: Boolean, default: false },
  buttoned: { type: Boolean, default: void 0 },
  disabled: { type: Boolean, default: void 0 },
  indeterminate: { type: Boolean, default: false },
  label: { type: String, default: void 0 },
  trueValue: { type: [String, Number, Boolean], default: true },
  falseValue: { type: [String, Number, Boolean], default: false },
  value: { type: null, default: void 0 },
  size: { type: String, default: void 0 },
  waveless: { type: Boolean, default: false },
  "onUpdate:checked": { type: [Function, Array] },
  onChange: {
    type: [Function, Array]
  },
  onBlur: { type: [Function, Array] },
  onFocus: { type: [Function, Array] }
};
const checkboxGroupProps = {
  control: {
    type: [String, Number, Object, Array],
    default: void 0
  },
  value: { type: Array, default: void 0 },
  buttoned: { type: Boolean, default: false },
  dataSource: { type: Array },
  disabled: { type: Boolean, default: false },
  gap: { type: [Number, String], default: void 0 },
  name: { type: String, default: void 0 },
  size: { type: String, default: void 0 },
  "onUpdate:value": { type: [Function, Array] },
  onChange: { type: [Function, Array] }
};
function getDefaultThemeTokens(tokens) {
  const { paddingSizeXs, colorInfoContainerBg, colorBorderSecondary } = tokens;
  return {
    labelPadding: `0 ${paddingSizeXs}px`,
    fieldsetBgColor: colorInfoContainerBg,
    fieldsetBorderColor: colorBorderSecondary
  };
}
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens) : getDefaultThemeTokens(tokens);
};
const buttonSizeMap = {
  sm: "xs",
  md: "sm",
  lg: "md"
};
var Checkbox = /* @__PURE__ */ defineComponent({
  name: "IxCheckbox",
  inheritAttrs: false,
  props: checkboxProps,
  setup(props, {
    attrs,
    expose,
    slots
  }) {
    const key = useKey();
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("checkbox");
    const {
      hashId: buttonHashId,
      registerToken: registerButtonToken
    } = useThemeToken("button");
    registerToken(getThemeTokens);
    registerButtonToken(getButtonThemeTokens);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-checkbox`);
    const config = useGlobalConfig("checkbox");
    const {
      elementRef,
      focus,
      blur
    } = useFormElement();
    const waveRef = ref();
    expose({
      focus,
      blur
    });
    const formContext = inject(FORM_TOKEN, null);
    const checkboxGroup = inject(checkboxGroupToken, null);
    const mergedName = computed(() => {
      var _a;
      return (_a = attrs.name) != null ? _a : checkboxGroup == null ? void 0 : checkboxGroup.props.name;
    });
    const mergedValue = computed(() => {
      const {
        value
      } = props;
      if (!isNil(value)) {
        return value;
      }
      return checkboxGroup ? key : void 0;
    });
    const isButtoned = computed(() => {
      var _a, _b;
      return (_b = (_a = props.buttoned) != null ? _a : checkboxGroup == null ? void 0 : checkboxGroup.props.buttoned) != null ? _b : false;
    });
    const size = computed(() => {
      var _a, _b, _c;
      return (_c = (_b = (_a = props.size) != null ? _a : checkboxGroup == null ? void 0 : checkboxGroup.props.size) != null ? _b : formContext == null ? void 0 : formContext.size.value) != null ? _c : config.size;
    });
    const mergedWaveless = computed(() => {
      var _a;
      return (_a = props.waveless) != null ? _a : config.waveless;
    });
    const {
      isChecked,
      isDisabled,
      isFocused,
      handleChange,
      handleBlur,
      handleFocus
    } = useCheckbox(props, checkboxGroup, mergedValue, waveRef, mergedWaveless);
    const classes = computed(() => {
      const {
        indeterminate
      } = props;
      const buttoned = isButtoned.value;
      const prefixCls = mergedPrefixCls.value;
      const commonPrefixCls = common.prefixCls;
      return normalizeClass({
        [globalHashId.value]: !!globalHashId.value,
        [buttonHashId.value]: !!buttonHashId.value,
        [hashId.value]: !!hashId.value,
        [prefixCls]: true,
        [`${prefixCls}-checked`]: !indeterminate && isChecked.value,
        [`${prefixCls}-disabled`]: isDisabled.value,
        [`${prefixCls}-focused`]: isFocused.value,
        [`${prefixCls}-indeterminate`]: indeterminate,
        [`${prefixCls}-${size.value}`]: buttoned,
        [`${commonPrefixCls}-button`]: buttoned,
        [`${commonPrefixCls}-button-default`]: buttoned,
        [`${commonPrefixCls}-button-disabled`]: buttoned && isDisabled.value,
        [`${commonPrefixCls}-button-${buttonSizeMap[size.value]}`]: buttoned
      });
    });
    return () => {
      const {
        autofocus,
        label
      } = props;
      const {
        class: className,
        style,
        type,
        tabindex,
        ...restAttrs
      } = attrs;
      const prefixCls = mergedPrefixCls.value;
      const checked = isChecked.value;
      const labelNode = convertStringVNode(slots.default, label);
      const checkboxNode = createVNode("label", {
        "class": slots.fieldset ? classes.value : normalizeClass([classes.value, className]),
        "style": slots.fieldset ? void 0 : style,
        "role": "checkbox",
        "aria-checked": checked,
        "aria-disabled": isDisabled.value
      }, [createVNode("span", {
        "class": `${prefixCls}-input`
      }, [createVNode("input", mergeProps({
        "ref": elementRef,
        "type": "checkbox",
        "class": `${prefixCls}-input-inner`,
        "aria-hidden": true,
        "autofocus": autofocus,
        "name": mergedName.value,
        "value": mergedValue.value,
        "checked": checked,
        "disabled": isDisabled.value,
        "onChange": handleChange,
        "onBlur": handleBlur,
        "onFocus": handleFocus
      }, restAttrs), null), !isButtoned.value && createVNode("span", {
        "class": `${prefixCls}-input-box`,
        "tabindex": tabindex
      }, [!mergedWaveless.value && createVNode(_Wave, {
        "ref": waveRef
      }, null)])]), isButtoned.value && createVNode("span", {
        "class": `${prefixCls}-input-tick`,
        "tabindex": tabindex
      }, null), isButtoned.value && !mergedWaveless.value && createVNode(_Wave, {
        "ref": waveRef
      }, null), labelNode && createVNode("span", {
        "class": `${prefixCls}-label`
      }, [labelNode])]);
      if (slots.fieldset) {
        return createVNode("div", {
          "class": normalizeClass([`${prefixCls}-wrapper`, className, globalHashId.value, hashId.value, buttonHashId.value]),
          "style": style
        }, [checkboxNode, createVNode("div", {
          "class": normalizeClass([`${prefixCls}-fieldset`, !checked ? `${prefixCls}-fieldset-hidden` : ""])
        }, [slots.fieldset()])]);
      }
      return checkboxNode;
    };
  }
});
const useCheckbox = (props, checkboxGroup, mergedValue, waveRef, mergedWaveless) => {
  let isChecked;
  let isDisabled;
  let handleChange;
  let handleBlur;
  const isFocused = ref(false);
  const handleFocus = (evt) => {
    isFocused.value = true;
    callEmit(props.onFocus, evt);
  };
  if (checkboxGroup) {
    const {
      props: groupProps,
      accessor
    } = checkboxGroup;
    isChecked = computed(() => (accessor.value || []).includes(mergedValue.value));
    isDisabled = computed(() => accessor.disabled || !!props.disabled);
    handleBlur = (evt) => {
      isFocused.value = false;
      accessor.markAsBlurred();
      callEmit(props.onBlur, evt);
    };
    handleChange = (evt) => {
      var _a;
      const checked = evt.target.checked;
      const {
        trueValue,
        falseValue
      } = props;
      const value = mergedValue.value;
      const checkValue = checked ? trueValue : falseValue;
      const oldCheckValue = !checked ? trueValue : falseValue;
      const oldValue = accessor.value || [];
      const newValue = [...oldValue];
      const checkValueIndex = newValue.indexOf(value);
      if (checkValueIndex === -1) {
        newValue.push(value);
      } else {
        newValue.splice(checkValueIndex, 1);
      }
      accessor.setValue(newValue);
      callEmit(props.onChange, checkValue, oldCheckValue);
      callEmit(groupProps.onChange, newValue, oldValue);
      !mergedWaveless.value && ((_a = waveRef.value) == null ? void 0 : _a.play());
    };
  } else {
    const {
      accessor,
      control
    } = useAccessorAndControl({
      valueKey: "checked"
    });
    useFormItemRegister(control);
    isChecked = computed(() => accessor.value === props.trueValue);
    isDisabled = computed(() => accessor.disabled);
    handleBlur = (evt) => {
      isFocused.value = false;
      accessor.markAsBlurred();
      callEmit(props.onBlur, evt);
    };
    handleChange = (evt) => {
      var _a;
      const checked = evt.target.checked;
      const {
        trueValue,
        falseValue
      } = props;
      const newChecked = checked ? trueValue : falseValue;
      const oldChecked = !checked ? trueValue : falseValue;
      accessor.setValue(newChecked);
      callEmit(props.onChange, newChecked, oldChecked);
      !mergedWaveless.value && ((_a = waveRef.value) == null ? void 0 : _a.play());
    };
  }
  return {
    isChecked,
    isDisabled,
    isFocused,
    handleChange,
    handleBlur,
    handleFocus
  };
};
var CheckboxGroup = /* @__PURE__ */ defineComponent({
  name: "IxCheckboxGroup",
  props: checkboxGroupProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId
    } = useThemeToken("checkbox");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-checkbox-group`);
    const {
      accessor,
      control
    } = useAccessorAndControl();
    useFormItemRegister(control);
    provide(checkboxGroupToken, {
      props,
      accessor
    });
    const mergedGap = computed(() => {
      var _a;
      return (_a = props.gap) != null ? _a : props.buttoned ? 0 : 8;
    });
    const classes = computed(() => {
      const {
        buttoned
      } = props;
      const gap = mergedGap.value;
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [prefixCls]: true,
        [`${common.prefixCls}-button-group`]: buttoned,
        [`${common.prefixCls}-button-group-compact`]: buttoned && (!gap || gap === "0")
      });
    });
    return () => {
      const {
        dataSource
      } = props;
      let children;
      if (dataSource) {
        children = dataSource.map((item) => {
          const {
            key,
            value
          } = item;
          return createVNode(Checkbox, mergeProps(item, {
            "key": key != null ? key : value,
            "value": value != null ? value : key
          }), null);
        });
      } else {
        children = slots.default ? slots.default() : void 0;
      }
      return createVNode(IxSpace, {
        "class": classes.value,
        "size": mergedGap.value
      }, {
        default: () => [children]
      });
    };
  }
});
const IxCheckbox = Checkbox;
const IxCheckboxGroup = CheckboxGroup;
export { IxCheckbox, IxCheckboxGroup, getThemeTokens as getCheckboxThemeTokens };
