/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { AbstractControl } from '@idux/cdk/forms';
import type { ExtractInnerPropTypes, ExtractPublicPropTypes, MaybeArray, VKey } from '@idux/cdk/utils';
import type { FormSize } from '@idux/components/form';
import type { SpaceProps } from '@idux/components/space';
import type { DefineComponent, HTMLAttributes, LabelHTMLAttributes, PropType } from 'vue';
export declare const checkboxProps: {
    readonly control: {
        readonly type: PropType<string | number | (string | number)[] | AbstractControl<any>>;
        readonly default: undefined;
    };
    readonly checked: {
        readonly type: PropType<CheckValue>;
        readonly default: undefined;
    };
    readonly autofocus: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly buttoned: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly indeterminate: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly label: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly trueValue: {
        readonly type: PropType<CheckValue>;
        readonly default: true;
    };
    readonly falseValue: {
        readonly type: PropType<CheckValue>;
        readonly default: false;
    };
    readonly value: {
        readonly type: null;
        readonly default: undefined;
    };
    readonly size: {
        readonly type: PropType<FormSize>;
        readonly default: undefined;
    };
    readonly waveless: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly 'onUpdate:checked': {
        readonly type: PropType<MaybeArray<(checked: any) => void>>;
    };
    readonly onChange: {
        readonly type: PropType<MaybeArray<(newChecked: any, oldChecked: any) => void>>;
    };
    readonly onBlur: {
        readonly type: PropType<MaybeArray<(evt: FocusEvent) => void>>;
    };
    readonly onFocus: {
        readonly type: PropType<MaybeArray<(evt: FocusEvent) => void>>;
    };
};
export type CheckboxProps = ExtractInnerPropTypes<typeof checkboxProps>;
export type CheckboxPublicProps = ExtractPublicPropTypes<typeof checkboxProps>;
export interface CheckboxBindings {
    blur: () => void;
    focus: (options?: FocusOptions) => void;
}
export type CheckboxComponent = DefineComponent<Omit<LabelHTMLAttributes, keyof CheckboxPublicProps> & CheckboxPublicProps, CheckboxBindings>;
export type CheckboxInstance = InstanceType<DefineComponent<CheckboxProps, CheckboxBindings>>;
export declare const checkboxGroupProps: {
    readonly control: {
        readonly type: PropType<string | number | (string | number)[] | AbstractControl<any>>;
        readonly default: undefined;
    };
    readonly value: {
        readonly type: ArrayConstructor;
        readonly default: undefined;
    };
    readonly buttoned: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly dataSource: {
        readonly type: PropType<CheckboxData<VKey>[]>;
    };
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly gap: {
        readonly type: PropType<string | number>;
        readonly default: undefined;
    };
    readonly name: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly size: {
        readonly type: PropType<FormSize>;
        readonly default: undefined;
    };
    readonly 'onUpdate:value': {
        readonly type: PropType<MaybeArray<(value: any) => void>>;
    };
    readonly onChange: {
        readonly type: PropType<MaybeArray<(value: any, oldValue: any) => void>>;
    };
};
export type CheckboxGroupProps = ExtractInnerPropTypes<typeof checkboxGroupProps>;
export type CheckboxGroupPublicProps = ExtractPublicPropTypes<typeof checkboxGroupProps> & Omit<SpaceProps, 'size'>;
export type CheckboxGroupComponent = DefineComponent<Omit<HTMLAttributes, keyof CheckboxGroupPublicProps> & CheckboxGroupPublicProps>;
export type CheckboxGroupInstance = InstanceType<DefineComponent<CheckboxGroupProps>>;
export type CheckValue = string | number | boolean;
export interface CheckboxData<K = VKey> extends Omit<CheckboxPublicProps, 'value'> {
    key?: K;
}
