import { defineComponent, computed, createVNode, inject, shallowRef, watch, toRaw, provide, toRef, normalizeClass, mergeProps } from "vue";
import { useGlobalConfig } from "@idux/components/config";
import { getAlphaColor, useThemeToken } from "@idux/components/theme";
import { isNaN as isNaN$1, isNil } from "lodash-es";
import { CdkDndMovable } from "@idux/cdk/dnd";
import { useState, convertElement, useControlledProp, callEmit } from "@idux/cdk/utils";
import { useElementSize } from "@idux/components/utils";
import { IxControlTrigger } from "@idux/components/control-trigger";
import { IxInputNumber } from "@idux/components/input-number";
import { IxSelect } from "@idux/components/select";
import { IxInput } from "@idux/components/input";
import { IxCollapse, IxCollapsePanel } from "@idux/components/collapse";
import { IxHeader } from "@idux/components/header";
import { IxIcon } from "@idux/components/icon";
import { useAccessorAndControl } from "@idux/cdk/forms";
import { useFormItemRegister, useFormSize, useFormStatus } from "@idux/components/form";
const colorPickerPanelProps = {
  value: String,
  format: String,
  presets: Array,
  paddingless: { type: Boolean, default: false },
  "onUpdate:format": [Function, Array],
  "onUpdate:value": [Function, Array],
  onChange: [Function, Array],
  onFormatChange: [Function, Array]
};
const colorPickerProps = {
  ...colorPickerPanelProps,
  control: {
    type: [String, Number, Object, Array],
    default: void 0
  },
  open: {
    type: Boolean,
    default: void 0
  },
  autofocus: {
    type: Boolean,
    default: false
  },
  clearable: {
    type: Boolean,
    default: void 0
  },
  clearIcon: String,
  disabled: {
    type: Boolean,
    default: false
  },
  overlayClassName: String,
  overlayContainer: {
    type: [String, HTMLElement, Function],
    default: void 0
  },
  overlayTabindex: { type: Number, default: void 0 },
  readonly: {
    type: Boolean,
    default: false
  },
  size: String,
  status: String,
  showText: { type: Boolean, default: false },
  "onUpdate:open": [Function, Array],
  onClear: [Function, Array],
  onFocus: [Function, Array],
  onBlur: [Function, Array]
};
const colorPickerIndicatorProps = {
  color: [String, Object],
  checked: Boolean,
  disabled: { type: Boolean, default: false },
  focused: { type: Boolean, default: void 0 },
  hoverable: { type: Boolean, default: true },
  showBoxShadow: { type: Boolean, default: true }
};
const colorPickerTriggerProps = {
  value: [String, Object],
  disabled: {
    type: Boolean,
    default: false
  },
  format: {
    type: String,
    default: "hex"
  },
  focused: { type: Boolean, default: void 0 },
  size: { type: String, default: "md" },
  showText: { type: Boolean, default: false }
};
function bound01(n, max) {
  if (isOnePointZero(n)) {
    n = "100%";
  }
  const isPercent = isPercentage(n);
  n = max === 360 ? n : Math.min(max, Math.max(0, parseFloat(n)));
  if (isPercent) {
    n = parseInt(String(n * max), 10) / 100;
  }
  if (Math.abs(n - max) < 1e-6) {
    return 1;
  }
  if (max === 360) {
    n = (n < 0 ? n % max + max : n % max) / parseFloat(String(max));
  } else {
    n = n % max / parseFloat(String(max));
  }
  return n;
}
function clamp01(val) {
  return Math.min(1, Math.max(0, val));
}
function isOnePointZero(n) {
  return typeof n === "string" && n.indexOf(".") !== -1 && parseFloat(n) === 1;
}
function isPercentage(n) {
  return typeof n === "string" && n.indexOf("%") !== -1;
}
function boundAlpha(a) {
  a = parseFloat(a);
  if (isNaN(a) || a < 0 || a > 1) {
    a = 1;
  }
  return a;
}
function convertToPercentage(n) {
  if (Number(n) <= 1) {
    return `${Number(n) * 100}%`;
  }
  return n;
}
function pad2(c) {
  return c.length === 1 ? "0" + c : String(c);
}
function rgbToRgb(r, g, b) {
  return {
    r: bound01(r, 255) * 255,
    g: bound01(g, 255) * 255,
    b: bound01(b, 255) * 255
  };
}
function rgbToHsl(r, g, b) {
  r = bound01(r, 255);
  g = bound01(g, 255);
  b = bound01(b, 255);
  const max = Math.max(r, g, b);
  const min = Math.min(r, g, b);
  let h = 0;
  let s = 0;
  const l = (max + min) / 2;
  if (max === min) {
    s = 0;
    h = 0;
  } else {
    const d = max - min;
    s = l > 0.5 ? d / (2 - max - min) : d / (max + min);
    switch (max) {
      case r:
        h = (g - b) / d + (g < b ? 6 : 0);
        break;
      case g:
        h = (b - r) / d + 2;
        break;
      case b:
        h = (r - g) / d + 4;
        break;
    }
    h /= 6;
  }
  return { h, s, l };
}
function hue2rgb(p, q, t) {
  if (t < 0) {
    t += 1;
  }
  if (t > 1) {
    t -= 1;
  }
  if (t < 1 / 6) {
    return p + (q - p) * (6 * t);
  }
  if (t < 1 / 2) {
    return q;
  }
  if (t < 2 / 3) {
    return p + (q - p) * (2 / 3 - t) * 6;
  }
  return p;
}
function hslToRgb(h, s, l) {
  let r;
  let g;
  let b;
  h = bound01(h, 360);
  s = bound01(s, 100);
  l = bound01(l, 100);
  if (s === 0) {
    g = l;
    b = l;
    r = l;
  } else {
    const q = l < 0.5 ? l * (1 + s) : l + s - l * s;
    const p = 2 * l - q;
    r = hue2rgb(p, q, h + 1 / 3);
    g = hue2rgb(p, q, h);
    b = hue2rgb(p, q, h - 1 / 3);
  }
  return { r: r * 255, g: g * 255, b: b * 255 };
}
function rgbToHsv(r, g, b) {
  r = bound01(r, 255);
  g = bound01(g, 255);
  b = bound01(b, 255);
  const max = Math.max(r, g, b);
  const min = Math.min(r, g, b);
  let h = 0;
  const v = max;
  const d = max - min;
  const s = max === 0 ? 0 : d / max;
  if (max === min) {
    h = 0;
  } else {
    switch (max) {
      case r:
        h = (g - b) / d + (g < b ? 6 : 0);
        break;
      case g:
        h = (b - r) / d + 2;
        break;
      case b:
        h = (r - g) / d + 4;
        break;
    }
    h /= 6;
  }
  return { h, s, v };
}
function hsvToRgb(h, s, v) {
  h = bound01(h, 360) * 6;
  s = bound01(s, 100);
  v = bound01(v, 100);
  const i = Math.floor(h);
  const f = h - i;
  const p = v * (1 - s);
  const q = v * (1 - f * s);
  const t = v * (1 - (1 - f) * s);
  const mod = i % 6;
  const r = [v, q, p, p, t, v][mod];
  const g = [t, v, v, q, p, p][mod];
  const b = [p, p, t, v, v, q][mod];
  return { r: r * 255, g: g * 255, b: b * 255 };
}
function rgbToHex(r, g, b, allow3Char) {
  const hex = [
    pad2(Math.round(r).toString(16)),
    pad2(Math.round(g).toString(16)),
    pad2(Math.round(b).toString(16))
  ];
  if (allow3Char && hex[0].startsWith(hex[0].charAt(1)) && hex[1].startsWith(hex[1].charAt(1)) && hex[2].startsWith(hex[2].charAt(1))) {
    return hex[0].charAt(0) + hex[1].charAt(0) + hex[2].charAt(0);
  }
  return hex.join("");
}
function rgbaToHex(r, g, b, a, allow4Char) {
  const hex = [
    pad2(Math.round(r).toString(16)),
    pad2(Math.round(g).toString(16)),
    pad2(Math.round(b).toString(16)),
    pad2(convertDecimalToHex(a))
  ];
  if (allow4Char && hex[0].startsWith(hex[0].charAt(1)) && hex[1].startsWith(hex[1].charAt(1)) && hex[2].startsWith(hex[2].charAt(1)) && hex[3].startsWith(hex[3].charAt(1))) {
    return hex[0].charAt(0) + hex[1].charAt(0) + hex[2].charAt(0) + hex[3].charAt(0);
  }
  return hex.join("");
}
function cmykToRgb(c, m, y, k) {
  const cConv = c / 100;
  const mConv = m / 100;
  const yConv = y / 100;
  const kConv = k / 100;
  const r = 255 * (1 - cConv) * (1 - kConv);
  const g = 255 * (1 - mConv) * (1 - kConv);
  const b = 255 * (1 - yConv) * (1 - kConv);
  return { r, g, b };
}
function rgbToCmyk(r, g, b) {
  let c = 1 - r / 255;
  let m = 1 - g / 255;
  let y = 1 - b / 255;
  let k = Math.min(c, m, y);
  if (k === 1) {
    c = 0;
    m = 0;
    y = 0;
  } else {
    c = (c - k) / (1 - k) * 100;
    m = (m - k) / (1 - k) * 100;
    y = (y - k) / (1 - k) * 100;
  }
  k *= 100;
  return {
    c: Math.round(c),
    m: Math.round(m),
    y: Math.round(y),
    k: Math.round(k)
  };
}
function convertDecimalToHex(d) {
  return Math.round(parseFloat(d) * 255).toString(16);
}
function convertHexToDecimal(h) {
  return parseIntFromHex(h) / 255;
}
function parseIntFromHex(val) {
  return parseInt(val, 16);
}
function numberInputToObject(color) {
  return {
    r: color >> 16,
    g: (color & 65280) >> 8,
    b: color & 255
  };
}
const names = {
  aliceblue: "#f0f8ff",
  antiquewhite: "#faebd7",
  aqua: "#00ffff",
  aquamarine: "#7fffd4",
  azure: "#f0ffff",
  beige: "#f5f5dc",
  bisque: "#ffe4c4",
  black: "#000000",
  blanchedalmond: "#ffebcd",
  blue: "#0000ff",
  blueviolet: "#8a2be2",
  brown: "#a52a2a",
  burlywood: "#deb887",
  cadetblue: "#5f9ea0",
  chartreuse: "#7fff00",
  chocolate: "#d2691e",
  coral: "#ff7f50",
  cornflowerblue: "#6495ed",
  cornsilk: "#fff8dc",
  crimson: "#dc143c",
  cyan: "#00ffff",
  darkblue: "#00008b",
  darkcyan: "#008b8b",
  darkgoldenrod: "#b8860b",
  darkgray: "#a9a9a9",
  darkgreen: "#006400",
  darkgrey: "#a9a9a9",
  darkkhaki: "#bdb76b",
  darkmagenta: "#8b008b",
  darkolivegreen: "#556b2f",
  darkorange: "#ff8c00",
  darkorchid: "#9932cc",
  darkred: "#8b0000",
  darksalmon: "#e9967a",
  darkseagreen: "#8fbc8f",
  darkslateblue: "#483d8b",
  darkslategray: "#2f4f4f",
  darkslategrey: "#2f4f4f",
  darkturquoise: "#00ced1",
  darkviolet: "#9400d3",
  deeppink: "#ff1493",
  deepskyblue: "#00bfff",
  dimgray: "#696969",
  dimgrey: "#696969",
  dodgerblue: "#1e90ff",
  firebrick: "#b22222",
  floralwhite: "#fffaf0",
  forestgreen: "#228b22",
  fuchsia: "#ff00ff",
  gainsboro: "#dcdcdc",
  ghostwhite: "#f8f8ff",
  goldenrod: "#daa520",
  gold: "#ffd700",
  gray: "#808080",
  green: "#008000",
  greenyellow: "#adff2f",
  grey: "#808080",
  honeydew: "#f0fff0",
  hotpink: "#ff69b4",
  indianred: "#cd5c5c",
  indigo: "#4b0082",
  ivory: "#fffff0",
  khaki: "#f0e68c",
  lavenderblush: "#fff0f5",
  lavender: "#e6e6fa",
  lawngreen: "#7cfc00",
  lemonchiffon: "#fffacd",
  lightblue: "#add8e6",
  lightcoral: "#f08080",
  lightcyan: "#e0ffff",
  lightgoldenrodyellow: "#fafad2",
  lightgray: "#d3d3d3",
  lightgreen: "#90ee90",
  lightgrey: "#d3d3d3",
  lightpink: "#ffb6c1",
  lightsalmon: "#ffa07a",
  lightseagreen: "#20b2aa",
  lightskyblue: "#87cefa",
  lightslategray: "#778899",
  lightslategrey: "#778899",
  lightsteelblue: "#b0c4de",
  lightyellow: "#ffffe0",
  lime: "#00ff00",
  limegreen: "#32cd32",
  linen: "#faf0e6",
  magenta: "#ff00ff",
  maroon: "#800000",
  mediumaquamarine: "#66cdaa",
  mediumblue: "#0000cd",
  mediumorchid: "#ba55d3",
  mediumpurple: "#9370db",
  mediumseagreen: "#3cb371",
  mediumslateblue: "#7b68ee",
  mediumspringgreen: "#00fa9a",
  mediumturquoise: "#48d1cc",
  mediumvioletred: "#c71585",
  midnightblue: "#191970",
  mintcream: "#f5fffa",
  mistyrose: "#ffe4e1",
  moccasin: "#ffe4b5",
  navajowhite: "#ffdead",
  navy: "#000080",
  oldlace: "#fdf5e6",
  olive: "#808000",
  olivedrab: "#6b8e23",
  orange: "#ffa500",
  orangered: "#ff4500",
  orchid: "#da70d6",
  palegoldenrod: "#eee8aa",
  palegreen: "#98fb98",
  paleturquoise: "#afeeee",
  palevioletred: "#db7093",
  papayawhip: "#ffefd5",
  peachpuff: "#ffdab9",
  peru: "#cd853f",
  pink: "#ffc0cb",
  plum: "#dda0dd",
  powderblue: "#b0e0e6",
  purple: "#800080",
  rebeccapurple: "#663399",
  red: "#ff0000",
  rosybrown: "#bc8f8f",
  royalblue: "#4169e1",
  saddlebrown: "#8b4513",
  salmon: "#fa8072",
  sandybrown: "#f4a460",
  seagreen: "#2e8b57",
  seashell: "#fff5ee",
  sienna: "#a0522d",
  silver: "#c0c0c0",
  skyblue: "#87ceeb",
  slateblue: "#6a5acd",
  slategray: "#708090",
  slategrey: "#708090",
  snow: "#fffafa",
  springgreen: "#00ff7f",
  steelblue: "#4682b4",
  tan: "#d2b48c",
  teal: "#008080",
  thistle: "#d8bfd8",
  tomato: "#ff6347",
  turquoise: "#40e0d0",
  violet: "#ee82ee",
  wheat: "#f5deb3",
  white: "#ffffff",
  whitesmoke: "#f5f5f5",
  yellow: "#ffff00",
  yellowgreen: "#9acd32"
};
function inputToRGB(color) {
  let rgb = { r: 0, g: 0, b: 0 };
  let a = 1;
  let s = null;
  let v = null;
  let l = null;
  let ok = false;
  let format = false;
  if (typeof color === "string") {
    color = stringInputToObject(color);
  }
  if (typeof color === "object") {
    if (isValidCSSUnit(color.r) && isValidCSSUnit(color.g) && isValidCSSUnit(color.b)) {
      rgb = rgbToRgb(color.r, color.g, color.b);
      ok = true;
      format = String(color.r).substr(-1) === "%" ? "prgb" : "rgb";
    } else if (isValidCSSUnit(color.h) && isValidCSSUnit(color.s) && isValidCSSUnit(color.v)) {
      s = convertToPercentage(color.s);
      v = convertToPercentage(color.v);
      rgb = hsvToRgb(color.h, s, v);
      ok = true;
      format = "hsv";
    } else if (isValidCSSUnit(color.h) && isValidCSSUnit(color.s) && isValidCSSUnit(color.l)) {
      s = convertToPercentage(color.s);
      l = convertToPercentage(color.l);
      rgb = hslToRgb(color.h, s, l);
      ok = true;
      format = "hsl";
    } else if (isValidCSSUnit(color.c) && isValidCSSUnit(color.m) && isValidCSSUnit(color.y) && isValidCSSUnit(color.k)) {
      rgb = cmykToRgb(color.c, color.m, color.y, color.k);
      ok = true;
      format = "cmyk";
    }
    if (Object.prototype.hasOwnProperty.call(color, "a")) {
      a = color.a;
    }
  }
  a = boundAlpha(a);
  return {
    ok,
    format: color.format || format,
    r: Math.min(255, Math.max(rgb.r, 0)),
    g: Math.min(255, Math.max(rgb.g, 0)),
    b: Math.min(255, Math.max(rgb.b, 0)),
    a
  };
}
const CSS_INTEGER = "[-\\+]?\\d+%?";
const CSS_NUMBER = "[-\\+]?\\d*\\.\\d+%?";
const CSS_UNIT = "(?:" + CSS_NUMBER + ")|(?:" + CSS_INTEGER + ")";
const PERMISSIVE_MATCH3 = "[\\s|\\(]+(" + CSS_UNIT + ")[,|\\s]+(" + CSS_UNIT + ")[,|\\s]+(" + CSS_UNIT + ")\\s*\\)?";
const PERMISSIVE_MATCH4 = "[\\s|\\(]+(" + CSS_UNIT + ")[,|\\s]+(" + CSS_UNIT + ")[,|\\s]+(" + CSS_UNIT + ")[,|\\s]+(" + CSS_UNIT + ")\\s*\\)?";
const matchers = {
  CSS_UNIT: new RegExp(CSS_UNIT),
  rgb: new RegExp("rgb" + PERMISSIVE_MATCH3),
  rgba: new RegExp("rgba" + PERMISSIVE_MATCH4),
  hsl: new RegExp("hsl" + PERMISSIVE_MATCH3),
  hsla: new RegExp("hsla" + PERMISSIVE_MATCH4),
  hsv: new RegExp("hsv" + PERMISSIVE_MATCH3),
  hsva: new RegExp("hsva" + PERMISSIVE_MATCH4),
  cmyk: new RegExp("cmyk" + PERMISSIVE_MATCH4),
  hex3: /^#?([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})$/,
  hex6: /^#?([0-9a-fA-F]{2})([0-9a-fA-F]{2})([0-9a-fA-F]{2})$/,
  hex4: /^#?([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})$/,
  hex8: /^#?([0-9a-fA-F]{2})([0-9a-fA-F]{2})([0-9a-fA-F]{2})([0-9a-fA-F]{2})$/
};
function stringInputToObject(color) {
  color = color.trim().toLowerCase();
  if (color.length === 0) {
    return false;
  }
  let named = false;
  if (names[color]) {
    color = names[color];
    named = true;
  } else if (color === "transparent") {
    return { r: 0, g: 0, b: 0, a: 0, format: "name" };
  }
  let match = matchers.rgb.exec(color);
  if (match) {
    return { r: match[1], g: match[2], b: match[3] };
  }
  match = matchers.rgba.exec(color);
  if (match) {
    return { r: match[1], g: match[2], b: match[3], a: match[4] };
  }
  match = matchers.hsl.exec(color);
  if (match) {
    return { h: match[1], s: match[2], l: match[3] };
  }
  match = matchers.hsla.exec(color);
  if (match) {
    return { h: match[1], s: match[2], l: match[3], a: match[4] };
  }
  match = matchers.hsv.exec(color);
  if (match) {
    return { h: match[1], s: match[2], v: match[3] };
  }
  match = matchers.hsva.exec(color);
  if (match) {
    return { h: match[1], s: match[2], v: match[3], a: match[4] };
  }
  match = matchers.cmyk.exec(color);
  if (match) {
    return {
      c: match[1],
      m: match[2],
      y: match[3],
      k: match[4]
    };
  }
  match = matchers.hex8.exec(color);
  if (match) {
    return {
      r: parseIntFromHex(match[1]),
      g: parseIntFromHex(match[2]),
      b: parseIntFromHex(match[3]),
      a: convertHexToDecimal(match[4]),
      format: named ? "name" : "hex8"
    };
  }
  match = matchers.hex6.exec(color);
  if (match) {
    return {
      r: parseIntFromHex(match[1]),
      g: parseIntFromHex(match[2]),
      b: parseIntFromHex(match[3]),
      format: named ? "name" : "hex"
    };
  }
  match = matchers.hex4.exec(color);
  if (match) {
    return {
      r: parseIntFromHex(match[1] + match[1]),
      g: parseIntFromHex(match[2] + match[2]),
      b: parseIntFromHex(match[3] + match[3]),
      a: convertHexToDecimal(match[4] + match[4]),
      format: named ? "name" : "hex8"
    };
  }
  match = matchers.hex3.exec(color);
  if (match) {
    return {
      r: parseIntFromHex(match[1] + match[1]),
      g: parseIntFromHex(match[2] + match[2]),
      b: parseIntFromHex(match[3] + match[3]),
      format: named ? "name" : "hex"
    };
  }
  return false;
}
function isValidCSSUnit(color) {
  if (typeof color === "number") {
    return !Number.isNaN(color);
  }
  return matchers.CSS_UNIT.test(color);
}
class TinyColor {
  constructor(color = "", opts = {}) {
    var _a;
    if (color instanceof TinyColor) {
      return color;
    }
    if (typeof color === "number") {
      color = numberInputToObject(color);
    }
    this.originalInput = color;
    const rgb = inputToRGB(color);
    this.originalInput = color;
    this.r = rgb.r;
    this.g = rgb.g;
    this.b = rgb.b;
    this.a = rgb.a;
    this.roundA = Math.round(100 * this.a) / 100;
    this.format = (_a = opts.format) != null ? _a : rgb.format;
    this.gradientType = opts.gradientType;
    if (this.r < 1) {
      this.r = Math.round(this.r);
    }
    if (this.g < 1) {
      this.g = Math.round(this.g);
    }
    if (this.b < 1) {
      this.b = Math.round(this.b);
    }
    this.isValid = rgb.ok;
  }
  isDark() {
    return this.getBrightness() < 128;
  }
  isLight() {
    return !this.isDark();
  }
  getBrightness() {
    const rgb = this.toRgb();
    return (rgb.r * 299 + rgb.g * 587 + rgb.b * 114) / 1e3;
  }
  getLuminance() {
    const rgb = this.toRgb();
    let R;
    let G;
    let B;
    const RsRGB = rgb.r / 255;
    const GsRGB = rgb.g / 255;
    const BsRGB = rgb.b / 255;
    if (RsRGB <= 0.03928) {
      R = RsRGB / 12.92;
    } else {
      R = Math.pow((RsRGB + 0.055) / 1.055, 2.4);
    }
    if (GsRGB <= 0.03928) {
      G = GsRGB / 12.92;
    } else {
      G = Math.pow((GsRGB + 0.055) / 1.055, 2.4);
    }
    if (BsRGB <= 0.03928) {
      B = BsRGB / 12.92;
    } else {
      B = Math.pow((BsRGB + 0.055) / 1.055, 2.4);
    }
    return 0.2126 * R + 0.7152 * G + 0.0722 * B;
  }
  getAlpha() {
    return this.a;
  }
  setAlpha(alpha) {
    this.a = boundAlpha(alpha);
    this.roundA = Math.round(100 * this.a) / 100;
    return this;
  }
  isMonochrome() {
    const { s } = this.toHsl();
    return s === 0;
  }
  toHsv() {
    const hsv = rgbToHsv(this.r, this.g, this.b);
    return { h: hsv.h * 360, s: hsv.s, v: hsv.v, a: this.a };
  }
  toHsvString() {
    const hsv = rgbToHsv(this.r, this.g, this.b);
    const h = Math.round(hsv.h * 360);
    const s = Math.round(hsv.s * 100);
    const v = Math.round(hsv.v * 100);
    return this.a === 1 ? `hsv(${h}, ${s}%, ${v}%)` : `hsva(${h}, ${s}%, ${v}%, ${this.roundA})`;
  }
  toHsl() {
    const hsl = rgbToHsl(this.r, this.g, this.b);
    return { h: hsl.h * 360, s: hsl.s, l: hsl.l, a: this.a };
  }
  toHslString() {
    const hsl = rgbToHsl(this.r, this.g, this.b);
    const h = Math.round(hsl.h * 360);
    const s = Math.round(hsl.s * 100);
    const l = Math.round(hsl.l * 100);
    return this.a === 1 ? `hsl(${h}, ${s}%, ${l}%)` : `hsla(${h}, ${s}%, ${l}%, ${this.roundA})`;
  }
  toHex(allow3Char = false) {
    return rgbToHex(this.r, this.g, this.b, allow3Char);
  }
  toHexString(allow3Char = false) {
    return "#" + this.toHex(allow3Char);
  }
  toHex8(allow4Char = false) {
    return rgbaToHex(this.r, this.g, this.b, this.a, allow4Char);
  }
  toHex8String(allow4Char = false) {
    return "#" + this.toHex8(allow4Char);
  }
  toHexShortString(allowShortChar = false) {
    return this.a === 1 ? this.toHexString(allowShortChar) : this.toHex8String(allowShortChar);
  }
  toRgb() {
    return {
      r: Math.round(this.r),
      g: Math.round(this.g),
      b: Math.round(this.b),
      a: this.a
    };
  }
  toRgbString() {
    const r = Math.round(this.r);
    const g = Math.round(this.g);
    const b = Math.round(this.b);
    return this.a === 1 ? `rgb(${r}, ${g}, ${b})` : `rgba(${r}, ${g}, ${b}, ${this.roundA})`;
  }
  toPercentageRgb() {
    const fmt = (x) => `${Math.round(bound01(x, 255) * 100)}%`;
    return {
      r: fmt(this.r),
      g: fmt(this.g),
      b: fmt(this.b),
      a: this.a
    };
  }
  toPercentageRgbString() {
    const rnd = (x) => Math.round(bound01(x, 255) * 100);
    return this.a === 1 ? `rgb(${rnd(this.r)}%, ${rnd(this.g)}%, ${rnd(this.b)}%)` : `rgba(${rnd(this.r)}%, ${rnd(this.g)}%, ${rnd(this.b)}%, ${this.roundA})`;
  }
  toCmyk() {
    return {
      ...rgbToCmyk(this.r, this.g, this.b)
    };
  }
  toCmykString() {
    const { c, m, y, k } = rgbToCmyk(this.r, this.g, this.b);
    return `cmyk(${c}, ${m}, ${y}, ${k})`;
  }
  toName() {
    if (this.a === 0) {
      return "transparent";
    }
    if (this.a < 1) {
      return false;
    }
    const hex = "#" + rgbToHex(this.r, this.g, this.b, false);
    for (const [key, value] of Object.entries(names)) {
      if (hex === value) {
        return key;
      }
    }
    return false;
  }
  toString(format) {
    const formatSet = Boolean(format);
    format = format != null ? format : this.format;
    let formattedString = false;
    const hasAlpha = this.a < 1 && this.a >= 0;
    const needsAlphaFormat = !formatSet && hasAlpha && (format.startsWith("hex") || format === "name");
    if (needsAlphaFormat) {
      if (format === "name" && this.a === 0) {
        return this.toName();
      }
      return this.toRgbString();
    }
    if (format === "rgb") {
      formattedString = this.toRgbString();
    }
    if (format === "prgb") {
      formattedString = this.toPercentageRgbString();
    }
    if (format === "hex" || format === "hex6") {
      formattedString = this.toHexString();
    }
    if (format === "hex3") {
      formattedString = this.toHexString(true);
    }
    if (format === "hex4") {
      formattedString = this.toHex8String(true);
    }
    if (format === "hex8") {
      formattedString = this.toHex8String();
    }
    if (format === "name") {
      formattedString = this.toName();
    }
    if (format === "hsl") {
      formattedString = this.toHslString();
    }
    if (format === "hsv") {
      formattedString = this.toHsvString();
    }
    if (format === "cmyk") {
      formattedString = this.toCmykString();
    }
    return formattedString || this.toHexString();
  }
  toNumber() {
    return (Math.round(this.r) << 16) + (Math.round(this.g) << 8) + Math.round(this.b);
  }
  clone() {
    return new TinyColor(this.toString());
  }
  lighten(amount = 10) {
    const hsl = this.toHsl();
    hsl.l += amount / 100;
    hsl.l = clamp01(hsl.l);
    return new TinyColor(hsl);
  }
  brighten(amount = 10) {
    const rgb = this.toRgb();
    rgb.r = Math.max(0, Math.min(255, rgb.r - Math.round(255 * -(amount / 100))));
    rgb.g = Math.max(0, Math.min(255, rgb.g - Math.round(255 * -(amount / 100))));
    rgb.b = Math.max(0, Math.min(255, rgb.b - Math.round(255 * -(amount / 100))));
    return new TinyColor(rgb);
  }
  darken(amount = 10) {
    const hsl = this.toHsl();
    hsl.l -= amount / 100;
    hsl.l = clamp01(hsl.l);
    return new TinyColor(hsl);
  }
  tint(amount = 10) {
    return this.mix("white", amount);
  }
  shade(amount = 10) {
    return this.mix("black", amount);
  }
  desaturate(amount = 10) {
    const hsl = this.toHsl();
    hsl.s -= amount / 100;
    hsl.s = clamp01(hsl.s);
    return new TinyColor(hsl);
  }
  saturate(amount = 10) {
    const hsl = this.toHsl();
    hsl.s += amount / 100;
    hsl.s = clamp01(hsl.s);
    return new TinyColor(hsl);
  }
  greyscale() {
    return this.desaturate(100);
  }
  spin(amount) {
    const hsl = this.toHsl();
    const hue = (hsl.h + amount) % 360;
    hsl.h = hue < 0 ? 360 + hue : hue;
    return new TinyColor(hsl);
  }
  mix(color, amount = 50) {
    const rgb1 = this.toRgb();
    const rgb2 = new TinyColor(color).toRgb();
    const p = amount / 100;
    const rgba = {
      r: (rgb2.r - rgb1.r) * p + rgb1.r,
      g: (rgb2.g - rgb1.g) * p + rgb1.g,
      b: (rgb2.b - rgb1.b) * p + rgb1.b,
      a: (rgb2.a - rgb1.a) * p + rgb1.a
    };
    return new TinyColor(rgba);
  }
  analogous(results = 6, slices = 30) {
    const hsl = this.toHsl();
    const part = 360 / slices;
    const ret = [this];
    for (hsl.h = (hsl.h - (part * results >> 1) + 720) % 360; --results; ) {
      hsl.h = (hsl.h + part) % 360;
      ret.push(new TinyColor(hsl));
    }
    return ret;
  }
  complement() {
    const hsl = this.toHsl();
    hsl.h = (hsl.h + 180) % 360;
    return new TinyColor(hsl);
  }
  monochromatic(results = 6) {
    const hsv = this.toHsv();
    const { h } = hsv;
    const { s } = hsv;
    let { v } = hsv;
    const res = [];
    const modification = 1 / results;
    while (results--) {
      res.push(new TinyColor({ h, s, v }));
      v = (v + modification) % 1;
    }
    return res;
  }
  splitcomplement() {
    const hsl = this.toHsl();
    const { h } = hsl;
    return [
      this,
      new TinyColor({ h: (h + 72) % 360, s: hsl.s, l: hsl.l }),
      new TinyColor({ h: (h + 216) % 360, s: hsl.s, l: hsl.l })
    ];
  }
  onBackground(background) {
    const fg = this.toRgb();
    const bg = new TinyColor(background).toRgb();
    const alpha = fg.a + bg.a * (1 - fg.a);
    return new TinyColor({
      r: (fg.r * fg.a + bg.r * bg.a * (1 - fg.a)) / alpha,
      g: (fg.g * fg.a + bg.g * bg.a * (1 - fg.a)) / alpha,
      b: (fg.b * fg.a + bg.b * bg.a * (1 - fg.a)) / alpha,
      a: alpha
    });
  }
  triad() {
    return this.polyad(3);
  }
  tetrad() {
    return this.polyad(4);
  }
  polyad(n) {
    const hsl = this.toHsl();
    const { h } = hsl;
    const result = [this];
    const increment = 360 / n;
    for (let i = 1; i < n; i++) {
      result.push(new TinyColor({ h: (h + i * increment) % 360, s: hsl.s, l: hsl.l }));
    }
    return result;
  }
  equals(color) {
    const comparedColor = new TinyColor(color);
    if (this.format === "cmyk" || comparedColor.format === "cmyk") {
      return this.toCmykString() === comparedColor.toCmykString();
    }
    return this.toRgbString() === comparedColor.toRgbString();
  }
}
var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, key + "", value);
  return value;
};
const isHsbColor = (color) => {
  return Boolean(color && typeof color === "object" && "h" in color && "b" in color);
};
const isBrightColor = (value, bgColorToken) => {
  const color = new Color(value);
  const { r, g, b, a } = color.toRgb();
  const hsv = color.onBackground(bgColorToken).toHsv();
  if (a <= 0.5) {
    return hsv.v > 0.5;
  }
  return r * 0.299 + g * 0.587 + b * 0.114 > 192;
};
const convertHsb2Hsv = (color) => {
  if (isHsbColor(color)) {
    const { b, ...resets } = color;
    return {
      ...resets,
      v: b
    };
  }
  if (typeof color === "string" && /hsb/.test(color)) {
    return color.replace(/hsb/, "hsv");
  }
  return color;
};
const getRoundNumber = (value) => Math.round(Number(value || 0));
const toHexFormat = (value, alpha) => (value == null ? void 0 : value.replace(/[^\w/]/gi, "").slice(0, alpha ? 8 : 6)) || "";
const getHex = (value, alpha) => value ? toHexFormat(value, alpha) : "";
const getTargetFormatColorStr = (color, colorFormat) => {
  switch (colorFormat) {
    case "hsb":
      return color.toHsbString();
    case "rgb":
      return color.toRgbString();
    case "hex":
    default:
      return color.toHexString();
  }
};
class Color {
  constructor(color) {
    __publicField(this, "metaColor");
    this.metaColor = new TinyColor(convertHsb2Hsv(color != null ? color : ""));
  }
  isValid() {
    return this.metaColor.isValid;
  }
  equals(color) {
    return this.metaColor.equals(convertHsb2Hsv(color));
  }
  toHsb() {
    let hsv = this.metaColor.toHsv();
    if (typeof this.metaColor.originalInput === "object" && this.metaColor.originalInput) {
      if ("h" in this.metaColor.originalInput) {
        hsv = this.metaColor.originalInput;
      }
    }
    const { v, a, ...resets } = hsv;
    return {
      ...resets,
      a,
      b: hsv.v
    };
  }
  toHsbString() {
    const hsb = this.toHsb();
    const saturation = getRoundNumber(hsb.s * 100);
    const lightness = getRoundNumber(hsb.b * 100);
    const hue = getRoundNumber(hsb.h);
    const a = hsb.a;
    const hsbString = `hsb(${hue}, ${saturation}%, ${lightness}%)`;
    const hsbaString = `hsba(${hue}, ${saturation}%, ${lightness}%, ${a.toFixed(a === 0 ? 0 : 2)})`;
    return a === 1 ? hsbString : hsbaString;
  }
  toHsv() {
    return this.metaColor.toHsv();
  }
  toHex(alpha = true) {
    return getHex(this.metaColor.toHexString(), alpha ? this.metaColor.getAlpha() < 1 : false);
  }
  toHexString() {
    return this.metaColor.getAlpha() === 1 ? this.metaColor.toHexString() : this.metaColor.toHex8String();
  }
  toRgb() {
    return this.metaColor.toRgb();
  }
  toRgbString() {
    return this.metaColor.toRgbString();
  }
  getAlpha() {
    return this.metaColor.getAlpha();
  }
  setAlpha(alpha) {
    const hsbVal = this.toHsb();
    hsbVal.a = alpha;
    this.metaColor = new TinyColor(convertHsb2Hsv(hsbVal));
    this.metaColor;
  }
  setHue(hue) {
    const hsbVal = this.toHsb();
    hsbVal.h = hue;
    this.metaColor = new TinyColor(convertHsb2Hsv(hsbVal));
    this.metaColor;
  }
  setSaturation(saturation) {
    const hsbVal = this.toHsb();
    hsbVal.s = saturation;
    this.metaColor = new TinyColor(convertHsb2Hsv(hsbVal));
    this.metaColor;
  }
  onBackground(bgColor) {
    return new Color(this.metaColor.onBackground(bgColor));
  }
}
function percentFormatter(num) {
  return `${num != null ? num : 0}%`;
}
function getDefaultThemeTokens(tokens, algorithms) {
  const { getGreyColors, getColorPalette } = algorithms;
  const {
    colorPrimary,
    colorContainerBg,
    borderRadiusSm,
    controlHeightSm,
    controlHeightMd,
    controlHeightLg,
    controlFontSizeSm,
    controlFontSizeMd,
    controlFontSizeLg
  } = tokens;
  const greyColors = getGreyColors();
  const primaryColorPalette = getColorPalette(colorPrimary);
  return {
    panelWidth: 358,
    sliderHeight: 12,
    sliderHandleSize: 12,
    sliderHandleBorderWidth: 2,
    sliderHandleBorderColor: colorContainerBg,
    sliderHandleBoxShadow: "inset 0 0 1px 0 rgb(0 0 0 / 25%), 0 0 0 1px rgb(0 0 0 / 6%)",
    indicatorBorderRadius: borderRadiusSm,
    indicatorCheckedIconColorLight: greyColors.d30,
    indicatorCheckedIconColorDark: greyColors.l40,
    indicatorHoverBoxShadow: `0px 0px 0px 2px ${getAlphaColor(primaryColorPalette.l40, 0.4)}`,
    indicatorCheckedBoxShadow: `0px 0px 0px 2px ${getAlphaColor(primaryColorPalette.l40, 0.4)}`,
    triggerSizeSm: controlHeightSm,
    triggerSizeMd: controlHeightMd,
    triggerSizeLg: controlHeightLg,
    triggerFontSizeSm: controlFontSizeSm,
    triggerFontSizeMd: controlFontSizeMd,
    triggerFontSizeLg: controlFontSizeLg,
    alphaGridColor: "rgb(0 0 0 / 6%)",
    alphaGridBoxShadow: "inset 0 0 1px 0 rgb(0 0 0 / 25%)"
  };
}
function getDarkThemeTokens(tokens, algorithms) {
  const defaultTokens = getDefaultThemeTokens(tokens, algorithms);
  const { getGreyColors } = algorithms;
  const greyColors = getGreyColors();
  return {
    ...defaultTokens,
    indicatorCheckedIconColorLight: greyColors.l40,
    indicatorCheckedIconColorDark: greyColors.d40,
    sliderHandleBoxShadow: "inset 0 0 1px 0 rgb(255 255 255 / 25%), 0 0 0 1px rgb(255 255 255 / 6%)",
    alphaGridColor: "rgb(255 255 255 / 6%)",
    alphaGridBoxShadow: "inset 0 0 1px 0 rgb(255 255 255 / 25%)"
  };
}
const getThemeTokens = (tokens, presetTheme, algorithms) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens, algorithms) : getDarkThemeTokens(tokens, algorithms);
};
var ColorPickerIndicator = /* @__PURE__ */ defineComponent({
  name: "IxColorPickerIndicator",
  props: colorPickerIndicatorProps,
  setup(props) {
    const {
      hashId,
      globalHashId,
      registerToken
    } = useThemeToken("colorPicker");
    const {
      themeTokens
    } = useThemeToken();
    registerToken(getThemeTokens);
    const {
      prefixCls
    } = useGlobalConfig("common");
    const indicatorPrefixCls = computed(() => `${prefixCls}-color-picker-indicator`);
    const color = computed(() => new Color(props.color));
    const bgColorToken = computed(() => themeTokens.value.colorBg);
    const isBlank = computed(() => !props.color || !color.value.isValid());
    const isBright = computed(() => isBrightColor(props.color, bgColorToken.value));
    const classes = computed(() => {
      const indicatorCls = indicatorPrefixCls.value;
      return {
        [indicatorCls]: true,
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [`${indicatorCls}-hoverable`]: props.hoverable,
        [`${indicatorCls}-with-box-shadow`]: props.showBoxShadow,
        [`${indicatorCls}-disabled`]: props.disabled,
        [`${indicatorCls}-focused`]: props.focused,
        [`${indicatorCls}-blank`]: isBlank.value,
        [`${indicatorCls}-bright`]: isBright.value,
        [`${indicatorCls}-checked`]: !!props.checked
      };
    });
    const innerStyle = computed(() => {
      if (isBlank.value) {
        return;
      }
      return {
        backgroundColor: color.value.toRgbString()
      };
    });
    return () => createVNode("div", {
      "class": classes.value
    }, [createVNode("div", {
      "class": `${indicatorPrefixCls.value}-inner`,
      "style": innerStyle.value
    }, null)]);
  }
});
const colorPickerPanelToken = Symbol("colorPickerPanelToken");
var Slider = /* @__PURE__ */ defineComponent({
  props: {
    value: Object,
    trackStyle: Object,
    handleStyle: Object,
    boundaryWithoutHandle: Boolean,
    xIsCircle: Boolean,
    yIsCircle: Boolean,
    onChange: Function
  },
  setup(props) {
    const {
      mergedPrefixCls
    } = inject(colorPickerPanelToken);
    const [xInSecondHalf, setXInSecondHalf] = useState(false);
    const [yInSecondHalf, setYInSecondHalf] = useState(false);
    const [offsetBuffer, setOffsetBuffer] = useState({
      x: 0,
      y: 0
    });
    const [isDragging, setIsDragging] = useState(false);
    const trackRef = shallowRef();
    const handleRef = shallowRef();
    const trackSize = useElementSize(trackRef);
    const handleSize = useElementSize(handleRef);
    const availableTrackWidth = computed(() => Math.max(props.boundaryWithoutHandle ? trackSize.value.width : trackSize.value.width - handleSize.value.width, 0));
    const availableTrackHeight = computed(() => Math.max(props.boundaryWithoutHandle ? trackSize.value.height : trackSize.value.height - handleSize.value.height, 0));
    const offset = computed(() => {
      var _a, _b;
      let xPercent = keepPercentInRange((_a = props.value) == null ? void 0 : _a.x);
      let yPercent = keepPercentInRange((_b = props.value) == null ? void 0 : _b.y);
      if (xPercent === 0 && xInSecondHalf.value && props.xIsCircle) {
        xPercent = 1;
      }
      if (yPercent === 0 && yInSecondHalf.value && props.yIsCircle) {
        yPercent = 1;
      }
      const xOffset = xPercent * availableTrackWidth.value;
      const yOffset = yPercent * availableTrackHeight.value;
      return {
        x: xOffset,
        y: yOffset
      };
    });
    const updateHandleOffset = (newOffset) => {
      var _a;
      const width = availableTrackWidth.value;
      const height = availableTrackHeight.value;
      const xPercent = Number(keepPercentInRange(newOffset.x / width).toFixed(2));
      const yPercent = Number(keepPercentInRange(newOffset.y / height).toFixed(2));
      setXInSecondHalf(xPercent > 0.5);
      setYInSecondHalf(yPercent > 0.5);
      (_a = props.onChange) == null ? void 0 : _a.call(props, {
        x: xPercent,
        y: yPercent
      });
      setOffsetBuffer(newOffset);
    };
    const handleOffsetChange = (newOffset) => {
      if (!isDragging.value) {
        return;
      }
      updateHandleOffset(newOffset);
    };
    const handleDragStart = () => {
      setOffsetBuffer({
        ...offset.value
      });
      setIsDragging(true);
    };
    const handleDragEnd = () => {
      setIsDragging(false);
    };
    const handleTrackMousedown = (evt) => {
      const trackEl = convertElement(trackRef.value);
      if (!trackEl) {
        return;
      }
      const {
        clientX,
        clientY
      } = evt;
      const {
        x,
        y
      } = trackEl.getBoundingClientRect();
      const newOffset = {
        x: clientX - x,
        y: clientY - y
      };
      updateHandleOffset(newOffset);
    };
    return () => {
      const prefixCls = `${mergedPrefixCls.value}-slider`;
      return createVNode("div", {
        "ref": trackRef,
        "class": `${prefixCls}-track`,
        "style": props.trackStyle,
        "tabindex": -1,
        "onMousedown": handleTrackMousedown
      }, [createVNode(CdkDndMovable, {
        "ref": handleRef,
        "class": `${prefixCls}-handle`,
        "style": props.boundaryWithoutHandle ? {
          width: 0,
          height: 0
        } : void 0,
        "tabindex": -1,
        "mode": "immediate",
        "offset": isDragging.value ? offsetBuffer.value : offset.value,
        "boundary": trackRef.value,
        "dropTargets": [handleRef.value],
        "onOffsetChange": handleOffsetChange,
        "onDragStart": handleDragStart,
        "onDrop": handleDragEnd
      }, {
        default: () => [createVNode("div", {
          "class": `${prefixCls}-handle-inner`,
          "style": props.handleStyle
        }, null)]
      })]);
    };
  }
});
function keepPercentInRange(precent) {
  const value = Math.min(1, Math.max(0, precent != null ? precent : 0));
  return isNaN$1(value) ? 0 : value;
}
var AlphaSlider = /* @__PURE__ */ defineComponent({
  setup() {
    const {
      mergedPrefixCls,
      alpha,
      rgbValue,
      setAlpha
    } = inject(colorPickerPanelToken);
    const trackStyle = computed(() => {
      const bgColor = new Color(Object.assign({
        ...rgbValue.value
      }, {
        a: 1
      })).toRgbString();
      return {
        background: `linear-gradient(to right, rgba(255, 0, 4, 0), ${bgColor})`
      };
    });
    const handleStyle = computed(() => {
      const color = new Color(rgbValue.value);
      color.setAlpha(alpha.value / 100);
      const backgroundColor = color.toRgbString();
      return {
        backgroundColor
      };
    });
    const handleChange = (value) => {
      setAlpha(value.x);
    };
    return () => createVNode("div", {
      "class": `${mergedPrefixCls.value}-alpha-slider-wrapper`
    }, [createVNode(Slider, {
      "class": `${mergedPrefixCls.value}-alpha-slider`,
      "value": {
        x: alpha.value,
        y: 0
      },
      "trackStyle": trackStyle.value,
      "handleStyle": handleStyle.value,
      "onChange": handleChange
    }, null)]);
  }
});
var ColorSelector = /* @__PURE__ */ defineComponent({
  setup() {
    const {
      mergedPrefixCls,
      hue,
      saturation,
      brightness,
      setHsb
    } = inject(colorPickerPanelToken);
    const sliderValue = computed(() => ({
      x: saturation.value,
      y: 1 - brightness.value
    }));
    const trackStyle = computed(() => {
      const backgroundColor = new Color({
        h: hue.value,
        s: 100,
        b: 100,
        a: 1
      }).toRgbString();
      return {
        backgroundColor
      };
    });
    const handleChange = (value) => {
      setHsb({
        s: value.x,
        b: 1 - value.y
      });
    };
    return () => createVNode(Slider, {
      "class": `${mergedPrefixCls.value}-color-selector`,
      "value": sliderValue.value,
      "trackStyle": trackStyle.value,
      "boundaryWithoutHandle": true,
      "onChange": handleChange
    }, null);
  }
});
var HueSlider = /* @__PURE__ */ defineComponent({
  setup() {
    const {
      mergedPrefixCls,
      hue,
      setHsb
    } = inject(colorPickerPanelToken);
    const handleStyle = computed(() => {
      const backgroundColor = new Color({
        h: hue.value,
        s: 100,
        b: 100,
        a: 1
      }).toRgbString();
      return {
        backgroundColor
      };
    });
    const sliderValue = computed(() => {
      return {
        x: hue.value / 360,
        y: 0
      };
    });
    const handleChange = (value) => {
      setHsb({
        h: Math.round(value.x * 360)
      });
    };
    return () => createVNode(Slider, {
      "class": `${mergedPrefixCls.value}-hue-slider`,
      "value": sliderValue.value,
      "handleStyle": handleStyle.value,
      "xIsCircle": true,
      "onChange": handleChange
    }, null);
  }
});
var ColorPickerPalette = /* @__PURE__ */ defineComponent({
  name: "IxColorPickerPalette",
  setup() {
    const {
      mergedPrefixCls,
      rgbValue
    } = inject(colorPickerPanelToken);
    return () => {
      const prefixCls = `${mergedPrefixCls.value}-palette`;
      return createVNode("div", {
        "class": prefixCls
      }, [createVNode(ColorSelector, null, null), createVNode("div", {
        "class": `${prefixCls}-bottom`
      }, [createVNode("div", {
        "class": `${prefixCls}-sliders`
      }, [createVNode(HueSlider, null, null), createVNode(AlphaSlider, null, null)]), createVNode(ColorPickerIndicator, {
        "class": `${prefixCls}-indicator`,
        "color": rgbValue.value,
        "hoverable": false,
        "showBoxShadow": false
      }, null)])]);
    };
  }
});
function useResolvedPresets(props) {
  const resolvedPresets = computed(() => {
    var _a, _b;
    return (_b = (_a = props.presets) == null ? void 0 : _a.map((preset) => {
      return {
        key: preset.key,
        label: preset.label,
        colors: preset.colors.map((color) => {
          if (!color) {
            return "rgba(0,0,0,0)";
          }
          const colorObj = new Color(color);
          if (!colorObj.isValid()) {
            return false;
          }
          return color;
        }).filter(Boolean),
        defaultOpen: preset.defaultOpen
      };
    })) != null ? _b : [];
  });
  return resolvedPresets;
}
function useSelectedColor(valueRef, format, handleChange) {
  const [selectedColor, setSelectedColor] = useState(new Color(valueRef.value));
  let tempSelectedColor;
  const hsbValue = computed(() => selectedColor.value.toHsb());
  const hexValue = computed(() => selectedColor.value.toHex());
  const rgbValue = computed(() => selectedColor.value.toRgb());
  const hue = computed(() => hsbValue.value.h);
  const saturation = computed(() => hsbValue.value.s);
  const brightness = computed(() => hsbValue.value.b);
  const alpha = computed(() => selectedColor.value.getAlpha());
  const updateColor = (color) => {
    if (!color.isValid()) {
      return;
    }
    tempSelectedColor = color;
    if (tempSelectedColor.equals(hsbValue.value)) {
      setSelectedColor(tempSelectedColor);
    }
    const colorStr = getTargetFormatColorStr(color, format.value);
    handleChange(colorStr);
  };
  watch(
    valueRef,
    (value) => {
      let color = new Color(value || { h: 0, s: 0, b: 0, a: 0 });
      if (tempSelectedColor && color.equals(tempSelectedColor.toRgb())) {
        color = tempSelectedColor;
      }
      setSelectedColor(color);
    },
    { immediate: true }
  );
  watch(format, () => {
    updateColor(selectedColor.value);
  });
  const setHsb = (v) => {
    updateColor(new Color(Object.assign({ ...hsbValue.value }, v)));
  };
  const setHex = (v, alpha2) => {
    const color = new Color(v);
    if (!isNil(alpha2)) {
      color.setAlpha(alpha2);
    }
    updateColor(color);
  };
  const setRgb = (v) => {
    updateColor(new Color(Object.assign({ ...rgbValue.value }, v)));
  };
  const setAlpha = (v) => {
    const color = new Color(hsbValue.value);
    color.setAlpha(v);
    updateColor(color);
  };
  return {
    selectedColor,
    hsbValue,
    hexValue,
    rgbValue,
    hue,
    saturation,
    brightness,
    alpha,
    setHsb,
    setHex,
    setRgb,
    setAlpha,
    updateColor
  };
}
function useSelectedFormat(props, config) {
  const [format, setFormat] = useControlledProp(props, "format");
  const mergedFormat = computed(() => {
    var _a;
    return (_a = format.value) != null ? _a : config.format;
  });
  const updateFormat = (newFormat) => {
    const oldFormat = toRaw(mergedFormat.value);
    setFormat(newFormat);
    callEmit(props.onFormatChange, newFormat, oldFormat);
  };
  return {
    format: mergedFormat,
    setFormat: updateFormat
  };
}
function useColorPickerPanelContext(props, valueRef, handleChange) {
  let context = inject(colorPickerPanelToken, null);
  if (!context) {
    const common = useGlobalConfig("common");
    const config = useGlobalConfig("colorPicker");
    const locale = useGlobalConfig("locale");
    const selectedFormatContext = useSelectedFormat(props, config);
    const selectedColorContext = useSelectedColor(valueRef, selectedFormatContext.format, handleChange);
    const resolvedPresets = useResolvedPresets(props);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-color-picker`);
    context = {
      mergedPrefixCls,
      locale: locale.colorPicker,
      ...selectedFormatContext,
      ...selectedColorContext,
      resolvedPresets
    };
    provide(colorPickerPanelToken, context);
  }
  return context;
}
var HexEditor = /* @__PURE__ */ defineComponent({
  setup() {
    const {
      mergedPrefixCls,
      hexValue,
      setHex
    } = inject(colorPickerPanelToken);
    const [hexInput, setHexInput] = useState("");
    watch(hexValue, setHexInput, {
      immediate: true
    });
    const handleInputChange = (value) => {
      setHexInput(value);
      setHex(value);
    };
    return () => {
      const prefixCls = `${mergedPrefixCls.value}-hex-editor`;
      return createVNode(IxInput, {
        "class": prefixCls,
        "size": "md",
        "value": hexInput.value,
        "onChange": handleInputChange
      }, null);
    };
  }
});
var HsbEditor = /* @__PURE__ */ defineComponent({
  setup() {
    const {
      mergedPrefixCls,
      hsbValue,
      setHsb
    } = inject(colorPickerPanelToken);
    const [hInput, setHInput] = useState(0);
    const [sInput, setSInput] = useState(0);
    const [bInput, setBInput] = useState(0);
    watch(hsbValue, (v) => {
      setHInput(v.h);
      setSInput(v.s * 100);
      setBInput(v.b * 100);
    }, {
      immediate: true
    });
    const handleHInputChange = (value) => {
      if (isNil(value)) {
        return;
      }
      setHInput(value);
      setHsb({
        h: value
      });
    };
    const handleSInputChange = (value) => {
      if (isNil(value)) {
        return;
      }
      setSInput(value);
      setHsb({
        s: value / 100
      });
    };
    const handleBInputChange = (value) => {
      if (isNil(value)) {
        return;
      }
      setBInput(value);
      setHsb({
        b: value / 100
      });
    };
    return () => {
      const prefixCls = `${mergedPrefixCls.value}-hsb-editor`;
      return createVNode("div", {
        "class": prefixCls
      }, [createVNode(IxInputNumber, {
        "class": [`${prefixCls}-input`, `${prefixCls}-input-h`],
        "value": hInput.value,
        "precision": 0,
        "max": 360,
        "min": 0,
        "onChange": handleHInputChange
      }, null), createVNode(IxInputNumber, {
        "class": [`${prefixCls}-input`, `${prefixCls}-input-s`],
        "value": sInput.value,
        "formatter": percentFormatter,
        "precision": 0,
        "max": 100,
        "min": 0,
        "onChange": handleSInputChange
      }, null), createVNode(IxInputNumber, {
        "class": [`${prefixCls}-input`, `${prefixCls}-input-b`],
        "value": bInput.value,
        "formatter": percentFormatter,
        "precision": 0,
        "max": 100,
        "min": 0,
        "onChange": handleBInputChange
      }, null)]);
    };
  }
});
var RgbEditor = /* @__PURE__ */ defineComponent({
  setup() {
    const {
      mergedPrefixCls,
      rgbValue,
      setRgb
    } = inject(colorPickerPanelToken);
    const [rInput, setRInput] = useState(0);
    const [gInput, setGInput] = useState(0);
    const [bInput, setBInput] = useState(0);
    watch(rgbValue, (v) => {
      setRInput(v.r);
      setGInput(v.g);
      setBInput(v.b);
    }, {
      immediate: true
    });
    const handleRInputChange = (value) => {
      if (isNil(value)) {
        return;
      }
      setRInput(value);
      setRgb({
        r: value
      });
    };
    const handleGInputChange = (value) => {
      if (isNil(value)) {
        return;
      }
      setGInput(value);
      setRgb({
        g: value
      });
    };
    const handleBInputChange = (value) => {
      if (isNil(value)) {
        return;
      }
      setBInput(value);
      setRgb({
        b: value
      });
    };
    return () => {
      const prefixCls = `${mergedPrefixCls.value}-rgb-editor`;
      return createVNode("div", {
        "class": prefixCls
      }, [createVNode(IxInputNumber, {
        "class": [`${prefixCls}-input`, `${prefixCls}-input-r`],
        "value": rInput.value,
        "precision": 0,
        "max": 255,
        "min": 0,
        "onChange": handleRInputChange
      }, null), createVNode(IxInputNumber, {
        "class": [`${prefixCls}-input`, `${prefixCls}-input-g`],
        "value": gInput.value,
        "precision": 0,
        "max": 255,
        "min": 0,
        "onChange": handleGInputChange
      }, null), createVNode(IxInputNumber, {
        "class": [`${prefixCls}-input`, `${prefixCls}-input-b`],
        "value": bInput.value,
        "precision": 0,
        "max": 255,
        "min": 0,
        "onChange": handleBInputChange
      }, null)]);
    };
  }
});
const formatSelectDataSource = [{
  key: "hex",
  label: "HEX"
}, {
  key: "hsb",
  label: "HSB"
}, {
  key: "rgb",
  label: "RGB"
}];
var ColorPickerEditor = /* @__PURE__ */ defineComponent({
  name: "IxColorPickerEditor",
  setup() {
    const {
      mergedPrefixCls,
      alpha,
      format,
      setAlpha,
      setFormat
    } = inject(colorPickerPanelToken);
    const alphaInput = computed(() => alpha.value * 100);
    const handleAlphaInputChange = (v) => {
      if (isNil(v)) {
        return;
      }
      setAlpha(v / 100);
    };
    return () => {
      const prefixCls = `${mergedPrefixCls.value}-editor`;
      const formatValue = format.value;
      const SubEditorComp = formatValue === "hex" ? HexEditor : formatValue === "hsb" ? HsbEditor : RgbEditor;
      return createVNode("div", {
        "class": prefixCls
      }, [createVNode(IxSelect, {
        "class": `${prefixCls}-format-select`,
        "value": format.value,
        "dataSource": formatSelectDataSource,
        "onChange": setFormat
      }, null), createVNode(SubEditorComp, {
        "class": `${prefixCls}-sub`
      }, null), createVNode(IxInputNumber, {
        "class": `${prefixCls}-alpha-input`,
        "value": alphaInput.value,
        "formatter": percentFormatter,
        "precision": 0,
        "max": 100,
        "min": 0,
        "onChange": handleAlphaInputChange
      }, null)]);
    };
  }
});
var PresetIndicator = /* @__PURE__ */ defineComponent({
  props: {
    color: String
  },
  setup(props) {
    const {
      mergedPrefixCls,
      hsbValue,
      updateColor
    } = inject(colorPickerPanelToken);
    const color = computed(() => new Color(props.color));
    const isChecked = computed(() => {
      return color.value.isValid() && color.value.equals(hsbValue.value);
    });
    const handleClick = () => {
      updateColor(color.value);
    };
    return () => createVNode(ColorPickerIndicator, {
      "class": `${mergedPrefixCls.value}-presets-indicator`,
      "color": props.color,
      "checked": isChecked.value,
      "onClick": handleClick
    }, null);
  }
});
var ColorPickerPresets = /* @__PURE__ */ defineComponent({
  name: "IxColorPickerPresets",
  setup() {
    const {
      mergedPrefixCls,
      locale,
      resolvedPresets
    } = inject(colorPickerPanelToken);
    const [expandedKeys, setExpandedKeys] = useState([]);
    const expandeKeySet = computed(() => new Set(expandedKeys.value));
    watch(resolvedPresets, (presets) => {
      const newExpandedKeySet = /* @__PURE__ */ new Set();
      presets.forEach((preset) => {
        if (preset.defaultOpen || expandeKeySet.value.has(preset.key)) {
          newExpandedKeySet.add(preset.key);
        }
      });
      setExpandedKeys([...newExpandedKeySet]);
    }, {
      immediate: true
    });
    const renderCollapseHeader = ({
      expanded,
      changeExpand,
      label
    }) => {
      return createVNode(IxHeader, {
        "title": label,
        "onClick": changeExpand,
        "size": "sm"
      }, {
        prefix: () => createVNode(IxIcon, {
          "class": `${mergedPrefixCls.value}-presets-collapse-icon`,
          "name": "right",
          "rotate": expanded ? 90 : 0
        }, null)
      });
    };
    return () => {
      const prefixCls = `${mergedPrefixCls.value}-presets`;
      return createVNode(IxCollapse, {
        "class": prefixCls,
        "expandedKeys": expandedKeys.value,
        "ghost": true,
        "borderless": true,
        "onUpdate:expandedKeys": setExpandedKeys
      }, {
        default: () => [resolvedPresets.value.map((preset) => createVNode(IxCollapsePanel, {
          "key": preset.key
        }, {
          default: () => [preset.colors.length ? createVNode("div", {
            "class": `${mergedPrefixCls.value}-presets-colors`
          }, [preset.colors.map((color) => createVNode(PresetIndicator, {
            "color": color
          }, null))]) : createVNode("span", {
            "class": `${mergedPrefixCls.value}-empty`
          }, [locale.presetEmpty])],
          header: (params) => renderCollapseHeader({
            ...params,
            label: preset.label
          })
        }))]
      });
    };
  }
});
var ColorPickerPanel = /* @__PURE__ */ defineComponent({
  name: "IxColorPickerPanel",
  props: colorPickerPanelProps,
  setup(props, {
    slots
  }) {
    const {
      hashId,
      globalHashId,
      registerToken
    } = useThemeToken("colorPicker");
    registerToken(getThemeTokens);
    let context = inject(colorPickerPanelToken, null);
    if (!context) {
      const [valueRef, setValue] = useControlledProp(props, "value");
      const handleChange = (value) => {
        const oldValue = toRaw(valueRef.value);
        setValue(value);
        callEmit(props.onChange, value, oldValue);
      };
      context = useColorPickerPanelContext(props, valueRef, handleChange);
    }
    const {
      mergedPrefixCls,
      resolvedPresets
    } = context;
    const classes = computed(() => {
      const prefixCls = `${mergedPrefixCls.value}-panel`;
      return {
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [prefixCls]: true,
        [`${prefixCls}-paddingless`]: props.paddingless
      };
    });
    const renderPanelContent = (prefixCls) => {
      if (slots.default) {
        return slots.default();
      }
      return createVNode("div", {
        "class": `${prefixCls}-inner`
      }, [createVNode(ColorPickerPalette, null, null), createVNode(ColorPickerEditor, null, null), resolvedPresets.value.length ? [createVNode("div", {
        "class": `${prefixCls}-separator`
      }, null), createVNode(ColorPickerPresets, null, null)] : void 0]);
    };
    return () => {
      const prefixCls = `${mergedPrefixCls.value}-panel`;
      return createVNode("div", {
        "class": classes.value
      }, [renderPanelContent(prefixCls)]);
    };
  }
});
var ColorPickerTrigger = /* @__PURE__ */ defineComponent({
  name: "IxColorPickerTrigger",
  props: colorPickerTriggerProps,
  setup(props, {
    slots
  }) {
    const {
      hashId,
      globalHashId,
      registerToken
    } = useThemeToken("colorPicker");
    registerToken(getThemeTokens);
    const common = useGlobalConfig("common");
    const triggerCls = computed(() => `${common.prefixCls}-color-picker-trigger`);
    const color = computed(() => new Color(props.value));
    const text = computed(() => color.value.isValid() ? getTargetFormatColorStr(color.value, props.format) : "");
    const classes = computed(() => {
      const prefixCls = triggerCls.value;
      return {
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [prefixCls]: true,
        [`${prefixCls}-${props.size}`]: props.size,
        [`${prefixCls}-focused`]: props.focused,
        [`${prefixCls}-disabled`]: props.disabled
      };
    });
    const renderContent = () => {
      if (slots.default) {
        return slots.default({
          value: props.value,
          format: props.format,
          text: text.value,
          disabled: props.disabled,
          focused: props.focused,
          size: props.size
        });
      }
      const indicator = createVNode(ColorPickerIndicator, {
        "color": props.value,
        "disabled": props.disabled,
        "focused": props.focused
      }, null);
      if (!props.showText) {
        return indicator;
      }
      return [indicator, createVNode("span", {
        "class": `${triggerCls.value}-text`
      }, [text.value])];
    };
    return () => createVNode("div", {
      "class": classes.value,
      "tabindex": -1
    }, [renderContent()]);
  }
});
function usePickerState(props, config) {
  const { accessor, control } = useAccessorAndControl();
  useFormItemRegister(control);
  const mergedSize = useFormSize(props, config);
  const mergedStatus = useFormStatus(props, control);
  const [focused, setFocused] = useState(false);
  const handleChange = (value) => {
    const oldValue = toRaw(accessor.value);
    accessor.setValue(value);
    callEmit(props.onChange, value, oldValue);
  };
  const handleFocus = (evt) => {
    setFocused(true);
    callEmit(props.onFocus, evt);
  };
  const handleBlur = (evt) => {
    setFocused(false);
    accessor.markAsBlurred();
    callEmit(props.onBlur, evt);
  };
  return {
    accessor,
    mergedSize,
    mergedStatus,
    focused,
    handleChange,
    handleFocus,
    handleBlur
  };
}
var ColorPicker = /* @__PURE__ */ defineComponent({
  name: "IxColorPicker",
  inheritAttrs: false,
  props: colorPickerProps,
  setup(props, {
    attrs,
    slots
  }) {
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("colorPicker");
    registerToken(getThemeTokens);
    const config = useGlobalConfig("colorPicker");
    const [overlayOpened, setOverlayOpened] = useControlledProp(props, "open", false);
    const {
      accessor,
      mergedSize,
      mergedStatus,
      focused,
      handleFocus,
      handleBlur,
      handleChange
    } = usePickerState(props, config);
    const valueRef = toRef(accessor, "value");
    const {
      mergedPrefixCls,
      format
    } = useColorPickerPanelContext(props, valueRef, handleChange);
    const renderTrigger = () => createVNode(ColorPickerTrigger, {
      "value": accessor.value,
      "disabled": accessor.disabled,
      "format": format.value,
      "focused": focused.value,
      "size": mergedSize.value,
      "showText": props.showText
    }, {
      default: slots.trigger
    });
    const renderContent = () => createVNode(ColorPickerPanel, null, {
      default: slots.panel
    });
    const overlayClass = computed(() => normalizeClass([`${mergedPrefixCls.value}-overlay`, globalHashId.value, hashId.value, props.overlayClassName]));
    const triggerProps = computed(() => {
      var _a, _b;
      return {
        autoFocus: props.autofocus,
        value: accessor.value,
        clearable: false,
        disabled: accessor.disabled,
        open: overlayOpened.value,
        overlayContainer: (_a = props.overlayContainer) != null ? _a : config.overlayContainer,
        overlayContainerFallback: `.${mergedPrefixCls.value}-overlay-container`,
        overlayTabindex: (_b = props.overlayTabindex) != null ? _b : config.overlayTabindex,
        readonly: props.readonly,
        size: mergedSize.value,
        status: mergedStatus.value,
        "onUpdate:open": setOverlayOpened,
        onFocus: handleFocus,
        onBlur: handleBlur
      };
    });
    return () => createVNode(IxControlTrigger, mergeProps(triggerProps.value, {
      "class": `${mergedPrefixCls.value} ${globalHashId.value} ${hashId.value}`,
      "overlayClassName": overlayClass.value
    }, attrs), {
      trigger: renderTrigger,
      overlay: renderContent
    });
  }
});
const IxColorPicker = ColorPicker;
const IxColorPickerPanel = ColorPickerPanel;
const IxColorPickerTrigger = ColorPickerTrigger;
const IxColorPickerIndicator = ColorPickerIndicator;
const IxColorPickerPalette = ColorPickerPalette;
const IxColorPickerEditor = ColorPickerEditor;
const IxColorPickerPresets = ColorPickerPresets;
export { IxColorPicker, IxColorPickerEditor, IxColorPickerIndicator, IxColorPickerPalette, IxColorPickerPanel, IxColorPickerPresets, IxColorPickerTrigger, getThemeTokens as getColorPickerThemeTokens };
