/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { ColorFormat } from './color';
import type { AbstractControl, ValidateStatus } from '@idux/cdk/forms';
import type { ExtractInnerPropTypes, ExtractPublicPropTypes, MaybeArray, VKey } from '@idux/cdk/utils';
import type { FormSize } from '@idux/components/form';
import type { OverlayContainerType } from '@idux/components/utils';
import type { DefineComponent, HTMLAttributes, PropType } from 'vue';
export interface ColorPreset {
    key: VKey;
    label: string;
    colors: string[];
    defaultOpen?: boolean;
}
export declare const colorPickerPanelProps: {
    value: StringConstructor;
    format: PropType<ColorFormat>;
    presets: PropType<ColorPreset[]>;
    paddingless: {
        type: BooleanConstructor;
        default: boolean;
    };
    'onUpdate:format': PropType<MaybeArray<(value: string | undefined) => void>>;
    'onUpdate:value': PropType<MaybeArray<(value: ColorFormat) => void>>;
    onChange: PropType<MaybeArray<(value: string | undefined, oldValue: string | undefined) => void>>;
    onFormatChange: PropType<MaybeArray<(format: ColorFormat, oldFormat: ColorFormat) => void>>;
};
export type ColorPickerPanelProps = ExtractInnerPropTypes<typeof colorPickerPanelProps>;
export type ColorPickerPanelPublicProps = ExtractPublicPropTypes<typeof colorPickerPanelProps>;
export type ColorPickerPanelComponent = DefineComponent<Omit<HTMLAttributes, keyof ColorPickerPanelPublicProps> & ColorPickerPanelPublicProps>;
export type ColorPickerPanelInstance = InstanceType<DefineComponent<ColorPickerPanelProps>>;
export declare const colorPickerProps: {
    readonly control: {
        readonly type: PropType<string | number | (string | number)[] | AbstractControl<any>>;
        readonly default: undefined;
    };
    readonly open: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly autofocus: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly clearable: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly clearIcon: StringConstructor;
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly overlayClassName: StringConstructor;
    readonly overlayContainer: {
        readonly type: PropType<OverlayContainerType>;
        readonly default: undefined;
    };
    readonly overlayTabindex: {
        readonly type: NumberConstructor;
        readonly default: undefined;
    };
    readonly readonly: {
        readonly type: PropType<boolean>;
        readonly default: false;
    };
    readonly size: PropType<FormSize>;
    readonly status: PropType<ValidateStatus>;
    readonly showText: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly 'onUpdate:open': PropType<MaybeArray<(isOpen: boolean) => void>>;
    readonly onClear: PropType<MaybeArray<(evt: MouseEvent) => void>>;
    readonly onFocus: PropType<MaybeArray<(evt: FocusEvent) => void>>;
    readonly onBlur: PropType<MaybeArray<(evt: FocusEvent) => void>>;
    readonly value: StringConstructor;
    readonly format: PropType<ColorFormat>;
    readonly presets: PropType<ColorPreset[]>;
    readonly paddingless: {
        type: BooleanConstructor;
        default: boolean;
    };
    readonly 'onUpdate:format': PropType<MaybeArray<(value: string | undefined) => void>>;
    readonly 'onUpdate:value': PropType<MaybeArray<(value: ColorFormat) => void>>;
    readonly onChange: PropType<MaybeArray<(value: string | undefined, oldValue: string | undefined) => void>>;
    readonly onFormatChange: PropType<MaybeArray<(format: ColorFormat, oldFormat: ColorFormat) => void>>;
};
export type ColorPickerProps = ExtractInnerPropTypes<typeof colorPickerProps>;
export type ColorPickerPublicProps = ExtractPublicPropTypes<typeof colorPickerProps>;
export type ColorPickerComponent = DefineComponent<Omit<HTMLAttributes, keyof ColorPickerPublicProps> & ColorPickerPublicProps>;
export type ColorPickerInstance = InstanceType<DefineComponent<ColorPickerProps>>;
