import { h, inject, reactive, provide } from "vue";
import { merge, cloneDeep, isNil, isString } from "lodash-es";
import { getMilliseconds, getSeconds, getMinutes, getHours, getDay, getDate, getWeek, getMonth, getQuarter, getYear, setMilliseconds, setSeconds, setMinutes, setHours, setDay, setDate, setWeek, setMonth, setQuarter, setYear, addDays, addWeeks, addMonths, addQuarters, addYears, startOfSecond, startOfMinute, startOfHour, startOfDay, startOfWeek, startOfMonth, startOfQuarter, startOfYear, endOfSecond, endOfMinute, endOfHour, endOfDay, endOfWeek, endOfMonth, endOfQuarter, endOfYear, toDate, isSameSecond, isSameMinute, isSameHour, isSameDay, isSameWeek, isSameMonth, isSameQuarter, isSameYear, isValid, format, parse } from "date-fns";
import { IxIcon } from "@idux/components/icon";
import { zhCN } from "@idux/components/locales";
function numFormatter(value, precision) {
  const separator = ".";
  const numReg = /^(\d+)(\.(\d+))?$/;
  const numMatchRet = String(value).match(numReg);
  if (!numMatchRet) {
    return {
      value: String(value),
      int: "",
      decimal: ""
    };
  } else {
    const int = String(numMatchRet[1]);
    let decimal = String(numMatchRet[3] || "").slice(0, precision).padEnd(precision, "0");
    decimal = decimal.length > 0 ? `${separator}${decimal}` : "";
    return {
      value: `${int}${decimal}`,
      int,
      decimal
    };
  }
}
const expandIconRenderer = ({ expanded }) => h(
  "svg",
  {
    viewBox: "0 0 16 16",
    style: {
      width: "var(--ix-font-size-icon)",
      height: "var(--ix-font-size-icon)"
    }
  },
  [
    h("rect", {
      x: 1,
      y: 1,
      width: 14,
      height: 14,
      rx: 1,
      style: { fill: "var(--ix-color-emphasized-container-bg)" }
    }),
    h("rect", { x: 5, y: 7, width: 6, height: 2, rx: 0.2, style: { fill: "var(--ix-color-icon)" } }),
    !expanded ? h("rect", { x: 7, y: 5, width: 2, height: 6, rx: 0.2, style: { fill: "var(--ix-color-icon)" } }) : null
  ]
);
const defaultConfig = {
  common: {
    prefixCls: "ix",
    overlayZIndex: 1e3
  },
  locale: zhCN,
  theme: {
    injectThemeStyle: true,
    presetTheme: "default",
    hashed: true
  },
  alert: {
    closable: false,
    icon: {
      success: "check-circle",
      error: "exclamation-circle",
      info: "info-circle",
      warning: "exclamation-circle",
      offline: "info-circle"
    }
  },
  anchor: {
    bounds: 5,
    hideLinkBall: false
  },
  avatar: {
    gap: 4,
    icon: "user",
    shape: "circle",
    size: "md"
  },
  backTop: {
    duration: 450,
    visibilityHeight: 400
  },
  badge: {
    showZero: false,
    dot: false,
    overflowCount: 99
  },
  button: {
    size: "sm",
    waveless: false
  },
  card: {
    borderless: false,
    hoverable: false,
    size: "sm",
    shadow: true
  },
  carousel: {
    autoplayTime: 0,
    dotPlacement: "bottom",
    showArrow: false,
    trigger: "click"
  },
  cascader: {
    borderless: false,
    clearIcon: "close-circle",
    childrenKey: "children",
    expandIcon: "right",
    fullPath: true,
    getKey: "key",
    labelKey: "label",
    overlayMatchWidth: false,
    size: "md",
    suffix: "down"
  },
  checkbox: {
    size: "md",
    waveless: false
  },
  collapse: {
    accordion: false,
    borderless: false,
    expandIcon: "right",
    ghost: false,
    size: "md"
  },
  colorPicker: {
    format: "hex",
    size: "md"
  },
  datePicker: {
    allowInput: false,
    borderless: false,
    clearable: false,
    clearIcon: "close-circle",
    size: "md",
    suffix: "calendar"
  },
  desc: {
    col: 3,
    colonless: true,
    labelAlign: "start",
    layout: "horizontal",
    size: "md"
  },
  divider: {
    dashed: false,
    labelPlacement: "center",
    plain: false,
    size: "md"
  },
  drawer: {
    closable: true,
    closeOnEsc: true,
    closeIcon: "close-filled",
    distance: 160,
    height: 256,
    mask: true,
    maskClosable: true,
    width: 480
  },
  dropdown: {
    autoAdjust: true,
    destroyOnHide: false,
    offset: [0, 4],
    placement: "bottomStart",
    showArrow: false,
    trigger: "hover"
  },
  empty: {},
  form: {
    colonless: true,
    labelAlign: "start",
    layout: "horizontal",
    size: "md",
    labelTooltipIcon: "question-circle",
    controlTooltipIcon: "info-circle"
  },
  icon: {},
  input: {
    borderless: false,
    clearable: false,
    clearIcon: "close-circle",
    size: "md",
    trim: false
  },
  inputNumber: {
    keyboard: true,
    size: "md"
  },
  list: {
    size: "md",
    borderless: true
  },
  loadingBar: {
    mask: false,
    animation: {
      loading: {
        duration: 3e3,
        progress: 80
      },
      finish: {
        duration: 300,
        progress: 100
      },
      error: {
        duration: 400,
        progress: 100
      }
    }
  },
  image: {
    preview: true
  },
  imageViewer: {
    draggable: true,
    loop: true,
    maskClosable: true,
    zoom: [0.5, 2]
  },
  menu: {
    getKey: "key",
    indent: 16,
    offset: [0, 4],
    suffix: "right",
    theme: "light"
  },
  message: {
    destroyOnHover: false,
    duration: 3e3,
    maxCount: 5,
    icon: {
      success: "check-circle-filled",
      error: "close-circle-filled",
      info: "info-circle-filled",
      warning: "exclamation-circle-filled",
      loading: "loading"
    }
  },
  modal: {
    animatable: true,
    centered: true,
    closable: true,
    closeIcon: "close-filled",
    closeOnEsc: true,
    mask: true,
    maskClosable: false,
    spinWithFullModal: false
  },
  notification: {
    destroyOnHover: false,
    duration: 4500,
    maxCount: 5,
    offset: 24,
    placement: "topEnd"
  },
  pagination: {
    pageSize: 10,
    pageSizes: [10, 20, 50, 100],
    showQuickJumper: false,
    showSizeChanger: false,
    showTitle: true,
    showTotal: true,
    simple: false,
    size: "sm"
  },
  popconfirm: {
    autoAdjust: true,
    delay: 100,
    destroyOnHide: false,
    placement: "bottomStart",
    showArrow: true,
    trigger: "click",
    offset: [0, 4]
  },
  popover: {
    autoAdjust: true,
    delay: 100,
    destroyOnHide: false,
    placement: "bottomStart",
    showArrow: true,
    trigger: "hover",
    closeIcon: "close",
    offset: [0, 4]
  },
  progress: {
    strokeLinecap: "square",
    size: "md",
    format: (percent) => percent + "%",
    icon: {
      success: "check",
      exception: "close"
    }
  },
  radio: {
    size: "md",
    waveless: false
  },
  rate: {
    allowHalf: false,
    clearable: false,
    count: 5,
    icon: "star-filled",
    size: "md"
  },
  result: {
    status: "info"
  },
  row: {
    wrap: true
  },
  select: {
    borderless: false,
    childrenKey: "children",
    getKey: "key",
    clearIcon: "close-circle",
    labelKey: "label",
    offset: [0, 4],
    overlayMatchWidth: true,
    size: "md",
    suffix: "down"
  },
  skeleton: {
    animated: true
  },
  space: {
    size: 8,
    wrap: true
  },
  spin: {
    tip: "",
    tipAlign: "vertical",
    size: "md"
  },
  statistic: {
    precision: 0,
    formatter: numFormatter
  },
  stepper: {
    clickable: false,
    labelPlacement: "end",
    size: "sm"
  },
  switch: {
    size: "md"
  },
  tabs: {
    size: "md",
    showAllTabsPanel: true
  },
  table: {
    autoHeight: false,
    borderless: true,
    childrenKey: "children",
    getKey: "key",
    size: "md",
    scrollToTopOnChange: true,
    insetShadow: true,
    pagination: {
      position: "bottomEnd"
    },
    columnBase: {
      align: "start",
      sortable: {
        nextTooltip: false,
        orders: ["ascend", "descend"]
      },
      filterable: {
        multiple: true,
        footer: true
      }
    },
    columnExpandable: {
      showLine: true,
      icon: expandIconRenderer
    },
    columnSelectable: {
      showIndex: false
    },
    columnIndexable: {
      align: "center",
      customCell: ({ rowIndex, pageSize, pageIndex }) => (pageIndex - 1) * pageSize + rowIndex + 1
    }
  },
  tag: {},
  tagGroup: {
    gap: 8,
    wrap: true
  },
  textarea: {
    autoRows: false,
    clearable: false,
    clearIcon: "close-circle",
    resize: "vertical",
    showCount: false,
    size: "md",
    trim: false
  },
  text: {
    copyIcon: ({ copied }) => h(IxIcon, { name: !copied ? "copy" : "check-circle-filled", style: copied ? { color: "#20CC94" } : void 0 })
  },
  timePicker: {
    borderless: false,
    clearable: true,
    clearIcon: "close-circle",
    size: "md",
    suffix: "clock-circle",
    allowInput: true,
    format: "HH:mm:ss"
  },
  transfer: {
    getKey: "key",
    clearable: true,
    clearIcon: "delete",
    showSelectAll: true
  },
  tooltip: {
    autoAdjust: true,
    showArrow: true,
    delay: 100,
    destroyOnHide: false,
    offset: [0, 4],
    placement: "bottomStart",
    trigger: "hover"
  },
  tour: {
    animatable: true,
    gap: { offset: 4, radius: 2 },
    offset: [0, 4],
    mask: true,
    placement: "bottomStart",
    showArrow: true,
    scrollIntoViewOptions: true,
    closeOnClick: false,
    closeOnEsc: true,
    closable: true
  },
  tree: {
    autoHeight: false,
    blocked: false,
    childrenKey: "children",
    expandIcon: expandIconRenderer,
    draggableIcon: "holder",
    getKey: "key",
    labelKey: "label",
    showLine: false
  },
  treeSelect: {
    borderless: false,
    childrenKey: "children",
    clearIcon: "close-circle",
    labelKey: "label",
    getKey: "key",
    offset: [0, 4],
    overlayMatchWidth: true,
    size: "md",
    suffix: "down"
  },
  upload: {
    multiple: false,
    dragable: false,
    directory: false,
    name: "file",
    withCredentials: false,
    requestMethod: "post"
  },
  uploadFiles: {
    type: "text",
    icon: {
      preview: "zoom-in",
      file: "paper-clip",
      remove: "close",
      retry: "edit"
    }
  }
};
const tokens = Object.keys(defaultConfig).map((key) => [key, Symbol(key)]);
const tokenMap = new Map(tokens);
const createGlobalConfig = (config) => {
  const install = (app) => {
    const compNames = Object.keys(config);
    compNames.forEach((compName) => {
      const token = tokenMap.get(compName);
      const currConfig = cloneDeep(defaultConfig[compName]);
      merge(currConfig, config[compName]);
      app.provide(token, currConfig);
    });
  };
  return { install };
};
function useGlobalConfig(compName, config) {
  const token = tokenMap.get(compName);
  const currConfig = inject(token, defaultConfig[compName]);
  if (!config) {
    return currConfig;
  }
  const newConfig = reactive(merge(cloneDeep(currConfig), config));
  provide(token, newConfig);
  return [newConfig, (config2) => merge(newConfig, config2)];
}
const DATE_CONFIG_TOKEN = Symbol("DATE_CONFIG_TOKEN");
let defaultDateConfig;
function useDateConfig() {
  const config = inject(DATE_CONFIG_TOKEN, null);
  if (config) {
    return config;
  }
  if (!defaultDateConfig) {
    defaultDateConfig = createDefaultDateConfig();
  }
  return defaultDateConfig;
}
function createDefaultDateConfig() {
  const locale = useGlobalConfig("locale");
  const now = () => new Date();
  const parse$1 = (dateString, format2, referenceDate) => parse(dateString, format2, referenceDate != null ? referenceDate : now(), { locale: locale.date });
  return {
    now,
    weekStartsOn: () => {
      var _a, _b;
      return (_b = (_a = locale.date.options) == null ? void 0 : _a.weekStartsOn) != null ? _b : 1;
    },
    get: (date, type) => {
      switch (type) {
        case "year":
          return getYear(date);
        case "quarter":
          return getQuarter(date);
        case "month":
          return getMonth(date);
        case "week":
          return getWeek(date, { locale: locale.date });
        case "date":
          return getDate(date);
        case "day":
          return getDay(date);
        case "hour":
          return getHours(date);
        case "minute":
          return getMinutes(date);
        case "second":
          return getSeconds(date);
        case "millisecond":
          return getMilliseconds(date);
      }
    },
    set: (date, amount, type) => {
      switch (type) {
        case "year":
          return setYear(date, amount);
        case "quarter":
          return setQuarter(date, amount);
        case "month":
          return setMonth(date, amount);
        case "week":
          return setWeek(date, amount, { locale: locale.date });
        case "date":
          return setDate(date, amount);
        case "day":
          return setDay(date, amount, { locale: locale.date });
        case "hour":
          return setHours(date, amount);
        case "minute":
          return setMinutes(date, amount);
        case "second":
          return setSeconds(date, amount);
        case "millisecond":
          return setMilliseconds(date, amount);
      }
    },
    add: (date, amount, type) => {
      switch (type) {
        case "year":
          return addYears(date, amount);
        case "quarter":
          return addQuarters(date, amount);
        case "month":
          return addMonths(date, amount);
        case "week":
          return addWeeks(date, amount);
        case "date":
        case "day":
          return addDays(date, amount);
      }
    },
    startOf: (date, type) => {
      switch (type) {
        case "year":
          return startOfYear(date);
        case "quarter":
          return startOfQuarter(date);
        case "month":
          return startOfMonth(date);
        case "week":
          return startOfWeek(date, { locale: locale.date });
        case "date":
        case "day":
          return startOfDay(date);
        case "hour":
          return startOfHour(date);
        case "minute":
          return startOfMinute(date);
        case "second":
        case "millisecond":
          return startOfSecond(date);
      }
    },
    endOf: (date, type) => {
      switch (type) {
        case "year":
          return endOfYear(date);
        case "quarter":
          return endOfQuarter(date);
        case "month":
          return endOfMonth(date);
        case "week":
          return endOfWeek(date, { locale: locale.date });
        case "date":
        case "day":
          return endOfDay(date);
        case "hour":
          return endOfHour(date);
        case "minute":
          return endOfMinute(date);
        case "second":
        case "millisecond":
          return endOfSecond(date);
      }
    },
    isSame: (date, dateToCompare, type) => {
      switch (type) {
        case "year":
          return isSameYear(date, dateToCompare);
        case "quarter":
          return isSameQuarter(date, dateToCompare);
        case "month":
          return isSameMonth(date, dateToCompare);
        case "week":
          return isSameWeek(date, dateToCompare, { locale: locale.date });
        case "date":
        case "day":
          return isSameDay(date, dateToCompare);
        case "hour":
          return isSameHour(date, dateToCompare);
        case "minute":
          return isSameMinute(date, dateToCompare);
        case "second":
          return isSameSecond(date, dateToCompare);
        case "millisecond":
          return toDate(date).getTime() === toDate(dateToCompare).getTime();
      }
    },
    isValid: (date) => isValid(date),
    format: (date, format$1) => format(date, format$1, { locale: locale.date }),
    parse: parse$1,
    convert: (date, format2) => {
      if (isNil(date)) {
        return now();
      }
      return isString(date) ? parse$1(date, format2) : toDate(date);
    },
    getLocalizedLabels: (type, length, width) => {
      const localize = locale.date.localize;
      switch (type) {
        case "month":
          return Array.from({ length }).map((_, i) => localize.month(i, { width }));
        case "day":
          return Array.from({ length }).map((_, i) => localize.day(i, { width }));
        case "dayPeriod":
          return Array.from({ length }).map((_, i) => localize.dayPeriod(i, { width }));
      }
    }
  };
}
export { DATE_CONFIG_TOKEN, createGlobalConfig, defaultConfig, useDateConfig, useGlobalConfig };
