/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import { type InjectionKey } from 'vue';
export type TimeConfigType = 'hour' | 'minute' | 'second' | 'millisecond';
export type DateConfigType = 'year' | 'quarter' | 'month' | 'week' | 'date' | 'day';
export type DateConfig<P = number | Date, R = Date> = {
    now: () => R;
    weekStartsOn: () => number;
    get: (date: P, type: DateConfigType | TimeConfigType) => number;
    set: (date: P, amount: number, type: DateConfigType | TimeConfigType) => R;
    add: (date: P, amount: number, type: DateConfigType) => R;
    startOf: (date: P, type: DateConfigType | TimeConfigType) => R;
    endOf: (date: P, type: DateConfigType | TimeConfigType) => R;
    isSame: (date: P, dateToCompare: P, type: DateConfigType | TimeConfigType) => boolean;
    isValid: (date: unknown) => boolean;
    format: (date: P, format: string) => string;
    parse: (dateString: string, format: string, referenceDate?: Date | number) => R;
    convert: (date: unknown, format: string) => R;
    getLocalizedLabels: (type: 'month' | 'day' | 'dayPeriod', maxIndex: number, width: 'narrow' | 'abbreviated' | 'wide') => string[];
};
export declare const DATE_CONFIG_TOKEN: InjectionKey<DateConfig>;
export declare function useDateConfig(): DateConfig<number | Date, Date>;
