import { defineComponent, inject, shallowRef, computed, watch, onMounted, createVNode, mergeProps, ref, watchEffect, nextTick, provide, normalizeClass } from "vue";
import { useEventListener, isFocusable, useState, useControlledProp, convertElement, convertCssPixel, callEmit } from "@idux/cdk/utils";
import { \u0275TriggerPropsDefs as _TriggerPropsDefs, \u0275Trigger as _Trigger } from "@idux/components/_private/trigger";
import { useGlobalConfig } from "@idux/components/config";
import { useThemeToken } from "@idux/components/theme";
import { useMergedCommonControlProps, useOverlayFocusMonitor } from "@idux/components/utils";
import { \u0275OverlayPlacementDef as _OverlayPlacementDef, \u0275OverlayTriggerDef as _OverlayTriggerDef, \u0275Overlay as _Overlay } from "@idux/components/_private/overlay";
import { useResizeObserver } from "@idux/cdk/resize";
const controlTriggerToken = Symbol("controlTriggerToken");
const controlTriggerProps = {
  ..._TriggerPropsDefs,
  autofocus: { type: Boolean, default: void 0 },
  offset: Array,
  open: { type: Boolean, default: void 0 },
  overlayClassName: { type: String, default: void 0 },
  overlayContainer: {
    type: [String, HTMLElement, Function],
    default: void 0
  },
  overlayTabindex: { type: Number, default: void 0 },
  overlayContainerFallback: String,
  overlayLazy: { type: Boolean, default: true },
  overlayMatchWidth: { type: [Boolean, String], default: void 0 },
  "onUpdate:open": [Function, Array],
  onClick: [Function, Array],
  onKeydown: [Function, Array],
  onOverlayAfterLeave: [Function, Array]
};
const controlTrigglerOverlayProps = {
  visible: {
    type: Boolean,
    default: void 0
  },
  lazy: {
    type: Boolean,
    default: void 0
  },
  placement: _OverlayPlacementDef,
  trigger: _OverlayTriggerDef,
  "onUpdate:visible": [Function, Array],
  showArrow: {
    type: Boolean,
    default: void 0
  },
  tabindex: { type: Number, default: void 0 },
  onAfterLeave: [Function, Array]
};
const defaultOffset = [0, 4];
var ControlTriggerOverlay = /* @__PURE__ */ defineComponent({
  name: "IxControlTriggerOverlay",
  inheritAttrs: false,
  props: controlTrigglerOverlayProps,
  setup(props, {
    expose,
    slots,
    attrs
  }) {
    const common = useGlobalConfig("common");
    const {
      props: controlTriggerProps2,
      mergedPrefixCls,
      overlayFocused,
      bindOverlayMonitor,
      resetTriggerFocus
    } = inject(controlTriggerToken);
    const overlayRef = shallowRef();
    const popperElRef = computed(() => {
      var _a;
      return (_a = overlayRef.value) == null ? void 0 : _a.getPopperElement();
    });
    const updatePopper = () => {
      var _a;
      (_a = overlayRef.value) == null ? void 0 : _a.updatePopper();
    };
    expose({
      updatePopper
    });
    const overlayProps = computed(() => {
      var _a, _b, _c, _d, _e;
      return {
        clickOutside: false,
        container: controlTriggerProps2.overlayContainer,
        containerFallback: (_a = controlTriggerProps2.overlayContainerFallback) != null ? _a : `.${mergedPrefixCls.value}-overlay-container`,
        disabled: controlTriggerProps2.disabled || controlTriggerProps2.readonly,
        lazy: (_b = props.lazy) != null ? _b : controlTriggerProps2.overlayLazy,
        offset: (_c = controlTriggerProps2.offset) != null ? _c : defaultOffset,
        placement: (_d = props.placement) != null ? _d : "bottomStart",
        showArrow: props.showArrow,
        transitionName: `${common.prefixCls}-slide-auto`,
        trigger: (_e = props.trigger) != null ? _e : "click",
        triggerId: attrs.id,
        visible: props.visible,
        "onUpdate:visible": props["onUpdate:visible"],
        onAfterLeave: props.onAfterLeave
      };
    });
    const overlayOpened = computed(() => {
      var _a;
      return (_a = props.visible) != null ? _a : false;
    });
    watch(overlayOpened, (opened) => {
      if (!opened && overlayFocused.value) {
        resetTriggerFocus();
      }
    });
    useEventListener(popperElRef, "mousedown", (evt) => {
      if (isFocusable(evt.target) || isFocusable(evt.currentTarget)) {
        return;
      }
      evt.preventDefault();
      resetTriggerFocus();
    });
    useEventListener(popperElRef, "wheel", () => {
      if (overlayFocused.value) {
        resetTriggerFocus();
      }
    });
    onMounted(() => {
      bindOverlayMonitor(overlayRef, overlayOpened);
    });
    return () => createVNode(_Overlay, mergeProps({
      "ref": overlayRef,
      "tabindex": props.tabindex
    }, overlayProps.value, attrs), slots);
  }
});
function useOverlayState(props, config, triggerRef, focused, triggerFocused) {
  const overlayRef = ref();
  const [overlayWidth, setOverlayWidth] = useState("");
  const overlayMatchWidth = computed(() => {
    var _a;
    return (_a = props.overlayMatchWidth) != null ? _a : config.overlayMatchWidth;
  });
  const overlayStyle = computed(() => {
    if (!overlayMatchWidth.value) {
      return;
    }
    return { [overlayMatchWidth.value === true ? "width" : "minWidth"]: overlayWidth.value };
  });
  const [_overlayOpened, setOverlayOpened] = useControlledProp(props, "open", false);
  const overlayOpened = computed(() => !props.disabled && _overlayOpened.value);
  const updateOverlay = (rect) => {
    var _a, _b;
    if (rect) {
      const { width } = rect;
      if (width > 0) {
        setOverlayWidth(convertCssPixel(width));
        overlayOpened.value && ((_a = overlayRef.value) == null ? void 0 : _a.updatePopper());
      }
    } else {
      overlayOpened.value && ((_b = overlayRef.value) == null ? void 0 : _b.updatePopper());
    }
  };
  onMounted(() => {
    var _a;
    if (props.autofocus) {
      (_a = triggerRef.value) == null ? void 0 : _a.focus();
      setOverlayOpened(true);
    }
    watchEffect(() => {
      const overlayInstance = overlayRef.value;
      if (overlayInstance && overlayOpened.value) {
        const triggerEl = convertElement(triggerRef.value);
        updateOverlay(triggerEl.getBoundingClientRect());
      }
    });
    watch(overlayOpened, () => {
      var _a2;
      if (focused.value && !triggerFocused.value) {
        (_a2 = triggerRef.value) == null ? void 0 : _a2.focus();
      }
    });
    useResizeObserver(triggerRef, ({ contentRect }) => updateOverlay(contentRect));
  });
  return { overlayRef, overlayStyle, overlayMatchWidth, updateOverlay, overlayOpened, setOverlayOpened };
}
function useTriggerFocusState(triggerRef, triggerFocused) {
  const [focusTarget, setFocusTarget] = useState(null);
  const [selectionStart, setSelectionStart] = useState(null);
  const updateSelectionStart = () => {
    const element = focusTarget.value;
    setTimeout(() => {
      if (element.selectionDirection === "backward") {
        setSelectionStart(element.selectionStart);
      } else {
        setSelectionStart(element.selectionEnd);
      }
    });
  };
  let stops;
  const destroyListeners = () => stops == null ? void 0 : stops.forEach((stop) => stop == null ? void 0 : stop());
  watch(focusTarget, (target, oldTarget) => {
    if (target === oldTarget) {
      return;
    }
    if (!(target instanceof HTMLInputElement)) {
      setSelectionStart(null);
      destroyListeners();
      return;
    }
    updateSelectionStart();
    if (stops == null ? void 0 : stops.length) {
      return;
    }
    stops = [
      useEventListener(focusTarget, "select", updateSelectionStart),
      useEventListener(focusTarget, "input", updateSelectionStart),
      useEventListener(focusTarget, "mousedown", updateSelectionStart),
      useEventListener(focusTarget, "keydown", updateSelectionStart)
    ];
  });
  const handleTriggerFocus = (evt) => {
    nextTick(() => {
      if (triggerFocused.value && evt.target instanceof HTMLElement) {
        setFocusTarget(evt.target);
      }
    });
  };
  const resetTriggerFocus = () => {
    var _a;
    const target = focusTarget.value;
    if (!target) {
      (_a = triggerRef.value) == null ? void 0 : _a.focus();
      return;
    }
    target.focus();
    if (target instanceof HTMLInputElement) {
      target.setSelectionRange(selectionStart.value, selectionStart.value);
    }
  };
  return {
    handleTriggerFocus,
    resetTriggerFocus
  };
}
const defaultTriggerProps = {
  borderless: false,
  clearIcon: "close-circle",
  size: "md",
  suffix: "down",
  overlayMatchWidth: false
};
var ControlTrigger = /* @__PURE__ */ defineComponent({
  name: "IxControlTrigger",
  inheritAttrs: false,
  props: controlTriggerProps,
  setup(props, {
    attrs,
    expose,
    slots: _slots
  }) {
    const slots = _slots;
    const common = useGlobalConfig("common");
    const {
      globalHashId
    } = useThemeToken();
    const mergedPrefixCls = computed(() => `${common.prefixCls}-control-trigger`);
    const triggerRef = ref();
    const mergedControlProps = useMergedCommonControlProps(props, defaultTriggerProps);
    const onFocus = (evt) => {
      callEmit(props.onFocus, evt);
      handleTriggerFocus(evt);
    };
    const onBlur = (evt) => {
      setOverlayOpened(false);
      callEmit(props.onBlur, evt);
    };
    const {
      focused,
      triggerFocused,
      overlayFocused,
      focusVia,
      blurVia,
      bindOverlayMonitor,
      handleFocus,
      handleBlur
    } = useOverlayFocusMonitor(onFocus, onBlur);
    const {
      resetTriggerFocus,
      handleTriggerFocus
    } = useTriggerFocusState(triggerRef, triggerFocused);
    const {
      overlayOpened,
      overlayRef,
      overlayMatchWidth,
      overlayStyle,
      setOverlayOpened
    } = useOverlayState(props, defaultTriggerProps, triggerRef, focused, triggerFocused);
    const focus = () => focusVia(triggerRef.value);
    const blur = () => blurVia(triggerRef.value);
    expose({
      focus,
      blur
    });
    provide(controlTriggerToken, {
      props,
      mergedPrefixCls,
      overlayFocused,
      resetTriggerFocus,
      bindOverlayMonitor
    });
    const handleClear = (evt) => {
      callEmit(props.onClear, evt);
    };
    const handleClick = (evt) => {
      const target = evt.target;
      if (!props.disabled && !props.readonly && !(overlayOpened.value && target instanceof HTMLInputElement && !target.readOnly && !target.disabled)) {
        setOverlayOpened(!overlayOpened.value);
      }
      callEmit(props.onClick, evt);
    };
    const handleKeyDown = (evt) => {
      if (!overlayOpened.value && !props.disabled && !props.readonly && !["Escape", "Backspace", "Tab"].includes(evt.code)) {
        setOverlayOpened(true);
      } else if (overlayOpened.value && evt.code === "Escape") {
        setOverlayOpened(false);
      }
      callEmit(props.onKeydown, evt);
    };
    const triggerProps = computed(() => {
      var _a;
      const {
        borderless,
        clearIcon,
        size,
        suffix
      } = mergedControlProps.value;
      return {
        value: props.value,
        size,
        status: props.status,
        borderless,
        clearIcon,
        clearable: props.clearable,
        disabled: props.disabled,
        readonly: props.readonly,
        focused: focused.value,
        opened: overlayOpened.value,
        placeholder: props.placeholder,
        suffix,
        suffixRotate: (_a = props.suffixRotate) != null ? _a : overlayOpened.value ? 180 : 0,
        onClear: handleClear,
        onFocus: handleFocus,
        onBlur: handleBlur,
        onClick: handleClick,
        onKeydown: handleKeyDown
      };
    });
    const overlayClasses = computed(() => {
      const {
        overlayClassName
      } = props;
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [globalHashId.value]: !!globalHashId.value,
        [`${prefixCls}-overlay`]: true,
        [`${prefixCls}-overlay-match-width`]: overlayMatchWidth.value === true,
        [overlayClassName || ""]: !!overlayClassName
      });
    });
    const renderTriggerContent = (params) => {
      var _a, _b;
      const triggerDefaultSlot = (_a = slots.trigger) != null ? _a : slots.default;
      return (_b = triggerDefaultSlot == null ? void 0 : triggerDefaultSlot({
        ...params,
        opened: overlayOpened.value
      })) != null ? _b : [];
    };
    const renderTrigger = () => {
      const triggerSlots = {
        suffix: slots.suffix,
        clearIcon: slots.clearIcon,
        default: renderTriggerContent
      };
      return createVNode(_Trigger, mergeProps({
        "class": mergedPrefixCls.value,
        "ref": triggerRef
      }, triggerProps.value, attrs, {
        "raw": !!slots.trigger
      }), triggerSlots);
    };
    const renderContent = () => {
      var _a;
      return (_a = slots.overlay) == null ? void 0 : _a.call(slots, {
        opened: overlayOpened.value
      });
    };
    return () => createVNode(ControlTriggerOverlay, {
      "ref": overlayRef,
      "class": overlayClasses.value,
      "style": overlayStyle.value,
      "visible": overlayOpened.value,
      "lazy": props.overlayLazy,
      "tabindex": props.overlayTabindex,
      "trigger": "manual",
      "onAfterLeave": props.onOverlayAfterLeave
    }, {
      default: renderTrigger,
      content: renderContent
    });
  }
});
const IxControlTrigger = ControlTrigger;
const IxControlTriggerOverlay = ControlTriggerOverlay;
export { IxControlTrigger, IxControlTriggerOverlay };
