/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { ExtractInnerPropTypes, ExtractPublicPropTypes, MaybeArray } from '@idux/cdk/utils';
import type { OverlayContainerType } from '@idux/components/utils';
import type { DefineComponent, HTMLAttributes, PropType, Slot } from 'vue';
import { ɵTriggerDefaultSlotParams, ɵTriggerSlots } from '@idux/components/_private/trigger';
export declare const controlTriggerProps: {
    readonly autofocus: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly offset: PropType<[number, number]>;
    readonly open: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly overlayClassName: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly overlayContainer: {
        readonly type: PropType<OverlayContainerType>;
        readonly default: undefined;
    };
    readonly overlayTabindex: {
        readonly type: NumberConstructor;
        readonly default: undefined;
    };
    readonly overlayContainerFallback: StringConstructor;
    readonly overlayLazy: {
        readonly type: BooleanConstructor;
        readonly default: true;
    };
    readonly overlayMatchWidth: {
        readonly type: PropType<boolean | "minWidth">;
        readonly default: undefined;
    };
    readonly 'onUpdate:open': PropType<MaybeArray<(opened: boolean) => void>>;
    readonly onClick: PropType<MaybeArray<(evt: Event) => void>>;
    readonly onKeydown: PropType<MaybeArray<(evt: KeyboardEvent) => void>>;
    readonly onOverlayAfterLeave: PropType<MaybeArray<() => void>>;
    readonly value: {
        readonly type: null;
        readonly default: undefined;
    };
    readonly raw: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly borderless: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly clearable: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly clearIcon: {
        readonly type: PropType<string>;
        readonly default: "close-circle";
    };
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly focused: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly monitorFocus: {
        readonly type: BooleanConstructor;
        readonly default: true;
    };
    readonly paddingless: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly placeholder: StringConstructor;
    readonly readonly: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly size: {
        readonly type: PropType<import("../..").FormSize>;
        readonly default: "md";
    };
    readonly status: PropType<import("../../../cdk").ValidateStatus>;
    readonly suffix: PropType<string>;
    readonly suffixRotate: {
        readonly type: PropType<string | number | boolean>;
        readonly default: undefined;
    };
    readonly onClear: PropType<MaybeArray<(evt: MouseEvent) => void>>;
    readonly onFocus: PropType<MaybeArray<(evt: FocusEvent) => void>>;
    readonly onBlur: PropType<MaybeArray<(evt: FocusEvent) => void>>;
};
export declare const controlTrigglerOverlayProps: {
    visible: {
        type: BooleanConstructor;
        default: undefined;
    };
    lazy: {
        type: BooleanConstructor;
        default: undefined;
    };
    placement: PropType<import("../../../cdk").PopperPlacement>;
    trigger: PropType<import("../../../cdk").PopperTrigger>;
    'onUpdate:visible': PropType<MaybeArray<(visible: boolean) => void>>;
    showArrow: {
        type: BooleanConstructor;
        default: undefined;
    };
    tabindex: {
        type: NumberConstructor;
        default: undefined;
    };
    onAfterLeave: PropType<MaybeArray<() => void>>;
};
export interface ControlTriggerBindings {
    blur: () => void;
    focus: (options?: FocusOptions) => void;
}
export type ControlTriggerSlots = {
    suffix?: ɵTriggerSlots['suffix'];
    clearIcon?: ɵTriggerSlots['clearIcon'];
    trigger?: Slot<ɵTriggerDefaultSlotParams & {
        opened: boolean;
    }>;
    default?: Slot<ɵTriggerDefaultSlotParams & {
        opened: boolean;
    }>;
    overlay?: Slot<{
        opened: boolean;
    }>;
};
export type ControlTriggerProps = ExtractInnerPropTypes<typeof controlTriggerProps>;
export type ControlTriggerPublicProps = ExtractPublicPropTypes<typeof controlTriggerProps>;
export type ControlTriggerComponent = DefineComponent<Omit<HTMLAttributes, keyof ControlTriggerPublicProps> & ControlTriggerPublicProps, ControlTriggerBindings>;
export type ControlTriggerInstance = InstanceType<DefineComponent<ControlTriggerProps, ControlTriggerBindings>>;
export interface ControlTriggerOverlayBindings {
    updatePopper: () => void;
}
export type ControlTriggerOverlayProps = ExtractInnerPropTypes<typeof controlTrigglerOverlayProps>;
export type ControlTriggerOverlayPublicProps = ExtractPublicPropTypes<typeof controlTrigglerOverlayProps>;
export type ControlTriggerOverlayComponent = DefineComponent<Omit<HTMLAttributes, keyof ControlTriggerOverlayPublicProps> & ControlTriggerOverlayPublicProps, ControlTriggerOverlayBindings>;
export type ControlTriggerOverlayInstance = InstanceType<DefineComponent<ControlTriggerOverlayProps, ControlTriggerOverlayBindings>>;
