/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { AbstractControl, ValidateStatus } from '@idux/cdk/forms';
import type { ExtractInnerPropTypes, ExtractPublicPropTypes, MaybeArray, VKey } from '@idux/cdk/utils';
import type { ɵFooterButtonProps } from '@idux/components/_private/footer';
import type { FormSize } from '@idux/components/form';
import type { OverlayContainerType } from '@idux/components/utils';
import type { DefineComponent, HTMLAttributes, PropType, Slots, VNode, VNodeChild } from 'vue';
export type PresetRangeShortcut = 'today' | 'yesterday' | 'last24h' | 'last7d' | 'last30d' | 'last180d' | 'custom';
export interface RangeShortcutPanelRenderContext {
    slots: Slots;
    setBuffer: (value: [Date, Date] | undefined) => void;
    setValue: (value: [Date, Date] | undefined) => void;
    ok: () => void;
    cancel: () => void;
}
export interface RangeShortcutOptions {
    key: VKey;
    label: string;
    confirmOnSelect?: boolean;
    selectedLabel?: string;
    value?: [Date, Date] | (() => [Date, Date]);
    onClick?: (evt: MouseEvent) => void;
    panelRenderer?: (context: RangeShortcutPanelRenderContext) => VNodeChild;
}
export type RangeShortcut = PresetRangeShortcut | RangeShortcutOptions;
export type RangeShortcutProp = RangeShortcut[] | {
    showPanel?: boolean;
    shortcuts: RangeShortcut[];
};
export interface PickerTimePanelOptions {
    disabledHours?: (selectedAmPm: string | undefined) => number[];
    disabledMinutes?: (selectedHour: number | undefined, selectedAmPm: string | undefined) => number[];
    disabledSeconds?: (selectedHour: number | undefined, selectedMinute: number | undefined, selectedAmPm: string | undefined) => number[];
    hideDisabledOptions?: boolean;
    hourStep?: number;
    minuteStep?: number;
    secondStep?: number;
}
export interface TimePanelOptions extends PickerTimePanelOptions {
    hourEnabled?: boolean;
    minuteEnabled?: boolean;
    secondEnabled?: boolean;
    use12Hours?: boolean;
}
declare const datePickerCommonProps: {
    readonly control: {
        readonly type: PropType<string | number | (string | number)[] | AbstractControl<any>>;
        readonly default: undefined;
    };
    readonly cellTooltip: PropType<(cell: {
        value: Date;
        disabled: boolean;
    }) => string | void>;
    readonly open: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly allowInput: {
        readonly type: PropType<boolean | "overlay">;
        readonly default: undefined;
    };
    readonly autofocus: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly borderless: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly clearable: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly clearIcon: StringConstructor;
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly disabledDate: PropType<(date: Date) => boolean>;
    readonly format: StringConstructor;
    readonly dateFormat: StringConstructor;
    readonly timeFormat: StringConstructor;
    readonly overlayClassName: StringConstructor;
    readonly overlayContainer: {
        readonly type: PropType<OverlayContainerType>;
        readonly default: undefined;
    };
    readonly overlayTabindex: {
        readonly type: NumberConstructor;
        readonly default: undefined;
    };
    readonly overlayRender: PropType<(children: VNodeChild) => VNodeChild>;
    readonly readonly: {
        readonly type: PropType<boolean>;
        readonly default: false;
    };
    readonly size: PropType<FormSize>;
    readonly status: PropType<ValidateStatus>;
    readonly suffix: StringConstructor;
    readonly type: {
        readonly type: PropType<DatePickerType>;
        readonly default: "date";
    };
    readonly 'onUpdate:open': PropType<MaybeArray<(isOpen: boolean) => void>>;
    readonly onClear: PropType<MaybeArray<(evt: MouseEvent) => void>>;
    readonly onFocus: PropType<MaybeArray<(evt: FocusEvent) => void>>;
    readonly onBlur: PropType<MaybeArray<(evt: FocusEvent) => void>>;
};
export type DatePickerCommonProps = ExtractInnerPropTypes<typeof datePickerCommonProps>;
export interface DatePickerCommonBindings {
    blur: () => void;
    focus: (options?: FocusOptions) => void;
}
export declare const datePickerProps: {
    readonly value: PropType<string | number | Date>;
    readonly footer: {
        readonly type: PropType<boolean | VNode<import("vue").RendererNode, import("vue").RendererElement, {
            [key: string]: any;
        }> | ɵFooterButtonProps[]>;
        readonly default: false;
    };
    readonly placeholder: StringConstructor;
    readonly timePanelOptions: PropType<PickerTimePanelOptions>;
    readonly onChange: PropType<MaybeArray<(value: Date | undefined, oldValue: Date | undefined) => void>>;
    readonly onInput: PropType<MaybeArray<(evt: Event) => void>>;
    readonly 'onUpdate:value': PropType<MaybeArray<(value: Date | undefined) => void>>;
    readonly control: {
        readonly type: PropType<string | number | (string | number)[] | AbstractControl<any>>;
        readonly default: undefined;
    };
    readonly cellTooltip: PropType<(cell: {
        value: Date;
        disabled: boolean;
    }) => string | void>;
    readonly open: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly allowInput: {
        readonly type: PropType<boolean | "overlay">;
        readonly default: undefined;
    };
    readonly autofocus: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly borderless: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly clearable: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly clearIcon: StringConstructor;
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly disabledDate: PropType<(date: Date) => boolean>;
    readonly format: StringConstructor;
    readonly dateFormat: StringConstructor;
    readonly timeFormat: StringConstructor;
    readonly overlayClassName: StringConstructor;
    readonly overlayContainer: {
        readonly type: PropType<OverlayContainerType>;
        readonly default: undefined;
    };
    readonly overlayTabindex: {
        readonly type: NumberConstructor;
        readonly default: undefined;
    };
    readonly overlayRender: PropType<(children: VNodeChild) => VNodeChild>;
    readonly readonly: {
        readonly type: PropType<boolean>;
        readonly default: false;
    };
    readonly size: PropType<FormSize>;
    readonly status: PropType<ValidateStatus>;
    readonly suffix: StringConstructor;
    readonly type: {
        readonly type: PropType<DatePickerType>;
        readonly default: "date";
    };
    readonly 'onUpdate:open': PropType<MaybeArray<(isOpen: boolean) => void>>;
    readonly onClear: PropType<MaybeArray<(evt: MouseEvent) => void>>;
    readonly onFocus: PropType<MaybeArray<(evt: FocusEvent) => void>>;
    readonly onBlur: PropType<MaybeArray<(evt: FocusEvent) => void>>;
};
export type DatePickerProps = ExtractInnerPropTypes<typeof datePickerProps>;
export type DatePickerPublicProps = ExtractPublicPropTypes<typeof datePickerProps>;
export type DatePickerComponent = DefineComponent<Omit<HTMLAttributes, keyof DatePickerPublicProps> & DatePickerPublicProps, DatePickerCommonBindings>;
export type DatePickerInstance = InstanceType<DefineComponent<DatePickerProps, DatePickerCommonBindings>>;
export declare const dateRangePickerProps: {
    readonly value: PropType<(string | number | Date)[]>;
    readonly footer: {
        readonly type: PropType<boolean | VNode<import("vue").RendererNode, import("vue").RendererElement, {
            [key: string]: any;
        }> | ɵFooterButtonProps[]>;
        readonly default: true;
    };
    readonly placeholder: PropType<string[]>;
    readonly separator: PropType<string | VNode<import("vue").RendererNode, import("vue").RendererElement, {
        [key: string]: any;
    }>>;
    readonly shortcuts: PropType<RangeShortcut[] | RangeShortcutProp>;
    readonly timePanelOptions: PropType<PickerTimePanelOptions | PickerTimePanelOptions[]>;
    readonly onChange: PropType<MaybeArray<(value: Date[] | undefined, oldValue: Date[] | undefined) => void>>;
    readonly onInput: PropType<MaybeArray<(isFrom: boolean, evt: Event) => void>>;
    readonly onSelect: PropType<MaybeArray<(selectingDate: (Date | undefined)[] | undefined) => void>>;
    readonly 'onUpdate:value': PropType<MaybeArray<(value: Date[] | undefined) => void>>;
    readonly control: {
        readonly type: PropType<string | number | (string | number)[] | AbstractControl<any>>;
        readonly default: undefined;
    };
    readonly cellTooltip: PropType<(cell: {
        value: Date;
        disabled: boolean;
    }) => string | void>;
    readonly open: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly allowInput: {
        readonly type: PropType<boolean | "overlay">;
        readonly default: undefined;
    };
    readonly autofocus: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly borderless: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly clearable: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly clearIcon: StringConstructor;
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly disabledDate: PropType<(date: Date) => boolean>;
    readonly format: StringConstructor;
    readonly dateFormat: StringConstructor;
    readonly timeFormat: StringConstructor;
    readonly overlayClassName: StringConstructor;
    readonly overlayContainer: {
        readonly type: PropType<OverlayContainerType>;
        readonly default: undefined;
    };
    readonly overlayTabindex: {
        readonly type: NumberConstructor;
        readonly default: undefined;
    };
    readonly overlayRender: PropType<(children: VNodeChild) => VNodeChild>;
    readonly readonly: {
        readonly type: PropType<boolean>;
        readonly default: false;
    };
    readonly size: PropType<FormSize>;
    readonly status: PropType<ValidateStatus>;
    readonly suffix: StringConstructor;
    readonly type: {
        readonly type: PropType<DatePickerType>;
        readonly default: "date";
    };
    readonly 'onUpdate:open': PropType<MaybeArray<(isOpen: boolean) => void>>;
    readonly onClear: PropType<MaybeArray<(evt: MouseEvent) => void>>;
    readonly onFocus: PropType<MaybeArray<(evt: FocusEvent) => void>>;
    readonly onBlur: PropType<MaybeArray<(evt: FocusEvent) => void>>;
};
export type DateRangePickerProps = ExtractInnerPropTypes<typeof dateRangePickerProps>;
export type DateRangePickerPublicProps = ExtractPublicPropTypes<typeof dateRangePickerProps>;
export type DateRangePickerComponent = DefineComponent<Omit<HTMLAttributes, keyof DateRangePickerPublicProps> & DateRangePickerPublicProps, DatePickerCommonBindings>;
export type DateRangePickerInstance = InstanceType<DefineComponent<DateRangePickerProps, DatePickerCommonBindings>>;
export type DatePickerType = 'date' | 'week' | 'month' | 'quarter' | 'year' | 'datetime';
export declare const datePanelProps: {
    readonly activeValue: PropType<Date | undefined>;
    readonly cellTooltip: PropType<(cell: {
        value: Date;
        disabled: boolean;
    }) => string | void>;
    readonly disabledDate: PropType<(date: Date) => boolean>;
    readonly value: PropType<Date>;
    readonly type: {
        readonly type: PropType<DatePickerType>;
        readonly default: undefined;
    };
    readonly timePanelOptions: {
        readonly type: PropType<TimePanelOptions>;
        readonly default: undefined;
    };
    readonly visible: {
        readonly type: PropType<boolean | "datePanel" | "timePanel">;
        readonly default: undefined;
    };
    readonly onChange: PropType<MaybeArray<(value: Date | undefined) => void>>;
    readonly 'onUpdate:activeValue': PropType<MaybeArray<(date: Date | undefined) => void>>;
};
export type DatePanelProps = ExtractInnerPropTypes<typeof datePanelProps>;
export type DatePanelPublicProps = ExtractPublicPropTypes<typeof datePanelProps>;
export type DatePanelComponent = DefineComponent<Omit<HTMLAttributes, keyof DatePanelPublicProps> & DatePanelPublicProps>;
export type DatePanelInstance = InstanceType<DefineComponent<DatePanelProps>>;
export declare const dateRangePanelProps: {
    readonly activeValue: PropType<Date[] | undefined>;
    readonly cellTooltip: PropType<(cell: {
        value: Date;
        disabled: boolean;
    }) => string | void>;
    readonly disabledDate: PropType<(date: Date) => boolean>;
    readonly value: PropType<Date[] | undefined>;
    readonly type: {
        readonly type: PropType<DatePickerType>;
        readonly default: undefined;
    };
    readonly timePanelOptions: {
        readonly type: PropType<TimePanelOptions[]>;
        readonly default: undefined;
    };
    readonly visible: {
        readonly type: PropType<boolean | "datePanel" | "timePanel">;
        readonly default: undefined;
    };
    readonly onChange: PropType<MaybeArray<(value: Date[] | undefined) => void>>;
    readonly onSelect: PropType<MaybeArray<(selectingDate: (Date | undefined)[] | undefined) => void>>;
    readonly 'onUpdate:activeValue': PropType<MaybeArray<(date: Date[] | undefined) => void>>;
};
export type DateRangePanelProps = ExtractInnerPropTypes<typeof dateRangePanelProps>;
export type DateRangePanelPublicProps = ExtractPublicPropTypes<typeof dateRangePanelProps>;
export type DateRangePanelComponent = DefineComponent<Omit<HTMLAttributes, keyof DateRangePanelPublicProps> & DateRangePanelPublicProps>;
export type DateRangePanelInstance = InstanceType<DefineComponent<DateRangePanelProps>>;
export {};
