import { defineComponent, computed, normalizeClass, createVNode } from "vue";
import { useGlobalConfig } from "@idux/components/config";
import { useThemeToken } from "@idux/components/theme";
import { convertStringVNode } from "@idux/components/utils";
const dividerProps = {
  dashed: {
    type: Boolean,
    default: void 0
  },
  label: String,
  labelPlacement: String,
  plain: {
    type: Boolean,
    default: void 0
  },
  size: String,
  vertical: {
    type: Boolean,
    default: void 0
  }
};
function getDefaultThemeTokens(tokens) {
  const {
    lineWidth,
    colorBorder,
    fontSizeSm,
    fontSizeMd,
    fontSizeLg,
    marginSizeXs,
    marginSizeSm,
    marginSizeMd,
    marginSizeLg,
    marginSizeXl
  } = tokens;
  return {
    lineWidth,
    lineColor: colorBorder,
    fontSizeSm,
    fontSizeMd,
    fontSizeLg,
    marginVerticalSm: marginSizeMd,
    marginVerticalMd: marginSizeLg,
    marginVerticalLg: marginSizeXl,
    marginHorizontalSm: marginSizeXs,
    marginHorizontalMd: marginSizeSm,
    marginHorizontalLg: marginSizeMd
  };
}
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens) : getDefaultThemeTokens(tokens);
};
var Divider = /* @__PURE__ */ defineComponent({
  name: "IxDivider",
  props: dividerProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("divider");
    registerToken(getThemeTokens);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-divider`);
    const config = useGlobalConfig("divider");
    const classes = computed(() => {
      const {
        dashed = config.dashed,
        label,
        labelPlacement = config.labelPlacement,
        plain = config.plain,
        size = config.size,
        vertical
      } = props;
      const withLabel = !!label || !!slots.default;
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [`${prefixCls}`]: true,
        [`${prefixCls}-${size}`]: true,
        [`${prefixCls}-dashed`]: dashed,
        [`${prefixCls}-horizontal`]: !vertical,
        [`${prefixCls}-vertical`]: vertical,
        [`${prefixCls}-plain`]: withLabel && plain,
        [`${prefixCls}-with-label`]: withLabel,
        [`${prefixCls}-with-label-${labelPlacement}`]: withLabel
      });
    });
    return () => {
      const prefixCls = mergedPrefixCls.value;
      const labelNode = convertStringVNode(slots.default, props.label);
      return createVNode("div", {
        "class": classes.value
      }, [labelNode && createVNode("span", {
        "class": `${prefixCls}-label`
      }, [labelNode])]);
    };
  }
});
const IxDivider = Divider;
export { IxDivider, getThemeTokens as getDividerThemeTokens };
