import { defineComponent, inject, computed, normalizeClass, ref, onMounted, withDirectives, createVNode, Transition, mergeProps, vShow, watch, onBeforeUnmount, toRef, provide, nextTick, onDeactivated, isVNode, cloneVNode, Fragment, shallowRef } from "vue";
import { CdkPortal } from "@idux/cdk/portal";
import { useResizeObserver } from "@idux/cdk/resize";
import { BlockScrollStrategy } from "@idux/cdk/scroll";
import { convertCssPixel, callEmit, useControlledProp, NoopFunction, convertArray, uniqueId, throwError } from "@idux/cdk/utils";
import { \u0275Mask as _Mask } from "@idux/components/_private/mask";
import { useGlobalConfig } from "@idux/components/config";
import { useThemeToken } from "@idux/components/theme";
import { usePortalTarget, useZIndex } from "@idux/components/utils";
import { isFunction } from "lodash-es";
import { \u0275Footer as _Footer } from "@idux/components/_private/footer";
import { \u0275Header as _Header } from "@idux/components/_private/header";
const drawerToken = Symbol("drawerToken");
const DRAWER_TOKEN = Symbol("DRAWER_TOKEN");
const DRAWER_PROVIDER_TOKEN = Symbol("DRAWER_PROVIDER_TOKEN");
const drawerTransitionMap = {
  top: "move-up",
  bottom: "move-down",
  start: "move-start",
  end: "move-end"
};
const horizontalPlacement = ["start", "end"];
var DrawerWrapper = /* @__PURE__ */ defineComponent({
  inheritAttrs: false,
  setup(_, {
    attrs
  }) {
    const {
      globalHashId,
      hashId
    } = useThemeToken("drawer");
    const {
      props,
      slots,
      common,
      config,
      mergedPrefixCls,
      drawerElRef,
      visible,
      delayedLoaded,
      animatedVisible,
      isAnimating,
      mergedVisible,
      currentZIndex,
      levelAction,
      distance
    } = inject(drawerToken);
    const {
      close
    } = inject(DRAWER_TOKEN);
    const {
      closable,
      closeIcon,
      closeOnEsc,
      mask,
      maskClosable
    } = useConfig(props, config);
    const transitionName = computed(() => `${common.prefixCls}-${drawerTransitionMap[props.placement]}`);
    const isHorizontal = computed(() => horizontalPlacement.includes(props.placement));
    const placementStyle = computed(() => {
      const {
        width,
        height,
        offset,
        placement
      } = props;
      const horizontal = isHorizontal.value;
      const offsetPixel = convertCssPixel(offset);
      let offsetObj;
      if (horizontal) {
        const transformPlacement = placement === "start" ? "left" : "right";
        offsetObj = {
          top: offsetPixel,
          [transformPlacement]: 0
        };
      } else {
        offsetObj = {
          left: offsetPixel,
          [placement]: 0
        };
      }
      return {
        width: convertCssPixel(width || (horizontal ? config.width : "100%")),
        height: convertCssPixel(height || (horizontal ? "100%" : config.height)),
        ...offsetObj
      };
    });
    const transformStyle = computed(() => {
      const {
        placement
      } = props;
      const horizontal = isHorizontal.value;
      let transform;
      if (horizontal) {
        transform = distance.value > 0 ? `translateX(${placement === "start" ? distance.value : -distance.value}px)` : void 0;
      } else {
        transform = distance.value > 0 ? `translateY(${placement === "top" ? distance.value : -distance.value}px)` : void 0;
      }
      return transform;
    });
    const wrapperClasses = computed(() => {
      const action = levelAction.value;
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [`${prefixCls}-wrapper`]: true,
        [`${prefixCls}-${props.placement}`]: true,
        [`${prefixCls}-opened`]: animatedVisible.value,
        [`${prefixCls}-${action}`]: !!action,
        [`${prefixCls}-with-mask`]: mask.value
      });
    });
    const wrapperStyle = computed(() => {
      const placement = mask.value ? void 0 : placementStyle.value;
      return {
        zIndex: currentZIndex.value,
        transform: transformStyle.value,
        ...placement
      };
    });
    const contentStyle = computed(() => {
      return mask.value ? placementStyle.value : void 0;
    });
    const wrapperRef = ref();
    const sentinelStartRef = ref();
    const sentinelEndRef = ref();
    const {
      onWrapperClick,
      onWrapperKeydown,
      onContentMousedown,
      onContentMouseup
    } = useEvent(close, mask, maskClosable, closeOnEsc, sentinelStartRef, sentinelEndRef);
    const {
      onEnter,
      onAfterEnter,
      onAfterLeave
    } = useEvents(props, wrapperRef, animatedVisible, isAnimating);
    onMounted(() => {
      watchVisibleChange(props, wrapperRef, sentinelStartRef, mask);
    });
    return () => {
      const prefixCls = mergedPrefixCls.value;
      return withDirectives(createVNode("div", {
        "ref": wrapperRef,
        "class": wrapperClasses.value,
        "style": wrapperStyle.value,
        "tabindex": -1,
        "onClick": onWrapperClick,
        "onKeydown": onWrapperKeydown
      }, [createVNode(Transition, {
        "name": props.animatable ? transitionName.value : void 0,
        "appear": true,
        "onEnter": onEnter,
        "onAfterEnter": onAfterEnter,
        "onAfterLeave": onAfterLeave
      }, {
        default: () => {
          var _a;
          return [delayedLoaded.value && withDirectives(createVNode("div", mergeProps({
            "ref": drawerElRef,
            "role": "document",
            "class": prefixCls,
            "style": contentStyle.value,
            "onMousedown": onContentMousedown,
            "onMouseup": onContentMouseup
          }, attrs), [createVNode("div", {
            "ref": sentinelStartRef,
            "tabindex": 0,
            "class": `${prefixCls}-sentinel`,
            "aria-hidden": true
          }, null), createVNode("div", {
            "class": `${prefixCls}-content`
          }, [createVNode(_Header, {
            "closable": closable.value,
            "closeIcon": closeIcon.value,
            "header": props.header,
            "onClose": close
          }, slots), createVNode("div", {
            "class": `${prefixCls}-body`
          }, [(_a = slots.default) == null ? void 0 : _a.call(slots)]), createVNode(_Footer, {
            "class": `${prefixCls}-footer`,
            "footer": props.footer
          }, slots)]), createVNode("div", {
            "ref": sentinelEndRef,
            "tabindex": 0,
            "class": `${prefixCls}-sentinel`,
            "aria-hidden": true
          }, null)]), [[vShow, visible.value]])];
        }
      })]), [[vShow, mergedVisible.value]]);
    };
  }
});
function useConfig(props, config) {
  const closable = computed(() => {
    var _a;
    return (_a = props.closable) != null ? _a : config.closable;
  });
  const closeIcon = computed(() => {
    var _a;
    return (_a = props.closeIcon) != null ? _a : config.closeIcon;
  });
  const closeOnEsc = computed(() => {
    var _a;
    return (_a = props.closeOnEsc) != null ? _a : config.closeOnEsc;
  });
  const mask = computed(() => {
    var _a;
    return (_a = props.mask) != null ? _a : config.mask;
  });
  const maskClosable = computed(() => {
    var _a;
    return (_a = props.maskClosable) != null ? _a : config.maskClosable;
  });
  return {
    closable,
    closeIcon,
    closeOnEsc,
    mask,
    maskClosable
  };
}
function watchVisibleChange(props, wrapperRef, sentinelStartRef, mask) {
  let lastOutSideActiveElement = null;
  watch(() => props.visible, (visible) => {
    var _a, _b;
    if (visible) {
      const wrapperElement = wrapperRef.value;
      const activeElement = document.activeElement;
      if (wrapperElement && !wrapperElement.contains(activeElement)) {
        lastOutSideActiveElement = activeElement;
        (_a = sentinelStartRef.value) == null ? void 0 : _a.focus();
      }
    } else {
      if (mask.value) {
        (_b = lastOutSideActiveElement == null ? void 0 : lastOutSideActiveElement.focus) == null ? void 0 : _b.call(lastOutSideActiveElement);
        lastOutSideActiveElement = null;
      }
    }
  }, {
    immediate: true
  });
}
function useEvent(close, mask, maskClosable, closeOnEsc, sentinelStartRef, sentinelEndRef) {
  let timeId;
  let mouseDown = false;
  const clearTimer = () => {
    if (timeId) {
      clearTimeout(timeId);
      timeId = void 0;
    }
  };
  const onWrapperClick = (evt) => {
    if (evt.target === evt.currentTarget && !mouseDown && mask.value && maskClosable.value) {
      close(evt);
    }
  };
  const onWrapperKeydown = (evt) => {
    if (closeOnEsc.value && evt.code === "Escape") {
      evt.stopPropagation();
      close(evt);
      return;
    }
    if (evt.code === "Tab") {
      const activeElement = document.activeElement;
      const sentinelStartElement = sentinelStartRef.value;
      const sentinelEndElement = sentinelEndRef.value;
      if (evt.shiftKey) {
        if (activeElement === sentinelStartElement) {
          sentinelEndElement == null ? void 0 : sentinelEndElement.focus();
        }
      } else if (activeElement === sentinelEndElement) {
        sentinelStartElement == null ? void 0 : sentinelStartElement.focus();
      }
    }
  };
  const onContentMousedown = () => {
    clearTimer();
    mouseDown = true;
  };
  const onContentMouseup = () => {
    if (mouseDown) {
      timeId = setTimeout(() => mouseDown = false);
    }
  };
  onBeforeUnmount(() => clearTimer());
  return {
    onContentMousedown,
    onContentMouseup,
    onWrapperClick,
    onWrapperKeydown
  };
}
function useEvents(props, wrapperRef, animatedVisible, isAnimating) {
  let lastOutSideActiveElement = null;
  const onEnter = () => {
    const wrapperElement = wrapperRef.value;
    const activeElement = document.activeElement;
    if (wrapperElement && !wrapperElement.contains(activeElement)) {
      lastOutSideActiveElement = activeElement;
    }
  };
  const onAfterEnter = () => {
    const wrapperElement = wrapperRef.value;
    const activeElement = document.activeElement;
    if (wrapperElement && !wrapperElement.contains(activeElement)) {
      wrapperElement.focus();
    }
    callEmit(props.onAfterOpen);
    animatedVisible.value = true;
    isAnimating.value = false;
  };
  const onAfterLeave = () => {
    if (lastOutSideActiveElement && isFunction(lastOutSideActiveElement.focus)) {
      const wrapperElement = wrapperRef.value;
      const activeElement = document.activeElement;
      if (!activeElement || activeElement === document.body || activeElement === wrapperElement || wrapperElement && wrapperElement.contains(activeElement)) {
        lastOutSideActiveElement.focus();
      }
    }
    callEmit(props.onAfterClose);
    animatedVisible.value = false;
    isAnimating.value = false;
  };
  return {
    onEnter,
    onAfterEnter,
    onAfterLeave
  };
}
const drawerProps = {
  visible: {
    type: Boolean,
    default: void 0
  },
  animatable: {
    type: Boolean,
    default: true
  },
  closable: {
    type: Boolean,
    default: void 0
  },
  closeIcon: [String, Object, Function],
  closeOnDeactivated: {
    type: Boolean,
    default: true
  },
  closeOnEsc: {
    type: Boolean,
    default: void 0
  },
  container: {
    type: [String, HTMLElement, Function],
    default: void 0
  },
  destroyOnHide: {
    type: Boolean,
    default: false
  },
  footer: [Array, Object, Function],
  header: [String, Object],
  height: [String, Number],
  mask: {
    type: Boolean,
    default: void 0
  },
  maskClosable: {
    type: Boolean,
    default: void 0
  },
  offset: {
    type: [String, Number],
    default: 0
  },
  placement: {
    type: String,
    default: "end"
  },
  distance: {
    type: Number,
    default: void 0
  },
  scrollStrategy: Object,
  width: [String, Number],
  zIndex: Number,
  "onUpdate:visible": [Function, Array],
  onAfterOpen: [Function, Array],
  onAfterClose: [Function, Array],
  onBeforeClose: [Function, Array],
  onClose: [Function, Array]
};
function getDefaultThemeTokens(tokens) {
  const { overlayBorderWidth, overlayBorderType, overlayBorderColor } = tokens;
  return {
    boxShadowStart: `6px 0 16px -8px rgba(0, 0, 0, 0.08), 9px 0 28px 0 rgba(0, 0, 0, 0.05), 12px 0 48px 16px rgba(0, 0, 0, 0.03)`,
    boxShadowEnd: `-6px 0 16px -8px rgba(0, 0, 0, 0.08), -9px 0 28px 0 rgba(0, 0, 0, 0.05), -12px 0 48px 16px rgba(0, 0, 0, 0.03)`,
    boxShadowTop: `0 -6px 16px -8px rgba(0, 0, 0, 0.08), 0 -9px 28px 0 rgba(0, 0, 0, 0.05), 0 -12px 48px 16px rgba(0, 0, 0, 0.03)`,
    borderWidth: overlayBorderWidth,
    borderType: overlayBorderType,
    borderColor: overlayBorderColor
  };
}
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens) : getDefaultThemeTokens(tokens);
};
var Drawer = /* @__PURE__ */ defineComponent({
  name: "IxDrawer",
  inheritAttrs: false,
  props: drawerProps,
  setup(props, {
    slots,
    expose,
    attrs
  }) {
    const common = useGlobalConfig("common");
    const {
      registerToken
    } = useThemeToken("drawer");
    registerToken(getThemeTokens);
    const config = useGlobalConfig("drawer");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-drawer`);
    const mergedPortalTarget = usePortalTarget(props, config, common, mergedPrefixCls);
    const mask = computed(() => {
      var _a;
      return (_a = props.mask) != null ? _a : config.mask;
    });
    const mergedDistance = computed(() => {
      var _a;
      return (_a = props.distance) != null ? _a : config.distance;
    });
    const {
      loaded,
      delayedLoaded,
      visible,
      setVisible,
      animatedVisible,
      isAnimating,
      mergedVisible
    } = useVisible(props);
    const currentZIndex = useZIndex(toRef(props, "zIndex"), toRef(common, "overlayZIndex"), visible);
    const drawerElRef = ref();
    const {
      open,
      close
    } = useTrigger(props, setVisible);
    const {
      levelAction,
      distance,
      push,
      pull
    } = useLevel(props, visible, mergedDistance, drawerElRef);
    provide(drawerToken, {
      props,
      slots,
      common,
      config,
      mergedPrefixCls,
      drawerElRef,
      visible,
      delayedLoaded,
      animatedVisible,
      isAnimating,
      mergedVisible,
      currentZIndex,
      levelAction,
      distance,
      push,
      pull
    });
    const apis = {
      open,
      close
    };
    provide(DRAWER_TOKEN, apis);
    expose(apis);
    useScrollStrategy(props, mask, mergedVisible);
    return () => {
      if (!mergedVisible.value && props.destroyOnHide) {
        return null;
      }
      return createVNode(CdkPortal, {
        "target": mergedPortalTarget.value,
        "load": loaded.value
      }, {
        default: () => [createVNode(_Mask, {
          "class": `${mergedPrefixCls.value}-mask`,
          "mask": mask.value,
          "visible": visible.value,
          "zIndex": currentZIndex.value
        }, null), createVNode(DrawerWrapper, attrs, null)]
      });
    };
  }
});
function useVisible(props) {
  const [visible, setVisible] = useControlledProp(props, "visible", false);
  const loaded = ref(false);
  const delayedLoaded = ref(false);
  const delayedVisible = ref(false);
  const isAnimating = ref(false);
  const animatedVisible = ref(false);
  watch(visible, (v) => {
    if (v && !loaded.value) {
      loaded.value = true;
      nextTick(() => {
        delayedLoaded.value = true;
        delayedVisible.value = true;
      });
    } else {
      delayedVisible.value = v;
    }
    isAnimating.value = true;
  }, {
    immediate: true
  });
  const mergedVisible = computed(() => visible.value || (isAnimating.value ? animatedVisible.value : visible.value));
  onDeactivated(() => {
    if (mergedVisible.value && props.closeOnDeactivated) {
      setVisible(false);
    }
  });
  return {
    loaded,
    delayedLoaded,
    visible: delayedVisible,
    setVisible,
    animatedVisible,
    isAnimating,
    mergedVisible
  };
}
function useScrollStrategy(props, mask, mergedVisible) {
  let scrollStrategy;
  onMounted(() => {
    watch([mask, mergedVisible], ([maskValue, visible]) => {
      var _a;
      if (!maskValue || !visible) {
        scrollStrategy == null ? void 0 : scrollStrategy.disable();
        return;
      }
      if (!scrollStrategy) {
        scrollStrategy = (_a = props.scrollStrategy) != null ? _a : new BlockScrollStrategy();
      }
      scrollStrategy.enable();
    }, {
      immediate: true
    });
  });
  onBeforeUnmount(() => scrollStrategy == null ? void 0 : scrollStrategy.disable());
}
function useTrigger(props, setVisible) {
  const open = () => setVisible(true);
  const close = async (evt) => {
    const result = await callEmit(props.onBeforeClose, evt);
    if (result === false) {
      return;
    }
    setVisible(false);
    callEmit(props.onClose, evt);
  };
  return {
    open,
    close
  };
}
function useLevel(props, visible, mergedDistance, drawerElRef) {
  const parentContext = inject(drawerToken, null);
  const level = ref(0);
  const distance = ref(0);
  const levelAction = ref();
  const drawerSize = ref(0);
  const sizeAttrName = computed(() => ["top", "bottom"].includes(props.placement) ? "height" : "width");
  let sizeUpdateCb;
  useResizeObserver(drawerElRef, ({
    contentRect
  }) => {
    var _a;
    drawerSize.value = (_a = contentRect[sizeAttrName.value]) != null ? _a : 0;
    sizeUpdateCb == null ? void 0 : sizeUpdateCb();
  });
  const onSizeUpdate = (cb) => {
    sizeUpdateCb = () => {
      cb();
      sizeUpdateCb = void 0;
    };
  };
  const updateDistance = (childSize) => {
    const minDisatance = Math.min(drawerSize.value, mergedDistance.value);
    if (!drawerSize.value || !childSize || drawerSize.value - childSize >= minDisatance) {
      distance.value = 0;
    } else {
      distance.value = minDisatance - (drawerSize.value - childSize);
    }
  };
  const push = (childSize) => {
    level.value++;
    updateDistance(childSize);
    pushParent();
  };
  const pull = (childSize) => {
    level.value--;
    updateDistance(childSize);
    pullParent();
  };
  const pushParent = () => {
    if ((parentContext == null ? void 0 : parentContext.props.placement) !== props.placement) {
      return;
    }
    parentContext == null ? void 0 : parentContext.push(drawerSize.value + distance.value);
  };
  const pullParent = () => {
    if ((parentContext == null ? void 0 : parentContext.props.placement) !== props.placement) {
      return;
    }
    parentContext == null ? void 0 : parentContext.pull(visible.value ? drawerSize.value + distance.value : 0);
  };
  watch(visible, (value) => {
    if (value) {
      onSizeUpdate(pushParent);
    } else {
      pullParent();
      levelAction.value = void 0;
    }
  });
  watch(level, (currLevel, prevLevel) => {
    levelAction.value = currLevel > prevLevel ? "push" : "pull";
  });
  onMounted(() => {
    if (visible.value) {
      pushParent();
    }
  });
  onBeforeUnmount(() => {
    if (visible.value) {
      pullParent();
    }
  });
  return {
    levelAction,
    distance,
    push,
    pull
  };
}
var DrawerProvider = /* @__PURE__ */ defineComponent({
  name: "IxDrawerProvider",
  setup(_, {
    expose,
    slots
  }) {
    const {
      drawers,
      setDrawerRef,
      open,
      update,
      destroy,
      destroyAll
    } = useDrawer$1();
    const apis = {
      open,
      update,
      destroy,
      destroyAll
    };
    provide(DRAWER_PROVIDER_TOKEN, apis);
    expose(apis);
    return () => {
      var _a;
      const children = drawers.value.map((item) => {
        const {
          key,
          visible = true,
          destroyOnHide,
          onDestroy,
          content,
          contentProps,
          ...rest
        } = item;
        const setRef = (instance) => setDrawerRef(key, instance);
        const onUpdateVisible = (visible2) => update(key, {
          visible: visible2
        });
        const onAfterClose = destroyOnHide ? () => destroy(key) : void 0;
        const mergedProps = {
          key,
          visible,
          ref: setRef,
          "onUpdate:visible": onUpdateVisible,
          onAfterClose
        };
        const contentNode = isFunction(content) ? content() : isVNode(content) ? cloneVNode(content, contentProps, true) : content;
        return createVNode(Drawer, mergeProps(mergedProps, rest), {
          default: () => [contentNode]
        });
      });
      return createVNode(Fragment, null, [(_a = slots.default) == null ? void 0 : _a.call(slots), children]);
    };
  }
});
function useDrawer$1() {
  const drawers = shallowRef([]);
  const drawerRefMap = /* @__PURE__ */ new Map();
  const setDrawerRef = (key, instance) => {
    const ref2 = drawerRefMap.get(key);
    if (instance) {
      if (ref2 && !ref2.open) {
        ref2.open = instance.open;
        ref2.close = instance.close;
      }
    } else {
      if (ref2) {
        drawerRefMap.delete(key);
        ref2.open = NoopFunction;
        ref2.close = NoopFunction;
      }
    }
  };
  const getCurrIndex = (key) => {
    return drawers.value.findIndex((message) => message.key === key);
  };
  const add = (item) => {
    var _a;
    const currIndex = item.key ? getCurrIndex(item.key) : -1;
    const tempDrawers = [...drawers.value];
    if (currIndex !== -1) {
      tempDrawers.splice(currIndex, 1, item);
      drawers.value = tempDrawers;
      return item.key;
    }
    item.key = (_a = item.key) != null ? _a : uniqueId("ix-drawer");
    tempDrawers.push(item);
    drawers.value = tempDrawers;
    return item.key;
  };
  const update = (key, item) => {
    const currIndex = getCurrIndex(key);
    if (currIndex !== -1) {
      const tempDrawers = [...drawers.value];
      const newItem = {
        ...drawers.value[currIndex],
        ...item
      };
      tempDrawers.splice(currIndex, 1, newItem);
      drawers.value = tempDrawers;
    }
  };
  const destroy = (key) => {
    const keys = convertArray(key);
    keys.forEach((key2) => {
      const currIndex = getCurrIndex(key2);
      if (currIndex !== -1) {
        const tempDrawers = [...drawers.value];
        const item = tempDrawers.splice(currIndex, 1);
        drawers.value = tempDrawers;
        callEmit(item[0].onDestroy, key2);
      }
    });
  };
  const destroyAll = () => {
    drawers.value = [];
  };
  const open = (options) => {
    const key = add(options);
    const drawerRef = {
      key,
      update: (options2) => update(key, options2),
      destroy: () => destroy(key)
    };
    drawerRefMap.set(key, drawerRef);
    return drawerRef;
  };
  return {
    drawers,
    setDrawerRef,
    open,
    update,
    destroy,
    destroyAll
  };
}
const useDrawer = () => {
  const drawerProviderRef = inject(DRAWER_PROVIDER_TOKEN, null);
  if (drawerProviderRef === null) {
    return throwError("components/drawer", "<IxDrawerProvider> not found.");
  }
  return drawerProviderRef;
};
const IxDrawer = Drawer;
const IxDrawerProvider = DrawerProvider;
export { DRAWER_PROVIDER_TOKEN, DRAWER_TOKEN, IxDrawer, IxDrawerProvider, getThemeTokens as getDrawerThemeTokens, useDrawer };
