/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { PortalTargetType } from '@idux/cdk/portal';
import type { ScrollStrategy } from '@idux/cdk/scroll';
import type { ExtractInnerPropTypes, ExtractPublicPropTypes, MaybeArray, VKey } from '@idux/cdk/utils';
import type { ɵFooterButtonProps } from '@idux/components/_private/footer';
import type { HeaderProps } from '@idux/components/header';
import type { DefineComponent, HTMLAttributes, PropType, VNode, VNodeChild, VNodeProps } from 'vue';
export type DrawerPlacement = 'top' | 'bottom' | 'start' | 'end';
export type DrawerButtonProps = ɵFooterButtonProps;
export interface DrawerOptions<K = VKey> extends DrawerPublicProps {
    key?: K;
    content?: string | VNode | (() => VNodeChild);
    contentProps?: Record<string, unknown> | VNodeProps;
    onDestroy?: (key: K) => void;
}
export interface DrawerRef<K = VKey> extends DrawerBindings {
    key: K;
    update: (options: DrawerOptions<K>) => void;
    destroy: () => void;
}
export declare const drawerProps: {
    readonly visible: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly animatable: {
        readonly type: BooleanConstructor;
        readonly default: true;
    };
    readonly closable: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly closeIcon: PropType<string | VNode<import("vue").RendererNode, import("vue").RendererElement, {
        [key: string]: any;
    }> | (() => VNodeChild)>;
    readonly closeOnDeactivated: {
        readonly type: BooleanConstructor;
        readonly default: true;
    };
    readonly closeOnEsc: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly container: {
        readonly type: PropType<PortalTargetType>;
        readonly default: undefined;
    };
    readonly destroyOnHide: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly footer: PropType<VNode<import("vue").RendererNode, import("vue").RendererElement, {
        [key: string]: any;
    }> | ɵFooterButtonProps[] | (() => VNodeChild)>;
    readonly header: PropType<string | HeaderProps>;
    readonly height: PropType<string | number>;
    readonly mask: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly maskClosable: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly offset: {
        readonly type: PropType<string | number>;
        readonly default: 0;
    };
    readonly placement: {
        readonly type: PropType<DrawerPlacement>;
        readonly default: "end";
    };
    readonly distance: {
        readonly type: NumberConstructor;
        readonly default: undefined;
    };
    readonly scrollStrategy: PropType<ScrollStrategy>;
    readonly width: PropType<string | number>;
    readonly zIndex: NumberConstructor;
    readonly 'onUpdate:visible': PropType<MaybeArray<(visible: boolean) => void>>;
    readonly onAfterOpen: PropType<MaybeArray<() => void>>;
    readonly onAfterClose: PropType<MaybeArray<() => void>>;
    readonly onBeforeClose: PropType<MaybeArray<(evt?: Event | unknown) => void | boolean | Promise<boolean>>>;
    readonly onClose: PropType<MaybeArray<(evt?: Event | unknown) => void>>;
};
export type DrawerProps = ExtractInnerPropTypes<typeof drawerProps>;
export type DrawerPublicProps = ExtractPublicPropTypes<typeof drawerProps>;
export interface DrawerBindings {
    open: () => void;
    close: (evt?: Event | unknown) => Promise<void>;
}
export type DrawerComponent = DefineComponent<Omit<HTMLAttributes, keyof DrawerPublicProps> & DrawerPublicProps, DrawerBindings>;
export type DrawerInstance = InstanceType<DefineComponent<DrawerProps, DrawerBindings>>;
export interface DrawerProviderRef<K = VKey> {
    open: (options: DrawerOptions) => DrawerRef;
    update: (key: K, options: DrawerOptions) => void;
    destroy: (key: K | K[]) => void;
    destroyAll: () => void;
}
export type DrawerProviderComponent = DefineComponent<HTMLAttributes, DrawerProviderRef>;
export type DrawerProviderInstance = InstanceType<DefineComponent<HTMLAttributes, DrawerProviderRef>>;
