import { defineComponent, computed, provide, toRef, createVNode } from "vue";
import { useControlledProp } from "@idux/cdk/utils";
import { \u0275OverlayPlacementDef as _OverlayPlacementDef, \u0275OverlayTriggerDef as _OverlayTriggerDef, \u0275Overlay as _Overlay } from "@idux/components/_private/overlay";
import { useGlobalConfig } from "@idux/components/config";
import { useThemeToken } from "@idux/components/theme";
const dropdownToken = Symbol("dropdownToken");
const dropdownProps = {
  visible: {
    type: Boolean,
    default: void 0
  },
  autoAdjust: {
    type: Boolean,
    default: void 0
  },
  destroyOnHide: {
    type: Boolean,
    default: void 0
  },
  disabled: {
    type: Boolean,
    default: false
  },
  hideOnClick: {
    type: Boolean,
    default: true
  },
  offset: Array,
  overlayContainer: {
    type: [String, HTMLElement, Function],
    default: void 0
  },
  placement: _OverlayPlacementDef,
  showArrow: {
    type: Boolean,
    default: void 0
  },
  trigger: _OverlayTriggerDef,
  "onUpdate:visible": [Function, Array]
};
function getDefaultThemeTokens(tokens) {
  const { paddingSizeXs } = tokens;
  return {
    minWidth: 128,
    menuContainerPadding: `${paddingSizeXs}px 0`
  };
}
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens) : getDefaultThemeTokens(tokens);
};
const defaultDelay = [0, 100];
var Dropdown = /* @__PURE__ */ defineComponent({
  name: "IxDropdown",
  props: dropdownProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("dropdown");
    registerToken(getThemeTokens);
    const config = useGlobalConfig("dropdown");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-dropdown`);
    const [visibility, setVisibility] = useControlledProp(props, "visible", false);
    provide(dropdownToken, {
      hideOnClick: toRef(props, "hideOnClick"),
      setVisibility
    });
    return () => {
      var _a, _b, _c, _d, _e;
      const {
        trigger = config.trigger,
        placement = config.placement
      } = props;
      const overlayProps = {
        class: [mergedPrefixCls.value, globalHashId.value, hashId.value],
        autoAdjust: (_a = props.autoAdjust) != null ? _a : config.autoAdjust,
        clickOutside: trigger === "click" || trigger === "contextmenu",
        container: (_b = props.overlayContainer) != null ? _b : config.overlayContainer,
        containerFallback: `.${mergedPrefixCls.value}-overlay-container`,
        delay: defaultDelay,
        destroyOnHide: (_c = props.destroyOnHide) != null ? _c : config.destroyOnHide,
        disabled: props.disabled,
        offset: (_d = props.offset) != null ? _d : config.offset,
        placement,
        showArrow: (_e = props.showArrow) != null ? _e : config.showArrow,
        transitionName: `${common.prefixCls}-slide-auto`,
        trigger,
        visible: visibility.value,
        ["onUpdate:visible"]: setVisibility
      };
      return createVNode(_Overlay, overlayProps, {
        default: slots.default,
        content: slots.overlay
      });
    };
  }
});
const IxDropdown = Dropdown;
export { IxDropdown, getThemeTokens as getDropdownThemeTokens, dropdownToken as \u0275DropdownToken };
