/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { AbstractControl, ValidateStatus } from '@idux/cdk/forms';
import type { ExtractInnerPropTypes, ExtractPublicPropTypes } from '@idux/cdk/utils';
import type { ColProps, RowProps } from '@idux/components/grid';
import type { TooltipProps } from '@idux/components/tooltip';
import type { DefineComponent, HTMLAttributes, PropType } from 'vue';
export declare const formProps: {
    readonly control: {
        readonly type: PropType<string | number | (string | number)[] | AbstractControl<any>>;
        readonly default: undefined;
    };
    readonly colonless: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly controlCol: PropType<string | number | ColProps>;
    readonly controlTooltipIcon: StringConstructor;
    readonly labelAlign: PropType<FormLabelAlign>;
    readonly labelCol: PropType<string | number | ColProps>;
    readonly labelTooltipIcon: StringConstructor;
    readonly layout: PropType<FormLayout>;
    readonly messageTooltip: {
        readonly type: PropType<boolean | TooltipProps>;
        readonly default: false;
    };
    readonly size: PropType<FormSize>;
    readonly onSubmit: {
        readonly type: PropType<(evt: Event) => void>;
        readonly default: (evt: Event) => void;
    };
};
export type FormProps = ExtractInnerPropTypes<typeof formProps>;
export type FormPublicProps = ExtractPublicPropTypes<typeof formProps>;
export type FormComponent = DefineComponent<Omit<HTMLAttributes, keyof FormPublicProps> & FormPublicProps>;
export type FormInstance = InstanceType<DefineComponent<FormProps>>;
export declare const formItemProps: {
    readonly colonless: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly control: {
        readonly type: PropType<string | number | (string | number)[] | AbstractControl<any>>;
        readonly default: undefined;
    };
    readonly controlCol: PropType<string | number | ColProps>;
    readonly controlTooltip: StringConstructor;
    readonly controlTooltipIcon: StringConstructor;
    readonly description: StringConstructor;
    readonly label: PropType<string | number>;
    readonly labelAlign: PropType<FormLabelAlign>;
    readonly labelCol: PropType<string | number | ColProps>;
    readonly labelFor: PropType<string | number>;
    readonly labelTooltip: StringConstructor;
    readonly labelTooltipIcon: StringConstructor;
    readonly required: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly message: PropType<string | ((control?: AbstractControl) => string) | Partial<Record<ValidateStatus, string>>>;
    readonly messageTooltip: {
        readonly type: PropType<boolean | TooltipProps>;
        readonly default: undefined;
    };
    readonly status: PropType<ValidateStatus>;
};
export type FormItemProps = ExtractInnerPropTypes<typeof formItemProps>;
export type FormItemPublicProps = ExtractPublicPropTypes<typeof formItemProps>;
export type FormItemComponent = DefineComponent<Omit<HTMLAttributes, keyof FormItemPublicProps> & FormItemPublicProps & RowProps>;
export type FormItemInstance = InstanceType<DefineComponent<FormItemProps>>;
export declare const formWrapperProps: {
    readonly control: {
        readonly type: PropType<string | number | (string | number)[] | AbstractControl<any>>;
        readonly default: undefined;
    };
};
export type FormWrapperProps = ExtractInnerPropTypes<typeof formWrapperProps>;
export type FormWrapperPublicProps = ExtractPublicPropTypes<typeof formWrapperProps>;
export type FormWrapperComponent = DefineComponent<Omit<HTMLAttributes, keyof FormWrapperPublicProps> & FormWrapperPublicProps>;
export type FormWrapperInstance = InstanceType<DefineComponent<FormItemProps>>;
export type FormColType = number | string | ColProps;
export type FormLabelAlign = 'start' | 'end';
export type FormLayout = 'horizontal' | 'vertical' | 'inline';
export type FormSize = 'sm' | 'md' | 'lg';
