/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import { type DefineComponent, type HTMLAttributes, PropType } from 'vue';
import { type BreakpointKey } from '@idux/cdk/breakpoint';
import { type ExtractInnerPropTypes, type ExtractPublicPropTypes } from '@idux/cdk/utils';
export type RowAlign = 'start' | 'center' | 'end' | 'baseline' | 'stretch';
export type RowJustify = 'start' | 'center' | 'end' | 'space-around' | 'space-between';
export type RowGutter = number | string | Array<number | string> | Partial<Record<BreakpointKey, number | string>>;
export declare const rowProps: {
    readonly align: PropType<RowAlign>;
    readonly justify: PropType<RowJustify>;
    readonly gutter: PropType<RowGutter>;
    readonly wrap: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
};
export type RowProps = ExtractInnerPropTypes<typeof rowProps>;
export type RowPublicProps = ExtractPublicPropTypes<typeof rowProps>;
export type RowComponent = DefineComponent<Omit<HTMLAttributes, keyof RowPublicProps> & RowPublicProps>;
export type RowInstance = InstanceType<DefineComponent<RowProps>>;
export interface ColBreakpointConfig {
    span?: number | string;
    order?: number | string;
    offset?: number | string;
    push?: number | string;
    pull?: number | string;
}
export declare const colProps: {
    readonly flex: PropType<string | number>;
    readonly span: PropType<string | number>;
    readonly order: PropType<string | number>;
    readonly offset: PropType<string | number>;
    readonly push: PropType<string | number>;
    readonly pull: PropType<string | number>;
    readonly xs: PropType<string | number | ColBreakpointConfig>;
    readonly sm: PropType<string | number | ColBreakpointConfig>;
    readonly md: PropType<string | number | ColBreakpointConfig>;
    readonly lg: PropType<string | number | ColBreakpointConfig>;
    readonly xl: PropType<string | number | ColBreakpointConfig>;
};
export type ColProps = ExtractInnerPropTypes<typeof colProps>;
export type ColPublicProps = ExtractPublicPropTypes<typeof colProps>;
export type ColComponent = DefineComponent<Omit<HTMLAttributes, keyof ColPublicProps> & ColPublicProps>;
export type ColInstance = InstanceType<DefineComponent<ColProps>>;
