/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { PortalTargetType } from '@idux/cdk/portal';
import type { ExtractInnerPropTypes, ExtractPublicPropTypes, MaybeArray } from '@idux/cdk/utils';
import type { DefineComponent, HTMLAttributes, PropType } from 'vue';
export type ImageStatus = 'loading' | 'loaded' | 'failed';
export declare const imageViewerProps: {
    readonly visible: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly activeIndex: NumberConstructor;
    readonly container: {
        readonly type: PropType<PortalTargetType>;
        readonly default: undefined;
    };
    readonly draggable: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly images: {
        readonly type: PropType<string[]>;
        readonly default: () => string[];
    };
    readonly zoom: {
        readonly type: PropType<number[]>;
        readonly validator: (val: number[]) => boolean;
    };
    readonly loop: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly maskClosable: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly zIndex: NumberConstructor;
    readonly 'onUpdate:visible': PropType<MaybeArray<(visible: boolean) => void>>;
    readonly 'onUpdate:activeIndex': PropType<MaybeArray<(curIndex: number) => void>>;
};
export declare const imageProps: {
    readonly src: {
        readonly type: StringConstructor;
        readonly required: true;
    };
    readonly preview: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly imageViewer: PropType<ImageViewerProps>;
    readonly onLoad: PropType<MaybeArray<(e: Event) => void>>;
    readonly onError: PropType<MaybeArray<(e: Event) => void>>;
};
export type ImageProps = ExtractInnerPropTypes<typeof imageProps>;
export type ImagePublicProps = ExtractPublicPropTypes<typeof imageProps>;
export type ImageComponent = DefineComponent<Omit<HTMLAttributes, keyof ImagePublicProps> & ImagePublicProps>;
export type ImageInstance = InstanceType<DefineComponent<ImageProps>>;
export type ImageViewerProps = ExtractInnerPropTypes<typeof imageViewerProps>;
export type ImageViewerPublicProps = ExtractPublicPropTypes<typeof imageViewerProps>;
export type ImageViewerComponent = DefineComponent<Omit<HTMLAttributes, keyof ImageViewerPublicProps> & ImageViewerPublicProps>;
export type ImageViewerInstance = InstanceType<DefineComponent<ImageViewerProps>>;
export interface OprType {
    icon: string;
    key: string;
    opr: () => void;
    visible: boolean;
    disabled?: boolean;
}
export type ScaleType = 'in' | 'out';
export type RotateType = 'left' | 'right';
export type GoType = 'previous' | 'next';
