/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { AbstractControl, ValidateStatus } from '@idux/cdk/forms';
import type { ExtractInnerPropTypes, ExtractPublicPropTypes, MaybeArray } from '@idux/cdk/utils';
import type { FormSize } from '@idux/components/form';
import type { DefineComponent, HTMLAttributes, PropType } from 'vue';
export type InputNumberButtonPosition = 'inner' | 'outer';
export declare const inputNumberProps: {
    readonly value: PropType<number | null>;
    readonly control: {
        readonly type: PropType<string | number | (string | number)[] | AbstractControl<any>>;
        readonly default: undefined;
    };
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly formatter: PropType<(value: number | null) => string>;
    readonly keyboard: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly max: {
        readonly type: NumberConstructor;
        readonly default: number;
    };
    readonly min: {
        readonly type: NumberConstructor;
        readonly default: number;
    };
    readonly placeholder: StringConstructor;
    readonly precision: NumberConstructor;
    readonly readonly: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly size: PropType<FormSize>;
    readonly status: PropType<ValidateStatus>;
    readonly step: {
        readonly type: NumberConstructor;
        readonly default: 1;
    };
    readonly 'onUpdate:value': PropType<MaybeArray<(value: number | null) => void>>;
    readonly onChange: PropType<MaybeArray<(value: number | null, oldValue: number | null | undefined) => void>>;
    readonly onFocus: PropType<MaybeArray<(evt: FocusEvent) => void>>;
    readonly onBlur: PropType<MaybeArray<(evt: FocusEvent) => void>>;
};
export type InputNumberProps = ExtractInnerPropTypes<typeof inputNumberProps>;
export type InputNumberPublicProps = ExtractPublicPropTypes<typeof inputNumberProps>;
export interface InputNumberBindings {
    focus: (options?: FocusOptions) => void;
    blur: () => void;
}
export type InputNumberComponent = DefineComponent<Omit<HTMLAttributes, keyof InputNumberPublicProps> & InputNumberPublicProps, InputNumberBindings>;
export type InputNumberInstance = InstanceType<DefineComponent<InputNumberProps, InputNumberBindings>>;
