/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { AbstractControl, ValidateStatus } from '@idux/cdk/forms';
import type { ExtractInnerPropTypes, ExtractPublicPropTypes, MaybeArray } from '@idux/cdk/utils';
import type { FormSize } from '@idux/components/form';
import type { DefineComponent, InputHTMLAttributes, PropType } from 'vue';
export declare const inputCommonProps: {
    readonly control: {
        readonly type: PropType<string | number | (string | number)[] | AbstractControl<any>>;
        readonly default: undefined;
    };
    readonly value: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly clearable: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly clearIcon: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly readonly: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly size: {
        readonly type: PropType<FormSize>;
        readonly default: undefined;
    };
    readonly status: PropType<ValidateStatus>;
    readonly trim: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly 'onUpdate:value': PropType<MaybeArray<(value: string) => void>>;
    readonly onChange: PropType<MaybeArray<(value: string, oldValue: string) => void>>;
    readonly onClear: PropType<MaybeArray<(evt: Event) => void>>;
    readonly onCompositionStart: PropType<MaybeArray<(evt: CompositionEvent) => void>>;
    readonly onCompositionEnd: PropType<MaybeArray<(evt: CompositionEvent) => void>>;
    readonly onInput: PropType<MaybeArray<(evt: Event) => void>>;
    readonly onFocus: PropType<MaybeArray<(evt: FocusEvent) => void>>;
    readonly onBlur: PropType<MaybeArray<(evt: FocusEvent) => void>>;
};
export type InputCommonProps = ExtractInnerPropTypes<typeof inputCommonProps>;
export declare const inputProps: {
    readonly addonAfter: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly addonBefore: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly borderless: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly prefix: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly suffix: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly control: {
        readonly type: PropType<string | number | (string | number)[] | AbstractControl<any>>;
        readonly default: undefined;
    };
    readonly value: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly clearable: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly clearIcon: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly readonly: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly size: {
        readonly type: PropType<FormSize>;
        readonly default: undefined;
    };
    readonly status: PropType<ValidateStatus>;
    readonly trim: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly 'onUpdate:value': PropType<MaybeArray<(value: string) => void>>;
    readonly onChange: PropType<MaybeArray<(value: string, oldValue: string) => void>>;
    readonly onClear: PropType<MaybeArray<(evt: Event) => void>>;
    readonly onCompositionStart: PropType<MaybeArray<(evt: CompositionEvent) => void>>;
    readonly onCompositionEnd: PropType<MaybeArray<(evt: CompositionEvent) => void>>;
    readonly onInput: PropType<MaybeArray<(evt: Event) => void>>;
    readonly onFocus: PropType<MaybeArray<(evt: FocusEvent) => void>>;
    readonly onBlur: PropType<MaybeArray<(evt: FocusEvent) => void>>;
};
export type InputProps = ExtractInnerPropTypes<typeof inputProps>;
export type InputPublicProps = ExtractPublicPropTypes<typeof inputProps>;
export interface InputBindings {
    focus: (options?: FocusOptions) => void;
    blur: () => void;
}
export type InputComponent = DefineComponent<Omit<InputHTMLAttributes, keyof InputPublicProps> & InputPublicProps, InputBindings>;
export type InputInstance = InstanceType<DefineComponent<InputProps, InputBindings>>;
