/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { ExtractInnerPropTypes, ExtractPublicPropTypes, MaybeArray } from '@idux/cdk/utils';
import type { DefineComponent, HTMLAttributes, PropType, VNodeChild } from 'vue';
export declare const layoutProps: {
    readonly fixed: {
        readonly type: PropType<LayoutFixedType>;
        readonly default: false;
    };
    readonly floatSider: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
};
export type LayoutProps = ExtractInnerPropTypes<typeof layoutProps>;
export type LayoutPublicProps = ExtractPublicPropTypes<typeof layoutProps>;
export type LayoutComponent = DefineComponent<Omit<HTMLAttributes, keyof LayoutPublicProps> & LayoutPublicProps>;
export type LayoutInstance = InstanceType<DefineComponent<LayoutProps>>;
export declare const layoutHeaderProps: {};
export type LayoutHeaderProps = ExtractInnerPropTypes<typeof layoutHeaderProps>;
export type LayoutHeaderPublicProps = ExtractPublicPropTypes<typeof layoutHeaderProps>;
export type LayoutHeaderComponent = DefineComponent<Omit<HTMLAttributes, keyof LayoutHeaderPublicProps> & LayoutHeaderPublicProps>;
export type LayoutHeaderInstance = InstanceType<DefineComponent<LayoutHeaderProps>>;
export declare const layoutContentProps: {};
export type LayoutContentProps = ExtractInnerPropTypes<typeof layoutContentProps>;
export type LayoutContentPublicProps = ExtractPublicPropTypes<typeof layoutContentProps>;
export type LayoutContentComponent = DefineComponent<Omit<HTMLAttributes, keyof LayoutContentPublicProps> & LayoutContentPublicProps>;
export type LayoutContentInstance = InstanceType<DefineComponent<LayoutContentProps>>;
export declare const layoutFooterProps: {};
export type LayoutFooterProps = ExtractInnerPropTypes<typeof layoutFooterProps>;
export type LayoutFooterPublicProps = ExtractPublicPropTypes<typeof layoutFooterProps>;
export type LayoutFooterComponent = DefineComponent<Omit<HTMLAttributes, keyof LayoutFooterPublicProps> & LayoutFooterPublicProps>;
export type LayoutFooterInstance = InstanceType<DefineComponent<LayoutFooterProps>>;
export declare const layoutSiderProps: {
    readonly collapsed: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly breakpoint: PropType<"xs" | "sm" | "md" | "lg" | "xl">;
    readonly pointer: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly pointerDelay: {
        readonly type: PropType<number | [number, number]>;
        readonly default: () => number[];
    };
    readonly 'onUpdate:collapsed': PropType<MaybeArray<(collapsed: boolean, changeType: LayoutCollapseChangeType) => void>>;
};
export type LayoutSiderProps = ExtractInnerPropTypes<typeof layoutSiderProps>;
export type LayoutSiderPublicProps = ExtractPublicPropTypes<typeof layoutSiderProps>;
export type LayoutSiderComponent = DefineComponent<Omit<HTMLAttributes, keyof LayoutSiderPublicProps> & LayoutSiderPublicProps>;
export type LayoutSiderInstance = InstanceType<DefineComponent<LayoutSiderProps>>;
export declare const layoutSiderTriggerProps: {
    readonly icon: {
        readonly type: PropType<VNodeChild | VNodeChild[]>;
        readonly default: undefined;
    };
};
export type LayoutSiderTriggerProps = ExtractInnerPropTypes<typeof layoutSiderTriggerProps>;
export type LayoutSiderTriggerPublicProps = ExtractPublicPropTypes<typeof layoutSiderTriggerProps>;
export type LayoutSiderTriggerComponent = DefineComponent<Omit<HTMLAttributes, keyof LayoutSiderTriggerPublicProps> & LayoutSiderTriggerPublicProps>;
export type LayoutSiderTriggerInstance = InstanceType<DefineComponent<LayoutSiderTriggerProps>>;
export type LayoutFixedType = boolean | {
    header: boolean;
    sider: boolean;
};
export type LayoutCollapseChangeType = 'breakpoint' | 'pointer' | 'trigger';
