import { defineComponent, inject, computed, createVNode, Fragment, provide, normalizeClass } from "vue";
import { IxCol, IxRow } from "@idux/components/grid";
import { getFirstValidNode } from "@idux/cdk/utils";
import { \u0275Empty as _Empty } from "@idux/components/_private/empty";
import { useGlobalConfig } from "@idux/components/config";
import { IxSpin } from "@idux/components/spin";
import { useThemeToken } from "@idux/components/theme";
const listToken = Symbol("list");
const listProps = {
  header: String,
  footer: String,
  empty: { type: [String, Object], default: "default" },
  borderless: {
    type: Boolean,
    default: void 0
  },
  loading: {
    type: [Boolean, Object],
    default: false
  },
  size: String,
  grid: Object
};
const listItemProps = {
  title: String,
  content: String,
  extra: String
};
var ListItem = /* @__PURE__ */ defineComponent({
  name: "IxListItem",
  props: listItemProps,
  setup(props, {
    slots
  }) {
    const {
      props: listProps2,
      mergedPrefixCls
    } = inject(listToken);
    const classPrefix = computed(() => `${mergedPrefixCls.value}-item`);
    const getGrid = (grid, dim) => {
      const v = grid[dim];
      if (typeof v === "number") {
        return Math.floor(24 / v);
      }
      return void 0;
    };
    const useGrid = (grid) => {
      const defaultSpan = getGrid(grid, "column");
      return {
        xs: getGrid(grid, "xs") || defaultSpan,
        sm: getGrid(grid, "sm") || defaultSpan,
        md: getGrid(grid, "md") || defaultSpan,
        ld: getGrid(grid, "lg") || defaultSpan,
        xl: getGrid(grid, "xl") || defaultSpan
      };
    };
    return () => {
      var _a, _b, _c, _d, _e;
      const grid = listProps2.grid && useGrid(listProps2.grid);
      const title = (_b = props.title) != null ? _b : (_a = slots.title) == null ? void 0 : _a.call(slots);
      const content = (_d = props.content) != null ? _d : (_c = slots.default) == null ? void 0 : _c.call(slots);
      const extra = (_e = slots.extra) == null ? void 0 : _e.call(slots);
      const renderContent = createVNode(Fragment, null, [title && createVNode("div", {
        "class": `${classPrefix.value}-title`
      }, [title]), createVNode("div", {
        "class": `${classPrefix.value}-content`
      }, [content]), extra && createVNode("div", {
        "class": `${classPrefix.value}-extra`
      }, [extra])]);
      return grid ? createVNode(IxCol, grid, {
        default: () => [renderContent]
      }) : createVNode("div", {
        "class": `${classPrefix.value}`
      }, [renderContent]);
    };
  }
});
var List = /* @__PURE__ */ defineComponent({
  name: "IxList",
  props: listProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId
    } = useThemeToken();
    const mergedPrefixCls = computed(() => `${common.prefixCls}-list`);
    const listConfig = useGlobalConfig("list");
    provide(listToken, {
      props,
      mergedPrefixCls
    });
    const loadingProp = computed(() => {
      if (typeof props.loading === "boolean") {
        return {
          spinning: props.loading
        };
      }
      return props.loading;
    });
    const classes = computed(() => {
      var _a, _b;
      const prefixCls = mergedPrefixCls.value;
      const borderless = (_a = props.borderless) != null ? _a : listConfig.borderless;
      const size = (_b = props.size) != null ? _b : listConfig.size;
      return normalizeClass({
        [globalHashId.value]: !globalHashId.value,
        [`${prefixCls}`]: true,
        [`${prefixCls}-split`]: true,
        [`${prefixCls}-${size}`]: true,
        [`${prefixCls}-border`]: !borderless
      });
    });
    return () => {
      var _a, _b, _c, _d;
      const header = (_b = props.header) != null ? _b : (_a = slots.header) == null ? void 0 : _a.call(slots);
      const footer = (_d = props.footer) != null ? _d : (_c = slots.footer) == null ? void 0 : _c.call(slots);
      const children = slots.default ? slots.default() : [];
      const content = getFirstValidNode(children) ? children : createVNode(_Empty, {
        "empty": props.empty
      }, slots);
      const renderContent = () => {
        if (props.grid) {
          return createVNode(IxRow, {
            "gutter": props.grid.gutter
          }, {
            default: () => [content]
          });
        }
        return createVNode("div", null, [content]);
      };
      return createVNode("div", {
        "class": classes.value
      }, [header && createVNode("div", {
        "class": `${mergedPrefixCls.value}-header`
      }, [header]), createVNode(IxSpin, loadingProp.value, {
        default: () => [renderContent()]
      }), footer && createVNode("div", {
        "class": `${mergedPrefixCls.value}-footer`
      }, [footer])]);
    };
  }
});
const IxListItem = ListItem;
const IxList = List;
export { IxList, IxListItem };
