import { computed, watch, inject, createVNode, mergeProps, defineComponent, normalizeClass, provide, withDirectives, vShow, nextTick, ref } from "vue";
import { flattenNode, Logger, useControlledProp, useKey, useState, callEmit } from "@idux/cdk/utils";
import { useGlobalConfig } from "@idux/components/config";
import { \u0275DropdownToken as _DropdownToken } from "@idux/components/dropdown";
import { useThemeToken } from "@idux/components/theme";
import { useGetKey } from "@idux/components/utils";
import { isString, debounce } from "lodash-es";
import { IxIcon } from "@idux/components/icon";
import { \u0275Overlay as _Overlay } from "@idux/components/_private/overlay";
import { \u0275CollapseTransition as _CollapseTransition } from "@idux/components/_private/collapse-transition";
const itemKey = "__IDUX_MENU_ITEM";
const MenuItem$1 = () => {
};
MenuItem$1.displayName = "IxMenuItem";
MenuItem$1[itemKey] = true;
const itemGroupKey = "__IDUX_MENU_ITEM_GROUP";
const MenuGroupItem = () => {
};
MenuGroupItem.displayName = "IxMenuItemGroup";
MenuGroupItem[itemGroupKey] = true;
const subKey = "__IDUX_MENU_SUB";
const MenuSub$1 = () => {
};
MenuSub$1.displayName = "IxMenuSub";
MenuSub$1[subKey] = true;
const dividerKey = "__IDUX_MENU_DIVIDER";
const MenuDivider$1 = () => {
};
MenuDivider$1.displayName = "IxMenuDivider";
MenuDivider$1[dividerKey] = true;
function useDataSource(props, slots) {
  return computed(() => {
    var _a;
    const { dataSource } = props;
    if (dataSource) {
      return dataSource;
    } else {
      return convertDataSource((_a = slots.default) == null ? void 0 : _a.call(slots));
    }
  });
}
const filterKeys = [itemKey, itemGroupKey, subKey, dividerKey];
function convertDataSource(nodes) {
  const dataSource = [];
  flattenNode(nodes, { key: filterKeys }).forEach((node) => {
    var _a, _b, _c, _d;
    const type = node.type;
    const slots = (_a = node.children) != null ? _a : {};
    const props = (_b = node.props) != null ? _b : {};
    let data;
    if (type[itemKey]) {
      const { key, disabled, icon, label } = props;
      const { icon: customIcon, label: customLabel, default: customLabel2 } = slots;
      data = {
        key,
        disabled: disabled || disabled === "",
        icon,
        label,
        customIcon,
        customLabel: customLabel != null ? customLabel : customLabel2
      };
    } else if (type[itemGroupKey]) {
      const { key, children, icon, label } = props;
      const { icon: customIcon, label: customLabel } = slots;
      data = {
        type: "itemGroup",
        key,
        children: children != null ? children : convertDataSource((_c = slots.default) == null ? void 0 : _c.call(slots)),
        icon,
        label,
        customIcon,
        customLabel
      };
    } else if (type[subKey]) {
      const { key, children, disabled, icon, label, suffix } = props;
      const { icon: customIcon, label: customLabel, suffix: customSuffix } = slots;
      data = {
        type: "sub",
        key,
        children: children != null ? children : convertDataSource((_d = slots.default) == null ? void 0 : _d.call(slots)),
        disabled: disabled || disabled === "",
        icon,
        label,
        suffix,
        customIcon,
        customLabel,
        customSuffix
      };
    } else {
      const { key } = props;
      data = { type: "divider", key };
    }
    if (process.env.NODE_ENV !== "production" && data.type !== "divider" && !data.key) {
      Logger.warn("components/menu", "key must exist", data);
    }
    dataSource.push(data);
  });
  return dataSource;
}
function useExpanded$1(props) {
  const [expandedKeys, setExpandedKeys] = useControlledProp(props, "expandedKeys", () => []);
  const handleExpand = (key, expanded) => {
    const index = expandedKeys.value.indexOf(key);
    if (expanded) {
      index === -1 && setExpandedKeys([...expandedKeys.value, key]);
    } else {
      if (index > -1) {
        const tempKeys = [...expandedKeys.value];
        tempKeys.splice(index, 1);
        setExpandedKeys(tempKeys);
      }
    }
  };
  let cachedExpandedKeys = [];
  watch(
    () => props.collapsed,
    (collapsed) => {
      if (collapsed) {
        cachedExpandedKeys = [...expandedKeys.value];
        setExpandedKeys([]);
      } else {
        setExpandedKeys(cachedExpandedKeys);
        cachedExpandedKeys = [];
      }
    }
  );
  return { expandedKeys, handleExpand };
}
function useSelected(props, dropdownContext) {
  const [selectedKeys, setSelectedKeys] = useControlledProp(props, "selectedKeys", () => []);
  const handleSelected = (key) => {
    if (dropdownContext) {
      const { hideOnClick, setVisibility } = dropdownContext;
      hideOnClick.value && setVisibility(false);
    }
    if (!props.selectable) {
      return;
    }
    const index = selectedKeys.value.indexOf(key);
    if (index > -1) {
      if (props.multiple) {
        const tempKeys = [...selectedKeys.value];
        tempKeys.splice(index, 1);
        setSelectedKeys(tempKeys);
      }
    } else {
      setSelectedKeys(props.multiple ? [...selectedKeys.value, key] : [key]);
    }
  };
  return { selectedKeys, handleSelected };
}
const menuToken = Symbol("menuToken");
const menuSubToken = Symbol("menuSubToken");
const menuItemGroupToken = Symbol("menuItemGroupToken");
const MenuDivider = (props) => {
  const {
    props: menuProps2,
    mergedPrefixCls
  } = inject(menuToken);
  const customAdditional = menuProps2.customAdditional ? menuProps2.customAdditional({
    data: props.data,
    index: props.index
  }) : void 0;
  return createVNode("li", mergeProps({
    "class": `${mergedPrefixCls.value}-divider`
  }, customAdditional), null);
};
MenuDivider.displayName = "MenuDivider";
const usePaddingLeft = (menuProps2, mode, indent, level, grouped) => {
  return computed(() => {
    if (mode.value !== "inline" || menuProps2.collapsed) {
      return void 0;
    }
    const groupedLeft = grouped ? 8 : 0;
    return `${indent.value * level + groupedLeft}px`;
  });
};
const menuProps = {
  expandedKeys: Array,
  selectedKeys: Array,
  collapsed: {
    type: Boolean,
    default: false
  },
  customAdditional: { type: Function, default: void 0 },
  dataSource: Array,
  getKey: {
    type: [String, Function],
    default: void 0
  },
  indent: Number,
  mode: {
    type: String,
    default: "vertical"
  },
  multiple: {
    type: Boolean,
    default: false
  },
  overlayClassName: String,
  overlayContainer: {
    type: [String, HTMLElement, Function],
    default: void 0
  },
  overlayDelay: {
    type: [Number, Array],
    default: () => [0, 100]
  },
  selectable: {
    type: Boolean,
    default: true
  },
  theme: String,
  "onUpdate:expandedKeys": [Function, Array],
  "onUpdate:selectedKeys": [Function, Array],
  onClick: [Function, Array]
};
const menuItemProps = {
  data: {
    type: Object,
    required: true
  },
  index: { type: Number, required: true }
};
const menuItemGroupProps = {
  data: {
    type: Object,
    required: true
  },
  index: { type: Number, required: true }
};
const menuSubProps = {
  data: {
    type: Object,
    required: true
  },
  index: { type: Number, required: true }
};
var MenuItem = /* @__PURE__ */ defineComponent({
  name: "MenuItem",
  props: menuItemProps,
  setup(props) {
    const key = useKey();
    const {
      props: menuProps2,
      slots: menuSlots,
      mergedPrefixCls,
      indent,
      selectedKeys,
      handleSelected,
      handleClick
    } = inject(menuToken);
    const menuSubContext = inject(menuSubToken, null);
    const menuItemGroupContext = inject(menuItemGroupToken, false);
    const level = menuSubContext ? menuSubContext.level + 1 : 1;
    const isSelected = computed(() => selectedKeys.value.includes(key));
    const classes = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [`${prefixCls}-item`]: true,
        [`${prefixCls}-level-${level}`]: true,
        [`${prefixCls}-item-disabled`]: props.data.disabled,
        [`${prefixCls}-item-selected`]: isSelected.value
      });
    });
    const mode = computed(() => menuProps2.mode);
    const paddingLeft = usePaddingLeft(menuProps2, mode, indent, level, menuItemGroupContext);
    const style = computed(() => {
      return {
        paddingLeft: paddingLeft.value
      };
    });
    const onClick = (evt) => {
      handleSelected(key);
      handleClick(key, "item", evt);
      menuSubContext == null ? void 0 : menuSubContext.handleItemClick();
    };
    return () => {
      const {
        disabled,
        icon,
        label,
        customIcon,
        customLabel,
        customSuffix
      } = props.data;
      const iconRender = customIcon != null ? customIcon : "itemIcon";
      const iconSlot = isString(iconRender) ? menuSlots[iconRender] : iconRender;
      const labelRender = customLabel != null ? customLabel : "itemLabel";
      const labelSlot = isString(labelRender) ? menuSlots[labelRender] : labelRender;
      const suffixRender = customSuffix != null ? customSuffix : "itemSuffix";
      const suffixSlot = isString(suffixRender) ? menuSlots[suffixRender] : suffixRender;
      const slotProps = iconSlot || labelSlot ? {
        ...props.data,
        selected: isSelected.value
      } : void 0;
      const iconNode = coverIcon(iconSlot, slotProps, icon);
      const labelNode = labelSlot ? labelSlot(slotProps) : label;
      const customAdditional = menuProps2.customAdditional ? menuProps2.customAdditional({
        data: props.data,
        index: props.index
      }) : void 0;
      const suffixNode = coverIcon(suffixSlot, slotProps, props.data.suffix);
      return createVNode("li", mergeProps({
        "class": classes.value,
        "style": style.value,
        "aria-label": label,
        "aria-selected": isSelected.value,
        "role": "menuitem",
        "onClick": disabled ? void 0 : onClick
      }, customAdditional), [iconNode, createVNode("span", null, [labelNode]), suffixNode && createVNode("span", {
        "class": `${mergedPrefixCls.value}-item-suffix`
      }, [suffixNode])]);
    };
  }
});
var MenuItemGroup = /* @__PURE__ */ defineComponent({
  name: "MenuItemGroup",
  props: menuItemGroupProps,
  setup(props) {
    const key = useKey();
    provide(menuItemGroupToken, true);
    const {
      props: menuProps2,
      slots: menuSlots,
      mergedPrefixCls,
      mergedGetKey,
      indent,
      handleClick
    } = inject(menuToken);
    const menuSubContext = inject(menuSubToken, null);
    const menuItemGroupContext = inject(menuItemGroupToken, null);
    const level = menuSubContext ? menuSubContext.level + 1 : 1;
    const mode = computed(() => menuProps2.mode);
    const paddingLeft = usePaddingLeft(menuProps2, mode, indent, level, !!menuItemGroupContext);
    const labelStyle = computed(() => ({
      paddingLeft: paddingLeft.value
    }));
    const classes = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [`${prefixCls}-item-group`]: true,
        [`${prefixCls}-level-${level}`]: true
      });
    });
    const onClick = (evt) => {
      handleClick(key, "itemGroup", evt);
    };
    return () => {
      const {
        icon,
        label,
        children,
        customIcon,
        customLabel
      } = props.data;
      const iconRender = customIcon != null ? customIcon : "itemGroupIcon";
      const iconSlot = isString(iconRender) ? menuSlots[iconRender] : iconRender;
      const labelRender = customLabel != null ? customLabel : "itemGroupLabel";
      const labelSlot = isString(labelRender) ? menuSlots[labelRender] : labelRender;
      const slotProps = props.data;
      const iconNode = coverIcon(iconSlot, slotProps, icon);
      const labelNode = labelSlot ? labelSlot(slotProps) : label;
      const prefixCls = `${mergedPrefixCls.value}-item-group`;
      const customAdditional = menuProps2.customAdditional ? menuProps2.customAdditional({
        data: props.data,
        index: props.index
      }) : void 0;
      return createVNode("li", mergeProps({
        "class": classes.value,
        "aria-label": label
      }, customAdditional), [createVNode("div", {
        "class": `${prefixCls}-label`,
        "style": labelStyle.value,
        "onClick": onClick
      }, [iconNode, createVNode("span", null, [labelNode])]), createVNode("ul", {
        "class": `${prefixCls}-content`
      }, [coverChildren(children, mergedGetKey.value)])]);
    };
  }
});
var InlineContent = /* @__PURE__ */ defineComponent({
  name: "MenuSubInlineContent",
  setup() {
    const {
      mergedPrefixCls,
      mergedGetKey
    } = inject(menuToken);
    const {
      props,
      isExpanded
    } = inject(menuSubToken);
    const classes = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [`${prefixCls}-inline`]: true
      });
    });
    return () => {
      return createVNode(_CollapseTransition, {
        "appear": true
      }, {
        default: () => [withDirectives(createVNode("ul", {
          "class": classes.value
        }, [coverChildren(props.data.children, mergedGetKey.value)]), [[vShow, isExpanded.value]])]
      });
    };
  }
});
var Label = /* @__PURE__ */ defineComponent({
  name: "MenuSubLabel",
  setup() {
    const {
      slots: menuSlots,
      config,
      mergedPrefixCls,
      handleClick
    } = inject(menuToken);
    const {
      props,
      key,
      isExpanded,
      isSelected,
      changeExpanded,
      handleMouseEvent,
      mode,
      paddingLeft
    } = inject(menuSubToken);
    const mergedSuffix = computed(() => {
      const {
        suffix
      } = props.data;
      if (suffix) {
        return suffix;
      }
      return mode.value !== "horizontal" ? config.suffix : void 0;
    });
    const rotate = computed(() => {
      if (mode.value === "inline") {
        return isExpanded.value ? -90 : 90;
      }
      return 0;
    });
    const events = computed(() => {
      if (props.data.disabled) {
        return void 0;
      }
      if (mode.value === "inline") {
        return {
          onClick: (evt) => {
            handleClick(key, "sub", evt);
            changeExpanded(!isExpanded.value);
          }
        };
      } else {
        return {
          onClick: (evt) => handleClick(key, "sub", evt),
          onMouseenter: () => handleMouseEvent(true),
          onMouseleave: () => handleMouseEvent(false)
        };
      }
    });
    const style = computed(() => {
      return {
        paddingLeft: paddingLeft.value
      };
    });
    return () => {
      const {
        icon,
        label,
        customIcon,
        customLabel,
        customSuffix
      } = props.data;
      const iconRender = customIcon != null ? customIcon : "subIcon";
      const iconSlot = isString(iconRender) ? menuSlots[iconRender] : iconRender;
      const labelRender = customLabel != null ? customLabel : "subLabel";
      const labelSlot = isString(labelRender) ? menuSlots[labelRender] : labelRender;
      const suffixRender = customSuffix != null ? customSuffix : "subSuffix";
      const suffixSlot = isString(suffixRender) ? menuSlots[suffixRender] : suffixRender;
      const slotProps = iconSlot || labelSlot || suffixSlot ? {
        ...props.data,
        expanded: isExpanded.value,
        selected: isSelected.value
      } : void 0;
      const iconNode = coverIcon(iconSlot, slotProps, icon);
      const labelNode = labelSlot ? labelSlot(slotProps) : label;
      const suffixNode = coverIcon(suffixSlot, slotProps, mergedSuffix.value, rotate.value);
      const prefixCls = `${mergedPrefixCls.value}-sub-label`;
      return createVNode("div", mergeProps({
        "class": prefixCls,
        "style": style.value
      }, events.value), [iconNode, createVNode("span", null, [labelNode]), suffixNode && createVNode("span", {
        "class": `${prefixCls}-suffix`
      }, [suffixNode])]);
    };
  }
});
var OverlayContent = /* @__PURE__ */ defineComponent({
  name: "MenuSubOverlayContent",
  setup() {
    const {
      mergedPrefixCls,
      theme,
      mergedGetKey
    } = inject(menuToken);
    const {
      props,
      isExpanded,
      handleMouseEvent
    } = inject(menuSubToken);
    const dropdownContext = inject(_DropdownToken, null);
    const classes = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [prefixCls]: true,
        [`${prefixCls}-dropdown`]: !!dropdownContext,
        [`${prefixCls}-vertical`]: true,
        [`${prefixCls}-${theme.value}`]: true
      });
    });
    const events = computed(() => {
      if (props.data.disabled || !isExpanded.value) {
        return void 0;
      }
      return {
        onMouseenter: () => handleMouseEvent(true),
        onMouseleave: () => handleMouseEvent(false)
      };
    });
    return () => createVNode("ul", mergeProps({
      "class": classes.value
    }, events.value), [coverChildren(props.data.children, mergedGetKey.value)]);
  }
});
var MenuSub = /* @__PURE__ */ defineComponent({
  name: "MenuSub",
  props: menuSubProps,
  setup(props) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId
    } = useThemeToken("menu");
    const {
      props: menuProps2,
      config,
      mergedPrefixCls,
      indent,
      selectedKeys,
      expandedKeys,
      handleExpand
    } = inject(menuToken);
    const menuSubContext = inject(menuSubToken, null);
    const menuItemGroupContext = inject(menuItemGroupToken, false);
    const key = useKey();
    const mode = useMode(menuProps2, menuSubContext);
    const level = menuSubContext ? menuSubContext.level + 1 : 1;
    const paddingLeft = usePaddingLeft(menuProps2, mode, indent, level, menuItemGroupContext);
    const isSelected = computed(() => getState(props.data.children, selectedKeys.value));
    const {
      isExpanded,
      changeExpanded,
      handleMouseEvent
    } = useExpanded(props, key, expandedKeys, handleExpand, mode);
    const handleItemClick = () => {
      if (!props.data.disabled && mode.value !== "inline" && !menuProps2.multiple) {
        handleMouseEvent(false);
        handleExpand(key, false);
      }
    };
    provide(menuSubToken, {
      props,
      key,
      isExpanded,
      isSelected,
      mode,
      level,
      paddingLeft,
      changeExpanded,
      handleMouseEvent,
      handleItemClick
    });
    const placement = computed(() => mode.value === "vertical" ? "rightStart" : "bottomStart");
    const classes = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [`${prefixCls}-sub`]: true,
        [`${prefixCls}-level-${level}`]: true,
        [`${prefixCls}-sub-disabled`]: props.data.disabled,
        [`${prefixCls}-sub-expanded`]: isExpanded.value,
        [`${prefixCls}-sub-selected`]: isSelected.value
      });
    });
    const overlayClasses = computed(() => {
      const {
        overlayClassName
      } = menuProps2;
      return normalizeClass({
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [`${mergedPrefixCls.value}-overlay`]: true,
        [overlayClassName || ""]: !!overlayClassName
      });
    });
    const offset = computed(() => {
      var _a;
      return (_a = props.data.offset) != null ? _a : config.offset;
    });
    return () => {
      var _a;
      const {
        disabled
      } = props.data;
      const isInline = mode.value === "inline";
      let children;
      if (isInline) {
        children = [createVNode(Label, null, null), createVNode(InlineContent, null, null)];
      } else {
        const trigger = () => createVNode(Label, null, null);
        const content = () => createVNode(OverlayContent, null, null);
        children = createVNode(_Overlay, {
          "class": overlayClasses.value,
          "autoAdjust": true,
          "container": (_a = menuProps2.overlayContainer) != null ? _a : config.overlayContainer,
          "containerFallback": `.${mergedPrefixCls.value}-overlay-container`,
          "destroyOnHide": false,
          "delay": menuProps2.overlayDelay,
          "disabled": disabled,
          "offset": offset.value,
          "placement": placement.value,
          "transitionName": `${common.prefixCls}-slide-up`,
          "trigger": "manual",
          "visible": isExpanded.value
        }, {
          default: trigger,
          content
        });
      }
      const customAdditional = menuProps2.customAdditional ? menuProps2.customAdditional({
        data: props.data,
        index: props.index
      }) : void 0;
      return createVNode("li", mergeProps({
        "class": classes.value,
        "aria-expanded": isExpanded.value,
        "aria-haspopup": !isInline,
        "role": "menuitem"
      }, customAdditional), [children]);
    };
  }
});
function useMode(menuProps2, menuSubContext) {
  const defaultMode = menuProps2.mode !== "inline" && !!menuSubContext ? "vertical" : menuProps2.mode;
  const [mode, setMode] = useState(defaultMode);
  watch(() => menuProps2.mode, (mode2) => {
    nextTick(() => setMode(mode2 !== "inline" && !!menuSubContext ? "vertical" : mode2));
  });
  return mode;
}
function useExpanded(props, key, expandedKeys, handleExpand, mode) {
  const isHover = ref(false);
  const isExpanded = computed(() => {
    if (mode.value === "inline") {
      return expandedKeys.value.includes(key);
    }
    return isHover.value && expandedKeys.value.includes(key) || childExpanded.value;
  });
  const changeExpanded = (expanded) => handleExpand(key, expanded);
  const childExpanded = computed(() => getState(props.data.children, expandedKeys.value));
  const handleMouseEvent = debounce((value) => isHover.value = value, 100);
  watch([mode, childExpanded, isHover], ([currMode, currChildExpanded, currIsHover]) => {
    if (currMode !== "inline") {
      changeExpanded(currChildExpanded || currIsHover);
    }
  });
  return {
    isExpanded,
    changeExpanded,
    handleMouseEvent
  };
}
function getState(children, selectedKeys) {
  if (!children || children.length === 0) {
    return false;
  }
  return children.some((item) => {
    return item.key && selectedKeys.includes(item.key) || "children" in item && getState(item.children, selectedKeys);
  });
}
function coverChildren(data, getKetFn) {
  if (!data || data.length === 0) {
    return [];
  }
  const nodes = [];
  data.forEach((item, index) => {
    const {
      type
    } = item;
    const key = getKetFn(item);
    if (!type || type === "item") {
      nodes.push(createVNode(MenuItem, {
        "key": key,
        "index": index,
        "data": item
      }, null));
    } else if (type === "sub") {
      nodes.push(createVNode(MenuSub, {
        "key": key,
        "index": index,
        "data": item
      }, null));
    } else if (type === "itemGroup") {
      nodes.push(createVNode(MenuItemGroup, {
        "key": key,
        "index": index,
        "data": item
      }, null));
    } else if (type === "divider") {
      nodes.push(createVNode(MenuDivider, {
        "key": key,
        "index": index,
        "data": item
      }, null));
    } else {
      process.env.NODE_ENV !== "production" && Logger.warn("components/menu", `type [${type}] not supported`);
    }
  });
  return nodes;
}
function coverIcon(slot, slotProps, iconOrName, rotate) {
  if (slot) {
    return slot(slotProps);
  }
  return isString(iconOrName) ? createVNode(IxIcon, {
    "name": iconOrName,
    "rotate": rotate
  }, null) : iconOrName;
}
function getDefaultThemeTokens(tokens, algorithms) {
  const { getGreyColors } = algorithms;
  const {
    heightMd,
    heightLg,
    fontSizeSm,
    fontSizeMd,
    fontWeightHeader,
    colorText,
    colorTextInfo,
    colorTextInverse,
    colorTextInverseDisabled,
    colorPrimary,
    colorPrimaryText,
    colorTextDisabled,
    colorContainerBg,
    colorContainerBgHover,
    colorContainerBgActive,
    colorBgInverse
  } = tokens;
  const greyColors = getGreyColors();
  return {
    borderRadius: 0,
    height: heightMd,
    heightLevel1: heightLg,
    fontSize: fontSizeSm,
    fontSizeLevel1: fontSizeMd,
    overlayMinWidth: 128,
    collapsedWidth: 44,
    itemColor: colorText,
    itemColorHover: colorText,
    itemColorActive: colorPrimaryText,
    itemColorDisabled: colorTextDisabled,
    itemGroupColor: colorTextInfo,
    itemBg: colorContainerBg,
    itemBgHover: colorContainerBgHover,
    itemBgActive: colorContainerBgActive,
    itemBgDisabled: colorContainerBg,
    itemGroupBg: colorContainerBg,
    horizontalItemColorHover: colorText,
    horizontalItemColorActive: colorPrimaryText,
    horizontalItemBgHover: colorContainerBgHover,
    horizontalItemBgActive: colorContainerBgActive,
    darkItemColor: colorTextInverse,
    darkItemColorHover: colorTextInverse,
    darkItemColorActive: colorTextInverse,
    darkItemColorDisabled: colorTextInverseDisabled,
    darkItemGroupColor: colorTextInfo,
    darkItemFontWeightActive: fontWeightHeader,
    darkItemBg: colorBgInverse,
    darkItemBgHover: greyColors.d50,
    darkItemBgActive: colorPrimary,
    darkItemBgDisabled: colorBgInverse,
    darkItemGroupBg: colorBgInverse,
    darkHorizontalItemColorHover: colorTextInverse,
    darkHorizontalItemColorActive: colorTextInverse,
    darkHorizontalItemBgHover: greyColors.d50,
    darkHorizontalItemBgActive: greyColors.d30
  };
}
function getDarkThemeTokens(tokens, algorithms) {
  const defaultTokens = getDefaultThemeTokens(tokens, algorithms);
  return {
    ...defaultTokens,
    darkItemColor: defaultTokens.itemColor,
    darkItemColorHover: defaultTokens.itemColorHover,
    darkItemColorActive: defaultTokens.itemColorActive,
    darkItemColorDisabled: defaultTokens.itemColorDisabled,
    darkItemGroupColor: defaultTokens.itemGroupColor,
    darkItemFontWeightActive: defaultTokens.darkItemFontWeightActive,
    darkItemBg: defaultTokens.itemBg,
    darkItemBgHover: defaultTokens.itemBgHover,
    darkItemBgActive: defaultTokens.itemBgActive,
    darkItemBgDisabled: defaultTokens.itemBgDisabled,
    darkItemGroupBg: defaultTokens.itemGroupBg,
    darkHorizontalItemColorHover: defaultTokens.horizontalItemColorHover,
    darkHorizontalItemColorActive: defaultTokens.horizontalItemColorActive,
    darkHorizontalItemBgHover: defaultTokens.horizontalItemBgHover,
    darkHorizontalItemBgActive: defaultTokens.horizontalItemBgActive
  };
}
const getThemeTokens = (tokens, presetTheme, algorithms) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens, algorithms) : getDarkThemeTokens(tokens, algorithms);
};
var Menu = /* @__PURE__ */ defineComponent({
  name: "IxMenu",
  props: menuProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("menu");
    registerToken(getThemeTokens);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-menu`);
    const config = useGlobalConfig("menu");
    const mergedGetKey = useGetKey(props, config, "components/menu");
    const indent = computed(() => {
      var _a;
      return (_a = props.indent) != null ? _a : config.indent;
    });
    const mode = computed(() => props.mode);
    const theme = computed(() => {
      var _a;
      return (_a = props.theme) != null ? _a : config.theme;
    });
    const dropdownContext = inject(_DropdownToken, null);
    const {
      expandedKeys,
      handleExpand
    } = useExpanded$1(props);
    const {
      selectedKeys,
      handleSelected
    } = useSelected(props, dropdownContext);
    const handleClick = (key, type, evt) => {
      callEmit(props.onClick, {
        key,
        type,
        event: evt
      });
    };
    provide(menuToken, {
      props,
      slots,
      config,
      mergedPrefixCls,
      mergedGetKey,
      indent,
      theme,
      expandedKeys,
      handleExpand,
      selectedKeys,
      handleSelected,
      handleClick
    });
    const classes = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [prefixCls]: true,
        [`${prefixCls}-${theme.value}`]: true,
        [`${prefixCls}-${mode.value}`]: true,
        [`${prefixCls}-collapsed`]: props.collapsed,
        [`${prefixCls}-dropdown`]: !!dropdownContext
      });
    });
    const dateSource = useDataSource(props, slots);
    return () => {
      return createVNode("ul", {
        "class": classes.value
      }, [coverChildren(dateSource.value, mergedGetKey.value)]);
    };
  }
});
const IxMenu = Menu;
const IxMenuDivider = MenuDivider$1;
const IxMenuItem = MenuItem$1;
const IxMenuItemGroup = MenuGroupItem;
const IxMenuSub = MenuSub$1;
export { IxMenu, IxMenuDivider, IxMenuItem, IxMenuItemGroup, IxMenuSub, getThemeTokens as getMenuThemeTokens };
