/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { ExtractInnerPropTypes, ExtractPublicPropTypes, MaybeArray, VKey } from '@idux/cdk/utils';
import type { OverlayContainerType } from '@idux/components/utils';
import type { DefineComponent, FunctionalComponent, HTMLAttributes, PropType, VNode, VNodeChild } from 'vue';
export type MenuMode = 'vertical' | 'horizontal' | 'inline';
export type MenuTheme = 'light' | 'dark';
export interface MenuClickOptions<K = VKey> {
    event: Event;
    key: K;
    type: 'item' | 'itemGroup' | 'sub';
}
export declare const menuProps: {
    expandedKeys: PropType<VKey[]>;
    selectedKeys: PropType<VKey[]>;
    collapsed: {
        type: BooleanConstructor;
        default: boolean;
    };
    customAdditional: {
        type: PropType<MenuCustomAdditional>;
        default: undefined;
    };
    dataSource: PropType<MenuData<VKey>[]>;
    getKey: {
        type: PropType<string | ((data: MenuData<any>) => any)>;
        default: undefined;
    };
    indent: NumberConstructor;
    mode: {
        type: PropType<MenuMode>;
        default: string;
    };
    multiple: {
        type: BooleanConstructor;
        default: boolean;
    };
    overlayClassName: StringConstructor;
    overlayContainer: {
        type: PropType<OverlayContainerType>;
        default: undefined;
    };
    overlayDelay: {
        type: PropType<number | [number | null, number | null]>;
        default: () => [number, number];
    };
    selectable: {
        type: BooleanConstructor;
        default: boolean;
    };
    theme: PropType<MenuTheme>;
    'onUpdate:expandedKeys': PropType<MaybeArray<(expandedKeys: any[]) => void>>;
    'onUpdate:selectedKeys': PropType<MaybeArray<(selectedKeys: any[]) => void>>;
    onClick: PropType<MaybeArray<(options: MenuClickOptions<any>) => void>>;
};
export type MenuProps = ExtractInnerPropTypes<typeof menuProps>;
export type MenuPublicProps = ExtractPublicPropTypes<typeof menuProps>;
export type MenuComponent = DefineComponent<Omit<HTMLAttributes, keyof MenuPublicProps> & MenuPublicProps>;
export type MenuInstance = InstanceType<DefineComponent<MenuProps>>;
export interface MenuItemProps<K = VKey> {
    key?: K;
    type?: 'item';
    disabled?: boolean;
    icon?: string | VNode;
    label?: string;
    suffix?: string;
    customIcon?: string | ((data: MenuItemProps<K> & {
        selected: boolean;
    }) => VNodeChild);
    customLabel?: string | ((data: MenuItemProps<K> & {
        selected: boolean;
    }) => VNodeChild);
    customSuffix?: string | ((data: MenuItemProps<K> & {
        selected: boolean;
    }) => VNodeChild);
}
export type MenuItemPublicProps = Omit<MenuItemProps, 'type'>;
export type MenuItemComponent = FunctionalComponent<Omit<HTMLAttributes, keyof MenuItemPublicProps> & MenuItemPublicProps>;
export type MenuItemInstance = InstanceType<DefineComponent<MenuItemProps>>;
export interface MenuItemGroupProps<K = VKey> {
    type: 'itemGroup';
    key?: K;
    children?: MenuData<K>[];
    icon?: string | VNode;
    label?: string;
    customIcon?: string | ((data: MenuItemGroupProps<K>) => VNodeChild);
    customLabel?: string | ((data: MenuItemGroupProps<K>) => VNodeChild);
}
export type MenuItemGroupPublicProps = Omit<MenuItemGroupProps, 'type'>;
export type MenuItemGroupComponent = FunctionalComponent<Omit<HTMLAttributes, keyof MenuItemGroupPublicProps> & MenuItemGroupPublicProps>;
export type MenuItemGroupInstance = InstanceType<DefineComponent<MenuItemGroupProps>>;
export interface MenuSubProps<K = VKey> {
    type: 'sub';
    key?: K;
    children?: MenuData<K>[];
    disabled?: boolean;
    icon?: string | VNode;
    label?: string;
    offset?: [number, number];
    suffix?: string;
    customIcon?: string | ((data: MenuSubProps<K> & {
        expanded: boolean;
        selected: boolean;
    }) => VNodeChild);
    customLabel?: string | ((data: MenuSubProps<K> & {
        expanded: boolean;
        selected: boolean;
    }) => VNodeChild);
    customSuffix?: string | ((data: MenuSubProps<K> & {
        expanded: boolean;
        selected: boolean;
    }) => VNodeChild);
}
export type MenuSubPublicProps = Omit<MenuSubProps, 'type'>;
export type MenuSubComponent = FunctionalComponent<Omit<HTMLAttributes, keyof MenuSubPublicProps> & MenuSubPublicProps>;
export interface MenuDividerProps<K = VKey> {
    type: 'divider';
    key?: K;
}
export type MenuDividerComponent = FunctionalComponent<HTMLAttributes>;
export type MenuData<K = VKey> = MenuItemProps<K> | MenuItemGroupProps<K> | MenuSubProps<K> | MenuDividerProps<K>;
export type MenuCustomAdditional = (options: {
    data: MenuData<any>;
    index: number;
}) => Record<string, any> | undefined;
export declare const menuItemProps: {
    readonly data: {
        readonly type: PropType<MenuItemProps<VKey>>;
        readonly required: true;
    };
    readonly index: {
        readonly type: NumberConstructor;
        readonly required: true;
    };
};
export declare const menuItemGroupProps: {
    readonly data: {
        readonly type: PropType<MenuItemGroupProps<VKey>>;
        readonly required: true;
    };
    readonly index: {
        readonly type: NumberConstructor;
        readonly required: true;
    };
};
export declare const menuSubProps: {
    readonly data: {
        readonly type: PropType<MenuSubProps<VKey>>;
        readonly required: true;
    };
    readonly index: {
        readonly type: NumberConstructor;
        readonly required: true;
    };
};
