import { defineComponent, inject, computed, createVNode, ref, onMounted, withDirectives, Transition, mergeProps, vShow, watch, onBeforeUnmount, toRef, provide, onDeactivated, isVNode, cloneVNode, Fragment, shallowRef } from "vue";
import { isFunction, isString, isBoolean } from "lodash-es";
import { CdkPortal } from "@idux/cdk/portal";
import { BlockScrollStrategy } from "@idux/cdk/scroll";
import { convertCssPixel, getOffset, callEmit, useControlledProp, isPromise, NoopFunction, convertArray, uniqueId, throwError } from "@idux/cdk/utils";
import { \u0275Mask as _Mask } from "@idux/components/_private/mask";
import { useGlobalConfig } from "@idux/components/config";
import { useThemeToken } from "@idux/components/theme";
import { usePortalTarget, useZIndex } from "@idux/components/utils";
import { CdkDndMovable } from "@idux/cdk/dnd";
import { \u0275Footer as _Footer } from "@idux/components/_private/footer";
import { \u0275Header as _Header } from "@idux/components/_private/header";
import { IxSpin } from "@idux/components/spin";
import { IxIcon } from "@idux/components/icon";
const modalToken = Symbol("modalToken");
const MODAL_TOKEN = Symbol("MODAL_TOKEN");
const MODAL_PROVIDER_TOKEN = Symbol("MODAL_PROVIDER_TOKEN");
const defaultIconTypes = {
  default: "",
  confirm: "exclamation-circle-filled",
  info: "info-circle-filled",
  success: "check-circle-filled",
  warning: "exclamation-circle-filled",
  error: "close-circle-filled"
};
var ModalBody = /* @__PURE__ */ defineComponent({
  setup() {
    const {
      props,
      slots,
      config,
      mergedPrefixCls,
      mergedSpin,
      mergedSpinWithFullModal
    } = inject(modalToken);
    const isDefault = computed(() => props.type === "default");
    const iconName = computed(() => {
      var _a, _b;
      const {
        icon,
        type
      } = props;
      return (_b = icon != null ? icon : (_a = config.icon) == null ? void 0 : _a[type]) != null ? _b : defaultIconTypes[type];
    });
    return () => {
      var _a, _b;
      const prefixCls = `${mergedPrefixCls.value}-body`;
      const defaultNode = (_b = (_a = slots.default) == null ? void 0 : _a.call(slots)) != null ? _b : props.__content_node;
      let bodyNode;
      if (isDefault.value) {
        bodyNode = createVNode("div", {
          "class": prefixCls
        }, [defaultNode]);
      } else {
        const classes = `${prefixCls} ${prefixCls}-${props.type}`;
        const iconNode = renderIcon(prefixCls, slots.icon, iconName.value);
        const titleNode = renderTitle(prefixCls, slots.title, props.title);
        bodyNode = createVNode("div", {
          "class": classes
        }, [iconNode, createVNode("div", {
          "class": [`${prefixCls}-content`, defaultNode ? "" : `${prefixCls}-content-only-title`]
        }, [titleNode, defaultNode])]);
      }
      const spinProps = mergedSpin.value;
      if (!mergedSpinWithFullModal.value && spinProps) {
        const spinSlots = {
          default: slots.spinContent,
          icon: slots.spinIcon
        };
        return createVNode(IxSpin, spinProps, {
          default: () => [bodyNode],
          ...spinSlots
        });
      }
      return bodyNode;
    };
  }
});
const renderIcon = (prefixCls, iconSlot, icon) => {
  if (!iconSlot && !icon) {
    return null;
  }
  let children;
  if (iconSlot) {
    children = iconSlot();
  } else if (isFunction(icon)) {
    children = icon();
  } else {
    children = isString(icon) ? createVNode(IxIcon, {
      "name": icon
    }, null) : icon;
  }
  return createVNode("div", {
    "class": `${prefixCls}-icon`
  }, [children]);
};
const renderTitle = (prefixCls, titleSlot, title) => {
  if (!titleSlot && !title) {
    return null;
  }
  let children;
  if (titleSlot) {
    children = titleSlot();
  } else if (isFunction(title)) {
    children = title();
  } else {
    children = title;
  }
  return createVNode("div", {
    "class": `${prefixCls}-title`
  }, [children]);
};
var ModalWrapper = /* @__PURE__ */ defineComponent({
  inheritAttrs: false,
  setup(_, {
    attrs
  }) {
    const {
      globalHashId,
      hashId
    } = useThemeToken("modal");
    const {
      props,
      slots,
      common,
      locale,
      config,
      mergedPrefixCls,
      visible,
      animatedVisible,
      mergedVisible,
      mergedSpin,
      mergedSpinWithFullModal,
      cancelLoading,
      okLoading,
      currentZIndex
    } = inject(modalToken);
    const {
      close,
      cancel,
      ok
    } = inject(MODAL_TOKEN);
    const {
      animatable,
      centered,
      closable,
      closeIcon,
      closeOnEsc,
      width,
      mask,
      maskClosable
    } = useConfig(props, config);
    const cancelVisible = computed(() => props.type === "default" || props.type === "confirm");
    const cancelText = computed(() => {
      var _a;
      return (_a = props.cancelText) != null ? _a : locale.modal.cancelText;
    });
    const okText = computed(() => {
      if (props.okText) {
        return props.okText;
      }
      return cancelVisible.value ? locale.modal.okText : locale.modal.justOkText;
    });
    const placementStyle = computed(() => {
      const top = centered.value ? 0 : convertCssPixel(props.offset);
      return {
        top,
        width: width.value
      };
    });
    const wrapperClasses = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      return {
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [`${prefixCls}-wrapper`]: true,
        [`${prefixCls}-centered`]: centered.value,
        [`${prefixCls}-draggable`]: !!props.draggable,
        [`${prefixCls}-with-mask`]: mask.value
      };
    });
    const wrapperStyle = computed(() => {
      return {
        zIndex: currentZIndex.value
      };
    });
    const modalTransformOrigin = ref();
    const contentStyle = computed(() => {
      return {
        transformOrigin: modalTransformOrigin.value,
        ...placementStyle.value
      };
    });
    const wrapperRef = ref();
    const modalRef = ref();
    const headerRef = ref();
    const sentinelStartRef = ref();
    const sentinelEndRef = ref();
    const movableRef = ref();
    const {
      onWrapperClick,
      onWrapperKeydown,
      onContentMousedown,
      onContentMouseup
    } = useEvent(props, close, mask, maskClosable, closeOnEsc, sentinelStartRef, sentinelEndRef);
    const {
      onEnter,
      onAfterEnter,
      onAfterLeave
    } = useEvents(props, wrapperRef, modalRef, animatedVisible, modalTransformOrigin);
    const handleOk = (evt) => {
      if (visible.value) {
        ok(evt);
      }
    };
    const handleCancel = (evt) => {
      if (visible.value) {
        cancel(evt);
      }
    };
    onMounted(() => watchVisibleChange(props, wrapperRef, sentinelStartRef, movableRef, mask));
    const renderContent = () => {
      const prefixCls = mergedPrefixCls.value;
      const okButton = {
        size: "md",
        ...props.okButton
      };
      const cancelButton = {
        size: "md",
        ...props.cancelButton
      };
      const headerSlots = {
        header: slots.header,
        closeIcon: slots.closeIcon
      };
      const spinSlots = {
        default: slots.spinContent,
        icon: slots.spinIcon
      };
      const spinProps = mergedSpin.value;
      const children = [createVNode(_Header, {
        "ref": headerRef,
        "class": `${prefixCls}-header`,
        "closable": closable.value,
        "closeIcon": closeIcon.value,
        "header": props.header,
        "size": props.header ? "md" : "sm",
        "onClose": close
      }, headerSlots), createVNode(ModalBody, null, null), createVNode(_Footer, {
        "class": `${prefixCls}-footer`,
        "cancel": handleCancel,
        "cancelButton": cancelButton,
        "cancelLoading": cancelLoading.value,
        "cancelText": cancelText.value,
        "cancelVisible": cancelVisible.value,
        "disabled": spinProps == null ? void 0 : spinProps.spinning,
        "footer": props.footer,
        "ok": handleOk,
        "okButton": okButton,
        "okLoading": okLoading.value,
        "okText": okText.value
      }, slots)];
      const contentNode = props.draggable ? createVNode(CdkDndMovable, {
        "ref": movableRef,
        "class": `${prefixCls}-content`,
        "dragHandle": headerRef.value,
        "boundary": wrapperRef.value,
        "mode": "immediate"
      }, {
        default: () => [children]
      }) : createVNode("div", {
        "class": `${prefixCls}-content`
      }, [children]);
      if (mergedSpinWithFullModal.value && spinProps) {
        return createVNode(IxSpin, spinProps, {
          default: () => [contentNode],
          ...spinSlots
        });
      }
      return contentNode;
    };
    return () => {
      const prefixCls = mergedPrefixCls.value;
      return withDirectives(createVNode("div", {
        "ref": wrapperRef,
        "class": wrapperClasses.value,
        "style": wrapperStyle.value,
        "tabindex": -1,
        "onClick": onWrapperClick,
        "onKeydown": onWrapperKeydown
      }, [createVNode(Transition, {
        "name": animatable.value ? `${common.prefixCls}-zoom` : void 0,
        "appear": true,
        "onEnter": onEnter,
        "onAfterEnter": onAfterEnter,
        "onAfterLeave": onAfterLeave
      }, {
        default: () => [withDirectives(createVNode("div", mergeProps({
          "ref": modalRef,
          "role": "document",
          "class": prefixCls,
          "style": contentStyle.value,
          "onMousedown": onContentMousedown,
          "onMouseup": onContentMouseup
        }, attrs), [createVNode("div", {
          "ref": sentinelStartRef,
          "tabindex": 0,
          "class": `${prefixCls}-sentinel`,
          "aria-hidden": true
        }, null), renderContent(), createVNode("div", {
          "ref": sentinelEndRef,
          "tabindex": 0,
          "class": `${prefixCls}-sentinel`,
          "aria-hidden": true
        }, null)]), [[vShow, visible.value]])]
      })]), [[vShow, mergedVisible.value]]);
    };
  }
});
function useConfig(props, config) {
  const animatable = computed(() => {
    var _a;
    return (_a = props.animatable) != null ? _a : config.animatable;
  });
  const centered = computed(() => {
    var _a;
    return (_a = props.centered) != null ? _a : config.centered;
  });
  const closable = computed(() => {
    var _a;
    return (_a = props.closable) != null ? _a : config.closable;
  });
  const closeIcon = computed(() => {
    var _a;
    return (_a = props.closeIcon) != null ? _a : config.closeIcon;
  });
  const closeOnEsc = computed(() => {
    var _a;
    return (_a = props.closeOnEsc) != null ? _a : config.closeOnEsc;
  });
  const mask = computed(() => {
    var _a;
    return (_a = props.mask) != null ? _a : config.mask;
  });
  const maskClosable = computed(() => {
    var _a;
    return (_a = props.maskClosable) != null ? _a : config.maskClosable;
  });
  const width = computed(() => {
    const {
      width: width2 = config.width
    } = props;
    if (width2) {
      return convertCssPixel(width2);
    } else {
      return props.type === "default" ? "480px" : "400px";
    }
  });
  return {
    animatable,
    centered,
    closable,
    closeIcon,
    closeOnEsc,
    width,
    mask,
    maskClosable
  };
}
function watchVisibleChange(props, wrapperRef, sentinelStartRef, movableRef, mask) {
  let lastOutSideActiveElement = null;
  watch(() => props.visible, (visible) => {
    var _a, _b, _c;
    if (visible) {
      const wrapperElement = wrapperRef.value;
      const activeElement = document.activeElement;
      if (wrapperElement && !wrapperElement.contains(activeElement)) {
        lastOutSideActiveElement = activeElement;
        (_a = sentinelStartRef.value) == null ? void 0 : _a.focus();
      }
      (_b = movableRef.value) == null ? void 0 : _b.init();
    } else {
      if (mask.value) {
        (_c = lastOutSideActiveElement == null ? void 0 : lastOutSideActiveElement.focus) == null ? void 0 : _c.call(lastOutSideActiveElement);
        lastOutSideActiveElement = null;
      }
    }
  }, {
    immediate: true
  });
}
function useEvent(props, close, mask, maskClosable, closeOnEsc, sentinelStartRef, sentinelEndRef) {
  let timeId;
  let mouseDown = false;
  const clearTimer = () => {
    if (timeId) {
      clearTimeout(timeId);
      timeId = void 0;
    }
  };
  const onWrapperClick = (evt) => {
    if (evt.target === evt.currentTarget && (!mouseDown || props.draggable) && mask.value && maskClosable.value) {
      close(evt);
    }
  };
  const onWrapperKeydown = (evt) => {
    if (closeOnEsc.value && evt.code === "Escape") {
      evt.stopPropagation();
      close(evt);
      return;
    }
    if (evt.code === "Tab") {
      const activeElement = document.activeElement;
      const sentinelStartElement = sentinelStartRef.value;
      const sentinelEndElement = sentinelEndRef.value;
      if (evt.shiftKey) {
        if (activeElement === sentinelStartElement) {
          sentinelEndElement == null ? void 0 : sentinelEndElement.focus();
        }
      } else if (activeElement === sentinelEndElement) {
        sentinelStartElement == null ? void 0 : sentinelStartElement.focus();
      }
    }
  };
  const onContentMousedown = () => {
    clearTimer();
    mouseDown = true;
  };
  const onContentMouseup = () => {
    if (mouseDown) {
      timeId = setTimeout(() => mouseDown = false);
    }
  };
  onBeforeUnmount(() => clearTimer());
  return {
    onWrapperClick,
    onWrapperKeydown,
    onContentMousedown,
    onContentMouseup
  };
}
function useEvents(props, wrapperRef, modalRef, animatedVisible, modalTransformOrigin) {
  let lastOutSideActiveElement = null;
  const onEnter = () => {
    const wrapperElement = wrapperRef.value;
    const activeElement = document.activeElement;
    if (wrapperElement && !wrapperElement.contains(activeElement)) {
      lastOutSideActiveElement = activeElement;
    }
    if (!modalTransformOrigin.value && lastOutSideActiveElement) {
      const modalElement = modalRef.value;
      const previouslyDOMRect = lastOutSideActiveElement.getBoundingClientRect();
      const lastPosition = getOffset(lastOutSideActiveElement);
      const x = lastPosition.left + previouslyDOMRect.width / 2;
      const y = lastPosition.top + previouslyDOMRect.height / 2;
      modalTransformOrigin.value = `${x - modalElement.offsetLeft}px ${y - modalElement.offsetTop}px`;
    }
  };
  const onAfterEnter = () => {
    const wrapperElement = wrapperRef.value;
    const activeElement = document.activeElement;
    if (wrapperElement && !wrapperElement.contains(activeElement)) {
      wrapperElement.focus();
    }
    callEmit(props.onAfterOpen);
    animatedVisible.value = true;
  };
  const onAfterLeave = () => {
    if (lastOutSideActiveElement && isFunction(lastOutSideActiveElement.focus)) {
      const wrapperElement = wrapperRef.value;
      const activeElement = document.activeElement;
      if (!activeElement || activeElement === document.body || activeElement === wrapperElement || wrapperElement && wrapperElement.contains(activeElement)) {
        lastOutSideActiveElement.focus();
      }
    }
    callEmit(props.onAfterClose);
    animatedVisible.value = false;
  };
  return {
    onEnter,
    onAfterEnter,
    onAfterLeave
  };
}
const modalProps = {
  visible: {
    type: Boolean,
    default: void 0
  },
  cancelButton: Object,
  cancelText: String,
  centered: {
    type: Boolean,
    default: void 0
  },
  closable: {
    type: Boolean,
    default: void 0
  },
  closeIcon: [String, Object, Function],
  closeOnDeactivated: {
    type: Boolean,
    default: true
  },
  closeOnEsc: {
    type: Boolean,
    default: void 0
  },
  container: {
    type: [String, HTMLElement, Function],
    default: void 0
  },
  destroyOnHide: {
    type: Boolean,
    default: false
  },
  draggable: { type: Boolean, default: false },
  footer: {
    type: [Boolean, Array, Object, Function],
    default: true
  },
  header: [String, Object],
  icon: [String, Object, Function],
  mask: {
    type: Boolean,
    default: void 0
  },
  maskClosable: {
    type: Boolean,
    default: void 0
  },
  animatable: {
    type: Boolean,
    default: void 0
  },
  offset: {
    type: [String, Number],
    default: 128
  },
  okButton: Object,
  okText: String,
  scrollStrategy: Object,
  spin: { type: [Boolean, Object], default: void 0 },
  spinWithFullModal: { type: Boolean, default: void 0 },
  title: [String, Object, Function],
  type: {
    type: String,
    default: "default"
  },
  width: [String, Number],
  zIndex: Number,
  "onUpdate:visible": [Function, Array],
  onAfterOpen: [Function, Array],
  onAfterClose: [Function, Array],
  onBeforeClose: [Function, Array],
  onClose: [Function, Array],
  onCancel: [Function, Array],
  onOk: [Function, Array],
  __content_node: [String, Object]
};
function getDefaultThemeTokens(tokens) {
  const { fontSizeMd, fontWeightMd, colorTextTitle, colorTextInfo } = tokens;
  return {
    titleFontSize: fontSizeMd,
    titleFontWeight: fontWeightMd,
    titleColor: colorTextTitle,
    titleWordBreak: "break-word",
    contentColor: colorTextInfo,
    iconSize: 40
  };
}
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens) : getDefaultThemeTokens(tokens);
};
var Modal = /* @__PURE__ */ defineComponent({
  name: "IxModal",
  inheritAttrs: false,
  props: modalProps,
  setup(props, {
    slots,
    expose,
    attrs
  }) {
    const common = useGlobalConfig("common");
    const {
      registerToken
    } = useThemeToken("modal");
    registerToken(getThemeTokens);
    const locale = useGlobalConfig("locale");
    const config = useGlobalConfig("modal");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-modal`);
    const mergedPortalTarget = usePortalTarget(props, config, common, mergedPrefixCls);
    const mergedSpin = computed(() => convertSpinProps(props.spin));
    const mergedSpinWithFullModal = computed(() => {
      var _a;
      return (_a = props.spinWithFullModal) != null ? _a : config.spinWithFullModal;
    });
    const mask = computed(() => {
      var _a;
      return (_a = props.mask) != null ? _a : config.mask;
    });
    const {
      visible,
      setVisible,
      animatedVisible,
      mergedVisible
    } = useVisible(props);
    const currentZIndex = useZIndex(toRef(props, "zIndex"), toRef(common, "overlayZIndex"), visible);
    const {
      cancelLoading,
      okLoading,
      open,
      close,
      cancel,
      ok
    } = useTrigger(props, setVisible);
    provide(modalToken, {
      props,
      slots,
      common,
      locale,
      config,
      mergedPrefixCls,
      mergedSpin,
      mergedSpinWithFullModal,
      visible,
      animatedVisible,
      mergedVisible,
      cancelLoading,
      okLoading,
      currentZIndex
    });
    const apis = {
      open,
      close,
      cancel,
      ok
    };
    provide(MODAL_TOKEN, apis);
    expose(apis);
    useScrollStrategy(props, mask, mergedVisible);
    return () => {
      if (!mergedVisible.value && props.destroyOnHide) {
        return null;
      }
      return createVNode(CdkPortal, {
        "target": mergedPortalTarget.value,
        "load": visible.value
      }, {
        default: () => [createVNode(_Mask, {
          "class": `${mergedPrefixCls.value}-mask`,
          "mask": mask.value,
          "visible": visible.value,
          "zIndex": currentZIndex.value
        }, null), createVNode(ModalWrapper, attrs, null)]
      });
    };
  }
});
function useVisible(props) {
  const [visible, setVisible] = useControlledProp(props, "visible", false);
  const animatedVisible = ref();
  const mergedVisible = computed(() => {
    const currVisible = visible.value;
    const currAnimatedVisible = animatedVisible.value;
    if (currAnimatedVisible === void 0 || currVisible) {
      return currVisible;
    }
    return currAnimatedVisible;
  });
  onDeactivated(() => {
    if (mergedVisible.value && props.closeOnDeactivated) {
      setVisible(false);
    }
  });
  return {
    visible,
    setVisible,
    animatedVisible,
    mergedVisible
  };
}
function useScrollStrategy(props, mask, mergedVisible) {
  let scrollStrategy;
  onMounted(() => {
    watch([mask, mergedVisible], ([maskValue, visible]) => {
      var _a;
      if (!maskValue || !visible) {
        scrollStrategy == null ? void 0 : scrollStrategy.disable();
        return;
      }
      if (!scrollStrategy) {
        scrollStrategy = (_a = props.scrollStrategy) != null ? _a : new BlockScrollStrategy();
      }
      scrollStrategy.enable();
    }, {
      immediate: true
    });
  });
  onBeforeUnmount(() => scrollStrategy == null ? void 0 : scrollStrategy.disable());
}
function useTrigger(props, setVisible) {
  const open = () => setVisible(true);
  const hide = (result) => {
    result !== false && setVisible(false);
  };
  const close = async (evt) => {
    const result = await callEmit(props.onBeforeClose, evt);
    hide(result);
    result !== false && callEmit(props.onClose, evt);
  };
  const cancelLoading = ref(false);
  const cancel = async (evt) => {
    let result = callEmit(props.onCancel, evt);
    if (isPromise(result)) {
      cancelLoading.value = true;
      result = await result;
      hide(result);
      cancelLoading.value = false;
      return;
    }
    hide(result);
  };
  const okLoading = ref(false);
  const ok = async (evt) => {
    let result = callEmit(props.onOk, evt);
    if (isPromise(result)) {
      okLoading.value = true;
      result = await result;
      hide(result);
      okLoading.value = false;
      return;
    }
    hide(result);
  };
  return {
    cancelLoading,
    okLoading,
    open,
    close,
    cancel,
    ok
  };
}
function convertSpinProps(spin) {
  return isBoolean(spin) ? {
    spinning: spin
  } : spin;
}
var ModalProvider = /* @__PURE__ */ defineComponent({
  name: "IxModalProvider",
  setup(_, {
    expose,
    slots
  }) {
    const {
      modals,
      setModalRef,
      open,
      confirm,
      info,
      success,
      warning,
      error,
      update,
      destroy,
      destroyAll
    } = useModal$1();
    const apis = {
      open,
      confirm,
      info,
      success,
      warning,
      error,
      update,
      destroy,
      destroyAll
    };
    provide(MODAL_PROVIDER_TOKEN, apis);
    expose(apis);
    return () => {
      var _a;
      const children = modals.value.map((item) => {
        const {
          key,
          visible = true,
          destroyOnHide,
          onDestroy,
          content,
          contentProps,
          ...restProps
        } = item;
        const setRef = (instance) => setModalRef(key, instance);
        const onUpdateVisible = (visible2) => update(key, {
          visible: visible2
        });
        const onAfterClose = destroyOnHide ? () => destroy(key) : void 0;
        const mergedProps = {
          key,
          visible,
          ref: setRef,
          "onUpdate:visible": onUpdateVisible,
          onAfterClose
        };
        const contentNode = isFunction(content) ? content() : isVNode(content) ? cloneVNode(content, contentProps, true) : content;
        return createVNode(Modal, mergeProps(mergedProps, restProps, {
          "__content_node": contentNode
        }), null);
      });
      return createVNode(Fragment, null, [(_a = slots.default) == null ? void 0 : _a.call(slots), children]);
    };
  }
});
function useModal$1() {
  const modals = shallowRef([]);
  const modalRefMap = /* @__PURE__ */ new Map();
  const setModalRef = (key, instance) => {
    const ref2 = modalRefMap.get(key);
    if (instance) {
      if (ref2 && !ref2.open) {
        ref2.open = instance.open;
        ref2.close = instance.close;
        ref2.cancel = instance.cancel;
        ref2.ok = instance.ok;
      }
    } else {
      if (ref2) {
        modalRefMap.delete(key);
        ref2.open = NoopFunction;
        ref2.close = NoopFunction;
        ref2.cancel = NoopFunction;
        ref2.ok = NoopFunction;
      }
    }
  };
  const getCurrIndex = (key) => {
    return modals.value.findIndex((message) => message.key === key);
  };
  const add = (item) => {
    var _a;
    const currIndex = item.key ? getCurrIndex(item.key) : -1;
    const tempModals = [...modals.value];
    if (currIndex !== -1) {
      tempModals.splice(currIndex, 1, item);
      modals.value = tempModals;
      return item.key;
    }
    item.key = (_a = item.key) != null ? _a : uniqueId("ix-modal");
    tempModals.push(item);
    modals.value = tempModals;
    return item.key;
  };
  const update = (key, item) => {
    const currIndex = getCurrIndex(key);
    if (currIndex !== -1) {
      const tempModals = [...modals.value];
      const newItem = {
        ...modals.value[currIndex],
        ...item
      };
      tempModals.splice(currIndex, 1, newItem);
      modals.value = tempModals;
    }
  };
  const destroy = (key) => {
    const keys = convertArray(key);
    keys.forEach((key2) => {
      const currIndex = getCurrIndex(key2);
      if (currIndex !== -1) {
        const tempModals = [...modals.value];
        const item = tempModals.splice(currIndex, 1);
        modals.value = tempModals;
        callEmit(item[0].onDestroy, key2);
      }
    });
  };
  const destroyAll = () => {
    modals.value = [];
  };
  const open = (options) => {
    const key = add(options);
    const modalRef = {
      key,
      update: (options2) => update(key, options2),
      destroy: () => destroy(key)
    };
    modalRefMap.set(key, modalRef);
    return modalRef;
  };
  const modalTypes = ["confirm", "info", "success", "warning", "error"];
  const [confirm, info, success, warning, error] = modalTypes.map((type) => {
    return (options) => open({
      ...options,
      type
    });
  });
  return {
    modals,
    setModalRef,
    open,
    confirm,
    info,
    success,
    warning,
    error,
    update,
    destroy,
    destroyAll
  };
}
const useModal = () => {
  const modalProviderRef = inject(MODAL_PROVIDER_TOKEN, null);
  if (modalProviderRef === null) {
    return throwError("components/modal", "<IxModalProvider> not found.");
  }
  return modalProviderRef;
};
const IxModal = Modal;
const IxModalProvider = ModalProvider;
export { IxModal, IxModalProvider, MODAL_PROVIDER_TOKEN, MODAL_TOKEN, getThemeTokens as getModalThemeTokens, useModal };
