/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { PortalTargetType } from '@idux/cdk/portal';
import type { ScrollStrategy } from '@idux/cdk/scroll';
import type { ExtractInnerPropTypes, ExtractPublicPropTypes, MaybeArray, VKey } from '@idux/cdk/utils';
import type { ɵFooterButtonProps } from '@idux/components/_private/footer';
import type { ButtonProps } from '@idux/components/button';
import type { HeaderProps } from '@idux/components/header';
import type { SpinProps } from '@idux/components/spin';
import type { DefineComponent, HTMLAttributes, PropType, VNode, VNodeChild, VNodeProps } from 'vue';
export type ModalType = 'default' | 'confirm' | 'info' | 'success' | 'warning' | 'error';
export type ModalButtonProps = ɵFooterButtonProps;
export interface ModalOptions<K = VKey> extends ModalPublicProps {
    key?: K;
    content?: string | VNode | (() => VNodeChild);
    contentProps?: Record<string, unknown> | VNodeProps;
    onDestroy?: (key: K) => void;
}
export interface ModalRef<K = VKey> extends ModalBindings {
    key: K;
    update: (options: ModalOptions<K>) => void;
    destroy: () => void;
}
export declare const modalProps: {
    readonly visible: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly cancelButton: PropType<ButtonProps>;
    readonly cancelText: StringConstructor;
    readonly centered: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly closable: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly closeIcon: PropType<string | VNode<import("vue").RendererNode, import("vue").RendererElement, {
        [key: string]: any;
    }> | (() => VNodeChild)>;
    readonly closeOnDeactivated: {
        readonly type: BooleanConstructor;
        readonly default: true;
    };
    readonly closeOnEsc: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly container: {
        readonly type: PropType<PortalTargetType>;
        readonly default: undefined;
    };
    readonly destroyOnHide: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly draggable: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly footer: {
        readonly type: PropType<boolean | VNode<import("vue").RendererNode, import("vue").RendererElement, {
            [key: string]: any;
        }> | ɵFooterButtonProps[] | (() => VNodeChild)>;
        readonly default: true;
    };
    readonly header: PropType<string | HeaderProps>;
    readonly icon: PropType<string | VNode<import("vue").RendererNode, import("vue").RendererElement, {
        [key: string]: any;
    }> | (() => VNodeChild)>;
    readonly mask: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly maskClosable: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly animatable: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly offset: {
        readonly type: PropType<string | number>;
        readonly default: 128;
    };
    readonly okButton: PropType<ButtonProps>;
    readonly okText: StringConstructor;
    readonly scrollStrategy: PropType<ScrollStrategy>;
    readonly spin: {
        readonly type: PropType<boolean | SpinProps>;
        readonly default: undefined;
    };
    readonly spinWithFullModal: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly title: PropType<string | VNode<import("vue").RendererNode, import("vue").RendererElement, {
        [key: string]: any;
    }> | (() => VNodeChild)>;
    readonly type: {
        readonly type: PropType<ModalType>;
        readonly default: "default";
    };
    readonly width: PropType<string | number>;
    readonly zIndex: NumberConstructor;
    readonly 'onUpdate:visible': PropType<MaybeArray<(visible: boolean) => void>>;
    readonly onAfterOpen: PropType<MaybeArray<() => void>>;
    readonly onAfterClose: PropType<MaybeArray<() => void>>;
    readonly onBeforeClose: PropType<MaybeArray<(evt?: Event | unknown) => void | boolean | Promise<boolean>>>;
    readonly onClose: PropType<MaybeArray<(evt?: Event | unknown) => void>>;
    readonly onCancel: PropType<MaybeArray<(evt?: Event | unknown) => unknown>>;
    readonly onOk: PropType<MaybeArray<(evt?: Event | unknown) => unknown>>;
    readonly __content_node: PropType<VNodeChild>;
};
export type ModalProps = ExtractInnerPropTypes<typeof modalProps>;
export type ModalPublicProps = Omit<ExtractPublicPropTypes<typeof modalProps>, '__content_node'>;
export interface ModalBindings {
    open: () => void;
    close: (evt?: Event | unknown) => Promise<void>;
    cancel: (evt?: Event | unknown) => Promise<void>;
    ok: (evt?: Event | unknown) => Promise<void>;
}
export type ModalComponent = DefineComponent<Omit<HTMLAttributes, keyof ModalPublicProps> & ModalPublicProps, ModalBindings>;
export type ModalInstance = InstanceType<DefineComponent<ModalProps, ModalBindings>>;
export interface ModalProviderRef<K = VKey> {
    open: (options: ModalOptions<K>) => ModalRef<K>;
    confirm: (options: Omit<ModalOptions<K>, 'type'>) => ModalRef<K>;
    info: (options: Omit<ModalOptions<K>, 'type'>) => ModalRef<K>;
    success: (options: Omit<ModalOptions<K>, 'type'>) => ModalRef<K>;
    warning: (options: Omit<ModalOptions<K>, 'type'>) => ModalRef<K>;
    error: (options: Omit<ModalOptions<K>, 'type'>) => ModalRef<K>;
    update: (key: K, options: ModalOptions<K>) => void;
    destroy: (key: K | K[]) => void;
    destroyAll: () => void;
}
export type ModalProviderComponent = DefineComponent<HTMLAttributes, ModalProviderRef>;
export type ModalProviderInstance = InstanceType<DefineComponent<HTMLAttributes, ModalProviderRef>>;
