/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { PortalTargetType } from '@idux/cdk/portal';
import type { ExtractInnerPropTypes, ExtractPublicPropTypes, MaybeArray, VKey } from '@idux/cdk/utils';
import type { ButtonProps } from '@idux/components/button';
import type { DefineComponent, HTMLAttributes, PropType, VNode, VNodeChild, VNodeProps } from 'vue';
export interface NotificationButtonProps<K = VKey> extends ButtonProps {
    key?: K;
    text?: string | VNode | (() => VNodeChild);
    onClick?: (evt: Event) => void;
}
export interface SlotProps {
    visible?: boolean;
    close?: () => void;
}
export declare const notificationType: readonly ["info", "success", "warning", "error"];
export declare const notificationPlacement: readonly ["topStart", "topEnd", "bottomStart", "bottomEnd"];
export declare const notificationProps: {
    readonly visible: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly destroyOnHover: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly duration: NumberConstructor;
    readonly icon: PropType<string | VNode<import("vue").RendererNode, import("vue").RendererElement, {
        [key: string]: any;
    }> | (() => VNodeChild)>;
    readonly closeIcon: PropType<string | VNode<import("vue").RendererNode, import("vue").RendererElement, {
        [key: string]: any;
    }> | (() => VNodeChild)>;
    readonly type: PropType<"info" | "error" | "success" | "warning">;
    readonly title: PropType<string | VNode<import("vue").RendererNode, import("vue").RendererElement, {
        [key: string]: any;
    }> | (() => VNodeChild)>;
    readonly content: PropType<string | VNode<import("vue").RendererNode, import("vue").RendererElement, {
        [key: string]: any;
    }> | (() => VNodeChild)>;
    readonly footer: PropType<string | VNode<import("vue").RendererNode, import("vue").RendererElement, {
        [key: string]: any;
    }> | NotificationButtonProps<VKey>[] | ((options: SlotProps) => VNodeChild)>;
    readonly placement: PropType<"topStart" | "topEnd" | "bottomStart" | "bottomEnd">;
    readonly 'onUpdate:visible': PropType<MaybeArray<(visible: boolean) => void>>;
    readonly onClose: PropType<MaybeArray<(evt?: Event) => void>>;
};
export declare const notificationProviderProps: {
    readonly container: {
        readonly type: PropType<PortalTargetType>;
        readonly default: undefined;
    };
    readonly maxCount: NumberConstructor;
    readonly offset: PropType<string | number | (string | number)[]>;
};
export type NotificationType = (typeof notificationType)[number];
export type NotificationPlacement = (typeof notificationPlacement)[number];
export type NotificationNodePropKey = 'title' | 'content' | 'footer';
export type NotificationProps = ExtractInnerPropTypes<typeof notificationProps>;
export type NotificationPublicProps = ExtractPublicPropTypes<typeof notificationProps>;
export type NotificationComponent = DefineComponent<Omit<HTMLAttributes, keyof NotificationPublicProps> & NotificationPublicProps>;
export type NotificationInstance = InstanceType<DefineComponent<NotificationProps>>;
export interface NotificationOptions<K = VKey> extends NotificationProps {
    key?: K;
    contentProps?: Record<string, unknown> | VNodeProps;
    onDestroy?: (key: K) => void;
}
export type NotificationPlacementMap = Record<NotificationPlacement, NotificationOptions[]>;
export interface NotificationProviderRef<K = VKey> {
    open: (options: NotificationOptions<K>) => NotificationRef<K>;
    info: (options: Omit<NotificationOptions<K>, 'type'>) => NotificationRef<K>;
    success: (options: Omit<NotificationOptions<K>, 'type'>) => NotificationRef<K>;
    warning: (options: Omit<NotificationOptions<K>, 'type'>) => NotificationRef<K>;
    error: (options: Omit<NotificationOptions<K>, 'type'>) => NotificationRef<K>;
    update: (key: K, options: Partial<Omit<NotificationOptions<K>, 'key'>>) => void;
    destroy: (key: K | K[]) => void;
    destroyAll: () => void;
}
export interface NotificationRef<K = VKey> {
    key: K;
    update: (options: Partial<NotificationOptions<K>>) => void;
    destroy: () => void;
}
export type NotificationProviderProps = ExtractInnerPropTypes<typeof notificationProviderProps>;
export type NotificationProviderPublicProps = ExtractPublicPropTypes<typeof notificationProviderProps>;
export type NotificationProviderComponent = DefineComponent<Omit<HTMLAttributes, keyof NotificationProviderPublicProps> & NotificationProviderPublicProps>;
export type NotificationProviderInstance = InstanceType<DefineComponent<NotificationProviderProps, NotificationProviderRef>>;
