/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { ExtractInnerPropTypes, ExtractPublicPropTypes, MaybeArray } from '@idux/cdk/utils';
import type { DefineComponent, HTMLAttributes, PropType, VNode } from 'vue';
export type PaginationSize = 'sm' | 'md' | 'lg';
export type PaginationItemType = 'page' | 'prev' | 'next' | 'prev5' | 'next5';
export interface PaginationItemRenderOptions {
    index?: number;
    type: PaginationItemType;
    active: boolean;
    disabled: boolean;
    original: VNode;
}
export declare const paginationProps: {
    readonly pageIndex: NumberConstructor;
    readonly pageSize: NumberConstructor;
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly pageSizes: PropType<number[]>;
    readonly showQuickJumper: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly showSizeChanger: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly showTitle: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly showTotal: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly simple: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly size: PropType<PaginationSize>;
    readonly total: {
        readonly type: NumberConstructor;
        readonly default: 0;
    };
    readonly 'onUpdate:pageIndex': PropType<MaybeArray<(pageIndex: number) => void>>;
    readonly 'onUpdate:pageSize': PropType<MaybeArray<(pageSize: number) => void>>;
    readonly onChange: PropType<MaybeArray<(pageIndex: number, pageSize: number) => void>>;
};
export type PaginationProps = ExtractInnerPropTypes<typeof paginationProps>;
export type PaginationPublicProps = ExtractPublicPropTypes<typeof paginationProps>;
export type PaginationComponent = DefineComponent<Omit<HTMLAttributes, keyof PaginationPublicProps> & PaginationPublicProps>;
export type PaginationInstance = InstanceType<DefineComponent<PaginationProps>>;
export declare const paginationItemProps: {
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly index: NumberConstructor;
    readonly type: {
        readonly type: PropType<PaginationItemType>;
        readonly required: true;
    };
};
export type PaginationItemProps = ExtractInnerPropTypes<typeof paginationItemProps>;
