import { defineComponent, inject, computed, createVNode, normalizeClass, provide, mergeProps, ref } from "vue";
import { callEmit, isPromise } from "@idux/cdk/utils";
import { \u0275Overlay as _Overlay } from "@idux/components/_private/overlay";
import { useGlobalConfig } from "@idux/components/config";
import { useThemeToken } from "@idux/components/theme";
import { \u0275TooltipProps as _TooltipProps, \u0275UseTooltipOverlay as _UseTooltipOverlay } from "@idux/components/tooltip";
import { \u0275Footer as _Footer } from "@idux/components/_private/footer";
import { IxIcon } from "@idux/components/icon";
const popconfirmToken = Symbol("popconfirmToken");
var PopconfirmContent = /* @__PURE__ */ defineComponent({
  name: "IxPopconfirmContent",
  setup(_, {
    slots
  }) {
    const {
      props,
      locale,
      mergedPrefixCls,
      cancelLoading,
      okLoading,
      cancel,
      ok
    } = inject(popconfirmToken);
    const cancelText = computed(() => {
      var _a;
      return (_a = props.cancelText) != null ? _a : locale.popconfirm.cancelText;
    });
    const okText = computed(() => {
      var _a;
      return (_a = props.okText) != null ? _a : locale.popconfirm.okText;
    });
    const cancelButton = computed(() => {
      return {
        size: "xs",
        ...props.cancelButton
      };
    });
    const okButton = computed(() => {
      return {
        size: "xs",
        ...props.okButton
      };
    });
    return () => {
      const prefixCls = mergedPrefixCls.value;
      const iconNode = slots.icon ? slots.icon() : props.icon ? createVNode(IxIcon, {
        "name": props.icon
      }, null) : void 0;
      const titleNode = slots.title ? slots.title() : props.title;
      const contentNode = slots.content ? slots.content() : props.content;
      const classes = normalizeClass({
        [`${prefixCls}-wrapper`]: true,
        [`${prefixCls}-with-icon`]: !!iconNode,
        [`${prefixCls}-with-content`]: !!contentNode
      });
      return createVNode("div", {
        "class": classes
      }, [(iconNode || titleNode) && createVNode("div", {
        "class": `${prefixCls}-title`
      }, [iconNode, createVNode("span", null, [titleNode])]), contentNode && createVNode("div", {
        "class": `${prefixCls}-content`
      }, [contentNode]), createVNode(_Footer, {
        "class": `${prefixCls}-footer`,
        "cancel": cancel,
        "cancelButton": cancelButton.value,
        "cancelLoading": cancelLoading.value,
        "cancelText": cancelText.value,
        "footer": props.footer,
        "ok": ok,
        "okButton": okButton.value,
        "okLoading": okLoading.value,
        "okText": okText.value
      }, slots)]);
    };
  }
});
const popconfirmProps = {
  ..._TooltipProps,
  cancelButton: Object,
  cancelText: String,
  okButton: Object,
  okText: String,
  content: String,
  icon: {
    type: String,
    default: "exclamation-circle-filled"
  },
  footer: {
    type: [Boolean, Array, Object],
    default: true
  },
  "onUpdate:visible": [Function, Array],
  onCancel: [Function, Array],
  onOk: [Function, Array]
};
function getDefaultThemeTokens(tokens) {
  const { fontSizeSm, fontSizeHeaderSm, colorText, colorWarningBg } = tokens;
  return {
    fontSize: fontSizeSm,
    titleFontSize: fontSizeHeaderSm,
    color: colorText,
    iconColor: colorWarningBg,
    minWidth: 240
  };
}
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens) : getDefaultThemeTokens(tokens);
};
var Popconfirm = /* @__PURE__ */ defineComponent({
  name: "IxPopconfirm",
  props: popconfirmProps,
  setup(props, {
    slots,
    expose
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("popconfirm");
    registerToken(getThemeTokens);
    const locale = useGlobalConfig("locale");
    const config = useGlobalConfig("popconfirm");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-popconfirm`);
    const {
      overlayRef,
      updatePopper,
      visible,
      setVisible,
      overlayProps
    } = _UseTooltipOverlay(props, config, mergedPrefixCls);
    const {
      cancelLoading,
      okLoading,
      cancel,
      ok
    } = useTrigger(props, setVisible);
    provide(popconfirmToken, {
      props,
      locale,
      mergedPrefixCls,
      visible,
      cancelLoading,
      okLoading,
      cancel,
      ok
    });
    expose({
      updatePopper,
      cancel,
      ok
    });
    return () => {
      const prefixCls = mergedPrefixCls.value;
      const {
        default: defaultSlot,
        ...restSlots
      } = slots;
      return createVNode(_Overlay, mergeProps({
        "ref": overlayRef,
        "class": [prefixCls, globalHashId.value, hashId.value],
        "transitionName": `${common.prefixCls}-fade`
      }, overlayProps.value), {
        default: defaultSlot,
        content: () => createVNode(PopconfirmContent, null, restSlots)
      });
    };
  }
});
function useTrigger(props, setVisible) {
  const cancelLoading = ref(false);
  const cancel = async (evt) => {
    let result = callEmit(props.onCancel, evt);
    if (isPromise(result)) {
      cancelLoading.value = true;
      result = await result;
      cancelLoading.value = false;
    }
    if (result === false) {
      return;
    }
    setVisible(false);
  };
  const okLoading = ref(false);
  const ok = async (evt) => {
    let result = callEmit(props.onOk, evt);
    if (isPromise(result)) {
      okLoading.value = true;
      result = await result;
      okLoading.value = false;
    }
    if (result === false) {
      return;
    }
    setVisible(false);
  };
  return {
    cancelLoading,
    okLoading,
    cancel,
    ok
  };
}
const IxPopconfirm = Popconfirm;
export { IxPopconfirm, getThemeTokens as getPopconfirmThemeTokens };
