/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { ExtractInnerPropTypes, ExtractPublicPropTypes, MaybeArray, VKey } from '@idux/cdk/utils';
import type { ButtonProps } from '@idux/components/button';
import type { DefineComponent, HTMLAttributes, PropType, VNode } from 'vue';
export declare const popconfirmProps: {
    readonly cancelButton: PropType<ButtonProps>;
    readonly cancelText: StringConstructor;
    readonly okButton: PropType<ButtonProps>;
    readonly okText: StringConstructor;
    readonly content: StringConstructor;
    readonly icon: {
        readonly type: StringConstructor;
        readonly default: "exclamation-circle-filled";
    };
    readonly footer: {
        readonly type: PropType<boolean | VNode<import("vue").RendererNode, import("vue").RendererElement, {
            [key: string]: any;
        }> | PopconfirmButtonProps<VKey>[]>;
        readonly default: true;
    };
    readonly 'onUpdate:visible': PropType<MaybeArray<(visible: boolean) => void>>;
    readonly onCancel: PropType<MaybeArray<(evt?: Event | unknown) => unknown>>;
    readonly onOk: PropType<MaybeArray<(evt?: Event | unknown) => unknown>>;
    readonly visible: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly autoAdjust: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly closeOnDeactivated: {
        readonly type: BooleanConstructor;
        readonly default: true;
    };
    readonly destroyOnHide: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly delay: PropType<number | [number | null, number | null]>;
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly offset: PropType<[number, number]>;
    readonly overlayContainer: {
        readonly type: PropType<import("../..").OverlayContainerType>;
        readonly default: undefined;
    };
    readonly placement: PropType<import("../../../cdk").PopperPlacement>;
    readonly showArrow: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly title: StringConstructor;
    readonly trigger: PropType<import("../../../cdk").PopperTrigger>;
    readonly zIndex: NumberConstructor;
};
export interface PopconfirmButtonProps<K = VKey> extends ButtonProps {
    key?: K;
    text?: string | VNode;
    onClick?: (evt: Event) => void;
}
export type PopconfirmProps = ExtractInnerPropTypes<typeof popconfirmProps>;
export type PopconfirmPublicProps = ExtractPublicPropTypes<typeof popconfirmProps>;
export interface PopconfirmBindings {
    updatePopper: () => void;
    cancel: (evt?: Event | unknown) => Promise<void>;
    ok: (evt?: Event | unknown) => Promise<void>;
}
export type PopconfirmComponent = DefineComponent<Omit<HTMLAttributes, keyof PopconfirmPublicProps> & PopconfirmPublicProps, PopconfirmBindings>;
export type PopconfirmInstance = InstanceType<DefineComponent<PopconfirmProps, PopconfirmBindings>>;
