import { defineComponent, computed, createVNode, mergeProps } from "vue";
import { \u0275Header as _Header } from "@idux/components/_private/header";
import { \u0275Overlay as _Overlay } from "@idux/components/_private/overlay";
import { useGlobalConfig } from "@idux/components/config";
import { useThemeToken } from "@idux/components/theme";
import { \u0275TooltipProps as _TooltipProps, \u0275UseTooltipOverlay as _UseTooltipOverlay } from "@idux/components/tooltip";
const popoverProps = {
  ..._TooltipProps,
  closable: {
    type: Boolean,
    default: false
  },
  closeIcon: [String, Object],
  header: [String, Object],
  content: String,
  showArrow: { type: Boolean, default: void 0 }
};
function getDefaultThemeTokens(tokens) {
  const { fontSizeSm, colorTextInfo } = tokens;
  return {
    fontSize: fontSizeSm,
    color: colorTextInfo,
    minWidth: 240
  };
}
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens) : getDefaultThemeTokens(tokens);
};
var Popover = /* @__PURE__ */ defineComponent({
  name: "IxPopover",
  props: popoverProps,
  setup(props, {
    slots,
    expose
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("popover");
    registerToken(getThemeTokens);
    const config = useGlobalConfig("popover");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-popover`);
    const {
      overlayRef,
      updatePopper,
      overlayProps,
      setVisible
    } = _UseTooltipOverlay(props, config, mergedPrefixCls);
    expose({
      updatePopper
    });
    const closeIcon = computed(() => {
      var _a;
      return (_a = props.closeIcon) != null ? _a : config.closeIcon;
    });
    return () => {
      const prefixCls = mergedPrefixCls.value;
      return createVNode(_Overlay, mergeProps({
        "ref": overlayRef,
        "class": [prefixCls, globalHashId.value, hashId.value]
      }, overlayProps.value, {
        "transitionName": `${common.prefixCls}-fade`
      }), {
        default: slots.default,
        content: () => renderContent(props, slots, closeIcon, setVisible, prefixCls)
      });
    };
  }
});
const renderContent = (props, slots, closeIcon, setVisibility, prefixCls) => {
  var _a;
  if (!(slots.header || props.header || slots.content || props.content)) {
    return null;
  }
  const children = [];
  if (slots.header || props.header) {
    children.push(createVNode(_Header, {
      "size": "sm",
      "closable": props.closable,
      "closeIcon": closeIcon.value,
      "header": (_a = props.header) != null ? _a : props.title,
      "onClose": () => setVisibility(false)
    }, {
      header: slots.header
    }));
  }
  if (slots.content || props.content) {
    children.push(createVNode("div", {
      "class": `${prefixCls}-content`
    }, [slots.content ? slots.content() : props.content]));
  }
  return createVNode("div", {
    "class": `${prefixCls}-wrapper`
  }, [children]);
};
const IxPopover = Popover;
export { IxPopover, getThemeTokens as getPopoverThemeTokens };
