/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { ExtractInnerPropTypes, ExtractPublicPropTypes } from '@idux/cdk/utils';
import type { DefineComponent, HTMLAttributes, PropType, VNode, VNodeChild } from 'vue';
export type ProgressSize = 'sm' | 'md' | 'lg';
export type ProgressFormat = (percent: number, successPercent?: number) => VNodeChild;
export type ProgressType = 'line' | 'circle' | 'dashboard';
export type ProgressGapPositionType = 'top' | 'bottom' | 'left' | 'right';
export type ProgressStatus = 'normal' | 'success' | 'exception' | 'active';
export type ProgressStrokeLinecap = 'round' | 'square';
export type StringGradients = {
    [percentage: string]: string;
};
export type FromToGradients = {
    from: string;
    to: string;
};
export type ProgressGradient = {
    direction?: string;
} & (StringGradients | FromToGradients);
export interface ProgressSuccess {
    percent?: number;
    strokeColor?: string;
}
export interface ConvertProgressSuccess extends ProgressSuccess {
    percent: number;
}
export interface ProgressIcons {
    success: string | VNode;
    exception: string | VNode;
}
export declare const progressStatus: readonly ["normal", "success", "exception", "active"];
export declare const progressProps: {
    readonly type: {
        readonly type: PropType<ProgressType>;
        readonly default: "line";
    };
    readonly format: PropType<ProgressFormat>;
    readonly percent: {
        readonly type: NumberConstructor;
        readonly default: 0;
    };
    readonly status: PropType<ProgressStatus>;
    readonly hideInfo: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly success: PropType<ProgressSuccess>;
    readonly trailColor: StringConstructor;
    readonly strokeColor: PropType<string | ProgressGradient>;
    readonly strokeLinecap: PropType<ProgressStrokeLinecap>;
    readonly strokeWidth: NumberConstructor;
    readonly gapDegree: NumberConstructor;
    readonly gapPosition: PropType<ProgressGapPositionType>;
    readonly width: {
        readonly type: NumberConstructor;
        readonly default: 132;
    };
    readonly size: PropType<ProgressSize>;
    readonly icons: PropType<ProgressIcons>;
};
export type ProgressProps = ExtractInnerPropTypes<typeof progressProps>;
export type ProgressPublicProps = ExtractPublicPropTypes<typeof progressProps>;
export type ProgressComponent = DefineComponent<Omit<HTMLAttributes, keyof ProgressPublicProps> & ProgressPublicProps>;
export type ProgressInstance = InstanceType<DefineComponent<ProgressProps>>;
