/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { AbstractControl } from '@idux/cdk/forms';
import type { ExtractInnerPropTypes, ExtractPublicPropTypes, MaybeArray, VKey } from '@idux/cdk/utils';
import type { FormSize } from '@idux/components/form';
import type { SpaceProps } from '@idux/components/space';
import type { DefineComponent, HTMLAttributes, LabelHTMLAttributes, PropType } from 'vue';
export declare const radioProps: {
    readonly control: {
        readonly type: PropType<string | number | (string | number)[] | AbstractControl<any>>;
        readonly default: undefined;
    };
    readonly checked: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly autofocus: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly buttoned: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly label: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly mode: {
        readonly type: PropType<RadioMode>;
        readonly default: undefined;
    };
    readonly size: {
        readonly type: PropType<FormSize>;
        readonly default: undefined;
    };
    readonly value: {
        readonly type: null;
        readonly default: undefined;
    };
    readonly waveless: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly 'onUpdate:checked': {
        readonly type: PropType<MaybeArray<(checked: boolean) => void>>;
    };
    readonly onChange: {
        readonly type: PropType<MaybeArray<(checked: boolean, oldChecked: boolean) => void>>;
    };
    readonly onBlur: {
        readonly type: PropType<MaybeArray<(evt: FocusEvent) => void>>;
    };
    readonly onFocus: {
        readonly type: PropType<MaybeArray<(evt: FocusEvent) => void>>;
    };
};
export type RadioProps = ExtractInnerPropTypes<typeof radioProps>;
export type RadioPublicProps = ExtractPublicPropTypes<typeof radioProps>;
export interface RadioBindings {
    blur: () => void;
    focus: (options?: FocusOptions) => void;
}
export type RadioComponent = DefineComponent<Omit<LabelHTMLAttributes, keyof RadioPublicProps> & RadioPublicProps, RadioBindings>;
export type RadioInstance = InstanceType<DefineComponent<RadioProps, RadioBindings>>;
export declare const radioGroupProps: {
    readonly control: {
        readonly type: PropType<string | number | (string | number)[] | AbstractControl<any>>;
        readonly default: undefined;
    };
    readonly value: {
        readonly type: null;
        readonly default: undefined;
    };
    readonly buttoned: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly dataSource: {
        readonly type: PropType<RadioData<VKey>[]>;
    };
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly gap: {
        readonly type: PropType<string | number>;
        readonly default: undefined;
    };
    readonly name: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly mode: {
        readonly type: PropType<RadioMode>;
        readonly default: undefined;
    };
    readonly size: {
        readonly type: PropType<FormSize>;
        readonly default: "md";
    };
    readonly 'onUpdate:value': {
        readonly type: PropType<MaybeArray<(value: any) => void>>;
    };
    readonly onChange: {
        readonly type: PropType<MaybeArray<(value: any, oldValue: any) => void>>;
    };
};
export type RadioGroupProps = ExtractInnerPropTypes<typeof radioGroupProps>;
export type RadioGroupPublicProps = ExtractPublicPropTypes<typeof radioGroupProps> & Omit<SpaceProps, 'size'>;
export type RadioGroupComponent = DefineComponent<Omit<HTMLAttributes, keyof RadioGroupPublicProps> & RadioGroupPublicProps>;
export type RadioGroupInstance = InstanceType<DefineComponent<RadioGroupProps>>;
export type RadioMode = 'default' | 'primary';
export interface RadioData<K = VKey> extends Omit<RadioPublicProps, 'value'> {
    key?: K;
}
