/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { AbstractControl } from '@idux/cdk/forms';
import type { ExtractInnerPropTypes, ExtractPublicPropTypes, MaybeArray } from '@idux/cdk/utils';
import type { FormSize } from '@idux/components/form';
import type { DefineComponent, HTMLAttributes, PropType } from 'vue';
export declare const rateProps: {
    readonly control: {
        readonly type: PropType<string | number | (string | number)[] | AbstractControl<any>>;
        readonly default: undefined;
    };
    readonly value: PropType<string | number>;
    readonly allowHalf: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly clearable: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly count: PropType<string | number>;
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly icon: StringConstructor;
    readonly tooltips: {
        readonly type: PropType<string[]>;
        readonly default: () => string[];
    };
    readonly size: PropType<FormSize>;
    readonly color: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly 'onUpdate:value': PropType<MaybeArray<(value: number) => void>>;
    readonly onChange: PropType<MaybeArray<(value: number) => void>>;
    readonly onBlur: PropType<MaybeArray<(evt: FocusEvent) => void>>;
    readonly onFocus: PropType<MaybeArray<(evt: FocusEvent) => void>>;
    readonly onKeyDown: PropType<MaybeArray<(evt: KeyboardEvent) => void>>;
};
export type RateProps = ExtractInnerPropTypes<typeof rateProps>;
export type RatePublicProps = ExtractPublicPropTypes<typeof rateProps>;
export interface RateBindings {
    blur: () => void;
    focus: (options?: FocusOptions) => void;
}
export type RateComponent = DefineComponent<Omit<HTMLAttributes, keyof RatePublicProps> & RatePublicProps, RateBindings>;
export type RateInstance = InstanceType<DefineComponent<RateProps, RateBindings>>;
export declare const rateItemProps: {
    readonly count: {
        readonly type: NumberConstructor;
        readonly required: true;
    };
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly required: true;
    };
    readonly focused: {
        readonly type: BooleanConstructor;
        readonly required: true;
    };
    readonly index: {
        readonly type: NumberConstructor;
        readonly required: true;
    };
    readonly prefixCls: {
        readonly type: StringConstructor;
        readonly required: true;
    };
    readonly tooltip: StringConstructor;
    readonly value: {
        readonly type: NumberConstructor;
        readonly required: true;
    };
    readonly color: StringConstructor;
    readonly onClick: {
        readonly type: PropType<(evt: MouseEvent, element: HTMLElement, index: number) => void>;
        readonly required: true;
    };
    readonly onMouseMove: {
        readonly type: PropType<(evt: MouseEvent, element: HTMLElement, index: number) => void>;
        readonly required: true;
    };
};
