import { defineComponent, computed, createVNode } from "vue";
import { isString } from "lodash-es";
import { useGlobalConfig } from "@idux/components/config";
import { IxIcon } from "@idux/components/icon";
import { useThemeToken } from "@idux/components/theme";
const resultProps = {
  icon: [String, Object],
  status: String,
  subtitle: String,
  title: String
};
function getDefaultThemeTokens(tokens) {
  const { marginSizeXs, marginSizeXl, fontSizeHeaderLg, fontSizeSm } = tokens;
  return {
    padding: `48px 32px`,
    iconSize: 48,
    titleFontSize: fontSizeHeaderLg,
    titleLineHeight: 32,
    subtitleFontSize: fontSizeSm,
    subtitleLineHeight: 18,
    subtitleMarginTop: marginSizeXs,
    extraMarginTop: marginSizeXl,
    contentMarginTop: marginSizeXl,
    contentPadding: `24px 40px`
  };
}
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens) : getDefaultThemeTokens(tokens);
};
const defaultIconMap = {
  success: "check-circle-filled",
  error: "close-circle-filled",
  info: "info-circle-filled",
  warning: "exclamation-circle-filled"
};
var Result = /* @__PURE__ */ defineComponent({
  name: "IxResult",
  components: {
    IxIcon
  },
  props: resultProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("result");
    registerToken(getThemeTokens);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-result`);
    const resultConfig = useGlobalConfig("result");
    const className = useClassName(props, mergedPrefixCls, globalHashId, hashId, resultConfig);
    const currentIcon = useIcon(props, resultConfig);
    return () => {
      const prefixCls = mergedPrefixCls.value;
      const {
        title,
        subtitle
      } = props;
      return createVNode("div", {
        "class": className.value
      }, [renderIcon(prefixCls, slots.icon, currentIcon.value), renderSlottedNode(`${prefixCls}-title`, slots.title, title), renderSlottedNode(`${prefixCls}-subtitle`, slots.subtitle, subtitle), renderSlottedNode(`${prefixCls}-extra`, slots.extra), renderSlottedNode(`${prefixCls}-content`, slots.default)]);
    };
  }
});
function renderSlottedNode(cls, slot, defaultContent) {
  if (!slot && !defaultContent) {
    return null;
  }
  const content = slot ? slot() : defaultContent;
  return content ? createVNode("div", {
    "class": cls
  }, [content]) : null;
}
function renderIcon(prefixCls, iconSlot, icon) {
  const content = isString(icon) ? createVNode(IxIcon, {
    "name": icon
  }, null) : icon;
  return renderSlottedNode(`${prefixCls}-icon`, iconSlot, content);
}
function useClassName(props, mergedPrefixCls, globalHashId, hashId, config) {
  return computed(() => {
    var _a;
    const prefixCls = mergedPrefixCls.value;
    const status = (_a = props.status) != null ? _a : config.status;
    return {
      [globalHashId.value]: !!globalHashId.value,
      [hashId.value]: !!hashId.value,
      [prefixCls]: true,
      [`${prefixCls}-${status}`]: !!status
    };
  });
}
function useIcon(props, config) {
  return computed(() => {
    var _a, _b, _c, _d;
    const status = (_a = props.status) != null ? _a : config.status;
    return (_d = (_c = props.icon) != null ? _c : (_b = config.icon) == null ? void 0 : _b[status]) != null ? _d : defaultIconMap[status];
  });
}
const IxResult = Result;
export { IxResult, getThemeTokens as getResultThemeTokens };
