/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { DndSortableOnDragStartArgs, DndSortableOnDropArgs, DndSortableReorderInfo } from '@idux/cdk/dnd';
import type { AbstractControl, ValidateStatus } from '@idux/cdk/forms';
import type { VirtualScrollMode, VirtualScrollToFn } from '@idux/cdk/scroll';
import type { ExtractInnerPropTypes, ExtractPublicPropTypes, MaybeArray, VKey } from '@idux/cdk/utils';
import type { EmptyProps } from '@idux/components/empty';
import type { FormSize } from '@idux/components/form';
import type { SpinProps } from '@idux/components/spin';
import type { OverlayContainerType } from '@idux/components/utils';
import type { DefineComponent, FunctionalComponent, HTMLAttributes, PropType, VNode, VNodeChild } from 'vue';
export interface SelectDndSortable {
    autoScroll?: boolean;
    dragHandle?: boolean | string;
}
export declare const selectPanelProps: {
    readonly activeValue: {
        readonly type: PropType<VKey>;
        readonly default: undefined;
    };
    readonly selectedKeys: {
        readonly type: PropType<VKey[]>;
        readonly default: undefined;
    };
    readonly childrenKey: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly customAdditional: {
        readonly type: PropType<SelectCustomAdditional>;
        readonly default: undefined;
    };
    readonly dataSource: {
        readonly type: PropType<SelectData<VKey>[]>;
        readonly default: undefined;
    };
    readonly dndSortable: {
        readonly type: PropType<boolean | SelectDndSortable>;
        readonly default: false;
    };
    readonly empty: {
        readonly type: PropType<"default" | "simple" | EmptyProps>;
        readonly default: "simple";
    };
    readonly getKey: {
        readonly type: PropType<string | ((data: SelectData<any>) => any)>;
        readonly default: undefined;
    };
    readonly labelKey: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly multiple: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly multipleLimit: {
        readonly type: NumberConstructor;
        readonly default: number;
    };
    readonly virtual: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly virtualScrollMode: {
        readonly type: PropType<VirtualScrollMode>;
        readonly default: undefined;
    };
    readonly virtualItemHeight: {
        readonly type: NumberConstructor;
        readonly default: 32;
    };
    readonly 'onUpdate:activeValue': PropType<MaybeArray<(value: any) => void>>;
    readonly onOptionClick: PropType<MaybeArray<(option: SelectData<any>, evt: MouseEvent) => void>>;
    readonly onScroll: PropType<MaybeArray<(evt: Event) => void>>;
    readonly onScrolledChange: PropType<MaybeArray<(startIndex: number, endIndex: number, visibleData: SelectData<any>[]) => void>>;
    readonly onScrolledBottom: PropType<MaybeArray<() => void>>;
    readonly onDndSortReorder: PropType<MaybeArray<(reorderInfo: DndSortableReorderInfo) => void>>;
    readonly onDndSortChange: PropType<MaybeArray<(newDataSource: SelectData[], oldDataSource: SelectData[]) => void>>;
    readonly _virtualScrollHeight: {
        readonly type: PropType<number | "auto" | "100%">;
        readonly default: 256;
    };
    readonly _onDragStart: PropType<(args: DndSortableOnDragStartArgs) => void>;
    readonly _onDrop: PropType<(args: DndSortableOnDropArgs) => void>;
};
export declare const selectProps: {
    readonly control: {
        readonly type: PropType<string | number | (string | number)[] | AbstractControl<any>>;
        readonly default: undefined;
    };
    readonly value: {
        readonly type: PropType<MaybeArray<VKey>>;
        readonly default: undefined;
    };
    readonly open: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly allowInput: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly autocomplete: {
        readonly type: StringConstructor;
        readonly default: "off";
    };
    readonly autofocus: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly borderless: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly childrenKey: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly clearable: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly clearIcon: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly customAdditional: {
        readonly type: PropType<SelectCustomAdditional>;
        readonly default: undefined;
    };
    readonly dataSource: {
        readonly type: PropType<SelectData<VKey>[]>;
        readonly default: undefined;
    };
    readonly dndSortable: {
        readonly type: PropType<boolean | SelectDndSortable>;
        readonly default: false;
    };
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly empty: {
        readonly type: PropType<"default" | "simple" | EmptyProps>;
        readonly default: "simple";
    };
    readonly getKey: {
        readonly type: PropType<string | ((data: SelectData) => VKey)>;
        readonly default: undefined;
    };
    readonly labelKey: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly maxLabel: {
        readonly type: PropType<number | "responsive">;
        readonly default: number;
    };
    readonly multiple: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly multipleLimit: {
        readonly type: NumberConstructor;
        readonly default: number;
    };
    readonly offset: PropType<[number, number]>;
    readonly overlayClassName: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly overlayContainer: {
        readonly type: PropType<OverlayContainerType>;
        readonly default: undefined;
    };
    readonly overlayTabindex: {
        readonly type: NumberConstructor;
        readonly default: undefined;
    };
    readonly overlayMatchWidth: {
        readonly type: PropType<boolean | "minWidth">;
        readonly default: undefined;
    };
    readonly overlayRender: {
        readonly type: PropType<(children: VNode[]) => VNodeChild>;
        readonly default: undefined;
    };
    readonly placeholder: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly readonly: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly searchable: {
        readonly type: PropType<boolean | "overlay">;
        readonly default: false;
    };
    readonly searchFn: {
        readonly type: PropType<boolean | SelectSearchFn>;
        readonly default: true;
    };
    readonly searchPlaceholder: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly size: {
        readonly type: PropType<FormSize>;
        readonly default: undefined;
    };
    readonly status: PropType<ValidateStatus>;
    readonly suffix: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly spin: {
        readonly type: PropType<boolean | SpinProps>;
        readonly default: undefined;
    };
    readonly virtual: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly virtualScrollMode: {
        readonly type: PropType<VirtualScrollMode>;
        readonly default: undefined;
    };
    readonly virtualItemHeight: {
        readonly type: NumberConstructor;
        readonly default: undefined;
    };
    readonly 'onUpdate:value': PropType<MaybeArray<(value: any) => void>>;
    readonly 'onUpdate:open': PropType<MaybeArray<(opened: boolean) => void>>;
    readonly onChange: PropType<MaybeArray<(value: any, oldValue: any) => void>>;
    readonly onClear: PropType<MaybeArray<(evt: Event) => void>>;
    readonly onFocus: PropType<MaybeArray<(evt: FocusEvent) => void>>;
    readonly onBlur: PropType<MaybeArray<(evt: FocusEvent) => void>>;
    readonly onSearch: PropType<MaybeArray<(searchText: string) => void>>;
    readonly onScroll: PropType<MaybeArray<(evt: Event) => void>>;
    readonly onScrolledChange: PropType<MaybeArray<(startIndex: number, endIndex: number, visibleData: SelectData<any>[]) => void>>;
    readonly onScrolledBottom: PropType<MaybeArray<() => void>>;
    readonly onDndSortReorder: PropType<MaybeArray<(reorderInfo: DndSortableReorderInfo) => void>>;
    readonly onDndSortChange: PropType<MaybeArray<(newDataSource: SelectData[], oldDataSource: SelectData[]) => void>>;
    readonly overlayHeight: {
        readonly type: NumberConstructor;
        readonly default: 256;
    };
};
export type SelectPanelProps = ExtractInnerPropTypes<typeof selectPanelProps>;
export type SelectPanelPublicProps = Omit<ExtractPublicPropTypes<typeof selectPanelProps>, '_virtualScrollHeight'>;
export interface SelectPanelBindings {
    scrollTo: VirtualScrollToFn;
    changeActiveIndex: (offset: number) => void;
}
export type SelectPanelComponent = DefineComponent<Omit<HTMLAttributes, keyof SelectPanelPublicProps> & SelectPanelPublicProps, SelectPanelBindings>;
export type SelectPanelInstance = InstanceType<DefineComponent<SelectPanelProps, SelectPanelBindings>>;
export type SelectProps = ExtractInnerPropTypes<typeof selectProps>;
export type SelectPublicProps = Omit<ExtractPublicPropTypes<typeof selectProps>, 'overlayHeight'>;
export interface SelectBindings {
    blur: () => void;
    focus: (options?: FocusOptions) => void;
    scrollTo: VirtualScrollToFn;
}
export type SelectComponent = DefineComponent<Omit<HTMLAttributes, keyof SelectPublicProps> & SelectPublicProps, SelectBindings>;
export type SelectInstance = InstanceType<DefineComponent<SelectProps, SelectBindings>>;
export type SelectCustomAdditional = (options: {
    data: SelectData<any>;
    index: number;
}) => Record<string, any> | undefined;
export interface SelectOptionProps<K = VKey> {
    disabled?: boolean;
    key?: K;
    label?: string | number;
    customLabel?: string | ((data: SelectOptionProps<K>) => VNodeChild);
    [key: string]: any;
}
export type SelectOptionPublicProps = SelectOptionProps;
export type SelectOptionComponent = FunctionalComponent<Omit<HTMLAttributes, keyof SelectOptionPublicProps> & SelectOptionPublicProps>;
export interface SelectOptionGroupProps<K = VKey> {
    children?: SelectOptionProps<K>[];
    key?: K;
    label?: string | number;
    customLabel?: string | ((data: SelectOptionGroupProps<K>) => VNodeChild);
    [key: string]: any;
}
export type SelectOptionGroupPublicProps = SelectOptionGroupProps;
export type SelectOptionGroupComponent = FunctionalComponent<Omit<HTMLAttributes, keyof SelectOptionGroupPublicProps> & SelectOptionGroupPublicProps>;
export type SelectData<K = VKey> = SelectOptionProps<K> | SelectOptionGroupProps<K>;
export interface FlattenedOption {
    key: VKey;
    label: string;
    disabled?: boolean;
    rawData: SelectData;
    type: 'group' | 'item';
    parentKey?: VKey;
}
export type SelectSearchFn = (data: SelectData<any>, searchText: string) => boolean;
export declare const optionProps: {
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly index: {
        readonly type: NumberConstructor;
        readonly required: true;
    };
    readonly label: {
        readonly type: PropType<string | number>;
        readonly default: undefined;
    };
    readonly optionKey: {
        readonly type: PropType<VKey>;
        readonly required: true;
    };
    readonly rawData: {
        readonly type: PropType<SelectOptionProps<VKey>>;
        readonly required: true;
    };
    readonly parentKey: {
        readonly type: PropType<VKey>;
        readonly default: undefined;
    };
};
export type OptionProps = ExtractInnerPropTypes<typeof optionProps>;
export declare const optionGroupProps: {
    readonly label: {
        readonly type: PropType<string | number>;
        readonly default: undefined;
    };
    readonly index: {
        readonly type: NumberConstructor;
        readonly required: true;
    };
    readonly rawData: {
        readonly type: PropType<SelectOptionGroupProps<VKey>>;
        readonly required: true;
    };
};
export type OptionGroupProps = ExtractInnerPropTypes<typeof optionGroupProps>;
