import { ref, computed, onMounted, watch, nextTick, defineComponent, inject, createVNode, normalizeClass, provide } from "vue";
import { isString } from "lodash-es";
import { useControlledProp, callEmit } from "@idux/cdk/utils";
import { \u0275Overflow as _Overflow } from "@idux/components/_private/overflow";
import { \u0275Trigger as _Trigger } from "@idux/components/_private/trigger";
import { useGlobalConfig } from "@idux/components/config";
import { useThemeToken } from "@idux/components/theme";
import { useGetKey } from "@idux/components/utils";
import { IxIcon } from "@idux/components/icon";
function useInputState(props, mergedSearchable) {
  const mirrorRef = ref();
  const inputRef = ref();
  const [inputValue, setInputValue] = useControlledProp(props, "input", "");
  const isComposing = ref(false);
  const focused = ref(false);
  const mergedFocused = computed(() => {
    var _a;
    return (_a = props.focused) != null ? _a : focused.value;
  });
  const handleFocus = (evt) => {
    focused.value = true;
    callEmit(props.onFocus, evt);
    nextTick(() => {
      var _a;
      (_a = inputRef.value) == null ? void 0 : _a.focus();
    });
  };
  const handleBlur = (evt) => {
    focused.value = false;
    callEmit(props.onBlur, evt);
  };
  const syncMirrorWidth = (evt) => {
    if (props.multiple) {
      const mirrorElement = mirrorRef.value;
      if (!mirrorElement) {
        return;
      }
      const inputText = evt ? evt.target.value : inputRef.value.value;
      mirrorElement.textContent = inputText;
      inputRef.value.style.width = `${mirrorElement.offsetWidth}px`;
    }
  };
  const handleCompositionStart = (evt) => {
    isComposing.value = true;
    callEmit(props.onCompositionStart, evt);
  };
  const handleCompositionEnd = (evt) => {
    callEmit(props.onCompositionEnd, evt);
    if (isComposing.value) {
      isComposing.value = false;
      handleInput(evt, false);
    }
  };
  const handleEnterDown = (evt) => {
    if (evt.code === "Enter" && isComposing.value) {
      evt.stopImmediatePropagation();
      handleCompositionEnd(evt);
    }
  };
  const handleInput = (evt, emitInput = true) => {
    emitInput && callEmit(props.onInput, evt);
    const inputEnabled = props.allowInput || mergedSearchable.value;
    if (isComposing.value) {
      inputEnabled && syncMirrorWidth(evt);
      return;
    }
    if (inputEnabled) {
      const { value } = evt.target;
      if (value !== inputValue.value) {
        setInputValue(value);
        callEmit(props.onInputValueChange, value);
      }
      syncMirrorWidth();
    }
  };
  const clearInput = () => {
    const inputElement = inputRef.value;
    if (inputElement) {
      inputElement.value = "";
    }
    setInputValue("");
    callEmit(props.onInputValueChange, "");
    syncMirrorWidth();
  };
  onMounted(() => syncMirrorWidth());
  watch(mirrorRef, (val) => {
    if (val) {
      syncMirrorWidth();
    }
  });
  return {
    inputRef,
    mirrorRef,
    inputValue,
    isComposing,
    mergedFocused,
    handleCompositionStart,
    handleCompositionEnd,
    handleInput,
    clearInput,
    handleEnterDown,
    handleFocus,
    handleBlur
  };
}
const selectorToken = Symbol("selectorToken");
var Input = /* @__PURE__ */ defineComponent({
  setup() {
    const {
      props,
      mergedPrefixCls,
      mergedSearchable,
      mergedFocused,
      mirrorRef,
      inputRef,
      inputValue,
      handleCompositionStart,
      handleCompositionEnd,
      handleInput,
      handleEnterDown
    } = inject(selectorToken);
    const innerInputValue = computed(() => props.allowInput || mergedSearchable.value ? inputValue.value : "");
    const inputReadonly = computed(() => props.readonly || !mergedFocused.value || !(props.allowInput || mergedSearchable.value));
    const innerStyle = computed(() => {
      return {
        opacity: inputReadonly.value && !innerInputValue.value ? 0 : void 0
      };
    });
    return () => {
      const {
        autocomplete,
        autofocus,
        disabled,
        multiple
      } = props;
      const prefixCls = `${mergedPrefixCls.value}-input`;
      return createVNode("div", {
        "class": prefixCls
      }, [createVNode("input", {
        "ref": inputRef,
        "class": `${prefixCls}-inner`,
        "style": innerStyle.value,
        "autocomplete": autocomplete,
        "autofocus": autofocus,
        "disabled": disabled,
        "readonly": inputReadonly.value,
        "value": innerInputValue.value,
        "onCompositionstart": handleCompositionStart,
        "onCompositionend": handleCompositionEnd,
        "onKeydown": handleEnterDown,
        "onInput": handleInput
      }, null), multiple && createVNode("span", {
        "ref": mirrorRef,
        "class": `${prefixCls}-mirror`,
        "aria-hidden": true
      }, null)]);
    };
  }
});
const Item = (props, {
  slots
}) => {
  const {
    disabled,
    prefixCls,
    removable,
    value,
    onRemove
  } = props;
  const classes = prefixCls + (disabled ? ` ${prefixCls}-disabled` : "");
  const handleClick = (evt) => {
    evt.stopPropagation();
    callEmit(onRemove, value);
  };
  return createVNode("div", {
    "class": classes
  }, [createVNode("span", {
    "class": `${prefixCls}-label`
  }, [slots.default()]), removable && createVNode("span", {
    "class": `${prefixCls}-remove`,
    "onClick": handleClick
  }, [createVNode(IxIcon, {
    "name": "close"
  }, null)])]);
};
const selectorProps = {
  input: String,
  allowInput: { type: [Boolean, String], default: void 0 },
  autocomplete: { type: String, default: void 0 },
  autofocus: { type: Boolean, default: void 0 },
  borderless: { type: Boolean, default: void 0 },
  clearable: { type: Boolean, default: void 0 },
  clearIcon: { type: String, default: "close-circle" },
  dataSource: Array,
  disabled: { type: Boolean, default: void 0 },
  focused: { type: Boolean, default: void 0 },
  getKey: { type: [String, Function], default: "key" },
  labelKey: { type: String, default: void 0 },
  maxLabel: { type: [Number, String], default: void 0 },
  multiple: { type: Boolean, default: void 0 },
  monitorFocus: { type: Boolean, default: true },
  opened: { type: Boolean, default: false },
  placeholder: { type: String, default: void 0 },
  readonly: { type: Boolean, default: void 0 },
  size: { type: String, default: "md" },
  status: String,
  suffix: { type: String, default: void 0 },
  suffixRotate: { type: [Boolean, Number, String], default: void 0 },
  "onUpdate:input": [Function, Array],
  onBlur: [Function, Array],
  onFocus: [Function, Array],
  onClear: [Function, Array],
  onCompositionStart: [Function, Array],
  onCompositionEnd: [Function, Array],
  onInput: [Function, Array],
  onInputValueChange: [Function, Array],
  onItemRemove: [Function, Array]
};
var Selector = /* @__PURE__ */ defineComponent({
  name: "IxSelector",
  props: selectorProps,
  setup(props, {
    expose,
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId
    } = useThemeToken();
    const getKey = useGetKey(props, {
      getKey: "key"
    }, "selector");
    const value = computed(() => {
      var _a, _b;
      return (_b = (_a = props.dataSource) == null ? void 0 : _a.map((data) => getKey.value(data))) != null ? _b : [];
    });
    const triggerRef = ref();
    const mergedPrefixCls = computed(() => `${common.prefixCls}-selector`);
    const mergedClearable = computed(() => {
      return !props.disabled && !props.readonly && props.clearable && value.value.length > 0;
    });
    const mergedSearchable = computed(() => {
      return !props.disabled && !props.readonly && props.allowInput === "searchable";
    });
    const mergedSuffix = computed(() => {
      var _a;
      return (_a = props.suffix) != null ? _a : mergedSearchable.value && props.focused ? "search" : "down";
    });
    const mergedSuffixRotate = computed(() => {
      return !mergedSearchable.value ? props.suffixRotate : void 0;
    });
    const showPlaceholder = computed(() => {
      var _a;
      return !((_a = props.dataSource) == null ? void 0 : _a.length) && !isComposing.value && !inputValue.value;
    });
    const mergedLabelKey = computed(() => {
      var _a;
      return (_a = props.labelKey) != null ? _a : "label";
    });
    const {
      mirrorRef,
      inputRef,
      inputValue,
      isComposing,
      mergedFocused,
      handleCompositionStart,
      handleCompositionEnd,
      handleInput,
      clearInput,
      handleEnterDown,
      handleBlur,
      handleFocus
    } = useInputState(props, mergedSearchable);
    const focus = (options) => {
      var _a;
      if (inputRef.value) {
        inputRef.value.focus(options);
      } else {
        (_a = triggerRef.value) == null ? void 0 : _a.focus(options);
      }
    };
    const blur = () => {
      var _a;
      if (inputRef.value) {
        inputRef.value.blur();
      } else {
        (_a = triggerRef.value) == null ? void 0 : _a.blur();
      }
    };
    expose({
      focus,
      blur,
      clearInput
    });
    const classes = computed(() => {
      const {
        allowInput,
        borderless,
        multiple,
        opened,
        size,
        status
      } = props;
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [globalHashId.value]: !!globalHashId.value,
        [prefixCls]: true,
        [`${prefixCls}-${size}`]: !!size,
        [`${prefixCls}-${status}`]: !!status,
        [`${prefixCls}-allow-input`]: allowInput,
        [`${prefixCls}-borderless`]: borderless,
        [`${prefixCls}-clearable`]: mergedClearable.value,
        [`${prefixCls}-disabled`]: props.disabled,
        [`${prefixCls}-focused`]: mergedFocused.value,
        [`${prefixCls}-multiple`]: multiple,
        [`${prefixCls}-opened`]: opened,
        [`${prefixCls}-readonly`]: props.readonly,
        [`${prefixCls}-searchable`]: mergedSearchable.value,
        [`${prefixCls}-single`]: !multiple
      });
    });
    const handleClear = (evt) => {
      const {
        disabled,
        readonly
      } = props;
      if (disabled || readonly) {
        return;
      }
      evt.stopPropagation();
      callEmit(props.onClear, evt);
    };
    provide(selectorToken, {
      props,
      mergedPrefixCls,
      mergedSearchable,
      mirrorRef,
      inputRef,
      inputValue,
      isComposing,
      mergedFocused,
      handleCompositionStart,
      handleCompositionEnd,
      handleInput,
      handleEnterDown
    });
    return () => {
      const {
        borderless,
        clearable,
        clearIcon,
        multiple,
        disabled,
        focused,
        readonly,
        dataSource,
        maxLabel,
        size,
        status
      } = props;
      const prefixCls = mergedPrefixCls.value;
      const itemPrefixCls = `${prefixCls}-item`;
      const renderItem = (item) => {
        const {
          value: value2,
          rawData
        } = item;
        const key = getKey.value(item);
        const label = item[mergedLabelKey.value];
        const _disabled = disabled || item.disabled;
        const removable = multiple && !_disabled && !readonly;
        const itemProps = {
          key,
          disabled: _disabled,
          prefixCls: itemPrefixCls,
          removable,
          value: value2 != null ? value2 : key,
          label,
          onRemove: props.onItemRemove
        };
        const selectedItemSlot = slots.selectedItem;
        if (selectedItemSlot) {
          return selectedItemSlot(itemProps);
        }
        const slotOrName = slots.selectedLabel || slots.label || (item == null ? void 0 : item.customLabel) || (rawData == null ? void 0 : rawData.customLabel);
        const selectedLabelRender = isString(slotOrName) ? slots[slotOrName] : slotOrName;
        const labelNode = selectedLabelRender ? selectedLabelRender(rawData != null ? rawData : item) : label;
        return createVNode(Item, itemProps, {
          default: () => [labelNode]
        });
      };
      const children = [];
      if (showPlaceholder.value) {
        const placeholderNode = slots.placeholder ? slots.placeholder() : props.placeholder;
        children.push(createVNode("div", {
          "key": "__placeholder",
          "class": `${prefixCls}-placeholder`
        }, [placeholderNode]));
      }
      if (multiple) {
        const renderRest = (rest) => {
          const key = "__IDUX_SELECT_MAX_ITEM";
          const itemProps = {
            key,
            prefixCls: itemPrefixCls,
            removable: false
          };
          const overflowedLabelSlot = slots.overflowedLabel || slots.maxLabel;
          const labelNode = overflowedLabelSlot ? overflowedLabelSlot(rest) : `+ ${rest.length} ...`;
          return createVNode(Item, itemProps, {
            default: () => [labelNode]
          });
        };
        const overflowSlot = {
          item: renderItem,
          rest: renderRest,
          suffix: () => createVNode(Input, null, null)
        };
        children.push(createVNode(_Overflow, {
          "prefixCls": prefixCls,
          "dataSource": dataSource,
          "getKey": getKey.value,
          "maxLabel": maxLabel
        }, overflowSlot));
      } else {
        if ((dataSource == null ? void 0 : dataSource.length) && !isComposing.value && !inputValue.value) {
          dataSource.forEach((item) => children.push(renderItem(item)));
        }
        children.push(createVNode(Input, null, null));
      }
      return createVNode(_Trigger, {
        "ref": triggerRef,
        "class": classes.value,
        "borderless": borderless,
        "clearable": clearable,
        "clearIcon": clearIcon,
        "disabled": disabled,
        "focused": focused,
        "monitorFocus": props.monitorFocus,
        "paddingless": multiple,
        "readonly": readonly,
        "size": size,
        "status": status,
        "suffix": mergedSuffix.value,
        "suffixRotate": mergedSuffixRotate.value,
        "value": value.value,
        "onClear": handleClear,
        "onFocus": handleFocus,
        "onBlur": handleBlur
      }, {
        default: () => [createVNode("div", {
          "class": `${prefixCls}-content`
        }, [children])],
        suffix: slots.suffix,
        clearIcon: slots.clearIcon
      });
    };
  }
});
const IxSelector = Selector;
export { IxSelector };
