/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { ValidateStatus } from '@idux/cdk/forms';
import type { ExtractInnerPropTypes, ExtractPublicPropTypes, MaybeArray, VKey } from '@idux/cdk/utils';
import type { FormSize } from '@idux/components/form';
import type { GetKeyFn } from '@idux/components/utils';
import type { DefineComponent, HTMLAttributes, PropType, VNodeChild } from 'vue';
export interface SelectorData<K = VKey> {
    disabled?: boolean;
    key?: K;
    label?: string | number;
    value?: any;
    rawData?: any;
    customLabel?: string | ((data: SelectorData<K>) => VNodeChild);
    [key: string]: any;
}
export declare const selectorProps: {
    readonly input: StringConstructor;
    readonly allowInput: {
        readonly type: PropType<boolean | "searchable">;
        readonly default: undefined;
    };
    readonly autocomplete: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly autofocus: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly borderless: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly clearable: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly clearIcon: {
        readonly type: StringConstructor;
        readonly default: "close-circle";
    };
    readonly dataSource: PropType<SelectorData<VKey>[]>;
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly focused: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly getKey: {
        readonly type: PropType<string | GetKeyFn>;
        readonly default: "key";
    };
    readonly labelKey: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly maxLabel: {
        readonly type: PropType<number | "responsive">;
        readonly default: undefined;
    };
    readonly multiple: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly monitorFocus: {
        readonly type: BooleanConstructor;
        readonly default: true;
    };
    readonly opened: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly placeholder: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly readonly: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly size: {
        readonly type: PropType<FormSize>;
        readonly default: "md";
    };
    readonly status: PropType<ValidateStatus>;
    readonly suffix: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly suffixRotate: {
        readonly type: PropType<string | number | boolean>;
        readonly default: undefined;
    };
    readonly 'onUpdate:input': PropType<MaybeArray<(input: string | undefined) => void>>;
    readonly onBlur: PropType<MaybeArray<(evt: FocusEvent) => void>>;
    readonly onFocus: PropType<MaybeArray<(evt: FocusEvent) => void>>;
    readonly onClear: PropType<MaybeArray<(evt: MouseEvent) => void>>;
    readonly onCompositionStart: PropType<MaybeArray<(evt: CompositionEvent) => void>>;
    readonly onCompositionEnd: PropType<MaybeArray<(evt: CompositionEvent) => void>>;
    readonly onInput: PropType<MaybeArray<(evt: Event) => void>>;
    readonly onInputValueChange: PropType<MaybeArray<(value: string) => void>>;
    readonly onItemRemove: PropType<MaybeArray<(value: any) => void>>;
};
export type SelectorProps = ExtractInnerPropTypes<typeof selectorProps>;
export type SelectorPublicProps = ExtractPublicPropTypes<typeof selectorProps>;
export interface SelectorBindings {
    blur: () => void;
    focus: (options?: FocusOptions) => void;
    clearInput: () => void;
    getBoundingClientRect: () => DOMRect | undefined;
}
export type SelectorComponent = DefineComponent<Omit<HTMLAttributes, keyof SelectorPublicProps> & SelectorPublicProps, SelectorBindings>;
export type SelectorInstance = InstanceType<DefineComponent<SelectorProps, SelectorBindings>>;
