import { defineComponent, computed, normalizeClass, createVNode, mergeProps } from "vue";
import { convertCssPixel } from "@idux/cdk/utils";
import { useGlobalConfig } from "@idux/components/config";
import { useThemeToken } from "@idux/components/theme";
const skeletonProps = {
  animated: {
    type: Boolean,
    default: void 0
  },
  loading: {
    type: Boolean,
    default: true
  },
  type: {
    type: String,
    default: "text"
  },
  width: [String, Number],
  height: [String, Number],
  repeat: {
    type: Number,
    default: 1
  }
};
function getDefaultThemeTokens(tokens, algorithms) {
  const { getGreyColors } = algorithms;
  const { marginSizeSm, marginSizeXs } = tokens;
  const greyColors = getGreyColors();
  return {
    marginBottom: marginSizeSm,
    loaderMarginBottom: marginSizeXs,
    startColor: greyColors.l40,
    endColor: greyColors.l30
  };
}
const getThemeTokens = (tokens, presetTheme, algorithms) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens, algorithms) : getDefaultThemeTokens(tokens, algorithms);
};
var Skeleton = /* @__PURE__ */ defineComponent({
  name: "IxSkeleton",
  inheritAttrs: false,
  props: skeletonProps,
  setup(props, {
    slots,
    attrs
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("skeleton");
    registerToken(getThemeTokens);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-skeleton`);
    const config = useGlobalConfig("skeleton");
    const loaderClass = computed(() => {
      var _a;
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [`${prefixCls}-loader`]: true,
        [`${prefixCls}-${props.type}`]: true,
        [`${prefixCls}-animated`]: (_a = props.animated) != null ? _a : config.animated
      });
    });
    const loaderStyle = computed(() => {
      const {
        width,
        height,
        type
      } = props;
      if (type === "circle") {
        const size = convertCssPixel(width || height);
        return {
          width: size,
          height: size
        };
      }
      return {
        width: convertCssPixel(width),
        height: convertCssPixel(height)
      };
    });
    return () => {
      var _a;
      if (!props.loading) {
        return (_a = slots.default) == null ? void 0 : _a.call(slots);
      }
      const prefixCls = mergedPrefixCls.value;
      const loader = createVNode("div", {
        "class": loaderClass.value,
        "style": loaderStyle.value
      }, null);
      const loaderList = Array.from({
        length: props.repeat
      }).map(() => loader);
      return createVNode("div", mergeProps({
        "class": [prefixCls, globalHashId.value, hashId.value]
      }, attrs), [loaderList]);
    };
  }
});
const IxSkeleton = Skeleton;
export { IxSkeleton, getThemeTokens as getSkeletonThemeTokens };
