/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { AbstractControl } from '@idux/cdk/forms';
import type { PopperPlacement } from '@idux/cdk/popper';
import type { ExtractInnerPropTypes, ExtractPublicPropTypes, MaybeArray } from '@idux/cdk/utils';
import type { TooltipInstance } from '@idux/components/tooltip';
import type { CSSProperties, DefineComponent, HTMLAttributes, PropType, VNode } from 'vue';
export declare const sliderProps: {
    readonly control: {
        readonly type: PropType<string | number | (string | number)[] | AbstractControl<any>>;
        readonly default: undefined;
    };
    readonly value: {
        readonly type: PropType<number | number[]>;
        readonly default: undefined;
    };
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly dots: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly keyboard: {
        readonly type: BooleanConstructor;
        readonly default: true;
    };
    readonly marks: {
        readonly type: PropType<Record<number, string | VNode<import("vue").RendererNode, import("vue").RendererElement, {
            [key: string]: any;
        }> | {
            style?: CSSProperties | undefined;
            label?: string | VNode<import("vue").RendererNode, import("vue").RendererElement, {
                [key: string]: any;
            }> | undefined;
        } | (() => string | VNode)>>;
        readonly default: undefined;
    };
    readonly max: {
        readonly type: NumberConstructor;
        readonly default: 100;
    };
    readonly min: {
        readonly type: NumberConstructor;
        readonly default: 0;
    };
    readonly range: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly reverse: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly step: {
        readonly type: NumberConstructor;
        readonly default: 1;
    };
    readonly tooltipFormatter: {
        readonly type: PropType<(value: number) => VNode | string | number>;
        readonly default: undefined;
    };
    readonly tooltipPlacement: {
        readonly type: PropType<PopperPlacement>;
        readonly default: undefined;
    };
    readonly tooltipVisible: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly vertical: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly 'onUpdate:value': PropType<MaybeArray<(value: number | number[]) => void>>;
    readonly onChange: PropType<MaybeArray<(newValue: number | number[], oldVal: number | number[]) => void>>;
    readonly onFocus: PropType<MaybeArray<(evt: FocusEvent) => void>>;
    readonly onBlur: PropType<MaybeArray<(evt: FocusEvent) => void>>;
};
export interface SliderBindings {
    focus: (options?: FocusOptions) => void;
    blur: () => void;
}
export type SliderProps = ExtractInnerPropTypes<typeof sliderProps>;
export type SliderPublicProps = ExtractPublicPropTypes<typeof sliderProps>;
export type SliderComponent = DefineComponent<Omit<HTMLAttributes, keyof SliderPublicProps> & SliderPublicProps, SliderBindings>;
export type SliderInstance = InstanceType<DefineComponent<SliderProps, SliderBindings>>;
export declare const sliderThumbProps: {
    readonly value: {
        readonly type: NumberConstructor;
        readonly default: undefined;
    };
    readonly onFocus: PropType<MaybeArray<(evt: FocusEvent) => void>>;
    readonly onBlur: PropType<MaybeArray<(evt: FocusEvent) => void>>;
};
export interface SliderThumbBindings {
    tooltipRef: TooltipInstance;
    thumbRef: HTMLElement;
    isHovering: boolean;
    showTooltip: () => void;
    hideTooltip: () => void;
}
export type SliderThumbProps = ExtractInnerPropTypes<typeof sliderThumbProps>;
export type SliderThumbPublicProps = ExtractPublicPropTypes<typeof sliderThumbProps>;
export type SliderThumbComponent = DefineComponent<Omit<HTMLAttributes, keyof SliderThumbPublicProps> & SliderThumbPublicProps, SliderThumbBindings>;
export type SliderThumbInstance = InstanceType<DefineComponent<SliderThumbProps, SliderThumbBindings>>;
export declare const sliderMarksProps: {
    readonly onClickMark: PropType<MaybeArray<(evt: MouseEvent | TouchEvent, markValue: number) => void>>;
};
export type SliderMarksProps = ExtractInnerPropTypes<typeof sliderMarksProps>;
export type SliderMarksPublicProps = ExtractPublicPropTypes<typeof sliderMarksProps>;
export type SliderMarksComponent = DefineComponent<Omit<HTMLAttributes, keyof SliderMarksPublicProps> & SliderMarksPublicProps>;
export type SliderMarksInstance = InstanceType<DefineComponent<SliderMarksProps>>;
