import { defineComponent, computed, createVNode, Fragment, createTextVNode } from "vue";
import { useGlobalConfig } from "@idux/components/config";
import { useThemeToken } from "@idux/components/theme";
const statisticProps = {
  formatter: Function,
  precision: Number,
  prefix: String,
  suffix: String,
  title: String,
  value: {
    type: [String, Number],
    default: ""
  }
};
function getDefaultThemeTokens(tokens) {
  const { fontSizeMd, fontSize2xl, fontSize3xl, colorTextTitle } = tokens;
  return {
    fontSizeSm: fontSizeMd,
    fontSizeMd,
    fontSizeLg: fontSize2xl,
    fontSizeXl: fontSize3xl,
    color: colorTextTitle
  };
}
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens) : getDefaultThemeTokens(tokens);
};
var Statistic = /* @__PURE__ */ defineComponent({
  name: "IxStatistic",
  props: statisticProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("statistic");
    registerToken(getThemeTokens);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-statistic`);
    const statisticConfig = useGlobalConfig("statistic");
    const formatedValue = useFomat(props, statisticConfig);
    return () => {
      var _a, _b, _c, _d, _e, _f, _g, _h;
      const prefixCls = mergedPrefixCls.value;
      const {
        title,
        prefix,
        suffix
      } = props;
      const titleNode = (title != null ? title : slots.title) && createVNode("div", {
        "class": `${prefixCls}-title`
      }, [(_b = (_a = slots.title) == null ? void 0 : _a.call(slots)) != null ? _b : title]);
      const prefixNode = (prefix != null ? prefix : slots.prefix) && createVNode("span", {
        "class": `${prefixCls}-content-prefix`
      }, [(_d = (_c = slots.prefix) == null ? void 0 : _c.call(slots)) != null ? _d : prefix]);
      const suffixNode = (suffix != null ? suffix : slots.suffix) && createVNode("span", {
        "class": `${prefixCls}-content-suffix`
      }, [(_f = (_e = slots.suffix) == null ? void 0 : _e.call(slots)) != null ? _f : suffix]);
      return createVNode("div", {
        "class": [prefixCls, globalHashId.value, hashId.value]
      }, [titleNode, createVNode("div", {
        "class": `${prefixCls}-content`
      }, [prefixNode, createVNode("span", {
        "class": `${prefixCls}-content-value`
      }, [(_h = (_g = slots.default) == null ? void 0 : _g.call(slots)) != null ? _h : createVNode(Fragment, null, [createVNode("span", {
        "class": `${prefixCls}-content-value-int`
      }, [formatedValue.value.int || formatedValue.value.value]), formatedValue.value.decimal && createVNode("span", {
        "class": `${prefixCls}-content-value-decimal`
      }, [createTextVNode(" "), formatedValue.value.decimal, createTextVNode(" ")])])]), suffixNode])]);
    };
  }
});
const useFomat = (props, config) => {
  return computed(() => {
    const precision = props.precision || config.precision;
    const formatter = props.formatter || config.formatter;
    return formatter(props.value, precision);
  });
};
const IxStatistic = Statistic;
export { IxStatistic, getThemeTokens as getStatisticThemeTokens };
