/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { AbstractControl } from '@idux/cdk/forms';
import type { ExtractInnerPropTypes, ExtractPublicPropTypes, MaybeArray } from '@idux/cdk/utils';
import type { DefineComponent, HTMLAttributes, PropType } from 'vue';
export declare const switchProps: {
    readonly control: {
        readonly type: PropType<string | number | (string | number)[] | AbstractControl<any>>;
        readonly default: undefined;
    };
    readonly checked: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly autofocus: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly labels: {
        readonly type: PropType<string[]>;
        readonly default: () => never[];
    };
    readonly loading: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly size: PropType<"sm" | "md" | "lg">;
    readonly 'onUpdate:checked': PropType<MaybeArray<(checked: boolean) => void>>;
    readonly onChange: PropType<MaybeArray<(checked: boolean, oldChecked: boolean) => void>>;
    readonly onBlur: PropType<MaybeArray<(evt: FocusEvent) => void>>;
    readonly onFocus: PropType<MaybeArray<(evt: FocusEvent) => void>>;
};
export type SwitchProps = ExtractInnerPropTypes<typeof switchProps>;
export type SwitchPublicProps = ExtractPublicPropTypes<typeof switchProps>;
export interface SwitchBindings {
    focus: (options?: FocusOptions) => void;
    blur: () => void;
}
export type SwitchComponent = DefineComponent<Omit<HTMLAttributes, keyof SwitchPublicProps> & SwitchPublicProps, SwitchBindings>;
export type SwitchInstance = InstanceType<DefineComponent<SwitchProps, SwitchBindings>>;
