/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import { type ComputedRef, type Slots, type VNode, type VNodeChild } from 'vue';
import { type VKey } from '@idux/cdk/utils';
import { type TableConfig } from '@idux/components/config';
import { type TableColumn, type TableColumnAlign, type TableColumnBase, type TableColumnExpandable, type TableColumnFixed, type TableColumnSelectable, type TableProps } from '../types';
export declare function useColumns(props: TableProps, slots: Slots, config: TableConfig, scrollBarSizeOnFixedHolder: ComputedRef<number>): ColumnsContext;
export interface ColumnsContext {
    flattedColumns: ComputedRef<TableColumnMerged[]>;
    scrollBarColumn: ComputedRef<TableColumnScrollBar | undefined>;
    flattedColumnsWithScrollBar: ComputedRef<(TableColumnMerged | TableColumnScrollBar)[]>;
    fixedColumns: ComputedRef<{
        fixedStartColumns: (TableColumnMerged | TableColumnScrollBar)[];
        fixedEndColumns: (TableColumnMerged | TableColumnScrollBar)[];
        fixedColumnIndexMap: Record<VKey, number>;
    }>;
    fixedColumnKeys: ComputedRef<{
        lastStartKey: VKey | undefined;
        firstEndKey: VKey | undefined;
    }>;
    hasEllipsis: ComputedRef<boolean>;
    hasFixed: ComputedRef<boolean>;
    mergedRows: ComputedRef<{
        rows: TableColumnMergedExtra[][];
        offsetIndexMap: Record<VKey, {
            colStart: number;
            colEnd: number;
        }>;
    }>;
}
export type TableColumnMerged = (TableColumnMergedBase | TableColumnMergedExpandable | TableColumnMergedSelectable) & {
    type?: 'selectable' | 'expandable' | 'scroll-bar' | 'indexable';
};
export type TableColumnMergedExtra = TableColumnMergedBaseExtra | TableColumnMergedExpandable | TableColumnMergedSelectable | TableColumnMergedScrollBar;
export interface TableColumnMergedBase extends TableColumnBase {
    align: {
        title: TableColumnAlign;
        cell: TableColumnAlign;
    };
    key: VKey;
}
export interface TableColumnMergedBaseExtra extends TableColumnMergedBase {
    colStart: number;
    colEnd: number;
    hasChildren: boolean;
    titleColSpan: number;
    titleRowSpan?: number;
}
export interface TableColumnMergedExpandable extends TableColumnMergedBaseExtra, TableColumnExpandable {
    align: {
        title: TableColumnAlign;
        cell: TableColumnAlign;
    };
    key: VKey;
    icon: string | VNodeChild | ((options: {
        expanded: boolean;
        record: unknown;
    }) => string | VNodeChild);
    titleColSpan: number;
}
export interface TableColumnMergedSelectable extends TableColumnMergedBaseExtra, TableColumnSelectable {
    align: {
        title: TableColumnAlign;
        cell: TableColumnAlign;
    };
    key: VKey;
    multiple: boolean;
    titleColSpan: number;
    customCell?: (data: unknown) => VNodeChild;
}
export interface TableColumnScrollBar {
    key: string;
    type: 'scroll-bar';
    fixed: TableColumnFixed | undefined;
    width: number;
}
export type TableColumnMergedScrollBar = TableColumnMergedBaseExtra & TableColumnScrollBar;
export declare function convertColumns(nodes: VNode[] | undefined): TableColumn[];
export declare function flatColumns<Col extends TableColumnMerged>(columns: Col[]): Col[];
