/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import { type ComputedRef, type Ref } from 'vue';
import { type VKey } from '@idux/cdk/utils';
import { type GetKeyFn } from '@idux/components/utils';
import { type ActiveFilter } from './useFilterable';
import { type ActiveSorter } from './useSortable';
import { type TablePagination, type TableProps } from '../types';
export interface DataSourceContext {
    isTreeData: ComputedRef<boolean>;
    filteredData: ComputedRef<MergedData[]>;
    flattedData: ComputedRef<FlattedData[]>;
    mergedData: ComputedRef<MergedData[]>;
    mergedMap: ComputedRef<Map<VKey, MergedData>>;
    paginatedData: ComputedRef<MergedData[]>;
    paginatedMap: ComputedRef<Map<VKey, MergedData>>;
    parentKeyMap: ComputedRef<Map<VKey, VKey | undefined>>;
    depthMap: ComputedRef<Map<VKey, number>>;
}
export interface MergedData {
    children?: MergedData[];
    parentKey?: VKey;
    record: unknown;
    rowKey: VKey;
    hasPrevSibling: boolean;
    hasNextSibling: boolean;
    showLineIndentIndexList: number[];
}
export interface FlattedData extends MergedData {
    expanded?: boolean;
    level?: number;
}
export declare function useDataSource(props: TableProps, mergedChildrenKey: ComputedRef<string>, mergedGetKey: ComputedRef<GetKeyFn>, activeSorters: Ref<ActiveSorter[]>, activeFilters: Ref<ActiveFilter[]>, expandedRowKeys: Ref<VKey[]>, mergedPagination: ComputedRef<TablePagination | null>): DataSourceContext;
