/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { TableColumnMerged, TableColumnMergedExtra } from './composables/useColumns';
import type { FlattedData } from './composables/useDataSource';
import type { ActiveFilter } from './composables/useFilterable';
import type { ActiveSorter } from './composables/useSortable';
import type { VirtualScrollMode, VirtualScrollToFn } from '@idux/cdk/scroll';
import type { ExtractInnerPropTypes, ExtractPublicPropTypes, MaybeArray, VKey } from '@idux/cdk/utils';
import type { CascaderStrategy } from '@idux/components/cascader';
import type { EmptyProps } from '@idux/components/empty';
import type { HeaderProps } from '@idux/components/header';
import type { MenuClickOptions, MenuData } from '@idux/components/menu';
import type { PaginationProps } from '@idux/components/pagination';
import type { SpinProps } from '@idux/components/spin';
import type { Component, DefineComponent, FunctionalComponent, HTMLAttributes, PropType, VNodeChild } from 'vue';
export declare const tableProps: {
    readonly expandedRowKeys: {
        readonly type: PropType<VKey[]>;
        readonly default: undefined;
    };
    readonly selectedRowKeys: {
        readonly type: PropType<VKey[]>;
        readonly default: undefined;
    };
    readonly autoHeight: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly borderless: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly childrenKey: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly cascaderStrategy: {
        readonly type: PropType<CascaderStrategy>;
        readonly default: "all";
    };
    readonly columns: {
        readonly type: PropType<TableColumn<any, VKey>[]>;
        readonly default: () => never[];
    };
    readonly customAdditional: {
        readonly type: PropType<TableCustomAdditional<any, any>>;
        readonly default: undefined;
    };
    readonly customTag: {
        readonly type: PropType<TableCustomTag>;
        readonly default: undefined;
    };
    readonly dataSource: {
        readonly type: PropType<any[]>;
        readonly default: () => never[];
    };
    readonly ellipsis: {
        readonly type: PropType<boolean | {
            title?: boolean | undefined;
            head?: boolean | undefined;
        }>;
        readonly default: false;
    };
    readonly empty: {
        readonly type: PropType<"default" | "simple" | EmptyProps>;
        readonly default: "default";
    };
    readonly emptyCell: {
        readonly type: PropType<string | ((options: TableEmptyCellOptions) => VNodeChild)>;
        readonly default: undefined;
    };
    readonly getKey: {
        readonly type: PropType<string | ((record: any) => any)>;
        readonly default: undefined;
    };
    readonly header: {
        readonly type: PropType<string | HeaderProps>;
        readonly default: undefined;
    };
    readonly headless: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly insetShadow: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly pagination: {
        readonly type: PropType<boolean | TablePagination>;
        readonly default: true;
    };
    readonly scroll: {
        readonly type: PropType<TableScroll>;
        readonly default: undefined;
    };
    readonly size: {
        readonly type: PropType<TableSize>;
        readonly default: undefined;
    };
    readonly spin: {
        readonly type: PropType<boolean | SpinProps>;
        readonly default: undefined;
    };
    readonly sticky: {
        readonly type: PropType<boolean | TableSticky>;
        readonly default: undefined;
    };
    readonly scrollToTopOnChange: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly tableLayout: {
        readonly type: PropType<"fixed" | "auto">;
        readonly default: undefined;
    };
    readonly virtual: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly virtualScrollMode: {
        readonly type: PropType<VirtualScrollMode>;
        readonly default: undefined;
    };
    readonly virtualHorizontal: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly virtualItemHeight: {
        readonly type: NumberConstructor;
        readonly default: undefined;
    };
    readonly virtualColWidth: {
        readonly type: NumberConstructor;
        readonly default: undefined;
    };
    readonly virtualBufferSize: {
        readonly type: NumberConstructor;
        readonly default: undefined;
    };
    readonly virtualBufferOffset: {
        readonly type: NumberConstructor;
        readonly default: undefined;
    };
    readonly 'onUpdate:expandedRowKeys': PropType<MaybeArray<(keys: any[]) => void>>;
    readonly 'onUpdate:selectedRowKeys': PropType<MaybeArray<(keys: any[]) => void>>;
    readonly onScroll: PropType<MaybeArray<(evt: Event) => void>>;
    readonly onScrolledChange: PropType<MaybeArray<(startIndex: number, endIndex: number, visibleData: any[]) => void>>;
    readonly onScrolledBottom: PropType<MaybeArray<() => void>>;
};
export type TableProps = ExtractInnerPropTypes<typeof tableProps>;
export type TablePublicProps = ExtractPublicPropTypes<typeof tableProps>;
export interface TableBindings {
    scrollTo: VirtualScrollToFn;
}
export type TableComponent = DefineComponent<Omit<HTMLAttributes, keyof TablePublicProps> & TablePublicProps, TableBindings>;
export type TableInstance = InstanceType<DefineComponent<TableProps, TableBindings>>;
export type TableColumnComponent = FunctionalComponent<Omit<HTMLAttributes, keyof TableColumnBase | keyof TableColumnExpandable | keyof TableColumnSelectable> & TableColumn>;
export type TableColumn<T = any, K = VKey> = TableColumnBase<T, K> | TableColumnExpandable<T, K> | TableColumnSelectable<T, K> | TableColumnIndexable<T>;
export interface TableColumnCommon<T = any> {
    key?: VKey;
    align?: TableColumnAlign | {
        title: TableColumnAlign;
        cell: TableColumnAlign;
    };
    colSpan?: (record: T, rowIndex: number) => number;
    rowSpan?: (record: T, rowIndex: number) => number;
    fixed?: TableColumnFixed;
    titleColSpan?: number;
    width?: string | number;
}
export interface TableColumnBase<T = any, K = VKey> extends TableColumnCommon<T> {
    dataKey?: VKey | VKey[];
    ellipsis?: boolean | {
        title?: boolean;
        head?: boolean;
    };
    sortable?: TableColumnSortable<T>;
    filterable?: TableColumnFilterable<T>;
    title?: string;
    children?: TableColumn<T, K>[];
    customCell?: string | ((data: {
        value: any;
        record: T;
        rowIndex: number;
    }) => VNodeChild);
    customTitle?: string | ((data: {
        title?: string;
    }) => VNodeChild);
}
export interface TableColumnExpandable<T = any, K = VKey> extends TableColumnBase<T, K> {
    type: 'expandable';
    disabled?: (record: T) => boolean;
    icon?: string | VNodeChild | ((options: {
        expanded: boolean;
        record: T;
    }) => string | VNodeChild);
    indent?: number;
    trigger?: 'click' | 'dblclick';
    showLine?: boolean;
    onChange?: (expendedRowKeys: K[]) => void;
    onExpand?: (expanded: boolean, record: T) => void;
    customExpand?: string | ((data: {
        record: T;
        rowIndex: number;
    }) => VNodeChild);
    customIcon?: string | ((data: {
        record: T;
        expanded: boolean;
    }) => VNodeChild);
}
export interface TableColumnSelectable<T = any, K = VKey> extends TableColumnCommon<T> {
    type: 'selectable';
    disabled?: (record: T) => boolean;
    multiple?: boolean;
    menus?: ('all' | 'invert' | 'none' | 'pageInvert' | MenuData)[];
    showIndex?: boolean;
    trigger?: 'click' | 'dblclick';
    onChange?: (selectedRowKeys: K[], selectedRows: T[]) => void;
    onMenuClick?: <MK = VKey>(options: MenuClickOptions<MK>, currentPageRowKeys: K[]) => void;
    onSelect?: (selected: boolean, record: T) => void;
    onSelectAll?: (selectedRowKeys: K[]) => void;
    onSelectInvert?: (selectedRowKeys: K[]) => void;
    onSelectNone?: () => void;
    onSelectPageInvert?: (selectedRowKeys: K[]) => void;
    customCell?: string | ((data: {
        checked: boolean;
        disabled: boolean;
        indeterminate?: boolean;
        record: T;
        rowIndex: number;
        onChange: () => void;
        onClick: () => void;
    }) => VNodeChild);
}
export interface TableColumnIndexable<T = any> extends TableColumnCommon<T> {
    type: 'indexable';
    ellipsis?: boolean | {
        title?: boolean;
        head?: boolean;
    };
    title?: string;
    customCell?: string | ((data: {
        record: T;
        rowIndex: number;
        pageSize: number;
        pageIndex: number;
    }) => VNodeChild);
    customTitle?: string | ((data: {
        title?: string;
    }) => VNodeChild);
}
export interface TableCustomAdditional<T = any, K = VKey> {
    bodyCell?: (data: {
        column: TableColumn<T, K>;
        record: T;
        rowIndex: number;
    }) => Record<string, any> | undefined;
    bodyRow?: (data: {
        record: T;
        rowIndex: number;
    }) => Record<string, any> | undefined;
    head?: (data: {
        rows: TableColumn<T, K>[][];
    }) => Record<string, any> | undefined;
    headCell?: (data: {
        column: TableColumn<T, K>;
    }) => Record<string, any> | undefined;
    headRow?: (data: {
        columns: TableColumn<T, K>[];
    }) => Record<string, any> | undefined;
}
export interface TableCustomTag {
    body?: string | Component;
    bodyCell?: string | Component;
    bodyRow?: string | Component;
    head?: string | Component;
    headCell?: string | Component;
    headRow?: string | Component;
}
export interface TablePagination extends PaginationProps {
    position?: TablePaginationPosition;
}
export type TablePaginationPosition = 'topStart' | 'top' | 'topEnd' | 'bottomStart' | 'bottom' | 'bottomEnd';
export interface TableScroll {
    width?: string | number;
    height?: string | number;
    fullHeight?: boolean;
}
export type TableSize = 'lg' | 'md' | 'sm';
export type TableColumnAlign = 'start' | 'center' | 'end';
export type TableColumnFixed = 'start' | 'end';
export type TableColumnSortOrder = 'ascend' | 'descend';
export interface TableColumnSortable<T = any> {
    multiple?: number;
    nextTooltip?: boolean;
    orderBy?: TableColumnSortOrder;
    orders?: TableColumnSortOrder[];
    sorter?: (curr: T, next: T) => number;
    onChange?: (orderBy: TableColumnSortOrder | undefined, sorters: ActiveSorter[]) => void;
}
export interface TableColumnFilterable<T = any> {
    filter?: (filterBy: any[], record: T) => boolean;
    filterBy?: VKey[];
    footer?: boolean;
    menus: MenuData[];
    multiple?: boolean;
    onChange?: (filterBy: any[], filters: ActiveFilter[]) => void;
    customTrigger?: string | (() => VNodeChild);
    customMenu?: string | ((options: {
        selectedKeys: VKey[];
        setSelectedKeys: (value: VKey[]) => void;
    }) => VNodeChild);
}
export interface TableSticky {
    offsetTop?: number;
    offsetBottom?: number;
    offsetScroll?: number;
    container?: Window | HTMLElement;
}
export interface TableEmptyCellOptions<T = any, K = VKey> {
    column: TableColumn<T, K>;
    record: T;
    rowIndex: number;
}
/** private components */
export declare const tableHeadRowProps: {
    readonly columns: {
        readonly type: PropType<TableColumnMergedExtra[]>;
        readonly required: true;
    };
};
export type TableHeadRowProps = ExtractInnerPropTypes<typeof tableHeadRowProps>;
export declare const tableHeadCellProps: {
    readonly column: {
        readonly type: PropType<TableColumnMerged>;
        readonly required: true;
    };
};
export type TableHeadCellProps = ExtractInnerPropTypes<typeof tableHeadCellProps>;
export declare const tableBodyRowProps: {
    readonly expanded: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly rowIndex: {
        readonly type: NumberConstructor;
        readonly required: true;
    };
    readonly level: {
        readonly type: NumberConstructor;
        readonly default: undefined;
    };
    readonly hasPrevSibling: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly hasNextSibling: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly showLineIndentIndexList: {
        readonly type: PropType<number[]>;
        readonly default: () => never[];
    };
    readonly hasChildren: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly record: {
        readonly type: PropType<any>;
        readonly required: true;
    };
    readonly rowData: {
        readonly type: PropType<FlattedData>;
        readonly required: true;
    };
    readonly rowKey: {
        readonly type: PropType<VKey>;
        readonly required: true;
    };
};
export type TableBodyRowProps = ExtractInnerPropTypes<typeof tableBodyRowProps>;
export declare const tableBodyCellProps: {
    readonly column: {
        readonly type: PropType<TableColumnMerged>;
        readonly required: true;
    };
    readonly isLast: {
        readonly type: BooleanConstructor;
        readonly required: true;
    };
    readonly colSpan: NumberConstructor;
    readonly rowSpan: NumberConstructor;
};
export type TableBodyCellProps = ExtractInnerPropTypes<typeof tableBodyCellProps>;
export declare const tableMeasureCellProps: {
    readonly cellKey: {
        readonly type: PropType<VKey>;
        readonly required: true;
    };
    readonly changeColumnWidth: {
        readonly type: PropType<(key: VKey, width: number | false) => void>;
        readonly required: true;
    };
};
export type TableMeasureCellProps = ExtractInnerPropTypes<typeof tableMeasureCellProps>;
export declare const tableFilterableTriggerProps: {
    readonly activeFilterBy: {
        readonly type: PropType<VKey[]>;
        readonly required: true;
    };
    readonly filterable: {
        readonly type: PropType<TableColumnFilterable<any>>;
        readonly required: true;
    };
    readonly onUpdateFilterBy: {
        readonly type: PropType<(filterBy: VKey[]) => void>;
        readonly required: true;
    };
};
export type TableFilterableTriggerProps = ExtractInnerPropTypes<typeof tableFilterableTriggerProps>;
export declare const tableSortableTriggerProps: {
    readonly activeOrderBy: {
        readonly type: PropType<TableColumnSortOrder>;
        readonly default: undefined;
    };
    readonly sortable: {
        readonly type: PropType<TableColumnSortable<any>>;
        readonly required: true;
    };
};
export type TableSortableTriggerProps = ExtractInnerPropTypes<typeof tableSortableTriggerProps>;
