import { computed, defineComponent, inject, createVNode, shallowRef, watch, onMounted, onUpdated, onBeforeUnmount, normalizeClass, watchEffect, nextTick, Transition, toRaw, mergeProps, withDirectives, vShow, ref, provide } from "vue";
import { isNil, isString } from "lodash-es";
import { flattenNode, uniqueId, callEmit, useState, useKey, convertElement, convertCssPixel, useControlledProp } from "@idux/cdk/utils";
import { useGlobalConfig } from "@idux/components/config";
import { useThemeToken } from "@idux/components/theme";
import { CdkDndSortableItem, useDndAutoScroll, reorderList, CdkDndSortable } from "@idux/cdk/dnd";
import { IxButton } from "@idux/components/button";
import { IxIcon } from "@idux/components/icon";
import { IxPopover } from "@idux/components/popover";
import { IxRow, IxCol } from "@idux/components/grid";
import { IxInput } from "@idux/components/input";
import { IxSelectPanel } from "@idux/components/select";
import { useResizeObserver } from "@idux/cdk/resize";
import { useScroll } from "@idux/cdk/scroll";
const tabKey = Symbol("IxTab");
const Tab = () => {
};
Tab.displayName = "IxTab";
Tab[tabKey] = true;
function useDataSource(props, slots) {
  return computed(() => {
    var _a, _b;
    return (_b = props.dataSource) != null ? _b : convertData((_a = slots.default) == null ? void 0 : _a.call(slots));
  });
}
function convertData(nodes) {
  const convertedData = [];
  flattenNode(nodes, { key: tabKey }).forEach((node) => {
    var _a, _b;
    const props = (_a = node.props) != null ? _a : {};
    const slots = (_b = node.children) != null ? _b : {};
    const { key = uniqueId("__IDUX_TAB_"), closable, content, disabled, forceRender, title, ...rest } = props;
    const { title: customTitle, default: customContent } = slots;
    const option = {
      key,
      content,
      closable: closable === "" || closable,
      disabled: disabled === "" || disabled,
      forceRender: forceRender === "" || forceRender,
      title,
      customTitle,
      customContent,
      ...rest
    };
    convertedData.push(option);
  });
  return convertedData;
}
const tabsToken = Symbol("tabsToken");
var AddBtn = /* @__PURE__ */ defineComponent({
  setup(_, {
    slots
  }) {
    const {
      props,
      mergedPrefixCls
    } = inject(tabsToken);
    const handleAdd = () => {
      callEmit(props.onAdd);
    };
    return () => {
      var _a, _b;
      const prefixCls = mergedPrefixCls.value;
      return createVNode("span", {
        "class": `${prefixCls}-nav-add-btn`,
        "onClick": handleAdd
      }, [(_b = (_a = slots.addIcon) == null ? void 0 : _a.call(slots)) != null ? _b : createVNode(IxIcon, {
        "name": "plus"
      }, null)]);
    };
  }
});
const tabsProps = {
  selectedKey: { type: [Number, String, Symbol], default: void 0 },
  allTabsPanelOpen: { type: Boolean, default: void 0 },
  addable: { type: Boolean, default: false },
  closable: { type: Boolean, default: false },
  dataSource: { type: Array, default: void 0 },
  dndSortable: { type: [Boolean, Object], default: false },
  forceRender: { type: Boolean, default: false },
  mode: { type: String, default: "default" },
  placement: { type: String, default: "top" },
  size: { type: String, default: void 0 },
  showAllTabsPanel: { type: Boolean, default: void 0 },
  type: { type: String, default: "card" },
  "onUpdate:selectedKey": [Function, Array],
  "onUpdate:allTabsPanelOpen": [Function, Array],
  onAdd: [Function, Array],
  onClose: [Function, Array],
  onTabClick: [Function, Array],
  onBeforeLeave: [Function, Array],
  onDndSortReorder: [Function, Array],
  onDndSortChange: [Function, Array]
};
const tabNavProps = {
  closable: { type: Boolean, default: void 0 },
  disabled: { type: Boolean, default: void 0 },
  selected: { type: Boolean, default: void 0 },
  title: { type: String, default: void 0 },
  onSelected: { type: Function, required: true }
};
const tabPaneProps = {
  content: { type: String, default: void 0 },
  forceRender: { type: Boolean, default: void 0 },
  selected: { type: Boolean, default: void 0 }
};
const moreSelectPaneProps = {
  dataSource: { type: Array, default: () => [] },
  visible: { type: Boolean, default: false },
  onSortReorder: Function,
  _virtualScrollHeight: { type: Number, default: 186 },
  _onDragStart: Function,
  _onDrop: Function
};
var MoreSelectPane = /* @__PURE__ */ defineComponent({
  props: moreSelectPaneProps,
  setup(props, {
    slots
  }) {
    const {
      props: tabsProps2,
      mergedPrefixCls,
      mergedDndSortable,
      selectedKey,
      handleTabClose,
      setSelectedKey
    } = inject(tabsToken);
    const [searchValue, setSearchValue] = useState("");
    const searchInputRef = shallowRef();
    const filteredDataSource = computed(() => {
      return props.dataSource.filter((data) => {
        return String(data.label).indexOf(searchValue.value) !== -1;
      });
    });
    const handleInput = (evt) => {
      const inputValue = evt.target.value;
      setSearchValue(inputValue);
    };
    const handleClear = () => {
      setSearchValue("");
    };
    const handleClose = (evt, key, disabled) => {
      if (!disabled) {
        evt.stopPropagation();
        handleTabClose(key);
      }
    };
    const handleSelectChange = (data) => {
      setSelectedKey(data.key);
    };
    const handleSortReorder = (reorderInfo) => {
      if (!props.onSortReorder) {
        return;
      }
      if (filteredDataSource.value.length === props.dataSource.length) {
        props.onSortReorder(reorderInfo);
      } else {
        const {
          sourceIndex,
          targetIndex,
          ...rest
        } = reorderInfo;
        const sourceKey = filteredDataSource.value[sourceIndex].key;
        const targetKey = filteredDataSource.value[targetIndex].key;
        const mergedSourceIndex = props.dataSource.findIndex((data) => data.key === sourceKey);
        const mergedTargetIndex = props.dataSource.findIndex((data) => data.key === targetKey);
        props.onSortReorder({
          sourceIndex: mergedSourceIndex,
          targetIndex: mergedTargetIndex,
          ...rest
        });
      }
    };
    const optionLabelRender = (data) => {
      var _a, _b;
      const {
        key,
        label,
        disabled,
        customTitle
      } = data;
      const titleSlot = (_b = (_a = isString(customTitle) ? slots[customTitle] : customTitle) != null ? _a : slots.allTabsPanelLabel) != null ? _b : slots.title;
      return createVNode(IxRow, null, {
        default: () => [createVNode(IxCol, {
          "flex": "auto"
        }, {
          default: () => {
            var _a2;
            return [(_a2 = titleSlot == null ? void 0 : titleSlot({
              key,
              disabled,
              selected: false,
              title: label,
              overflowed: true
            })) != null ? _a2 : label];
          }
        }), tabsProps2.closable && createVNode(IxCol, {
          "flex": "none"
        }, {
          default: () => [createVNode(IxIcon, {
            "class": `${mergedPrefixCls.value}-nav-close-icon`,
            "name": "close",
            "onClick": (evt) => handleClose(evt, key, disabled)
          }, null)]
        })]
      });
    };
    watch(() => props.visible, (visible) => {
      if (visible) {
        setSearchValue("");
        setTimeout(() => {
          var _a, _b;
          (_b = (_a = searchInputRef.value) == null ? void 0 : _a.focus) == null ? void 0 : _b.call(_a);
        });
      }
    }, {
      flush: "post",
      immediate: true
    });
    return () => {
      return createVNode("div", null, [createVNode(IxInput, {
        "autofocus": true,
        "ref": searchInputRef,
        "size": "sm",
        "suffix": "search",
        "clearable": true,
        "value": searchValue.value,
        "onInput": handleInput,
        "onClear": handleClear
      }, null), createVNode(IxSelectPanel, {
        "selectedKeys": !isNil(selectedKey.value) ? [selectedKey.value] : void 0,
        "dataSource": filteredDataSource.value,
        "dndSortable": mergedDndSortable.value,
        "onDndSortReorder": handleSortReorder,
        "onOptionClick": handleSelectChange,
        "_virtualScrollHeight": props._virtualScrollHeight,
        "_onDragStart": props._onDragStart,
        "_onDrop": props._onDrop
      }, {
        optionLabel: optionLabelRender
      })]);
    };
  }
});
function getMarginSize(el, isHorizontal) {
  const style = window.getComputedStyle(el);
  if (isHorizontal) {
    const marginLeft = parseFloat(style.marginLeft) || 0;
    const marginRight = parseFloat(style.marginRight) || 0;
    return marginLeft + marginRight;
  } else {
    const marginTop = parseFloat(style.marginTop) || 0;
    const marginBottom = parseFloat(style.marginBottom) || 0;
    return marginTop + marginBottom;
  }
}
var TabNav = /* @__PURE__ */ defineComponent({
  name: "IxTabNav",
  props: tabNavProps,
  setup(props, {
    slots
  }) {
    const key = useKey();
    const {
      props: tabsProps2,
      mergedPrefixCls,
      handleTabClick,
      handleTabClose,
      navAttrs,
      isHorizontal
    } = inject(tabsToken);
    const prefixCls = computed(() => `${mergedPrefixCls.value}-nav-tab`);
    const mergedClosable = computed(() => {
      var _a;
      return (_a = props.closable) != null ? _a : tabsProps2.closable;
    });
    const sizeProp = computed(() => isHorizontal.value ? "offsetWidth" : "offsetHeight");
    const offsetProp = computed(() => isHorizontal.value ? "offsetLeft" : "offsetTop");
    const elementRef = shallowRef();
    const setNavAttr = (el) => {
      let attr = void 0;
      if (el) {
        attr = {
          size: el[sizeProp.value] + getMarginSize(el, isHorizontal.value),
          offset: el[offsetProp.value]
        };
      }
      navAttrs.value[key] = attr;
    };
    onMounted(() => setNavAttr(convertElement(elementRef.value)));
    onUpdated(() => setNavAttr(convertElement(elementRef.value)));
    onBeforeUnmount(() => setNavAttr(void 0));
    const classes = computed(() => {
      const {
        disabled,
        selected
      } = props;
      return normalizeClass({
        [`${prefixCls.value}`]: true,
        [`${prefixCls.value}-disabled`]: disabled,
        [`${prefixCls.value}-selected`]: selected
      });
    });
    watchEffect(() => {
      const element = convertElement(elementRef.value);
      if (element && props.selected) {
        props.onSelected(element);
      }
    });
    const handleClick = (evt) => {
      if (!props.disabled) {
        handleTabClick(key, evt);
      }
    };
    const handleClose = (evt) => {
      if (!props.disabled) {
        evt.stopPropagation();
        handleTabClose(key);
      }
    };
    return () => {
      const titleNode = slots.title ? slots.title({
        key,
        disabled: props.disabled,
        selected: props.selected,
        title: props.title,
        overflowed: false
      }) : props.title;
      const contentNodes = [createVNode("span", {
        "class": `${prefixCls.value}-label`
      }, [titleNode]), mergedClosable.value && createVNode("span", {
        "class": `${mergedPrefixCls.value}-nav-close-icon`,
        "onClick": handleClose
      }, [createVNode(IxIcon, {
        "name": "close"
      }, null)])];
      return tabsProps2.dndSortable ? createVNode(CdkDndSortableItem, {
        "ref": elementRef,
        "class": classes.value,
        "onClick": handleClick,
        "itemKey": key,
        "canDrag": !props.disabled
      }, {
        default: () => [contentNodes]
      }) : createVNode("div", {
        "ref": elementRef,
        "class": classes.value,
        "onClick": handleClick
      }, [contentNodes]);
    };
  }
});
function useNavListScroll(navListRef, navListInnerRef, isHorizontal, selectedNavSize, selectedNavOffset, navAttrs, closedKeys) {
  const [navListSize, setNavListSize] = useState(0);
  const [hasScroll, setHasScroll] = useState(false);
  const calcHasScroll = () => {
    if (!navListRef.value) {
      setHasScroll(false);
      return;
    }
    const { scrollWidth, scrollHeight, clientWidth, clientHeight } = navListRef.value;
    setHasScroll(isHorizontal.value ? scrollWidth > clientWidth : scrollHeight > clientHeight);
  };
  const {
    init,
    destroy,
    update,
    syncScroll,
    scrollLeft,
    scrollTop,
    scrolledLeft,
    scrolledRight,
    scrolledTop,
    scrolledBottom
  } = useScroll(navListRef, { updateOnResize: false, simulatedScroll: true });
  useResizeObserver(navListRef, (entry) => {
    nextTick(() => {
      const target = entry.target;
      setNavListSize(target[isHorizontal.value ? "offsetWidth" : "offsetHeight"]);
      update();
      calcHasScroll();
    });
  });
  useResizeObserver(navListInnerRef, () => {
    nextTick(() => {
      update();
      calcHasScroll();
    });
  });
  const scrollOffset = computed(() => isHorizontal.value ? scrollLeft.value : scrollTop.value);
  const scrolledStart = computed(() => isHorizontal.value ? scrolledLeft.value : scrolledTop.value);
  const scrolledEnd = computed(() => isHorizontal.value ? scrolledRight.value : scrolledBottom.value);
  const offsetSequence = computed(
    () => Object.values(navAttrs.value).map((attr) => attr == null ? void 0 : attr.offset).filter((offset) => !isNil(offset)).sort((v1, v2) => v1 - v2)
  );
  const findPageOffset = (offset, pre2 = false) => {
    const offsets = offsetSequence.value;
    let index = -1;
    for (let i = pre2 ? offsets.length - 1 : 0; pre2 ? i >= -1 : i <= offsets.length; pre2 ? i-- : i++) {
      const currentOffset = offsets[i];
      index = i;
      if (!isNil(currentOffset) && (pre2 ? currentOffset <= offset : currentOffset >= offset)) {
        break;
      }
    }
    if (index !== 0 && index !== offsets.length - 1) {
      index = pre2 ? index + 1 : index - 1;
    }
    index = Math.min(Math.max(index, 0), offsets.length - 1);
    return index === 0 ? 0 : index === offsets.length - 1 ? Number.MAX_SAFE_INTEGER : offsets[index];
  };
  const setScrollOffset = (offset) => {
    syncScroll(
      {
        [isHorizontal.value ? "left" : "top"]: offset
      },
      true
    );
  };
  const syncNavListScrollBySelectedNav = () => {
    if (!hasScroll.value) {
      return;
    }
    const _wrapperSize = navListSize.value;
    const _selectedNavSize = selectedNavSize.value;
    const _scrollOffset = scrollOffset.value;
    const _selectedNavOffset = selectedNavOffset.value;
    if (_selectedNavOffset === offsetSequence.value[offsetSequence.value.length - 1]) {
      setScrollOffset(Number.MAX_SAFE_INTEGER);
      return;
    }
    const selectedNavOffsetFromList = _selectedNavOffset - (_wrapperSize + _scrollOffset);
    const isAfterView = selectedNavOffsetFromList > -_selectedNavSize;
    const isBeforeView = selectedNavOffsetFromList < -_wrapperSize;
    if (isAfterView) {
      setScrollOffset(selectedNavOffsetFromList + _scrollOffset + _selectedNavSize);
    } else if (isBeforeView) {
      setScrollOffset(_selectedNavOffset);
    }
  };
  watch([selectedNavSize, selectedNavOffset], syncNavListScrollBySelectedNav, { flush: "post" });
  watch(closedKeys, (cur, old) => {
    const curSet = new Set(cur);
    const oldSet = new Set(old);
    const closeTabKey = [...curSet].find((item) => !oldSet.has(item));
    if (closeTabKey !== void 0) {
      const closeTabAttr = navAttrs.value[closeTabKey] || { size: 0, offset: 0 };
      const { size: closeTabSize, offset: closeTabOffset } = closeTabAttr;
      let nextTabOffset = 0;
      for (const attr of Object.values(navAttrs)) {
        if (!attr) {
          continue;
        }
        const { offset } = attr;
        if (offset > closeTabOffset) {
          nextTabOffset = offset;
          break;
        }
      }
      const diffOffset = scrollOffset.value - (nextTabOffset ? nextTabOffset - closeTabOffset : closeTabSize);
      setScrollOffset(diffOffset > 0 ? diffOffset : 0);
    }
  });
  onMounted(init);
  onBeforeUnmount(destroy);
  const pre = () => {
    const preOffset = Math.max(scrollOffset.value - navListSize.value, 0);
    const _offset = findPageOffset(preOffset, true);
    if (!isNil(_offset)) {
      setScrollOffset(_offset);
    }
  };
  const next = () => {
    const nextOffset = scrollOffset.value + navListSize.value;
    const _offset = findPageOffset(nextOffset, false);
    if (!isNil(_offset)) {
      setScrollOffset(_offset);
    }
  };
  return {
    scrollOffset,
    hasScroll,
    scrolledStart,
    scrolledEnd,
    pre,
    next
  };
}
function useSelectedNav(props, selectedNavRef, isHorizontal, closedKeys, navAttrs) {
  const [selectedNavSize, setSelectedNavSize] = useState(0);
  const [selectedNavOffset, setSelectedNavOffset] = useState(0);
  useResizeObserver(selectedNavRef, (entry) => {
    nextTick(() => {
      const target = entry.target;
      setSelectedNavSize(target[isHorizontal.value ? "offsetWidth" : "offsetHeight"]);
    });
  });
  const updateSelectedNavOffset = () => {
    nextTick(() => {
      const target = selectedNavRef.value;
      if (target) {
        setSelectedNavOffset(target[isHorizontal.value ? "offsetLeft" : "offsetTop"]);
      }
    });
  };
  onMounted(() => {
    watch([() => props.dataSource, closedKeys, selectedNavRef], updateSelectedNavOffset, { immediate: true });
    watch(navAttrs, updateSelectedNavOffset, { deep: true });
  });
  return {
    selectedNavSize,
    selectedNavOffset
  };
}
var TabNavWrapper = /* @__PURE__ */ defineComponent({
  props: {
    selectedKey: {
      type: [Number, String, Symbol]
    }
  },
  setup(props, {
    slots
  }) {
    const {
      common,
      props: tabsProps2,
      config,
      locale,
      mergedPrefixCls,
      mergedDataSource,
      mergedDndSortable,
      allTabsPanelVisible,
      isHorizontal,
      closedKeys,
      navAttrs,
      setAllTabsPanelVisible
    } = inject(tabsToken);
    const navListRef = shallowRef();
    const navListInnerRef = shallowRef();
    const operationsRef = shallowRef();
    const selectedNavRef = shallowRef();
    const {
      selectedNavSize,
      selectedNavOffset
    } = useSelectedNav(tabsProps2, selectedNavRef, isHorizontal, closedKeys, navAttrs);
    const {
      hasScroll,
      scrolledStart,
      scrolledEnd,
      pre,
      next
    } = useNavListScroll(navListRef, navListInnerRef, isHorizontal, selectedNavSize, selectedNavOffset, navAttrs, closedKeys);
    const autoScrollListRef = computed(() => {
      if (!mergedDndSortable.value) {
        return;
      }
      return mergedDndSortable.value.autoScroll ? navListRef.value : void 0;
    });
    useDndAutoScroll(autoScrollListRef, {
      canScroll: true,
      allowedAxis: "horizontal"
    });
    const showAllTabs = computed(() => {
      var _a;
      return hasScroll.value && ((_a = tabsProps2.showAllTabsPanel) != null ? _a : config.showAllTabsPanel);
    });
    const allNavDataSource = computed(() => {
      const currClosedKeys = closedKeys.value;
      return mergedDataSource.value.filter((data) => !currClosedKeys.includes(data.key));
    });
    const moreNavDataSource = computed(() => {
      return allNavDataSource.value.map((item) => ({
        key: item.key,
        label: item.title,
        disabled: item.disabled,
        customTitle: item.customTitle
      }));
    });
    watch(hasScroll, (_hasScroll) => {
      if (!_hasScroll) {
        setAllTabsPanelVisible(false);
      }
    });
    const classes = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [`${prefixCls}-nav-wrapper`]: true,
        [`${prefixCls}-nav-wrapper-has-scroll`]: hasScroll.value
      });
    });
    const navBarStyle = computed(() => {
      const size = convertCssPixel(selectedNavSize.value);
      const offset = convertCssPixel(selectedNavOffset.value);
      if (isHorizontal.value) {
        return {
          width: size,
          left: offset
        };
      } else {
        return {
          height: size,
          top: offset
        };
      }
    });
    const popoverPlacement = computed(() => {
      const {
        placement
      } = tabsProps2;
      switch (placement) {
        case "top":
          return "bottomEnd";
        case "bottom":
          return "topStart";
        case "start":
          return "rightEnd";
        case "end":
          return "leftEnd";
        default:
          return "bottomStart";
      }
    });
    let popoverHideDelayTmr;
    let popoverLockTmr;
    let popoverLocked = false;
    watch(allNavDataSource, () => {
      clearTimeout(popoverLockTmr);
      popoverLocked = true;
      popoverLockTmr = setTimeout(() => {
        popoverLocked = false;
      }, 50);
    });
    const handleSelectedNavChange = (element) => {
      selectedNavRef.value = element;
    };
    const handleMouseenter = () => {
      clearTimeout(popoverHideDelayTmr);
      setAllTabsPanelVisible(true);
    };
    const handleMouseleave = () => {
      if (popoverLocked) {
        return;
      }
      clearTimeout(popoverHideDelayTmr);
      popoverHideDelayTmr = setTimeout(() => {
        setAllTabsPanelVisible(false);
      }, 100);
    };
    const handleDragStart = () => {
      popoverLocked = true;
    };
    const handleDrop = (args) => {
      const {
        data
      } = args;
      popoverLocked = false;
      if (!data) {
        handleMouseleave();
      }
    };
    const handleSortReorder = (reorderInfo) => {
      callEmit(tabsProps2.onDndSortReorder, reorderInfo);
    };
    const handleNavListSortChange = (newData, oldData) => {
      callEmit(tabsProps2.onDndSortChange, newData, oldData);
    };
    const handleSelectPanelSortReorder = (reorderInfo) => {
      const {
        sourceIndex,
        targetIndex,
        ...rest
      } = reorderInfo;
      const sourceKey = moreNavDataSource.value[sourceIndex].key;
      const targetKey = moreNavDataSource.value[targetIndex].key;
      const oldData = toRaw(mergedDataSource.value);
      const mergedSourceIndex = oldData.findIndex((data) => data.key === sourceKey);
      const mergedTargetIndex = oldData.findIndex((data) => data.key === targetKey);
      const mergedReorderInfo = {
        sourceIndex: mergedSourceIndex,
        targetIndex: mergedTargetIndex,
        ...rest
      };
      const newData = reorderList(oldData, mergedReorderInfo);
      callEmit(tabsProps2.onDndSortReorder, mergedReorderInfo);
      callEmit(tabsProps2.onDndSortChange, newData, oldData);
    };
    const renderContent = () => {
      const prefixCls = mergedPrefixCls.value;
      const {
        selectedKey
      } = props;
      const {
        addable,
        type
      } = tabsProps2;
      const contentNodes = createVNode("div", {
        "ref": navListRef,
        "class": `${prefixCls}-nav-list`
      }, [createVNode("div", {
        "ref": navListInnerRef,
        "class": `${prefixCls}-nav-list-inner`
      }, [allNavDataSource.value.map((data) => {
        const {
          key,
          content,
          customContent,
          customTitle = "title",
          ...navProps
        } = data;
        const titleSlot = isString(customTitle) ? slots[customTitle] : customTitle;
        return createVNode(TabNav, mergeProps(navProps, {
          "onSelected": handleSelectedNavChange,
          "key": key,
          "selected": selectedKey === key
        }), {
          title: titleSlot
        });
      }), addable && !hasScroll.value && createVNode(AddBtn, null, slots), isHorizontal.value && hasScroll.value && createVNode("div", {
        "class": `${prefixCls}-nav-list-gap-filler`
      }, null), type === "line" && createVNode("div", {
        "class": `${prefixCls}-nav-bar`,
        "style": navBarStyle.value
      }, null)])]);
      return mergedDndSortable.value ? createVNode(CdkDndSortable, {
        "dataSource": mergedDataSource.value,
        "direction": "horizontal",
        "onSortReorder": handleSortReorder,
        "onSortChange": handleNavListSortChange
      }, {
        default: () => [contentNodes]
      }) : contentNodes;
    };
    return () => {
      const {
        addable,
        type
      } = tabsProps2;
      const prefixCls = mergedPrefixCls.value;
      return createVNode("div", {
        "class": classes.value
      }, [createVNode("div", {
        "class": `${prefixCls}-nav`
      }, [createVNode(Transition, {
        "appear": true,
        "name": `${common}-fade`
      }, {
        default: () => [hasScroll.value && !scrolledStart.value && createVNode("button", {
          "class": `${prefixCls}-nav-pre-btn`,
          "onClick": pre
        }, [createVNode("span", {
          "class": `${prefixCls}-nav-btn-trigger`
        }, [createVNode(IxIcon, {
          "name": isHorizontal.value ? "left" : "up"
        }, null)])])]
      }), renderContent(), createVNode(Transition, {
        "appear": true,
        "name": `${common}-fade`
      }, {
        default: () => [hasScroll.value && !scrolledEnd.value && createVNode("button", {
          "class": `${prefixCls}-nav-next-btn`,
          "onClick": next
        }, [createVNode("span", {
          "class": `${prefixCls}-nav-btn-trigger`
        }, [createVNode(IxIcon, {
          "name": isHorizontal.value ? "right" : "down"
        }, null)])])]
      })]), createVNode("div", {
        "ref": operationsRef,
        "class": [`${prefixCls}-nav-operations`, !hasScroll.value && `${prefixCls}-nav-operations-hidden`]
      }, [showAllTabs.value && createVNode("span", {
        "class": `${prefixCls}-nav-operations-item`
      }, [createVNode(IxPopover, {
        "trigger": "manual",
        "placement": popoverPlacement.value,
        "showArrow": false,
        "visible": allTabsPanelVisible.value,
        "props": {
          "onUpdate:visible": setAllTabsPanelVisible
        },
        "class": `${prefixCls}-nav-operations-popover`,
        "onMouseenter": handleMouseenter,
        "onMouseleave": handleMouseleave
      }, {
        default: () => [createVNode(IxButton, {
          "icon": "more",
          "mode": "text",
          "shape": isHorizontal.value ? void 0 : "square",
          "onMouseenter": handleMouseenter,
          "onMouseleave": handleMouseleave
        }, {
          default: () => [isHorizontal.value ? locale.allTabs : ""]
        })],
        content: () => {
          return createVNode(MoreSelectPane, {
            "visible": allTabsPanelVisible.value,
            "dataSource": moreNavDataSource.value,
            "onSortReorder": handleSelectPanelSortReorder,
            "_onDragStart": handleDragStart,
            "_onDrop": handleDrop
          }, slots);
        }
      })]), addable && createVNode("span", {
        "class": `${prefixCls}-nav-operations-item`
      }, [createVNode(AddBtn, null, slots)])]), type !== "segment" && createVNode("div", {
        "class": `${prefixCls}-nav-border`
      }, null)]);
    };
  }
});
var TabPane = /* @__PURE__ */ defineComponent({
  name: "IxTabPane",
  props: tabPaneProps,
  setup(props, {
    slots
  }) {
    const key = useKey();
    const {
      props: tabsProps2,
      mergedPrefixCls
    } = inject(tabsToken);
    const mergedForceRender = computed(() => {
      var _a;
      return (_a = props.forceRender) != null ? _a : tabsProps2.forceRender;
    });
    let rendered = false;
    return () => {
      if (!(rendered || props.selected || mergedForceRender.value)) {
        return null;
      }
      rendered = true;
      const contentNode = slots.content ? slots.content({
        key,
        content: props.content,
        selected: props.selected
      }) : props.content;
      return withDirectives(createVNode("div", {
        "class": `${mergedPrefixCls.value}-pane`
      }, [contentNode]), [[vShow, props.selected]]);
    };
  }
});
function getDefaultThemeTokens(tokens) {
  const { fontSizeSm, fontSizeMd, paddingSizeLg, heightMd, heightLg, borderRadiusSm } = tokens;
  return {
    navFontSizeMd: fontSizeSm,
    navFontSizeLg: fontSizeMd,
    navTabPadding: `0 ${paddingSizeLg}px`,
    navTabMinWidth: 72,
    navTabHeight: heightLg,
    navBarHeight: 2,
    segmentNavHeight: heightMd,
    borderRadius: borderRadiusSm,
    panelPaddingSize: paddingSizeLg
  };
}
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens) : getDefaultThemeTokens(tokens);
};
var Tabs = /* @__PURE__ */ defineComponent({
  name: "IxTabs",
  props: tabsProps,
  setup(props, {
    slots
  }) {
    const common = useGlobalConfig("common");
    const locale = useGlobalConfig("locale");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("tabs");
    registerToken(getThemeTokens);
    const config = useGlobalConfig("tabs");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-tabs`);
    const mergedSize = computed(() => {
      var _a;
      return (_a = props.size) != null ? _a : config.size;
    });
    const mergedDndSortable = computed(() => {
      var _a, _b;
      const dndSortable = props.dndSortable;
      if (!dndSortable) {
        return false;
      }
      if (dndSortable === true) {
        return {
          autoScroll: true,
          dragHandle: false
        };
      }
      return {
        autoScroll: (_a = dndSortable.autoScroll) != null ? _a : true,
        dragHandle: (_b = dndSortable.dragHandle) != null ? _b : false
      };
    });
    const mergedDataSource = useDataSource(props, slots);
    const horizontalPlacement = ["top", "bottom"];
    const isHorizontal = computed(() => horizontalPlacement.includes(props.placement));
    const [selectedKey, setSelectedKey] = useControlledProp(props, "selectedKey");
    const [allTabsPanelVisible, setAllTabsPanelVisible] = useControlledProp(props, "allTabsPanelOpen", false);
    const [closedKeys, setClosedKeys] = useState([]);
    const navAttrs = ref({});
    const handleTabClick = async (key, evt) => {
      const result = await callEmit(props.onBeforeLeave, key, selectedKey.value);
      if (result !== false) {
        setSelectedKey(key);
        callEmit(props.onTabClick, key, evt);
      }
    };
    const handleTabClose = async (key) => {
      const result = await callEmit(props.onClose, key);
      if (result !== false) {
        const currSelectedKey = selectedKey.value;
        const currClosedKeys = closedKeys.value;
        if (key === selectedKey.value) {
          setSelectedKey(getNextSelectedKey(mergedDataSource.value, currClosedKeys, currSelectedKey));
        }
        setClosedKeys([...currClosedKeys, key]);
      }
    };
    provide(tabsToken, {
      common,
      config,
      props,
      locale: locale.tabs,
      mergedPrefixCls,
      mergedDataSource,
      mergedDndSortable,
      allTabsPanelVisible,
      isHorizontal,
      closedKeys,
      navAttrs,
      selectedKey,
      handleTabClick,
      handleTabClose,
      setSelectedKey,
      setAllTabsPanelVisible
    });
    const classes = computed(() => {
      const {
        type,
        placement,
        mode
      } = props;
      const prefixCls = mergedPrefixCls.value;
      const size = mergedSize.value;
      return normalizeClass({
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [prefixCls]: true,
        [`${prefixCls}-${size}`]: true,
        [`${prefixCls}-${type}`]: true,
        [`${prefixCls}-nav-${placement}`]: placement === "top" || type === "line",
        [`${prefixCls}-nav-${mode}`]: type === "segment"
      });
    });
    return () => {
      var _a;
      const dataSource = mergedDataSource.value;
      const currClosedKeys = closedKeys.value;
      const currSelectedKey = (_a = selectedKey.value) != null ? _a : getNextSelectedKey(dataSource, currClosedKeys);
      return createVNode("div", {
        "class": classes.value
      }, [createVNode(TabNavWrapper, {
        "selectedKey": currSelectedKey
      }, slots), createVNode("div", {
        "class": `${mergedPrefixCls.value}-pane-wrapper`
      }, [dataSource.map((data) => {
        const {
          key,
          content,
          forceRender,
          customContent = "content"
        } = data;
        const contentSlot = isString(customContent) ? slots[customContent] : customContent;
        return createVNode(TabPane, {
          "key": key,
          "content": content,
          "forceRender": forceRender,
          "selected": key === currSelectedKey
        }, {
          content: contentSlot
        });
      })])]);
    };
  }
});
function getNextSelectedKey(dataSource, closedKeys, currSelectedKey) {
  var _a;
  const isValidNext = (data) => !data.disabled && !closedKeys.includes(data.key);
  const currSelectedIndex = isNil(currSelectedKey) ? -1 : dataSource.findIndex((item) => item.key === currSelectedKey);
  if (currSelectedIndex === -1) {
    return (_a = dataSource.find(isValidNext)) == null ? void 0 : _a.key;
  }
  for (let index = currSelectedIndex + 1; index < dataSource.length; index++) {
    const data = dataSource[index];
    if (isValidNext(data)) {
      return data.key;
    }
  }
  for (let index = currSelectedIndex - 1; index > 0; index--) {
    const data = dataSource[index];
    if (isValidNext(data)) {
      return data.key;
    }
  }
  return;
}
const IxTabs = Tabs;
const IxTab = Tab;
export { IxTab, IxTabs, getThemeTokens as getTabsThemeTokens };
