/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { DndSortableOnDragStartArgs, DndSortableOnDropArgs, DndSortableReorderInfo } from '@idux/cdk/dnd';
import type { ExtractInnerPropTypes, ExtractPublicPropTypes, MaybeArray, VKey } from '@idux/cdk/utils';
import type { DefineComponent, FunctionalComponent, HTMLAttributes, PropType, VNodeChild } from 'vue';
import { SelectData } from '@idux/components/select';
export type TabsMode = 'default' | 'primary';
export type TabsPlacement = 'top' | 'bottom' | 'start' | 'end';
export type TabsSize = 'lg' | 'md';
export type TabsType = 'card' | 'line' | 'segment';
export interface TabsDndSortable {
    autoScroll?: boolean;
    dragHandle?: boolean | string;
}
export declare const tabsProps: {
    readonly selectedKey: {
        readonly type: PropType<VKey>;
        readonly default: undefined;
    };
    readonly allTabsPanelOpen: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly addable: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly closable: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly dataSource: {
        readonly type: PropType<TabsData<VKey>[]>;
        readonly default: undefined;
    };
    readonly dndSortable: {
        readonly type: PropType<boolean | TabsDndSortable>;
        readonly default: false;
    };
    readonly forceRender: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly mode: {
        readonly type: PropType<TabsMode>;
        readonly default: "default";
    };
    readonly placement: {
        readonly type: PropType<TabsPlacement>;
        readonly default: "top";
    };
    readonly size: {
        readonly type: PropType<TabsSize>;
        readonly default: undefined;
    };
    readonly showAllTabsPanel: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly type: {
        readonly type: PropType<TabsType>;
        readonly default: "card";
    };
    readonly 'onUpdate:selectedKey': PropType<MaybeArray<(key: any) => void>>;
    readonly 'onUpdate:allTabsPanelOpen': PropType<MaybeArray<(opened: boolean) => void>>;
    readonly onAdd: PropType<MaybeArray<() => void | boolean | Promise<boolean>>>;
    readonly onClose: PropType<MaybeArray<(key: any) => void | boolean | Promise<boolean>>>;
    /**
     * @deprecated
     */
    readonly onTabClick: PropType<MaybeArray<(key: any, evt: Event) => void>>;
    readonly onBeforeLeave: PropType<MaybeArray<(key: any, oldKey?: any) => void | boolean | Promise<boolean>>>;
    readonly onDndSortReorder: PropType<MaybeArray<(reorderInfo: DndSortableReorderInfo) => void>>;
    readonly onDndSortChange: PropType<MaybeArray<(newDataSource: TabsData[], oldDataSource: TabsData[]) => void>>;
};
export type TabsProps = ExtractInnerPropTypes<typeof tabsProps>;
export type TabsPublicProps = Omit<ExtractPublicPropTypes<typeof tabsProps>, 'onTabClick'>;
export type TabsComponent = DefineComponent<Omit<HTMLAttributes, keyof TabsPublicProps> & TabsPublicProps>;
export type TabsInstance = InstanceType<DefineComponent<TabsProps>>;
export interface TabProps {
    closable?: boolean;
    content?: string;
    disabled?: boolean;
    forceRender?: boolean;
    title?: string;
}
export type TabComponent = FunctionalComponent<Omit<HTMLAttributes, keyof TabProps> & TabProps>;
export interface TabsData<K = VKey> extends TabProps {
    key: K;
    customContent?: string | ((data: {
        key: VKey;
        content?: string;
        selected?: boolean;
    }) => VNodeChild);
    customTitle?: string | ((data: {
        key: VKey;
        disabled?: boolean;
        selected?: boolean;
        title?: string;
        overflowed: boolean;
    }) => VNodeChild);
    [key: string]: any;
}
export declare const tabNavProps: {
    readonly closable: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly selected: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly title: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly onSelected: {
        readonly type: FunctionConstructor;
        readonly required: true;
    };
};
export declare const tabPaneProps: {
    readonly content: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly forceRender: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly selected: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
};
export declare const moreSelectPaneProps: {
    readonly dataSource: {
        readonly type: PropType<SelectData<VKey>[]>;
        readonly default: () => never[];
    };
    readonly visible: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly onSortReorder: PropType<(reorderInfo: DndSortableReorderInfo) => void>;
    readonly _virtualScrollHeight: {
        readonly type: NumberConstructor;
        readonly default: 186;
    };
    readonly _onDragStart: PropType<(args: DndSortableOnDragStartArgs) => void>;
    readonly _onDrop: PropType<(args: DndSortableOnDropArgs) => void>;
};
