/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { ExtractInnerPropTypes, ExtractPublicPropTypes, MaybeArray, VKey } from '@idux/cdk/utils';
import type { DefineComponent, HTMLAttributes, PropType } from 'vue';
export type TagShape = 'round' | 'rect';
export type TagStatus = 'normal' | 'success' | 'info' | 'warning' | 'risk' | 'error' | 'fatal';
export declare const tagProps: {
    readonly bordered: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly filled: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly icon: StringConstructor;
    readonly number: NumberConstructor;
    readonly shape: PropType<TagShape>;
    readonly status: {
        readonly type: PropType<TagStatus>;
        readonly default: "normal";
    };
};
export type TagProps = ExtractInnerPropTypes<typeof tagProps>;
export type TagPublicProps = ExtractPublicPropTypes<typeof tagProps>;
export type TagComponent = DefineComponent<Omit<HTMLAttributes, keyof TagPublicProps> & TagPublicProps>;
export type TagInstance = InstanceType<DefineComponent<TagProps>>;
export interface TagData<K = VKey> extends Omit<TagPublicProps, 'shape'> {
    key?: K;
    label?: string;
}
export declare const tagGroupProps: {
    readonly activeKeys: {
        readonly type: PropType<VKey[]>;
        readonly default: () => VKey[];
    };
    readonly bordered: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly clickable: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly closable: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly closeIcon: {
        readonly type: StringConstructor;
        readonly default: "close";
    };
    readonly filled: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly dataSource: PropType<TagData<VKey>[]>;
    readonly gap: PropType<string | number>;
    readonly wrap: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly shape: PropType<TagShape>;
    readonly 'onUpdate:activeKeys': PropType<MaybeArray<(keys: any[]) => void>>;
    readonly onClick: PropType<MaybeArray<(key: any, evt: MouseEvent) => void>>;
    readonly onClose: PropType<MaybeArray<(key: any) => void | boolean | Promise<boolean>>>;
};
export type TagGroupProps = ExtractInnerPropTypes<typeof tagGroupProps>;
export type TagGroupPublicProps = ExtractPublicPropTypes<typeof tagGroupProps>;
export type TagGroupComponent = DefineComponent<Omit<HTMLAttributes, keyof TagGroupPublicProps> & TagGroupPublicProps>;
export type TagGroupInstance = InstanceType<DefineComponent<TagGroupProps>>;
