/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { ExtractInnerPropTypes, ExtractPublicPropTypes, MaybeArray } from '@idux/cdk/utils';
import type { TooltipProps } from '@idux/components/tooltip';
import type { Component, DefineComponent, HTMLAttributes, PropType, VNodeChild } from 'vue';
export type TextCopyIconRenderer = (options: {
    copied: boolean;
}) => VNodeChild | string;
export type TextExpandIconRenderer = (options: {
    expanded: boolean;
}) => VNodeChild | string;
export interface TextEllipsis {
    rows?: number;
    expandable?: boolean;
}
export declare const textProps: {
    readonly copyable: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly copyIcon: {
        readonly type: PropType<string | [string, string] | TextCopyIconRenderer>;
        readonly default: undefined;
    };
    readonly copyTooltip: {
        readonly type: PropType<boolean | TooltipProps | [string, string] | [TooltipProps, TooltipProps]>;
        readonly default: true;
    };
    readonly expanded: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    /**
     * @deprecated please use `ellipsis` instead
     */
    readonly expandable: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly expandIcon: {
        readonly type: PropType<string | [string, string] | TextExpandIconRenderer>;
        readonly default: undefined;
    };
    readonly ellipsis: {
        readonly type: PropType<boolean | TextEllipsis>;
        readonly default: undefined;
    };
    /**
     * @deprecated please use `ellipsis` instead
     */
    readonly lineClamp: {
        readonly type: PropType<string | number>;
        readonly default: undefined;
    };
    readonly tag: {
        readonly type: PropType<string | Component<any, any, any, import("vue").ComputedOptions, import("vue").MethodOptions, {}, any>>;
        readonly default: "span";
    };
    readonly tooltip: {
        readonly type: PropType<boolean | TooltipProps | "native">;
        readonly default: true;
    };
    readonly onCopy: PropType<MaybeArray<(success: boolean, text: string) => void>>;
    readonly 'onUpdate:expanded': PropType<MaybeArray<(expanded: boolean) => void>>;
};
export type TextProps = ExtractInnerPropTypes<typeof textProps>;
export type TextPublicProps = ExtractPublicPropTypes<typeof textProps>;
export type TextComponent = DefineComponent<Omit<HTMLAttributes, keyof TextPublicProps> & TextPublicProps>;
export type TextInstance = InstanceType<DefineComponent<TextProps>>;
