/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { ExtractInnerPropTypes, ExtractPublicPropTypes } from '@idux/cdk/utils';
import type { DefineComponent, PropType, TextareaHTMLAttributes } from 'vue';
export declare const textareaProps: {
    readonly autoRows: {
        readonly type: PropType<boolean | TextareaAutoRows>;
        readonly default: undefined;
    };
    readonly computeCount: {
        readonly type: PropType<(value: string) => string>;
        readonly default: undefined;
    };
    readonly maxCount: {
        readonly type: PropType<string | number>;
        readonly default: undefined;
    };
    readonly resize: {
        readonly type: PropType<TextareaResize>;
        readonly default: undefined;
    };
    readonly showCount: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly control: {
        readonly type: PropType<string | number | (string | number)[] | import("../../../cdk").AbstractControl<any>>;
        readonly default: undefined;
    };
    readonly value: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly clearable: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly clearIcon: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly readonly: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly size: {
        readonly type: PropType<import("../..").FormSize>;
        readonly default: undefined;
    };
    readonly status: PropType<import("../../../cdk").ValidateStatus>;
    readonly trim: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly 'onUpdate:value': PropType<import("@idux/cdk/utils").MaybeArray<(value: string) => void>>;
    readonly onChange: PropType<import("@idux/cdk/utils").MaybeArray<(value: string, oldValue: string) => void>>;
    readonly onClear: PropType<import("@idux/cdk/utils").MaybeArray<(evt: Event) => void>>;
    readonly onCompositionStart: PropType<import("@idux/cdk/utils").MaybeArray<(evt: CompositionEvent) => void>>;
    readonly onCompositionEnd: PropType<import("@idux/cdk/utils").MaybeArray<(evt: CompositionEvent) => void>>;
    readonly onInput: PropType<import("@idux/cdk/utils").MaybeArray<(evt: Event) => void>>;
    readonly onFocus: PropType<import("@idux/cdk/utils").MaybeArray<(evt: FocusEvent) => void>>;
    readonly onBlur: PropType<import("@idux/cdk/utils").MaybeArray<(evt: FocusEvent) => void>>;
};
export type TextareaProps = ExtractInnerPropTypes<typeof textareaProps>;
export type TextareaPublicProps = ExtractPublicPropTypes<typeof textareaProps>;
export interface TextareaBindings {
    focus: (options?: FocusOptions) => void;
    blur: () => void;
}
export type TextareaComponent = DefineComponent<Omit<TextareaHTMLAttributes, keyof TextareaPublicProps> & TextareaPublicProps, TextareaBindings>;
export type TextareaInstance = InstanceType<DefineComponent<TextareaProps, TextareaBindings>>;
export type TextareaResize = 'none' | 'both' | 'horizontal' | 'vertical';
export type TextareaAutoRows = {
    minRows?: number;
    maxRows?: number;
};
