import { computed, reactive, watch, defineComponent, inject, provide, createVNode, Fragment, effectScope } from "vue";
import { tryOnScopeDispose, useState, createSharedComposable, Logger } from "@idux/cdk/utils";
import { merge, pick, isString, kebabCase, isFunction } from "lodash-es";
import { useGlobalConfig } from "@idux/components/config";
import { isBrowser } from "@idux/cdk/platform";
import hash from "@emotion/hash";
function getBaseColors$1() {
  return {
    red: "#f52727",
    orange: "#fa721b",
    brown: "#fdaa1d",
    yellow: "#f8d81a",
    canary: "#fcd200",
    bud: "#8dd118",
    green: "#39c317",
    turquoise: "#20cc94",
    cyan: "#19d2db",
    glacier: "#1ba0f2",
    blue: "#1c6eff",
    indigo: "#2229f6",
    purple: "#7824ff",
    magenta: "#d619be",
    graphite: "#a1a7b3",
    grey: "#a3a3a3",
    white: "#ffffff",
    black: "#000000",
    gold: "#e7aa40",
    silver: "#a9b4c8",
    bronze: "#c97f58"
  };
}
function bound01(n, max) {
  if (isOnePointZero(n)) {
    n = "100%";
  }
  const isPercent = isPercentage(n);
  n = max === 360 ? n : Math.min(max, Math.max(0, parseFloat(n)));
  if (isPercent) {
    n = parseInt(String(n * max), 10) / 100;
  }
  if (Math.abs(n - max) < 1e-6) {
    return 1;
  }
  if (max === 360) {
    n = (n < 0 ? n % max + max : n % max) / parseFloat(String(max));
  } else {
    n = n % max / parseFloat(String(max));
  }
  return n;
}
function clamp01(val) {
  return Math.min(1, Math.max(0, val));
}
function isOnePointZero(n) {
  return typeof n === "string" && n.indexOf(".") !== -1 && parseFloat(n) === 1;
}
function isPercentage(n) {
  return typeof n === "string" && n.indexOf("%") !== -1;
}
function boundAlpha(a) {
  a = parseFloat(a);
  if (isNaN(a) || a < 0 || a > 1) {
    a = 1;
  }
  return a;
}
function convertToPercentage(n) {
  if (Number(n) <= 1) {
    return `${Number(n) * 100}%`;
  }
  return n;
}
function pad2(c) {
  return c.length === 1 ? "0" + c : String(c);
}
function rgbToRgb(r, g, b) {
  return {
    r: bound01(r, 255) * 255,
    g: bound01(g, 255) * 255,
    b: bound01(b, 255) * 255
  };
}
function rgbToHsl(r, g, b) {
  r = bound01(r, 255);
  g = bound01(g, 255);
  b = bound01(b, 255);
  const max = Math.max(r, g, b);
  const min = Math.min(r, g, b);
  let h = 0;
  let s = 0;
  const l = (max + min) / 2;
  if (max === min) {
    s = 0;
    h = 0;
  } else {
    const d = max - min;
    s = l > 0.5 ? d / (2 - max - min) : d / (max + min);
    switch (max) {
      case r:
        h = (g - b) / d + (g < b ? 6 : 0);
        break;
      case g:
        h = (b - r) / d + 2;
        break;
      case b:
        h = (r - g) / d + 4;
        break;
    }
    h /= 6;
  }
  return { h, s, l };
}
function hue2rgb(p, q, t) {
  if (t < 0) {
    t += 1;
  }
  if (t > 1) {
    t -= 1;
  }
  if (t < 1 / 6) {
    return p + (q - p) * (6 * t);
  }
  if (t < 1 / 2) {
    return q;
  }
  if (t < 2 / 3) {
    return p + (q - p) * (2 / 3 - t) * 6;
  }
  return p;
}
function hslToRgb(h, s, l) {
  let r;
  let g;
  let b;
  h = bound01(h, 360);
  s = bound01(s, 100);
  l = bound01(l, 100);
  if (s === 0) {
    g = l;
    b = l;
    r = l;
  } else {
    const q = l < 0.5 ? l * (1 + s) : l + s - l * s;
    const p = 2 * l - q;
    r = hue2rgb(p, q, h + 1 / 3);
    g = hue2rgb(p, q, h);
    b = hue2rgb(p, q, h - 1 / 3);
  }
  return { r: r * 255, g: g * 255, b: b * 255 };
}
function rgbToHsv(r, g, b) {
  r = bound01(r, 255);
  g = bound01(g, 255);
  b = bound01(b, 255);
  const max = Math.max(r, g, b);
  const min = Math.min(r, g, b);
  let h = 0;
  const v = max;
  const d = max - min;
  const s = max === 0 ? 0 : d / max;
  if (max === min) {
    h = 0;
  } else {
    switch (max) {
      case r:
        h = (g - b) / d + (g < b ? 6 : 0);
        break;
      case g:
        h = (b - r) / d + 2;
        break;
      case b:
        h = (r - g) / d + 4;
        break;
    }
    h /= 6;
  }
  return { h, s, v };
}
function hsvToRgb(h, s, v) {
  h = bound01(h, 360) * 6;
  s = bound01(s, 100);
  v = bound01(v, 100);
  const i = Math.floor(h);
  const f = h - i;
  const p = v * (1 - s);
  const q = v * (1 - f * s);
  const t = v * (1 - (1 - f) * s);
  const mod = i % 6;
  const r = [v, q, p, p, t, v][mod];
  const g = [t, v, v, q, p, p][mod];
  const b = [p, p, t, v, v, q][mod];
  return { r: r * 255, g: g * 255, b: b * 255 };
}
function rgbToHex(r, g, b, allow3Char) {
  const hex = [
    pad2(Math.round(r).toString(16)),
    pad2(Math.round(g).toString(16)),
    pad2(Math.round(b).toString(16))
  ];
  if (allow3Char && hex[0].startsWith(hex[0].charAt(1)) && hex[1].startsWith(hex[1].charAt(1)) && hex[2].startsWith(hex[2].charAt(1))) {
    return hex[0].charAt(0) + hex[1].charAt(0) + hex[2].charAt(0);
  }
  return hex.join("");
}
function rgbaToHex(r, g, b, a, allow4Char) {
  const hex = [
    pad2(Math.round(r).toString(16)),
    pad2(Math.round(g).toString(16)),
    pad2(Math.round(b).toString(16)),
    pad2(convertDecimalToHex(a))
  ];
  if (allow4Char && hex[0].startsWith(hex[0].charAt(1)) && hex[1].startsWith(hex[1].charAt(1)) && hex[2].startsWith(hex[2].charAt(1)) && hex[3].startsWith(hex[3].charAt(1))) {
    return hex[0].charAt(0) + hex[1].charAt(0) + hex[2].charAt(0) + hex[3].charAt(0);
  }
  return hex.join("");
}
function cmykToRgb(c, m, y, k) {
  const cConv = c / 100;
  const mConv = m / 100;
  const yConv = y / 100;
  const kConv = k / 100;
  const r = 255 * (1 - cConv) * (1 - kConv);
  const g = 255 * (1 - mConv) * (1 - kConv);
  const b = 255 * (1 - yConv) * (1 - kConv);
  return { r, g, b };
}
function rgbToCmyk(r, g, b) {
  let c = 1 - r / 255;
  let m = 1 - g / 255;
  let y = 1 - b / 255;
  let k = Math.min(c, m, y);
  if (k === 1) {
    c = 0;
    m = 0;
    y = 0;
  } else {
    c = (c - k) / (1 - k) * 100;
    m = (m - k) / (1 - k) * 100;
    y = (y - k) / (1 - k) * 100;
  }
  k *= 100;
  return {
    c: Math.round(c),
    m: Math.round(m),
    y: Math.round(y),
    k: Math.round(k)
  };
}
function convertDecimalToHex(d) {
  return Math.round(parseFloat(d) * 255).toString(16);
}
function convertHexToDecimal(h) {
  return parseIntFromHex(h) / 255;
}
function parseIntFromHex(val) {
  return parseInt(val, 16);
}
function numberInputToObject(color) {
  return {
    r: color >> 16,
    g: (color & 65280) >> 8,
    b: color & 255
  };
}
const names = {
  aliceblue: "#f0f8ff",
  antiquewhite: "#faebd7",
  aqua: "#00ffff",
  aquamarine: "#7fffd4",
  azure: "#f0ffff",
  beige: "#f5f5dc",
  bisque: "#ffe4c4",
  black: "#000000",
  blanchedalmond: "#ffebcd",
  blue: "#0000ff",
  blueviolet: "#8a2be2",
  brown: "#a52a2a",
  burlywood: "#deb887",
  cadetblue: "#5f9ea0",
  chartreuse: "#7fff00",
  chocolate: "#d2691e",
  coral: "#ff7f50",
  cornflowerblue: "#6495ed",
  cornsilk: "#fff8dc",
  crimson: "#dc143c",
  cyan: "#00ffff",
  darkblue: "#00008b",
  darkcyan: "#008b8b",
  darkgoldenrod: "#b8860b",
  darkgray: "#a9a9a9",
  darkgreen: "#006400",
  darkgrey: "#a9a9a9",
  darkkhaki: "#bdb76b",
  darkmagenta: "#8b008b",
  darkolivegreen: "#556b2f",
  darkorange: "#ff8c00",
  darkorchid: "#9932cc",
  darkred: "#8b0000",
  darksalmon: "#e9967a",
  darkseagreen: "#8fbc8f",
  darkslateblue: "#483d8b",
  darkslategray: "#2f4f4f",
  darkslategrey: "#2f4f4f",
  darkturquoise: "#00ced1",
  darkviolet: "#9400d3",
  deeppink: "#ff1493",
  deepskyblue: "#00bfff",
  dimgray: "#696969",
  dimgrey: "#696969",
  dodgerblue: "#1e90ff",
  firebrick: "#b22222",
  floralwhite: "#fffaf0",
  forestgreen: "#228b22",
  fuchsia: "#ff00ff",
  gainsboro: "#dcdcdc",
  ghostwhite: "#f8f8ff",
  goldenrod: "#daa520",
  gold: "#ffd700",
  gray: "#808080",
  green: "#008000",
  greenyellow: "#adff2f",
  grey: "#808080",
  honeydew: "#f0fff0",
  hotpink: "#ff69b4",
  indianred: "#cd5c5c",
  indigo: "#4b0082",
  ivory: "#fffff0",
  khaki: "#f0e68c",
  lavenderblush: "#fff0f5",
  lavender: "#e6e6fa",
  lawngreen: "#7cfc00",
  lemonchiffon: "#fffacd",
  lightblue: "#add8e6",
  lightcoral: "#f08080",
  lightcyan: "#e0ffff",
  lightgoldenrodyellow: "#fafad2",
  lightgray: "#d3d3d3",
  lightgreen: "#90ee90",
  lightgrey: "#d3d3d3",
  lightpink: "#ffb6c1",
  lightsalmon: "#ffa07a",
  lightseagreen: "#20b2aa",
  lightskyblue: "#87cefa",
  lightslategray: "#778899",
  lightslategrey: "#778899",
  lightsteelblue: "#b0c4de",
  lightyellow: "#ffffe0",
  lime: "#00ff00",
  limegreen: "#32cd32",
  linen: "#faf0e6",
  magenta: "#ff00ff",
  maroon: "#800000",
  mediumaquamarine: "#66cdaa",
  mediumblue: "#0000cd",
  mediumorchid: "#ba55d3",
  mediumpurple: "#9370db",
  mediumseagreen: "#3cb371",
  mediumslateblue: "#7b68ee",
  mediumspringgreen: "#00fa9a",
  mediumturquoise: "#48d1cc",
  mediumvioletred: "#c71585",
  midnightblue: "#191970",
  mintcream: "#f5fffa",
  mistyrose: "#ffe4e1",
  moccasin: "#ffe4b5",
  navajowhite: "#ffdead",
  navy: "#000080",
  oldlace: "#fdf5e6",
  olive: "#808000",
  olivedrab: "#6b8e23",
  orange: "#ffa500",
  orangered: "#ff4500",
  orchid: "#da70d6",
  palegoldenrod: "#eee8aa",
  palegreen: "#98fb98",
  paleturquoise: "#afeeee",
  palevioletred: "#db7093",
  papayawhip: "#ffefd5",
  peachpuff: "#ffdab9",
  peru: "#cd853f",
  pink: "#ffc0cb",
  plum: "#dda0dd",
  powderblue: "#b0e0e6",
  purple: "#800080",
  rebeccapurple: "#663399",
  red: "#ff0000",
  rosybrown: "#bc8f8f",
  royalblue: "#4169e1",
  saddlebrown: "#8b4513",
  salmon: "#fa8072",
  sandybrown: "#f4a460",
  seagreen: "#2e8b57",
  seashell: "#fff5ee",
  sienna: "#a0522d",
  silver: "#c0c0c0",
  skyblue: "#87ceeb",
  slateblue: "#6a5acd",
  slategray: "#708090",
  slategrey: "#708090",
  snow: "#fffafa",
  springgreen: "#00ff7f",
  steelblue: "#4682b4",
  tan: "#d2b48c",
  teal: "#008080",
  thistle: "#d8bfd8",
  tomato: "#ff6347",
  turquoise: "#40e0d0",
  violet: "#ee82ee",
  wheat: "#f5deb3",
  white: "#ffffff",
  whitesmoke: "#f5f5f5",
  yellow: "#ffff00",
  yellowgreen: "#9acd32"
};
function inputToRGB(color) {
  let rgb = { r: 0, g: 0, b: 0 };
  let a = 1;
  let s = null;
  let v = null;
  let l = null;
  let ok = false;
  let format = false;
  if (typeof color === "string") {
    color = stringInputToObject(color);
  }
  if (typeof color === "object") {
    if (isValidCSSUnit(color.r) && isValidCSSUnit(color.g) && isValidCSSUnit(color.b)) {
      rgb = rgbToRgb(color.r, color.g, color.b);
      ok = true;
      format = String(color.r).substr(-1) === "%" ? "prgb" : "rgb";
    } else if (isValidCSSUnit(color.h) && isValidCSSUnit(color.s) && isValidCSSUnit(color.v)) {
      s = convertToPercentage(color.s);
      v = convertToPercentage(color.v);
      rgb = hsvToRgb(color.h, s, v);
      ok = true;
      format = "hsv";
    } else if (isValidCSSUnit(color.h) && isValidCSSUnit(color.s) && isValidCSSUnit(color.l)) {
      s = convertToPercentage(color.s);
      l = convertToPercentage(color.l);
      rgb = hslToRgb(color.h, s, l);
      ok = true;
      format = "hsl";
    } else if (isValidCSSUnit(color.c) && isValidCSSUnit(color.m) && isValidCSSUnit(color.y) && isValidCSSUnit(color.k)) {
      rgb = cmykToRgb(color.c, color.m, color.y, color.k);
      ok = true;
      format = "cmyk";
    }
    if (Object.prototype.hasOwnProperty.call(color, "a")) {
      a = color.a;
    }
  }
  a = boundAlpha(a);
  return {
    ok,
    format: color.format || format,
    r: Math.min(255, Math.max(rgb.r, 0)),
    g: Math.min(255, Math.max(rgb.g, 0)),
    b: Math.min(255, Math.max(rgb.b, 0)),
    a
  };
}
const CSS_INTEGER = "[-\\+]?\\d+%?";
const CSS_NUMBER = "[-\\+]?\\d*\\.\\d+%?";
const CSS_UNIT = "(?:" + CSS_NUMBER + ")|(?:" + CSS_INTEGER + ")";
const PERMISSIVE_MATCH3 = "[\\s|\\(]+(" + CSS_UNIT + ")[,|\\s]+(" + CSS_UNIT + ")[,|\\s]+(" + CSS_UNIT + ")\\s*\\)?";
const PERMISSIVE_MATCH4 = "[\\s|\\(]+(" + CSS_UNIT + ")[,|\\s]+(" + CSS_UNIT + ")[,|\\s]+(" + CSS_UNIT + ")[,|\\s]+(" + CSS_UNIT + ")\\s*\\)?";
const matchers = {
  CSS_UNIT: new RegExp(CSS_UNIT),
  rgb: new RegExp("rgb" + PERMISSIVE_MATCH3),
  rgba: new RegExp("rgba" + PERMISSIVE_MATCH4),
  hsl: new RegExp("hsl" + PERMISSIVE_MATCH3),
  hsla: new RegExp("hsla" + PERMISSIVE_MATCH4),
  hsv: new RegExp("hsv" + PERMISSIVE_MATCH3),
  hsva: new RegExp("hsva" + PERMISSIVE_MATCH4),
  cmyk: new RegExp("cmyk" + PERMISSIVE_MATCH4),
  hex3: /^#?([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})$/,
  hex6: /^#?([0-9a-fA-F]{2})([0-9a-fA-F]{2})([0-9a-fA-F]{2})$/,
  hex4: /^#?([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})$/,
  hex8: /^#?([0-9a-fA-F]{2})([0-9a-fA-F]{2})([0-9a-fA-F]{2})([0-9a-fA-F]{2})$/
};
function stringInputToObject(color) {
  color = color.trim().toLowerCase();
  if (color.length === 0) {
    return false;
  }
  let named = false;
  if (names[color]) {
    color = names[color];
    named = true;
  } else if (color === "transparent") {
    return { r: 0, g: 0, b: 0, a: 0, format: "name" };
  }
  let match = matchers.rgb.exec(color);
  if (match) {
    return { r: match[1], g: match[2], b: match[3] };
  }
  match = matchers.rgba.exec(color);
  if (match) {
    return { r: match[1], g: match[2], b: match[3], a: match[4] };
  }
  match = matchers.hsl.exec(color);
  if (match) {
    return { h: match[1], s: match[2], l: match[3] };
  }
  match = matchers.hsla.exec(color);
  if (match) {
    return { h: match[1], s: match[2], l: match[3], a: match[4] };
  }
  match = matchers.hsv.exec(color);
  if (match) {
    return { h: match[1], s: match[2], v: match[3] };
  }
  match = matchers.hsva.exec(color);
  if (match) {
    return { h: match[1], s: match[2], v: match[3], a: match[4] };
  }
  match = matchers.cmyk.exec(color);
  if (match) {
    return {
      c: match[1],
      m: match[2],
      y: match[3],
      k: match[4]
    };
  }
  match = matchers.hex8.exec(color);
  if (match) {
    return {
      r: parseIntFromHex(match[1]),
      g: parseIntFromHex(match[2]),
      b: parseIntFromHex(match[3]),
      a: convertHexToDecimal(match[4]),
      format: named ? "name" : "hex8"
    };
  }
  match = matchers.hex6.exec(color);
  if (match) {
    return {
      r: parseIntFromHex(match[1]),
      g: parseIntFromHex(match[2]),
      b: parseIntFromHex(match[3]),
      format: named ? "name" : "hex"
    };
  }
  match = matchers.hex4.exec(color);
  if (match) {
    return {
      r: parseIntFromHex(match[1] + match[1]),
      g: parseIntFromHex(match[2] + match[2]),
      b: parseIntFromHex(match[3] + match[3]),
      a: convertHexToDecimal(match[4] + match[4]),
      format: named ? "name" : "hex8"
    };
  }
  match = matchers.hex3.exec(color);
  if (match) {
    return {
      r: parseIntFromHex(match[1] + match[1]),
      g: parseIntFromHex(match[2] + match[2]),
      b: parseIntFromHex(match[3] + match[3]),
      format: named ? "name" : "hex"
    };
  }
  return false;
}
function isValidCSSUnit(color) {
  if (typeof color === "number") {
    return !Number.isNaN(color);
  }
  return matchers.CSS_UNIT.test(color);
}
class TinyColor {
  constructor(color = "", opts = {}) {
    var _a;
    if (color instanceof TinyColor) {
      return color;
    }
    if (typeof color === "number") {
      color = numberInputToObject(color);
    }
    this.originalInput = color;
    const rgb = inputToRGB(color);
    this.originalInput = color;
    this.r = rgb.r;
    this.g = rgb.g;
    this.b = rgb.b;
    this.a = rgb.a;
    this.roundA = Math.round(100 * this.a) / 100;
    this.format = (_a = opts.format) != null ? _a : rgb.format;
    this.gradientType = opts.gradientType;
    if (this.r < 1) {
      this.r = Math.round(this.r);
    }
    if (this.g < 1) {
      this.g = Math.round(this.g);
    }
    if (this.b < 1) {
      this.b = Math.round(this.b);
    }
    this.isValid = rgb.ok;
  }
  isDark() {
    return this.getBrightness() < 128;
  }
  isLight() {
    return !this.isDark();
  }
  getBrightness() {
    const rgb = this.toRgb();
    return (rgb.r * 299 + rgb.g * 587 + rgb.b * 114) / 1e3;
  }
  getLuminance() {
    const rgb = this.toRgb();
    let R;
    let G;
    let B;
    const RsRGB = rgb.r / 255;
    const GsRGB = rgb.g / 255;
    const BsRGB = rgb.b / 255;
    if (RsRGB <= 0.03928) {
      R = RsRGB / 12.92;
    } else {
      R = Math.pow((RsRGB + 0.055) / 1.055, 2.4);
    }
    if (GsRGB <= 0.03928) {
      G = GsRGB / 12.92;
    } else {
      G = Math.pow((GsRGB + 0.055) / 1.055, 2.4);
    }
    if (BsRGB <= 0.03928) {
      B = BsRGB / 12.92;
    } else {
      B = Math.pow((BsRGB + 0.055) / 1.055, 2.4);
    }
    return 0.2126 * R + 0.7152 * G + 0.0722 * B;
  }
  getAlpha() {
    return this.a;
  }
  setAlpha(alpha) {
    this.a = boundAlpha(alpha);
    this.roundA = Math.round(100 * this.a) / 100;
    return this;
  }
  isMonochrome() {
    const { s } = this.toHsl();
    return s === 0;
  }
  toHsv() {
    const hsv = rgbToHsv(this.r, this.g, this.b);
    return { h: hsv.h * 360, s: hsv.s, v: hsv.v, a: this.a };
  }
  toHsvString() {
    const hsv = rgbToHsv(this.r, this.g, this.b);
    const h = Math.round(hsv.h * 360);
    const s = Math.round(hsv.s * 100);
    const v = Math.round(hsv.v * 100);
    return this.a === 1 ? `hsv(${h}, ${s}%, ${v}%)` : `hsva(${h}, ${s}%, ${v}%, ${this.roundA})`;
  }
  toHsl() {
    const hsl = rgbToHsl(this.r, this.g, this.b);
    return { h: hsl.h * 360, s: hsl.s, l: hsl.l, a: this.a };
  }
  toHslString() {
    const hsl = rgbToHsl(this.r, this.g, this.b);
    const h = Math.round(hsl.h * 360);
    const s = Math.round(hsl.s * 100);
    const l = Math.round(hsl.l * 100);
    return this.a === 1 ? `hsl(${h}, ${s}%, ${l}%)` : `hsla(${h}, ${s}%, ${l}%, ${this.roundA})`;
  }
  toHex(allow3Char = false) {
    return rgbToHex(this.r, this.g, this.b, allow3Char);
  }
  toHexString(allow3Char = false) {
    return "#" + this.toHex(allow3Char);
  }
  toHex8(allow4Char = false) {
    return rgbaToHex(this.r, this.g, this.b, this.a, allow4Char);
  }
  toHex8String(allow4Char = false) {
    return "#" + this.toHex8(allow4Char);
  }
  toHexShortString(allowShortChar = false) {
    return this.a === 1 ? this.toHexString(allowShortChar) : this.toHex8String(allowShortChar);
  }
  toRgb() {
    return {
      r: Math.round(this.r),
      g: Math.round(this.g),
      b: Math.round(this.b),
      a: this.a
    };
  }
  toRgbString() {
    const r = Math.round(this.r);
    const g = Math.round(this.g);
    const b = Math.round(this.b);
    return this.a === 1 ? `rgb(${r}, ${g}, ${b})` : `rgba(${r}, ${g}, ${b}, ${this.roundA})`;
  }
  toPercentageRgb() {
    const fmt = (x) => `${Math.round(bound01(x, 255) * 100)}%`;
    return {
      r: fmt(this.r),
      g: fmt(this.g),
      b: fmt(this.b),
      a: this.a
    };
  }
  toPercentageRgbString() {
    const rnd = (x) => Math.round(bound01(x, 255) * 100);
    return this.a === 1 ? `rgb(${rnd(this.r)}%, ${rnd(this.g)}%, ${rnd(this.b)}%)` : `rgba(${rnd(this.r)}%, ${rnd(this.g)}%, ${rnd(this.b)}%, ${this.roundA})`;
  }
  toCmyk() {
    return {
      ...rgbToCmyk(this.r, this.g, this.b)
    };
  }
  toCmykString() {
    const { c, m, y, k } = rgbToCmyk(this.r, this.g, this.b);
    return `cmyk(${c}, ${m}, ${y}, ${k})`;
  }
  toName() {
    if (this.a === 0) {
      return "transparent";
    }
    if (this.a < 1) {
      return false;
    }
    const hex = "#" + rgbToHex(this.r, this.g, this.b, false);
    for (const [key, value] of Object.entries(names)) {
      if (hex === value) {
        return key;
      }
    }
    return false;
  }
  toString(format) {
    const formatSet = Boolean(format);
    format = format != null ? format : this.format;
    let formattedString = false;
    const hasAlpha = this.a < 1 && this.a >= 0;
    const needsAlphaFormat = !formatSet && hasAlpha && (format.startsWith("hex") || format === "name");
    if (needsAlphaFormat) {
      if (format === "name" && this.a === 0) {
        return this.toName();
      }
      return this.toRgbString();
    }
    if (format === "rgb") {
      formattedString = this.toRgbString();
    }
    if (format === "prgb") {
      formattedString = this.toPercentageRgbString();
    }
    if (format === "hex" || format === "hex6") {
      formattedString = this.toHexString();
    }
    if (format === "hex3") {
      formattedString = this.toHexString(true);
    }
    if (format === "hex4") {
      formattedString = this.toHex8String(true);
    }
    if (format === "hex8") {
      formattedString = this.toHex8String();
    }
    if (format === "name") {
      formattedString = this.toName();
    }
    if (format === "hsl") {
      formattedString = this.toHslString();
    }
    if (format === "hsv") {
      formattedString = this.toHsvString();
    }
    if (format === "cmyk") {
      formattedString = this.toCmykString();
    }
    return formattedString || this.toHexString();
  }
  toNumber() {
    return (Math.round(this.r) << 16) + (Math.round(this.g) << 8) + Math.round(this.b);
  }
  clone() {
    return new TinyColor(this.toString());
  }
  lighten(amount = 10) {
    const hsl = this.toHsl();
    hsl.l += amount / 100;
    hsl.l = clamp01(hsl.l);
    return new TinyColor(hsl);
  }
  brighten(amount = 10) {
    const rgb = this.toRgb();
    rgb.r = Math.max(0, Math.min(255, rgb.r - Math.round(255 * -(amount / 100))));
    rgb.g = Math.max(0, Math.min(255, rgb.g - Math.round(255 * -(amount / 100))));
    rgb.b = Math.max(0, Math.min(255, rgb.b - Math.round(255 * -(amount / 100))));
    return new TinyColor(rgb);
  }
  darken(amount = 10) {
    const hsl = this.toHsl();
    hsl.l -= amount / 100;
    hsl.l = clamp01(hsl.l);
    return new TinyColor(hsl);
  }
  tint(amount = 10) {
    return this.mix("white", amount);
  }
  shade(amount = 10) {
    return this.mix("black", amount);
  }
  desaturate(amount = 10) {
    const hsl = this.toHsl();
    hsl.s -= amount / 100;
    hsl.s = clamp01(hsl.s);
    return new TinyColor(hsl);
  }
  saturate(amount = 10) {
    const hsl = this.toHsl();
    hsl.s += amount / 100;
    hsl.s = clamp01(hsl.s);
    return new TinyColor(hsl);
  }
  greyscale() {
    return this.desaturate(100);
  }
  spin(amount) {
    const hsl = this.toHsl();
    const hue = (hsl.h + amount) % 360;
    hsl.h = hue < 0 ? 360 + hue : hue;
    return new TinyColor(hsl);
  }
  mix(color, amount = 50) {
    const rgb1 = this.toRgb();
    const rgb2 = new TinyColor(color).toRgb();
    const p = amount / 100;
    const rgba = {
      r: (rgb2.r - rgb1.r) * p + rgb1.r,
      g: (rgb2.g - rgb1.g) * p + rgb1.g,
      b: (rgb2.b - rgb1.b) * p + rgb1.b,
      a: (rgb2.a - rgb1.a) * p + rgb1.a
    };
    return new TinyColor(rgba);
  }
  analogous(results = 6, slices = 30) {
    const hsl = this.toHsl();
    const part = 360 / slices;
    const ret = [this];
    for (hsl.h = (hsl.h - (part * results >> 1) + 720) % 360; --results; ) {
      hsl.h = (hsl.h + part) % 360;
      ret.push(new TinyColor(hsl));
    }
    return ret;
  }
  complement() {
    const hsl = this.toHsl();
    hsl.h = (hsl.h + 180) % 360;
    return new TinyColor(hsl);
  }
  monochromatic(results = 6) {
    const hsv = this.toHsv();
    const { h } = hsv;
    const { s } = hsv;
    let { v } = hsv;
    const res = [];
    const modification = 1 / results;
    while (results--) {
      res.push(new TinyColor({ h, s, v }));
      v = (v + modification) % 1;
    }
    return res;
  }
  splitcomplement() {
    const hsl = this.toHsl();
    const { h } = hsl;
    return [
      this,
      new TinyColor({ h: (h + 72) % 360, s: hsl.s, l: hsl.l }),
      new TinyColor({ h: (h + 216) % 360, s: hsl.s, l: hsl.l })
    ];
  }
  onBackground(background) {
    const fg = this.toRgb();
    const bg = new TinyColor(background).toRgb();
    const alpha = fg.a + bg.a * (1 - fg.a);
    return new TinyColor({
      r: (fg.r * fg.a + bg.r * bg.a * (1 - fg.a)) / alpha,
      g: (fg.g * fg.a + bg.g * bg.a * (1 - fg.a)) / alpha,
      b: (fg.b * fg.a + bg.b * bg.a * (1 - fg.a)) / alpha,
      a: alpha
    });
  }
  triad() {
    return this.polyad(3);
  }
  tetrad() {
    return this.polyad(4);
  }
  polyad(n) {
    const hsl = this.toHsl();
    const { h } = hsl;
    const result = [this];
    const increment = 360 / n;
    for (let i = 1; i < n; i++) {
      result.push(new TinyColor({ h: (h + i * increment) % 360, s: hsl.s, l: hsl.l }));
    }
    return result;
  }
  equals(color) {
    const comparedColor = new TinyColor(color);
    if (this.format === "cmyk" || comparedColor.format === "cmyk") {
      return this.toCmykString() === comparedColor.toCmykString();
    }
    return this.toRgbString() === comparedColor.toRgbString();
  }
}
const defaultHueStep = 2;
const defaultSaturationStepLight = 0.16;
const defaultSaturationStepDark = 0.05;
const defaultBrightnessStepLight = 0.05;
const defaultBrightnessStepDark = 0.15;
const lightColorCount = 5;
const darkColorCount = 5;
function toHsv({ r, g, b }) {
  const hsv = rgbToHsv(r, g, b);
  return { h: hsv.h * 360, s: hsv.s, v: hsv.v };
}
function toHex({ r, g, b }) {
  return `#${rgbToHex(r, g, b, false)}`;
}
function getHue(hsv, i, hueStep, light) {
  let hue;
  if (Math.round(hsv.h) >= 60 && Math.round(hsv.h) <= 240) {
    hue = light ? Math.round(hsv.h) - hueStep * i : Math.round(hsv.h) + hueStep * i;
  } else {
    hue = light ? Math.round(hsv.h) + hueStep * i : Math.round(hsv.h) - hueStep * i;
  }
  if (hue < 0) {
    hue += 360;
  } else if (hue >= 360) {
    hue -= 360;
  }
  return hue;
}
function getSaturation(hsv, i, saturationStepLight, saturationStepDark, light) {
  if (hsv.h === 0 && hsv.s === 0) {
    return hsv.s;
  }
  let saturation;
  if (light) {
    saturation = hsv.s - saturationStepLight * i;
  } else if (i === darkColorCount) {
    saturation = hsv.s + saturationStepLight;
  } else {
    saturation = hsv.s + saturationStepDark * i;
  }
  if (saturation > 1) {
    saturation = 1;
  }
  if (light && i === lightColorCount && saturation > 0.1) {
    saturation = 0.1;
  }
  if (saturation < 0.06) {
    saturation = 0.06;
  }
  return Number(saturation.toFixed(2));
}
function getValue(hsv, i, brightnessStep, light) {
  let value;
  if (light) {
    value = hsv.v + brightnessStep * i;
  } else {
    value = hsv.v - brightnessStep * i;
  }
  if (value > 1) {
    value = 1;
  }
  return Number(value.toFixed(2));
}
const colorPaletteCache$1 = /* @__PURE__ */ new Map();
function getColorPalette$1(color, options) {
  const {
    brightnessStepLight = defaultBrightnessStepLight,
    brightnessStepDark = defaultBrightnessStepDark,
    saturationStepLight = defaultSaturationStepLight,
    saturationStepDark = defaultSaturationStepDark,
    hueStep = defaultHueStep
  } = options != null ? options : {};
  const pColor = inputToRGB(color);
  const hexColor = toHex(pColor);
  if (colorPaletteCache$1.has(hexColor)) {
    return colorPaletteCache$1.get(hexColor);
  }
  const colorPalette = { base: hexColor };
  for (let i = 1; i <= lightColorCount; i++) {
    const hsv = toHsv(pColor);
    const colorString = toHex(
      inputToRGB({
        h: getHue(hsv, i, hueStep, true),
        s: getSaturation(hsv, i, saturationStepLight, saturationStepDark, true),
        v: getValue(hsv, i, brightnessStepLight, true)
      })
    );
    colorPalette[`l${i}0`] = colorString;
  }
  for (let i = 1; i <= darkColorCount; i++) {
    const hsv = toHsv(pColor);
    const colorString = toHex(
      inputToRGB({
        h: getHue(hsv, i, hueStep),
        s: getSaturation(hsv, i, saturationStepLight, saturationStepDark),
        v: getValue(hsv, i, brightnessStepDark)
      })
    );
    colorPalette[`d${i}0`] = colorString;
  }
  colorPaletteCache$1.set(hexColor, colorPalette);
  return colorPalette;
}
const darkColorMap = {
  l50: { key: "d20", opacity: 0.15 },
  l40: { key: "d10", opacity: 0.25 },
  l30: { key: "base", opacity: 0.3 },
  l20: { key: "base", opacity: 0.45 },
  l10: { key: "base", opacity: 0.65 },
  base: { key: "base", opacity: 0.85 },
  d10: { key: "l10", opacity: 0.9 },
  d20: { key: "l20", opacity: 0.95 },
  d30: { key: "l30", opacity: 0.97 },
  d40: { key: "l40", opacity: 0.98 },
  d50: { key: "l50", opacity: 1 }
};
function mix(rgb1, rgb2, amount) {
  const p = amount;
  const rgb = {
    r: (rgb2.r - rgb1.r) * p + rgb1.r,
    g: (rgb2.g - rgb1.g) * p + rgb1.g,
    b: (rgb2.b - rgb1.b) * p + rgb1.b
  };
  return rgb;
}
const colorPaletteCache = /* @__PURE__ */ new Map();
function getDarkColorPalette(color, bgColor, options) {
  const lightColorPalette = getColorPalette$1(color, options);
  const mergedBgColor = bgColor || "#0a0c0f";
  const cacheKey = `${color}-${mergedBgColor}`;
  if (colorPaletteCache.has(cacheKey)) {
    return colorPaletteCache.get(cacheKey);
  }
  const darkColorPalette = {};
  Object.entries(darkColorMap).forEach(([key, map]) => {
    darkColorPalette[key] = toHex(
      mix(inputToRGB(mergedBgColor), inputToRGB(lightColorPalette[map.key]), map.opacity)
    );
  });
  colorPaletteCache.set(cacheKey, darkColorPalette);
  return darkColorPalette;
}
function getGreyColors$2(color) {
  const baseColor = getBaseColors$1();
  if (color === "grey") {
    return {
      l50: "#fafafa",
      l40: "#f5f5f5",
      l30: "#eeeeee",
      l20: "#e0e0e0",
      l10: "#c2c2c2",
      base: baseColor.grey,
      d10: "#7a7a7a",
      d20: "#5c5c5c",
      d30: "#3d3d3d",
      d40: "#1f1f1f",
      d50: "#141414"
    };
  }
  return {
    l50: "#f7f9fc",
    l40: "#edf1f7",
    l30: "#e1e5eb",
    l20: "#d3d7de",
    l10: "#bec3cc",
    base: baseColor.graphite,
    d10: "#6f7785",
    d20: "#5e6573",
    d30: "#454c59",
    d40: "#2f3540",
    d50: "#1e232b"
  };
}
function getAlphaColor(baseColor, alpha, bgColor) {
  const alphaColor = new TinyColor(baseColor).setAlpha(alpha);
  if (!bgColor) {
    return alphaColor.toRgbString();
  }
  return alphaColor.onBackground(bgColor).toRgbString();
}
function getSolidColor(baseColor, brightness) {
  const instance = new TinyColor(baseColor);
  return instance.darken(brightness).toHexString();
}
function getDerivedColorTokens(basicTokens, getBaseColors2, getColorPalette2, getGreyColors2) {
  const primaryColorPalette = getColorPalette2(basicTokens.colorPrimary);
  const successColorPalette = getColorPalette2(basicTokens.colorSuccess);
  const errorColorPalette = getColorPalette2(basicTokens.colorError);
  const warningColorPalette = getColorPalette2(basicTokens.colorWarning);
  const riskColorPalette = getColorPalette2(basicTokens.colorRisk);
  const fatalColorPalette = getColorPalette2(basicTokens.colorFatal);
  const blueColorPalette = getColorPalette2(getBaseColors2().blue);
  const infoColorPalette = getColorPalette2(basicTokens.colorInfo);
  const primaryStatusColors = {
    colorPrimaryHover: primaryColorPalette.l10,
    colorPrimaryActive: primaryColorPalette.d10
  };
  const primaryColors = {
    ...primaryStatusColors,
    colorPrimaryBorder: primaryColorPalette.base,
    colorPrimaryBorderHover: primaryColorPalette.l10,
    colorPrimaryBorderActive: primaryColorPalette.d10,
    colorPrimaryText: primaryColorPalette.base,
    colorPrimaryTextHover: primaryStatusColors.colorPrimaryHover,
    colorPrimaryTextActive: primaryStatusColors.colorPrimaryActive,
    colorPrimaryIcon: primaryColorPalette.base
  };
  const successColors = {
    colorSuccessBg: successColorPalette.base,
    colorSuccessBgHover: successColorPalette.l10,
    colorSuccessBgActive: successColorPalette.d10,
    colorSuccessBorder: successColorPalette.d10,
    colorSuccessBorderHover: successColorPalette.base,
    colorSuccessBorderActive: successColorPalette.d20,
    colorSuccessText: successColorPalette.d10,
    colorSuccessTextHover: successColorPalette.base,
    colorSuccessTextActive: successColorPalette.d20,
    colorSuccessIcon: successColorPalette.base
  };
  const errorColors = {
    colorErrorBg: errorColorPalette.l10,
    colorErrorBgHover: errorColorPalette.l20,
    colorErrorBgActive: errorColorPalette.d10,
    colorErrorBorder: errorColorPalette.d10,
    colorErrorBorderHover: errorColorPalette.base,
    colorErrorBorderActive: errorColorPalette.d20,
    colorErrorText: errorColorPalette.d10,
    colorErrorTextHover: errorColorPalette.base,
    colorErrorTextActive: errorColorPalette.d10,
    colorErrorIcon: errorColorPalette.l10
  };
  const warningColors = {
    colorWarningBg: warningColorPalette.base,
    colorWarningBgHover: warningColorPalette.l10,
    colorWarningBgActive: warningColorPalette.d10,
    colorWarningBorder: warningColorPalette.base,
    colorWarningBorderHover: warningColorPalette.l10,
    colorWarningBorderActive: warningColorPalette.d10,
    colorWarningText: warningColorPalette.base,
    colorWarningTextHover: warningColorPalette.l10,
    colorWarningTextActive: warningColorPalette.d10,
    colorWarningIcon: warningColorPalette.l10
  };
  const riskColors = {
    colorRiskBg: riskColorPalette.base,
    colorRiskBgHover: riskColorPalette.l10,
    colorRiskBgActive: riskColorPalette.d10,
    colorRiskBorder: riskColorPalette.base,
    colorRiskBorderHover: riskColorPalette.l10,
    colorRiskBorderActive: riskColorPalette.d10,
    colorRiskText: riskColorPalette.base,
    colorRiskTextHover: riskColorPalette.l10,
    colorRiskTextActive: riskColorPalette.d10,
    colorRiskIcon: riskColorPalette.l10
  };
  const fatalColors = {
    colorFatalBg: fatalColorPalette.base,
    colorFatalBgHover: fatalColorPalette.l10,
    colorFatalBgActive: fatalColorPalette.d10,
    colorFatalBorder: fatalColorPalette.base,
    colorFatalBorderHover: fatalColorPalette.l10,
    colorFatalBorderActive: fatalColorPalette.d10,
    colorFatalText: fatalColorPalette.base,
    colorFatalTextHover: fatalColorPalette.l10,
    colorFatalTextActive: fatalColorPalette.d10,
    colorFatalIcon: fatalColorPalette.l10
  };
  const infoColors = basicTokens.colorInfo === blueColorPalette.l10 ? {
    colorInfoBg: blueColorPalette.l10,
    colorInfoBgHover: blueColorPalette.l20,
    colorInfoBgActive: blueColorPalette.base,
    colorInfoBorder: blueColorPalette.l10,
    colorInfoBorderHover: blueColorPalette.l20,
    colorInfoBorderActive: blueColorPalette.base,
    colorInfoText: blueColorPalette.l10,
    colorInfoTextHover: blueColorPalette.l20,
    colorInfoTextActive: blueColorPalette.base,
    colorInfoIcon: blueColorPalette.l10
  } : {
    colorInfoBg: infoColorPalette.base,
    colorInfoBgHover: infoColorPalette.l10,
    colorInfoBgActive: infoColorPalette.d10,
    colorInfoBorder: infoColorPalette.base,
    colorInfoBorderHover: infoColorPalette.l10,
    colorInfoBorderActive: infoColorPalette.d10,
    colorInfoText: infoColorPalette.base,
    colorInfoTextHover: infoColorPalette.l10,
    colorInfoTextActive: infoColorPalette.d10,
    colorInfoIcon: infoColorPalette.base
  };
  const greyColors = getGreyColors2();
  return {
    ...primaryColors,
    ...successColors,
    ...errorColors,
    ...warningColors,
    ...riskColors,
    ...fatalColors,
    ...infoColors,
    colorOffline: greyColors.base,
    colorOfflineBg: greyColors.base,
    colorOfflineText: greyColors.d10
  };
}
function getDerivedFontTokens(basicTokens) {
  const { fontSize, fontWeight } = basicTokens;
  const fontSizeUnit = 2;
  const minFontSize = 10;
  const fontWeightUnit = 100;
  const fontSizeMap = {
    fontSizeXs: Math.max(fontSize - fontSizeUnit * 2, minFontSize),
    fontSizeSm: Math.max(fontSize - fontSizeUnit, minFontSize),
    fontSizeMd: Math.max(fontSize, minFontSize),
    fontSizeLg: Math.max(fontSize + fontSizeUnit, minFontSize),
    fontSizeXl: Math.max(fontSize + fontSizeUnit * 3, minFontSize),
    fontSize2xl: Math.max(fontSize + fontSizeUnit * 5, minFontSize),
    fontSize3xl: Math.max(fontSize + fontSizeUnit * 8, minFontSize)
  };
  return {
    ...fontSizeMap,
    fontSizeHeaderSm: fontSizeMap.fontSizeMd,
    fontSizeHeaderMd: fontSizeMap.fontSizeLg,
    fontSizeHeaderLg: fontSizeMap.fontSizeXl,
    fontSizeHeaderXl: fontSizeMap.fontSize2xl,
    fontWeightXs: fontWeight - fontWeightUnit * 2,
    fontWeightSm: fontWeight - fontWeightUnit,
    fontWeightMd: fontWeight,
    fontWeightLg: fontWeight + fontWeightUnit,
    fontWeightXl: fontWeight + fontWeightUnit * 2
  };
}
function getDerivedMotionTokens(basicTokens) {
  const { motionDuration } = basicTokens;
  return {
    motionDurationFast: 0.18,
    motionDurationMedium: motionDuration,
    motionDurationSlow: 0.3
  };
}
function getDerivedSizeTokens(basicTokens) {
  const { spacing, height, borderRadius, screenSm, screenMd, screenLg, screenXl } = basicTokens;
  const spacingSizes = {
    xxs: Math.ceil(spacing * 0.25),
    xs: Math.ceil(spacing * 0.5),
    sm: spacing,
    md: Math.ceil(spacing * 1.5),
    lg: Math.ceil(spacing * 2),
    xl: spacing * 3,
    xxl: Math.ceil(spacing * 4)
  };
  return {
    paddingSize2Xs: spacingSizes.xxs,
    paddingSizeXs: spacingSizes.xs,
    paddingSizeSm: spacingSizes.sm,
    paddingSizeMd: spacingSizes.md,
    paddingSizeLg: spacingSizes.lg,
    paddingSizeXl: spacingSizes.xl,
    paddingSize2Xl: spacingSizes.xxl,
    marginSize2Xs: spacingSizes.xxs,
    marginSizeXs: spacingSizes.xs,
    marginSizeSm: spacingSizes.sm,
    marginSizeMd: spacingSizes.md,
    marginSizeLg: spacingSizes.lg,
    marginSizeXl: spacingSizes.xl,
    marginSize2Xl: spacingSizes.xxl,
    heightXs: height * 0.5,
    heightSm: height * 0.75,
    heightMd: height,
    heightLg: height * 1.25,
    heightXl: height * 1.5,
    height2xl: height * 1.75,
    height3xl: height * 2,
    borderRadiusXs: Math.max(2, Math.floor(borderRadius / 4)),
    borderRadiusSm: Math.max(2, Math.floor(borderRadius / 2)),
    borderRadiusMd: borderRadius,
    borderRadiusLg: Math.min(8, borderRadius * 2),
    lineWidthBold: 2,
    arrowSize: 6,
    screenXsMax: screenSm - 0.01,
    screenSmMin: screenSm,
    screenSmMax: screenMd - 0.01,
    screenMdMin: screenMd,
    screenMdMax: screenLg - 0.01,
    screenLgMin: screenLg,
    screenLgMax: screenXl - 0.01,
    screenXlMin: screenXl
  };
}
function getShadowTokens(getGreyColors2) {
  const greyColors = getGreyColors2();
  return {
    boxShadowSm: `0 2px 10px 0px ${getAlphaColor(greyColors.d50, 0.16)}`,
    boxShadowMd: `0 4px 16px 0px ${getAlphaColor(greyColors.d50, 0.14)}`,
    boxShadowLg: `0 8px 22px 0px ${getAlphaColor(greyColors.d50, 0.12)}`
  };
}
function getDerivedTokens(basicTokens, options) {
  const { getBaseColors: getBaseColors2, getColorPalette: getColorPalette2, getGreyColors: getGreyColors2 } = options;
  return {
    ...getDerivedColorTokens(basicTokens, getBaseColors2, getColorPalette2, getGreyColors2),
    ...getDerivedFontTokens(basicTokens),
    ...getDerivedSizeTokens(basicTokens),
    ...getDerivedMotionTokens(basicTokens),
    ...getShadowTokens(getGreyColors2)
  };
}
function getControlTokens(tokens) {
  const {
    colorPrimary,
    colorError,
    colorPrimaryBorderHover,
    colorPrimaryBorderActive,
    colorContainerBg,
    colorContainerBgDisabled,
    heightXs,
    heightSm,
    heightMd,
    heightLg,
    fontSizeXs,
    fontSizeSm,
    fontSizeMd,
    borderRadiusSm,
    lineWidth,
    lineType,
    lineHeightGutter
  } = tokens;
  const controlHeightXs = heightXs;
  const controlHeightSm = heightSm;
  const controlHeightMd = heightMd;
  const controlHeightLg = heightLg;
  const controlLineWidth = lineWidth;
  const controlFontSizeXs = fontSizeXs;
  const controlFontSizeSm = fontSizeSm;
  const controlFontSizeMd = fontSizeSm;
  const controlFontSizeLg = fontSizeMd;
  const getVerticalPadding = (height, fontSize) => {
    return Math.round((height - (fontSize + lineHeightGutter)) / 2 - controlLineWidth);
  };
  return {
    controlHeightXs,
    controlHeightSm,
    controlHeightMd,
    controlHeightLg,
    controlLineWidth,
    controlLineType: lineType,
    controlFontSizeXs,
    controlFontSizeSm,
    controlFontSizeMd,
    controlFontSizeLg,
    controlBoxShadowFocus: `0 0 0 2px ${getAlphaColor(colorPrimary, 0.2)}`,
    controlBoxShadowInvalid: `0 0 0 2px ${getAlphaColor(colorError, 0.2)}`,
    controlBgColor: colorContainerBg,
    controlBgColorDisabled: colorContainerBgDisabled,
    controlBorderColorHover: colorPrimaryBorderHover,
    controlBorderColorActive: colorPrimaryBorderActive,
    controlPaddingSizeHorizontalXs: 7,
    controlPaddingSizeHorizontalSm: 7,
    controlPaddingSizeHorizontalMd: 11,
    controlPaddingSizeHorizontalLg: 11,
    controlPaddingSizeVerticalXs: Math.max(getVerticalPadding(controlHeightXs, controlFontSizeXs), 0),
    controlPaddingSizeVerticalSm: Math.max(getVerticalPadding(controlHeightSm, controlFontSizeSm), 0),
    controlPaddingSizeVerticalMd: Math.max(getVerticalPadding(controlHeightMd, controlFontSizeMd), 2),
    controlPaddingSizeVerticalLg: Math.ceil(getVerticalPadding(controlHeightLg, controlFontSizeLg)),
    controlBorderRadiusXs: borderRadiusSm,
    controlBorderRadiusSm: borderRadiusSm,
    controlBorderRadiusMd: borderRadiusSm,
    controlBorderRadiusLg: borderRadiusSm
  };
}
function getExtendedColorTokens(tokens, getColorPalette2, getGreyColors2) {
  const greyColors = getGreyColors2();
  const { colorPrimary, colorWarning, colorError, colorBg } = tokens;
  const primaryColorPalette = getColorPalette2(colorPrimary);
  return {
    colorContainerBg: colorBg,
    colorContainerBgHover: greyColors.l50,
    colorContainerBgActive: primaryColorPalette.l50,
    colorContainerBgDisabled: greyColors.l40,
    colorAddonContainerBg: greyColors.l50,
    colorEmphasizedContainerBg: greyColors.l40,
    colorEmphasizedContainerBgHover: greyColors.l30,
    colorEmphasizedContainerBgDisabled: greyColors.l10,
    colorInfoContainerBg: greyColors.l50,
    colorInfoContainerBgHover: greyColors.l50,
    colorInfoContainerBgDisabled: greyColors.l30,
    colorFillContainerBg: greyColors.l10,
    colorSeparator: greyColors.l30,
    colorTextPlaceholder: greyColors.base,
    colorTextDisabled: greyColors.l10,
    colorTextInverse: colorBg,
    colorTextInverseDisabled: greyColors.l10,
    colorTextTitle: greyColors.d40,
    colorTextTitleSecondary: greyColors.d30,
    colorTextInfo: greyColors.d10,
    colorIcon: greyColors.d20,
    colorIconInfo: greyColors.l10,
    colorIconHover: colorPrimary,
    colorIconActive: primaryColorPalette.d10,
    colorIconDisabled: greyColors.l10,
    colorWarningOutline: colorWarning,
    colorErrorOutline: colorError,
    colorBgInverse: greyColors.d40,
    colorBgInverseDisabled: greyColors.d50,
    colorBorderInverse: colorBg,
    colorBorderSecondary: greyColors.l30,
    colorMask: "rgba(0, 0, 0, 0.45)",
    tagCompColorAlpha: 0.1,
    alertCompColorAlpha: 0.1
  };
}
function getExtendedFontTokens(tokens) {
  return {
    fontSizeIcon: 16,
    fontWeightHeader: tokens.fontWeightXl
  };
}
function getExtendedSizeTokens(tokens) {
  return {
    componentBorderRadius: tokens.borderRadiusSm
  };
}
function getOverlayTokens(tokens) {
  const { borderRadiusSm, arrowSize, colorContainerBg, lineType } = tokens;
  return {
    overlayBgColor: colorContainerBg,
    overlayArrowSize: arrowSize,
    overlayBorderRadius: borderRadiusSm,
    overlayBorderWidth: 0,
    overlayBorderColor: "transparent",
    overlayBorderType: lineType
  };
}
function getScollbarTokens(getGreyColors2) {
  const greyColors = getGreyColors2();
  return {
    scrollbarWidth: 12,
    scrollbarHeight: 12,
    scrollbarThumbBg: greyColors.l30,
    scrollbarThumbBgHover: greyColors.l10,
    scrollbarThumbBgActive: greyColors.l10,
    scrollbarThumbBorderWidth: 2,
    scrollbarThumbBorderRadius: 6,
    scrollbarThumbBoxShadow: "none",
    scrollbarTrackBg: "unset",
    scrollbarTrackBorderRadius: 0,
    scrollbarTrackBoxShadow: "unset"
  };
}
function getExtendedTokens(tokens, options) {
  const { getColorPalette: getColorPalette2, getGreyColors: getGreyColors2 } = options;
  const extendedColorTokens = getExtendedColorTokens(tokens, getColorPalette2, getGreyColors2);
  const mergedTokens = { ...tokens, ...extendedColorTokens };
  return {
    ...extendedColorTokens,
    ...getExtendedFontTokens(tokens),
    ...getExtendedSizeTokens(tokens),
    ...getScollbarTokens(getGreyColors2),
    ...getControlTokens(mergedTokens),
    ...getOverlayTokens(mergedTokens)
  };
}
function getBasicTokens(getBaseColors2, getColorPalette2, getGreyColors2) {
  const baseColors = getBaseColors2();
  const redColorPalette = getColorPalette2(baseColors.red);
  const blueColorPalette = getColorPalette2(baseColors.blue);
  const greyColors = getGreyColors2();
  return {
    colorPrimary: baseColors.blue,
    colorError: baseColors.red,
    colorSuccess: baseColors.turquoise,
    colorWarning: baseColors.brown,
    colorRisk: baseColors.orange,
    colorFatal: redColorPalette.d30,
    colorInfo: blueColorPalette.l10,
    colorLink: baseColors.blue,
    colorWhite: baseColors.white,
    colorText: greyColors.d40,
    colorBg: baseColors.white,
    colorBorder: greyColors.l20,
    colorBorderSecondary: greyColors.l30,
    fontSize: 14,
    fontFamily: `pingfang SC, helvetica neue, arial, hiragino sans gb, microsoft yahei ui, microsoft yahei, simsun, sans-serif`,
    fontFamilyCode: `'SFMono-Regular', Consolas, 'Liberation Mono', Menlo, Courier, monospace`,
    fontWeight: 400,
    borderRadius: 4,
    height: 32,
    lineHeightGutter: 8,
    spacing: 8,
    lineWidth: 1,
    lineType: "solid",
    motionDuration: 0.24,
    motionEaseIn: "cubic-bezier(0.12, 0, 0.39, 0)",
    motionEaseOut: "cubic-bezier(0.61, 1, 0.88, 1)",
    motionEaseInOut: "cubic-bezier(0.37, 0, 0.63, 1)",
    motionEaseInBack: "cubic-bezier(0.36, 0, 0.66, -0.56)",
    motionEaseOutBack: "cubic-bezier(0.34, 1.56, 0.64, 1)",
    motionEaseInCirc: "cubic-bezier(0.55, 0, 1, 0.45)",
    motionEaseOutCirc: "cubic-bezier(0, 0.55, 0.45, 1)",
    motionEaseInQuint: "cubic-bezier(0.64, 0, 0.78, 0)",
    motionEaseOutQuint: "cubic-bezier(0.22, 1, 0.36, 1)",
    screenSm: 600,
    screenMd: 960,
    screenLg: 1280,
    screenXl: 1720
  };
}
function getResetTokens(tokens) {
  const {
    colorBg,
    colorText,
    colorTextTitle,
    colorLink,
    fontFamily,
    fontFamilyCode,
    fontSizeSm,
    fontWeight,
    fontWeightHeader,
    lineHeightGutter,
    scrollbarWidth,
    scrollbarHeight,
    scrollbarThumbBg,
    scrollbarThumbBgHover,
    scrollbarThumbBgActive,
    scrollbarThumbBorderWidth,
    scrollbarThumbBorderRadius,
    scrollbarThumbBoxShadow,
    scrollbarTrackBg,
    scrollbarTrackBorderRadius,
    scrollbarTrackBoxShadow
  } = tokens;
  return {
    colorText,
    colorTextTitle,
    colorBg,
    colorLink,
    fontFamily,
    fontFamilyCode,
    fontSize: fontSizeSm,
    fontWeight,
    fontWeightHeader,
    lineHeightGutter,
    scrollbarWidth,
    scrollbarHeight,
    scrollbarThumbBg,
    scrollbarThumbBgHover,
    scrollbarThumbBgActive,
    scrollbarThumbBorderWidth,
    scrollbarThumbBorderRadius,
    scrollbarThumbBoxShadow,
    scrollbarTrackBg,
    scrollbarTrackBorderRadius,
    scrollbarTrackBoxShadow
  };
}
const colors = {
  red: {
    l50: "#33151A",
    l40: "#4A1618",
    l30: "#821D1F",
    l20: "#A72122",
    l10: "#D42525",
    base: "#E8514C",
    d10: "#F37E75",
    d20: "#F8A89F",
    d30: "#FAC9C0",
    d40: "#FAD8D2"
  },
  orange: {
    l50: "#331C15",
    l40: "#4A2916",
    l30: "#7A3D16",
    l20: "#AB521B",
    l10: "#D8641B",
    base: "#E88641",
    d10: "#F3A76A",
    d20: "#F8C492",
    d30: "#FAD6AF",
    d40: "#FAE4CD"
  },
  brown: {
    l50: "#332515",
    l40: "#4A3616",
    l30: "#7A5517",
    l20: "#AD771C",
    l10: "#DB941D",
    base: "#FDAA1D",
    d10: "#F3CA6A",
    d20: "#F8DD92",
    d30: "#ECEBA1",
    d40: "#FAF2D7"
  },
  yellow: {
    l50: "#332C15",
    l40: "#4A4219",
    l30: "#736516",
    l20: "#A9941A",
    l10: "#D6BB1A",
    base: "#E8D43E",
    d10: "#F3E867",
    d20: "#F8F390",
    d30: "#FAF9AA",
    d40: "#FAFADC"
  },
  bud: {
    l50: "#243315",
    l40: "#364A16",
    l30: "#4C6B19",
    l20: "#649019",
    l10: "#7BB518",
    base: "#9ACA39",
    d10: "#B9E061",
    d20: "#D6F08E",
    d30: "#EAFAB4",
    d40: "#F3FADC"
  },
  green: {
    l50: "#153315",
    l40: "#1F4B16",
    l30: "#29641D",
    l20: "#2D8718",
    l10: "#34A917",
    base: "#56BD37",
    d10: "#7CD15D",
    d20: "#A3E288",
    d30: "#BCF1A1",
    d40: "#DAF1CF"
  },
  turquoise: {
    l50: "#153330",
    l40: "#164A3C",
    l30: "#0F694E",
    l20: "#1D8D69",
    l10: "#1FB182",
    base: "#40C695",
    d10: "#6ADCAE",
    d20: "#97EBC7",
    d30: "#B6FADB",
    d40: "#D9FAEB"
  },
  cyan: {
    l50: "#152B33",
    l40: "#16464A",
    l30: "#15686E",
    l20: "#189197",
    l10: "#19B6BE",
    base: "#3BD0D3",
    d10: "#65EAE7",
    d20: "#92F8F3",
    d30: "#B4FAF5",
    d40: "#CAFAF7"
  },
  glacier: {
    l50: "#162433",
    l40: "#16354A",
    l30: "#17547B",
    l20: "#1A70A6",
    l10: "#1A8BD2",
    base: "#40ADE8",
    d10: "#6AC8F4",
    d20: "#92DBF8",
    d30: "#A8E6FA",
    d40: "#C3EDFA"
  },
  blue: {
    l50: "#151D33",
    l40: "#16274A",
    l30: "#173B7A",
    l20: "#1A50AF",
    l10: "#1B61DD",
    base: "#4083E8",
    d10: "#6AA6F4",
    d20: "#92C2F8",
    d30: "#A7D1FA",
    d40: "#C3DEFA"
  },
  indigo: {
    l50: "#1E1533",
    l40: "#29164A",
    l30: "#3D177A",
    l20: "#561FAF",
    l10: "#6922DD",
    base: "#4852E8",
    d10: "#AB70F4",
    d20: "#C89AF8",
    d30: "#DAB6FA",
    d40: "#E7D2FA"
  },
  purple: {
    l50: "#2D132E",
    l40: "#451541",
    l30: "#66135D",
    l20: "#931885",
    l10: "#B919A5",
    base: "#8A48E8",
    d10: "#E565CC",
    d20: "#F593DF",
    d30: "#FAB6E9",
    d40: "#FAD0EF"
  },
  magenta: {
    l50: "#2D132E",
    l40: "#451541",
    l30: "#66135D",
    l20: "#931885",
    l10: "#B919A5",
    base: "#CF3CB7",
    d10: "#E565CC",
    d20: "#F593DF",
    d30: "#FAB6E9",
    d40: "#FAD0EF"
  },
  gold: {
    l50: "#472D06",
    l40: "#674109",
    l30: "#75460F",
    l20: "#9C641C",
    l10: "#C2862D",
    base: "#E7AA40",
    d10: "#F5C76C",
    d20: "#FFE099",
    d30: "#FFEFC2",
    d40: "#FFFCF0"
  },
  silver: {
    l50: "#222731",
    l40: "#2C303E",
    l30: "#3A4154",
    l20: "#5B647A",
    l10: "#7F8AA1",
    base: "#A9B4C8",
    d10: "#C7CCD4",
    d20: "#D3D9E0",
    d30: "#DFE5ED",
    d40: "#EBF2FA"
  },
  bronze: {
    l50: "#24100D",
    l40: "#30140C",
    l30: "#572919",
    l20: "#7D422A",
    l10: "#A35F40",
    base: "#C97F58",
    d10: "#D6A181",
    d20: "#E3C3AC",
    d30: "#F0E5DD",
    d40: "#FCF5ED"
  }
};
function getColorPalette(color, bgColor) {
  const baseColors = Object.keys(colors).map((name) => ({ name, base: colors[name].base }));
  const baseColor = baseColors.find((c) => c.base === color);
  if (baseColor) {
    return colors[baseColor.name];
  }
  return getDarkColorPalette(color, bgColor);
}
function getGreyColors$1() {
  return {
    l50: "#0A0C0F",
    l40: "#171A1F",
    l30: "#1F2329",
    l20: "#3C424D",
    l10: "#687080",
    base: "#525966",
    d10: "#808999",
    d20: "#A1A7B3",
    d30: "#C2C7D1",
    d40: "#F4F8FF",
    d50: "#FFFFFF"
  };
}
const getBaseColors = () => {
  return Object.keys(colors).reduce(
    (baseColors, key) => {
      baseColors[key] = colors[key].base;
      return baseColors;
    },
    {
      white: "#fff",
      graphite: getGreyColors$1().base,
      black: "#000",
      grey: getGreyColors$2("grey").base
    }
  );
};
function getPresetTokens() {
  const greyColors = getGreyColors$1();
  const baseColors = getBaseColors();
  const colorError = baseColors.red;
  const errorColorPalette = getColorPalette(colorError);
  const colorSuccess = baseColors.turquoise;
  const colorInfo = baseColors.blue;
  const colorWarning = baseColors.brown;
  const colorRisk = baseColors.orange;
  const colorFatal = errorColorPalette.l20;
  const successColorPalette = getColorPalette(colorSuccess);
  return {
    colorSuccess,
    colorError,
    colorInfo,
    colorWarning,
    colorRisk,
    colorFatal,
    colorBg: greyColors.l50,
    colorContainerBg: greyColors.l50,
    colorContainerBgHover: greyColors.l40,
    colorEmphasizedContainerBg: greyColors.l30,
    colorEmphasizedContainerBgHover: greyColors.l30,
    colorEmphasizedContainerBgDisabled: greyColors.l10,
    colorAddonContainerBg: greyColors.l40,
    colorInfoContainerBg: greyColors.l40,
    colorInfoContainerBgHover: greyColors.l40,
    colorInfoContainerBgDisabled: greyColors.l30,
    colorTextInverse: baseColors.white,
    colorBgInverse: baseColors.white,
    colorMask: getAlphaColor(greyColors.l40, 0.88),
    overlayBorderWidth: 1,
    overlayBorderColor: greyColors.l30,
    boxShadowSm: `0 2px 10px 0px ${greyColors.l50}`,
    boxShadowMd: `0 4px 16px 0px ${greyColors.l50}`,
    boxShadowLg: `0 8px 22px 0px ${greyColors.l50}`,
    colorSuccessBg: successColorPalette.base,
    colorSuccessBgHover: successColorPalette.l10,
    colorSuccessBgActive: successColorPalette.d10,
    colorSuccessBorder: successColorPalette.base,
    colorSuccessBorderHover: successColorPalette.l10,
    colorSuccessBorderActive: successColorPalette.d10,
    colorSuccessText: successColorPalette.base,
    colorSuccessTextHover: successColorPalette.l10,
    colorSuccessTextActive: successColorPalette.d10,
    colorSuccessIcon: successColorPalette.base,
    colorErrorBg: errorColorPalette.base,
    colorErrorBgHover: errorColorPalette.l10,
    colorErrorBgActive: errorColorPalette.d10,
    colorErrorBorder: errorColorPalette.base,
    colorErrorBorderHover: errorColorPalette.l10,
    colorErrorBorderActive: errorColorPalette.d10,
    colorErrorText: errorColorPalette.base,
    colorErrorTextHover: errorColorPalette.l10,
    colorErrorTextActive: errorColorPalette.d10,
    colorErrorIcon: errorColorPalette.base,
    colorInfoIcon: colorInfo,
    colorWarningIcon: colorWarning,
    colorRiskIcon: colorRisk,
    colorFatalIcon: colorFatal,
    tagCompColorAlpha: 0.2,
    alertCompColorAlpha: 0.2
  };
}
const getGreyColors = () => getGreyColors$2("graphite");
const presetTokens = {};
function getPresetAlgorithms(presetTheme) {
  return presetTheme === "default" ? {
    getBaseColors: getBaseColors$1,
    getColorPalette: getColorPalette$1,
    getGreyColors
  } : {
    getBaseColors,
    getColorPalette,
    getGreyColors: getGreyColors$1
  };
}
function getThemeTokens(presetTheme, tokens, algorithms) {
  var _a, _b, _c;
  const presetAlgorithms = getPresetAlgorithms(presetTheme);
  const getBaseColors2 = (_a = algorithms == null ? void 0 : algorithms.getBaseColors) != null ? _a : presetAlgorithms.getBaseColors;
  const getColorPalette2 = (_b = algorithms == null ? void 0 : algorithms.getColorPalette) != null ? _b : presetAlgorithms.getColorPalette;
  const getGreyColors2 = (_c = algorithms == null ? void 0 : algorithms.getGreyColors) != null ? _c : presetAlgorithms.getGreyColors;
  const presetTokens$1 = presetTheme === "default" ? presetTokens : getPresetTokens();
  const mergedPresetTokens = merge({ ...presetTokens$1 }, tokens);
  const presetBasicTokens = getBasicTokens(getBaseColors2, getColorPalette2, getGreyColors2);
  const mergedBasicTokens = merge(presetBasicTokens, pick(mergedPresetTokens, Object.keys(presetBasicTokens)));
  const presetDerivedTokens = getDerivedTokens(mergedBasicTokens, { getBaseColors: getBaseColors2, getColorPalette: getColorPalette2, getGreyColors: getGreyColors2 });
  const mergedDerivedTokens = merge(presetDerivedTokens, pick(mergedPresetTokens, Object.keys(presetDerivedTokens)));
  const presetExtendedTokens = getExtendedTokens(
    { ...mergedBasicTokens, ...mergedDerivedTokens },
    { getColorPalette: getColorPalette2, getGreyColors: getGreyColors2 }
  );
  const mergedExtendedTokens = merge(presetExtendedTokens, pick(mergedPresetTokens, Object.keys(presetExtendedTokens)));
  return {
    ...mergedBasicTokens,
    ...mergedDerivedTokens,
    ...mergedExtendedTokens
  };
}
const globalTokenKey = "global";
const resetTokenKey = "reset";
const themeTokenPrefix = "ix";
const themeProviderProps = {
  presetTheme: {
    type: String,
    default: void 0
  },
  injectThemeStyle: {
    type: Boolean,
    default: void 0
  },
  hashed: {
    type: Boolean,
    default: void 0
  },
  tag: String,
  inherit: {
    type: [Boolean, String],
    default: true
  },
  attachTo: [String, Object, Function],
  tokens: Object,
  algorithm: Object
};
function useTokenMerge(props, config, supperContext, mergedPresetTheme) {
  const mergedAlgorithms = computed(() => {
    var _a;
    const presetAlgorithms = getPresetAlgorithms(mergedPresetTheme.value);
    const { getBaseColors: getBaseColors2, getColorPalette: getColorPalette2, getGreyColors: getGreyColors2 } = (_a = props == null ? void 0 : props.algorithm) != null ? _a : {};
    return {
      getBaseColors: getBaseColors2 != null ? getBaseColors2 : presetAlgorithms.getBaseColors,
      getColorPalette: getColorPalette2 != null ? getColorPalette2 : presetAlgorithms.getColorPalette,
      getGreyColors: getGreyColors2 != null ? getGreyColors2 : presetAlgorithms.getGreyColors
    };
  });
  const mergedTokens = computed(() => {
    var _a, _b, _c, _d;
    const configGlobalTokens = config.global;
    const configComponentTokens = config.components;
    const overwrittenTokens = merge(
      { ...(props == null ? void 0 : props.inherit) && !props.presetTheme ? (_a = supperContext == null ? void 0 : supperContext.mergedTokens.value.global) != null ? _a : {} : {} },
      { ...configGlobalTokens },
      (_b = props == null ? void 0 : props.tokens) == null ? void 0 : _b.global
    );
    const mergedGlobalTokens = getThemeTokens(mergedPresetTheme.value, overwrittenTokens, mergedAlgorithms.value);
    const mergedComponentTokens = merge(
      { ...(props == null ? void 0 : props.inherit) ? (_c = supperContext == null ? void 0 : supperContext.mergedTokens.value.components) != null ? _c : {} : {} },
      { ...configComponentTokens },
      (_d = props == null ? void 0 : props.tokens) == null ? void 0 : _d.components
    );
    return {
      global: mergedGlobalTokens,
      components: mergedComponentTokens
    };
  });
  const getMergedTokens = (key, tokens) => {
    var _a;
    if (key === globalTokenKey) {
      return merge({ ...tokens }, mergedTokens.value.global);
    }
    return merge({ ...tokens }, (_a = mergedTokens.value.components) == null ? void 0 : _a[key]);
  };
  return {
    mergedAlgorithms,
    mergedTokens,
    getMergedTokens
  };
}
const IDUX_THEME_KEY = "idux-theme-key";
const IDUX_THEME_REF_CNT_KEY = "__IDUX_THEME_REF_CNT__";
function findStyles(container) {
  return Array.from(container.children).filter((node) => node.tagName === "STYLE");
}
function getContainer(option) {
  if (option.attachTo) {
    return option.attachTo;
  }
  const head = document.querySelector("head");
  return head || document.body;
}
function findExistNode(key, option = {}) {
  const container = getContainer(option);
  return findStyles(container).find((node) => node.getAttribute(IDUX_THEME_KEY) === key);
}
function getStyleRefCnt(node) {
  var _a;
  return (_a = node[IDUX_THEME_REF_CNT_KEY]) != null ? _a : 0;
}
function setStyleRefCnt(node, cnt) {
  node[IDUX_THEME_REF_CNT_KEY] = cnt;
}
function injectCSS(css, option = {}) {
  if (!isBrowser) {
    return null;
  }
  const { csp } = option;
  const styleNode = document.createElement("style");
  if (csp == null ? void 0 : csp.nonce) {
    styleNode.nonce = csp == null ? void 0 : csp.nonce;
  }
  styleNode.innerHTML = css;
  const container = getContainer(option);
  container.appendChild(styleNode);
  return styleNode;
}
function removeCSS(key, option = {}) {
  const existNode = findExistNode(key, option);
  if (existNode) {
    const container = getContainer(option);
    container.removeChild(existNode);
  }
}
function updateCSS(css, key, option = {}) {
  var _a, _b, _c;
  const existNode = findExistNode(key, option);
  if (existNode) {
    if (((_a = option.csp) == null ? void 0 : _a.nonce) && existNode.nonce !== ((_b = option.csp) == null ? void 0 : _b.nonce)) {
      existNode.nonce = (_c = option.csp) == null ? void 0 : _c.nonce;
    }
    if (existNode.innerHTML !== css) {
      existNode.innerHTML = css;
    }
    return existNode;
  }
  const newNode = injectCSS(css, option);
  newNode == null ? void 0 : newNode.setAttribute(IDUX_THEME_KEY, key);
  return newNode;
}
const unhasedHashSalt = "__unhashed__";
const sequenceCache = /* @__PURE__ */ new Map();
function createTokensHash(key, tokens, hashed = true) {
  let sequence = sequenceCache.get(key);
  if (!sequence) {
    sequence = getTokenSequence(tokens);
    sequenceCache.set(key, sequence);
  }
  let str = flattenTokens(tokens, sequence);
  if (!hashed) {
    str += unhasedHashSalt;
  }
  return `${themeTokenPrefix}-${key}-${hash(str)}`;
}
function flattenTokens(tokens, sequence) {
  let str = "";
  sequence.forEach((key) => {
    const value = tokens[key];
    str += `${key}${value || ""}`;
  });
  return str;
}
function getTokenSequence(tokens) {
  return Object.keys(tokens).sort();
}
function numUnitTransform(value, unit) {
  if (isString(value)) {
    return value;
  }
  if (value === 0) {
    return value.toString();
  }
  return `${value}${unit ? unit : ""}`;
}
const checkKeyWithSearch = (key, searchStrs) => {
  return searchStrs.some((search) => key.includes(search));
};
function defaultTokenTransform(key, value) {
  if (isString(value)) {
    return value;
  }
  const lowerCasedKey = key.toLowerCase();
  const unit = checkKeyWithSearch(lowerCasedKey, [
    "width",
    "height",
    "size",
    "gap",
    "indent",
    "fontsize",
    "spacing",
    "padding",
    "margin",
    "borderradius",
    "linewidth",
    "iconsize",
    "arrowsize",
    "lineheightgutter",
    "screen"
  ]) ? "px" : checkKeyWithSearch(lowerCasedKey, ["duration"]) ? "s" : void 0;
  return numUnitTransform(value, unit);
}
function tokenToCss(record, prefix, transforms) {
  const { key, tokens, hashId } = record;
  const resolvedPrefix = prefix != null ? prefix : themeTokenPrefix;
  let cssVarContents = "";
  const varNamePrefix = `--${resolvedPrefix}${key === globalTokenKey ? "" : `-${kebabCase(key)}`}`;
  Object.entries(tokens).forEach(([tokenName, value]) => {
    var _a;
    const _tokenName = tokenName;
    const varName = `${varNamePrefix}-${getTokenCssVarName(_tokenName)}`;
    const transform = (_a = transforms == null ? void 0 : transforms[_tokenName]) != null ? _a : (value2) => defaultTokenTransform(tokenName, value2);
    cssVarContents += `${varName}: ${transform(value)};`;
  });
  if (!hashId) {
    return `:root { ${cssVarContents} }`;
  }
  return `.${hashId} { ${cssVarContents} }`;
}
function getTokenCssVarName(tokenName) {
  let name = "";
  let preChar;
  for (let i = 0; i < tokenName.length; i++) {
    const char = tokenName[i];
    if (preChar && !isUppercaseOrNumber(preChar) && isUppercaseOrNumber(char)) {
      name += `-${char}`;
    } else {
      name += char;
    }
    preChar = char;
  }
  return name.toLowerCase();
}
function isUppercaseOrNumber(char) {
  const charCode = char.charCodeAt(0);
  return charCode >= 65 && charCode <= 90 || charCode >= 48 && charCode <= 57;
}
const injectedStyles = [];
function useDynamicCss(mergedAttachTo) {
  const localInjectedStyleCnts = /* @__PURE__ */ new Map();
  const incrementCnt = (hashId) => {
    if (!localInjectedStyleCnts.has(hashId)) {
      localInjectedStyleCnts.set(hashId, 1);
    } else {
      localInjectedStyleCnts.set(hashId, localInjectedStyleCnts.get(hashId) + 1);
    }
  };
  const decrementCnt = (hashId) => {
    if (!localInjectedStyleCnts.has(hashId)) {
      return;
    }
    const currentCnt = localInjectedStyleCnts.get(hashId) - 1;
    if (currentCnt <= 0) {
      localInjectedStyleCnts.delete(hashId);
    } else {
      localInjectedStyleCnts.set(hashId, currentCnt);
    }
  };
  const findInjectedStyle = (hashId, attachTo) => {
    let index = injectedStyles.findIndex((style2) => style2.hashId === hashId && style2.attachTo === attachTo);
    let style = injectedStyles[index];
    const styleElement = findExistNode(hashId, { attachTo });
    if (styleElement) {
      const styleRefCnt = getStyleRefCnt(styleElement);
      if (!style) {
        style = {
          hashId,
          attachTo: mergedAttachTo.value,
          refCnt: styleRefCnt
        };
        index = injectedStyles.length;
        injectedStyles.push(style);
      } else {
        style.refCnt = styleRefCnt;
      }
    }
    return {
      index,
      style,
      styleElement
    };
  };
  const setInjectedStyle = (injectedStyle, styleElement) => {
    injectedStyles.push(injectedStyle);
    setStyleRefCnt(styleElement, injectedStyle.refCnt);
  };
  const updateInjectedStyleRefCnt = (injectedStyle, styleElement, cnt) => {
    injectedStyle.refCnt = cnt;
    setStyleRefCnt(styleElement, cnt);
  };
  const inject2 = (hashId, css) => {
    incrementCnt(hashId);
    const { style, styleElement } = findInjectedStyle(hashId, mergedAttachTo.value);
    if (style && styleElement) {
      updateInjectedStyleRefCnt(style, styleElement, style.refCnt + 1);
      style.refCnt++;
    } else {
      const newInjectedStyleElement = updateCSS(css, hashId, { attachTo: mergedAttachTo.value });
      newInjectedStyleElement && setInjectedStyle(
        {
          hashId,
          attachTo: mergedAttachTo.value,
          refCnt: 1
        },
        newInjectedStyleElement
      );
    }
  };
  const remove = (hashId) => {
    decrementCnt(hashId);
    const { index, style, styleElement } = findInjectedStyle(hashId, mergedAttachTo.value);
    if (!style || !styleElement) {
      return;
    }
    updateInjectedStyleRefCnt(style, styleElement, style.refCnt - 1);
    if (style.refCnt <= 0) {
      injectedStyles.splice(index, 1);
      removeCSS(hashId, { attachTo: mergedAttachTo.value });
    }
  };
  const updateThemeStyle = (css, hashId, oldHashId) => {
    if (oldHashId) {
      remove(oldHashId);
    }
    inject2(hashId, css);
  };
  tryOnScopeDispose(() => {
    for (const [hashId, cnt] of localInjectedStyleCnts.entries()) {
      for (let i = 0; i < cnt; i++) {
        remove(hashId);
      }
    }
  });
  return updateThemeStyle;
}
function useTokenRegister(injectThemeStyle, mergedPresetTheme, mergedAlgorithms, mergedAttachTo, mergedHashed, getMergedTokens) {
  const tokenRecordMap = reactive(/* @__PURE__ */ new Map());
  const tokenGettersMap = /* @__PURE__ */ new Map();
  const tokenTransformsMap = /* @__PURE__ */ new Map();
  const tokenHashedMap = /* @__PURE__ */ new Map();
  const tokenPrefixMap = /* @__PURE__ */ new Map();
  const updateThemeStyle = useDynamicCss(mergedAttachTo);
  const [globalHashId, setGlobalHashId] = useState("");
  const _updateToken = (key, force, existedHashId) => {
    var _a, _b;
    let record = tokenRecordMap.get(key != null ? key : globalTokenKey);
    if (record && !force) {
      return record.hashId;
    }
    const globalTokens = key === globalTokenKey ? {} : (_a = tokenRecordMap.get(globalTokenKey)) == null ? void 0 : _a.tokens;
    if (!globalTokens) {
      return;
    }
    const getTokens = tokenGettersMap.get(key);
    const transforms = tokenTransformsMap.get(key);
    const prefix = tokenPrefixMap.get(key);
    const hashed = (_b = tokenHashedMap.get(key)) != null ? _b : mergedHashed.value;
    if (!getTokens) {
      return;
    }
    const tokens = getTokens(globalTokens, mergedPresetTheme.value, mergedAlgorithms.value);
    if (!tokens) {
      return;
    }
    const mergedCompTokens = getMergedTokens(key, tokens);
    const hashId = existedHashId != null ? existedHashId : createTokensHash(key, mergedCompTokens, hashed);
    if ((record == null ? void 0 : record.hashId) === hashId) {
      return hashId;
    }
    const oldHashId = record == null ? void 0 : record.hashId;
    record = {
      key,
      hashId,
      tokens: mergedCompTokens
    };
    tokenRecordMap.set(key, record);
    if (injectThemeStyle.value && !existedHashId) {
      const cssContent = tokenToCss(
        { ...record, hashId: hashed ? record.hashId : "" },
        prefix,
        transforms
      );
      updateThemeStyle(cssContent, record.hashId, oldHashId);
    }
    if (key === globalTokenKey) {
      setGlobalHashId(record.hashId);
      [...tokenRecordMap.keys()].filter((key2) => key2 !== globalTokenKey).forEach((componentTokenKey) => {
        updateToken(componentTokenKey);
      });
    }
    return record.hashId;
  };
  const registerToken = (key, getTokens, transforms, prefix, hashed, hashId) => {
    tokenGettersMap.set(key, getTokens);
    tokenTransformsMap.set(key, transforms);
    tokenPrefixMap.set(key, prefix);
    tokenHashedMap.set(key, hashed);
    return _updateToken(key, false, hashId);
  };
  const updateToken = (key, hashId) => {
    if (!isTokensRegistered(key)) {
      return;
    }
    return _updateToken(key, true, hashId);
  };
  const isTokensRegistered = (key) => {
    return tokenRecordMap.has(key);
  };
  const getThemeTokens2 = (key) => {
    return tokenRecordMap.get(key).tokens;
  };
  const getThemeHashId = (key) => {
    var _a;
    return (_a = tokenRecordMap.get(key)) == null ? void 0 : _a.hashId;
  };
  return {
    globalHashId,
    isTokensRegistered,
    getThemeTokens: getThemeTokens2,
    getThemeHashId,
    registerToken,
    updateToken
  };
}
function createThemeProviderContext(supperContext, props) {
  const themeConfig = useGlobalConfig("theme");
  const injectThemeStyle = computed(() => {
    var _a;
    return (_a = props == null ? void 0 : props.injectThemeStyle) != null ? _a : themeConfig.injectThemeStyle;
  });
  const mergedPresetTheme = computed(
    () => {
      var _a;
      return (_a = (props == null ? void 0 : props.inherit) && !props.presetTheme ? supperContext == null ? void 0 : supperContext.presetTheme.value : props == null ? void 0 : props.presetTheme) != null ? _a : themeConfig.presetTheme;
    }
  );
  const mergedHashed = computed(
    () => {
      var _a, _b;
      return (_b = (_a = (props == null ? void 0 : props.inherit) ? supperContext == null ? void 0 : supperContext.hashed.value : void 0) != null ? _a : props == null ? void 0 : props.hashed) != null ? _b : themeConfig.hashed;
    }
  );
  const mergedAttachTo = computed(() => {
    var _a, _b, _c;
    const attachTo = (_b = (_a = (props == null ? void 0 : props.inherit) ? supperContext == null ? void 0 : supperContext.attachTo.value : void 0) != null ? _a : props == null ? void 0 : props.attachTo) != null ? _b : themeConfig.attachTo;
    if (!attachTo) {
      return;
    }
    if (attachTo instanceof Element) {
      return attachTo;
    }
    if (isFunction(attachTo)) {
      return attachTo();
    }
    return (_c = document.querySelector(attachTo)) != null ? _c : void 0;
  });
  const { mergedAlgorithms, mergedTokens, getMergedTokens } = useTokenMerge(
    props,
    themeConfig,
    supperContext,
    mergedPresetTheme
  );
  const { globalHashId, registerToken, updateToken, getThemeTokens: getThemeTokens2, getThemeHashId, isTokensRegistered } = useTokenRegister(
    injectThemeStyle,
    mergedPresetTheme,
    mergedAlgorithms,
    mergedAttachTo,
    mergedHashed,
    getMergedTokens
  );
  watch(
    () => mergedTokens.value.global,
    () => {
      var _a;
      const useSupper = (props == null ? void 0 : props.inherit) && !((_a = props.tokens) == null ? void 0 : _a.global) && !!supperContext;
      if (!isTokensRegistered(globalTokenKey)) {
        registerToken(
          globalTokenKey,
          () => useSupper ? supperContext.getThemeTokens(globalTokenKey) : mergedTokens.value.global,
          void 0,
          void 0,
          void 0,
          useSupper ? supperContext.getThemeHashId(globalTokenKey) : void 0
        );
      } else {
        updateToken(globalTokenKey, useSupper ? supperContext.getThemeHashId(globalTokenKey) : void 0);
      }
      if ((props == null ? void 0 : props.inherit) && !!supperContext || isTokensRegistered(resetTokenKey)) {
        return;
      }
      registerToken(
        resetTokenKey,
        (globalTokens) => useSupper ? supperContext.getThemeTokens(resetTokenKey) : getResetTokens(globalTokens),
        void 0,
        void 0,
        false,
        useSupper ? supperContext.getThemeHashId(resetTokenKey) : void 0
      );
    },
    {
      immediate: true,
      deep: true
    }
  );
  watch(
    () => mergedTokens.value.components,
    (components) => {
      Object.keys(components).forEach((key) => {
        updateToken(key);
      });
    },
    {
      deep: true
    }
  );
  const useThemeTokenContextMap = /* @__PURE__ */ new Map();
  return {
    globalHashId,
    hashed: mergedHashed,
    presetTheme: mergedPresetTheme,
    attachTo: mergedAttachTo,
    mergedTokens,
    useThemeTokenContextMap,
    getThemeHashId,
    registerToken,
    updateToken,
    getThemeTokens: getThemeTokens2,
    isTokensRegistered
  };
}
const useSharedThemeProvider = createSharedComposable(() => createThemeProviderContext(null));
const THEME_PROVIDER_TOKEN = Symbol("THEME_PROVIDER_TOKEN");
var ThemeProvider = /* @__PURE__ */ defineComponent({
  name: "IxThemeProvider",
  props: themeProviderProps,
  setup(props, {
    slots,
    attrs
  }) {
    let supperContext = inject(THEME_PROVIDER_TOKEN, null);
    if (props.inherit === "all" && !supperContext) {
      if (process.env.NODE_ENV !== "production") {
        Logger.warn("components/theme", `parent IxThemeProvider not found when using inherit 'all', this may cause unexpected theme errors`);
      }
      supperContext = useSharedThemeProvider();
    }
    const context = createThemeProviderContext(supperContext, props);
    provide(THEME_PROVIDER_TOKEN, context);
    return () => {
      var _a;
      return props.tag ? createVNode(props.tag, attrs, {
        default: () => {
          var _a2;
          return [(_a2 = slots.default) == null ? void 0 : _a2.call(slots)];
        }
      }) : createVNode(Fragment, null, [(_a = slots.default) == null ? void 0 : _a.call(slots)]);
    };
  }
});
function useThemeToken(key) {
  let themeProviderContext = inject(THEME_PROVIDER_TOKEN, null);
  if (!themeProviderContext) {
    themeProviderContext = useSharedThemeProvider();
  }
  const {
    useThemeTokenContextMap,
    presetTheme,
    globalHashId: _globalHashId,
    hashed,
    getThemeTokens: getThemeTokens2,
    getThemeHashId,
    registerToken: _registerToken
  } = themeProviderContext;
  const globalContext = getSharedContext(
    globalTokenKey,
    useThemeTokenContextMap,
    () => ({
      presetTheme,
      globalHashId: computed(() => hashed.value ? _globalHashId.value : ""),
      themeTokens: computed(() => getThemeTokens2(globalTokenKey))
    })
  );
  if (!key) {
    return globalContext;
  }
  const context = getSharedContext(key, useThemeTokenContextMap, () => {
    const globalHashId = globalContext.globalHashId;
    const hashId = computed(() => {
      var _a;
      return hashed.value ? (_a = getThemeHashId(key)) != null ? _a : "" : "";
    });
    const themeTokens = computed(() => getThemeTokens2(key));
    const registerToken = (getTokens, transforms, prefixCls) => {
      var _a;
      return (_a = _registerToken(key, getTokens, transforms, prefixCls)) != null ? _a : "";
    };
    return {
      presetTheme,
      globalHashId,
      hashId,
      themeTokens,
      registerToken
    };
  });
  return context;
}
function getSharedContext(key, map, getContext) {
  let states = map.get(key);
  if (!states) {
    const scope = effectScope(true);
    const context = scope.run(() => getContext());
    states = {
      scope,
      context,
      subscribers: 1
    };
    tryOnScopeDispose(() => {
      if (states && scope && --states.subscribers <= 0) {
        states.scope.stop();
        map.set(key, void 0);
      }
    });
    map.set(key, states);
  } else {
    states.subscribers++;
  }
  return states.context;
}
const IxThemeProvider = ThemeProvider;
export { IxThemeProvider, getAlphaColor, getBaseColors$1 as getBaseColors, getColorPalette$1 as getColorPalette, getDarkColorPalette, getGreyColors$2 as getGreyColors, getPresetAlgorithms, getSolidColor, getThemeTokens, numUnitTransform, useThemeToken };
