/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { BaseColors, ColorPalette } from '../themeTokens';
import { type ComputedRef } from 'vue';
import { type CertainThemeTokens, type GlobalThemeTokens, type PresetTheme, type ThemeKeys, type ThemeTokenAlgorithms, type TokenTransforms } from '../types';
export type TokenGetter<K extends ThemeKeys | keyof Ext, Ext extends object = object> = (globalTokens: GlobalThemeTokens, presetTheme: PresetTheme, algorithms: {
    getColorPalette: (color: string) => ColorPalette;
    getGreyColors: () => ColorPalette;
    getBaseColors: () => BaseColors;
}) => CertainThemeTokens<K, Ext>;
export type RegisterToken<K extends ThemeKeys | keyof Ext, Ext extends object = object> = (key: K, getTokens: TokenGetter<K, Ext>, transforms?: TokenTransforms<K, Ext> | undefined, prefix?: string, hashed?: boolean, hashId?: string) => string | undefined;
export type UpdateToken<K extends ThemeKeys | keyof Ext, Ext extends object = object> = (key: K, hashId?: string) => string | undefined;
export interface TokenRegisterContext<Ext extends object = object> {
    globalHashId: ComputedRef<string>;
    isTokensRegistered: (key: ThemeKeys | keyof Ext) => boolean;
    registerToken: RegisterToken<ThemeKeys | keyof Ext, Ext>;
    updateToken: UpdateToken<ThemeKeys | keyof Ext, Ext>;
    getThemeTokens: <K extends ThemeKeys | keyof Ext>(key: K) => CertainThemeTokens<K, Ext>;
    getThemeHashId: (key: ThemeKeys | keyof Ext) => string | undefined;
}
export declare function useTokenRegister(injectThemeStyle: ComputedRef<boolean>, mergedPresetTheme: ComputedRef<PresetTheme>, mergedAlgorithms: ComputedRef<ThemeTokenAlgorithms>, mergedAttachTo: ComputedRef<Element | undefined>, mergedHashed: ComputedRef<boolean>, getMergedTokens: <K extends ThemeKeys>(key: K, tokens: CertainThemeTokens<K>) => CertainThemeTokens<K>): TokenRegisterContext;
