/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { DeepPartialThemeTokens, ThemeKeys, ThemeTokenAlgorithms } from './themeTokens';
import type { UseThemeTokenContext } from '../useThemeToken';
import type { ExtractInnerPropTypes, ExtractPublicPropTypes } from '@idux/cdk/utils';
import type { DefineComponent, EffectScope, HTMLAttributes, PropType } from 'vue';
export type ThemeInherit = boolean | 'all';
export type PresetTheme = 'default' | 'dark';
export type ThemeProviderAttachTo = string | HTMLElement | (() => HTMLElement);
export interface UsetThemeProviderStates<Ext extends object = object> {
    scope: EffectScope;
    subscribers: number;
    context: UseThemeTokenContext<ThemeKeys | keyof Ext, Ext>;
}
export declare const themeProviderProps: {
    readonly presetTheme: {
        readonly type: PropType<PresetTheme>;
        readonly default: undefined;
    };
    readonly injectThemeStyle: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly hashed: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly tag: StringConstructor;
    readonly inherit: {
        readonly type: PropType<ThemeInherit>;
        readonly default: true;
    };
    readonly attachTo: PropType<ThemeProviderAttachTo>;
    readonly tokens: PropType<DeepPartialThemeTokens>;
    readonly algorithm: PropType<Partial<ThemeTokenAlgorithms>>;
};
export type ThemeProviderProps = ExtractInnerPropTypes<typeof themeProviderProps>;
export type ThemeProviderPublicProps = ExtractPublicPropTypes<typeof themeProviderProps>;
export type ThemeProviderComponent = DefineComponent<Omit<HTMLAttributes, keyof ThemeProviderPublicProps> & ThemeProviderPublicProps>;
export type ThemeProviderInstance = InstanceType<DefineComponent<ThemeProviderProps>>;
