/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { TokenGetter } from './composables/useTokenRegister';
import { type ComputedRef } from 'vue';
import { type CertainThemeTokens, type GlobalThemeTokens, type GlobalTokenKey, type PresetTheme, type ThemeKeys, type TokenTransforms } from './types';
export type UseThemeTokenScope = Exclude<ThemeKeys, GlobalTokenKey>;
export interface ScopedUseThemeTokenContext<K extends UseThemeTokenScope | keyof Ext, Ext extends object = object> {
    presetTheme: ComputedRef<PresetTheme>;
    globalHashId: ComputedRef<string>;
    hashId: ComputedRef<string>;
    themeTokens: ComputedRef<CertainThemeTokens<K, Ext>>;
    registerToken: (getTokens: TokenGetter<K, Ext>, transforms?: TokenTransforms<K, Ext>, prefixCls?: string) => string;
}
export interface GlobalUseThemeTokenContext {
    globalHashId: ComputedRef<string>;
    themeTokens: ComputedRef<GlobalThemeTokens>;
    presetTheme: ComputedRef<PresetTheme>;
}
export type UseThemeTokenContext<K extends UseThemeTokenScope | keyof Ext | undefined, Ext extends object = object> = K extends undefined ? GlobalUseThemeTokenContext : K extends UseThemeTokenScope | keyof Ext ? ScopedUseThemeTokenContext<K, Ext> : never;
export declare function useThemeToken(): GlobalUseThemeTokenContext;
export declare function useThemeToken<Ext extends object, K extends UseThemeTokenScope | keyof Ext | undefined>(key: K): UseThemeTokenContext<K, Ext>;
