import { computed, watch, onMounted, toRaw, defineComponent, inject, ref, onUpdated, createVNode, mergeProps, toRef, provide, normalizeClass } from "vue";
import { useGlobalConfig, useDateConfig } from "@idux/components/config";
import { IxControlTrigger } from "@idux/components/control-trigger";
import { useFormItemRegister, useFormSize, useFormStatus, useFormElement } from "@idux/components/form";
import { useThemeToken } from "@idux/components/theme";
import { isString, isArray } from "lodash-es";
import { useState, useControlledProp, callEmit, convertArray } from "@idux/cdk/utils";
import { convertToDate as convertToDate$1, applyDateTime } from "@idux/components/utils";
import { useAccessorAndControl } from "@idux/cdk/forms";
import { \u0275NormalizeAmPm as _NormalizeAmPm, \u0275CalculateViewHour as _CalculateViewHour, \u0275TimePanel as _TimePanel, \u0275baseTimePanelProps as _baseTimePanelProps } from "@idux/components/_private/time-panel";
import { \u0275Footer as _Footer } from "@idux/components/_private/footer";
import { \u0275Input as _Input } from "@idux/components/_private/input";
function convertToDate(dateConfig, value, format) {
  if (!value) {
    return void 0;
  }
  return dateConfig.convert(value, format);
}
function sortRangeValue(values) {
  return values.sort((v1, v2) => {
    if (!v1) {
      return 1;
    }
    if (!v2) {
      return 0;
    }
    return v1.valueOf() - v2.valueOf();
  });
}
function checkHourEnabled(format) {
  return /[hH]/.test(format);
}
function checkMinuteEnabled(format) {
  return /m/.test(format);
}
function checkSecondEnabled(format) {
  return /s/.test(format);
}
function checkUse12Hours(format) {
  return /[aA]/.test(format);
}
function checkAmPmCapital(format) {
  return /A/.test(format);
}
function usePickerControl(dateConfig, formatRef, handleChange, valueProp) {
  const [inputValue, setInputValue] = useState(void 0);
  const [inputFocused, setInputFocused] = useState(false);
  const [panelValue, setPanelValue] = useState(void 0);
  const { parse, format: formatDate } = dateConfig;
  const dateValue = computed(() => convertToDate$1(dateConfig, valueProp.value, formatRef.value));
  const formatedDateValue = computed(
    () => isString(valueProp.value) ? valueProp.value : dateValue.value ? formatDate(dateValue.value, formatRef.value) : ""
  );
  const timeInputEnabledStatus = computed(() => ({
    hourEnabled: checkHourEnabled(formatRef.value) || checkUse12Hours(formatRef.value),
    minuteEnabled: checkMinuteEnabled(formatRef.value),
    secondEnabled: checkSecondEnabled(formatRef.value)
  }));
  function init(force = false) {
    if (force || !inputValue.value || parse(inputValue.value, formatRef.value).valueOf() !== parse(formatedDateValue.value, formatRef.value).valueOf()) {
      setInputValue(formatedDateValue.value);
    }
    setPanelValue(dateValue.value);
  }
  watch([valueProp, formatRef], () => init(), { immediate: true });
  function adjustTimeOfInput(value) {
    if (!value || !dateValue.value) {
      return value;
    }
    const { hourEnabled, minuteEnabled, secondEnabled } = timeInputEnabledStatus.value;
    const typesToApply = [
      "year",
      "month",
      "date",
      !hourEnabled && "hour",
      !minuteEnabled && "minute",
      !secondEnabled && "second",
      "millisecond"
    ].filter(Boolean);
    return applyDateTime(dateConfig, dateValue.value, value, typesToApply);
  }
  function parseInput(value, format, referenceDate) {
    return value ? dateConfig.parse(value, format, referenceDate) : void 0;
  }
  function checkInputValid(date) {
    return !date || dateConfig.isValid(date);
  }
  function handleInput(evt) {
    const value = evt.target.value;
    setInputValue(value);
    const currDate = parseInput(value, formatRef.value, dateValue.value);
    if (checkInputValid(currDate)) {
      handleChange(inputValue.value ? adjustTimeOfInput(currDate) : void 0);
    }
  }
  function handleInputClear() {
    setInputValue("");
  }
  function handleInputFocus() {
    setInputFocused(true);
  }
  function handleInputBlur() {
    setInputFocused(false);
  }
  function handlePanelChange(value) {
    handleChange(value);
  }
  return {
    inputValue,
    panelValue,
    inputFocused,
    setInputValue,
    setPanelValue,
    init,
    handleInput,
    handleInputClear,
    handleInputFocus,
    handleInputBlur,
    handlePanelChange
  };
}
function useInputEnableStatus(props, config) {
  return computed(() => {
    var _a;
    const allowInput = (_a = props.allowInput) != null ? _a : config.allowInput;
    return {
      enableExternalInput: allowInput === true,
      enableInternalInput: allowInput === "overlay"
    };
  });
}
function useKeyboardEvents(overlayOpened, setOverlayOpened) {
  return (evt) => {
    if (evt.code === "Escape") {
      setOverlayOpened(false);
      return;
    }
    if (!overlayOpened.value && !["Backspace", "Tab"].includes(evt.code)) {
      evt.preventDefault();
      setOverlayOpened(true);
      return;
    }
    if (evt.code === "Enter") {
      setOverlayOpened(false);
    }
  };
}
function useRangeKeyboardEvents(rangeControl, overlayOpened, setOverlayOpened, handleChange) {
  const { bufferUpdated, buffer } = rangeControl;
  return (evt) => {
    if (evt.code === "Escape") {
      setOverlayOpened(false);
      return;
    }
    if (!overlayOpened.value && !["Backspace", "Tab"].includes(evt.code)) {
      evt.preventDefault();
      setOverlayOpened(true);
      return;
    }
    switch (evt.code) {
      case "Enter":
        if (bufferUpdated.value) {
          handleChange(buffer.value);
        }
        setOverlayOpened(false);
        break;
    }
  };
}
function useOverlayState(props, focus) {
  const [overlayOpened, setOverlayOpened] = useControlledProp(props, "open", false);
  onMounted(() => {
    if (props.autofocus) {
      focus();
    }
  });
  return { overlayOpened, setOverlayOpened };
}
function usePickerState(props, config, dateConfig, formatRef) {
  const { accessor, control } = useAccessorAndControl();
  useFormItemRegister(control);
  const mergedSize = useFormSize(props, config);
  const mergedStatus = useFormStatus(props, control);
  const [focused, setFocused] = useState(false);
  const checkValueDisabled = (value) => {
    const { get } = dateConfig;
    const use12Hours = checkUse12Hours(formatRef.value);
    return convertArray(value).some((v) => {
      const _hour = get(v, "hour");
      const amPm = _NormalizeAmPm(_hour, use12Hours);
      const hour = _CalculateViewHour(_hour, !!use12Hours);
      const minute = get(v, "minute");
      const second = get(v, "second");
      return props.disabledHours(amPm).includes(hour) || props.disabledMinutes(hour, amPm).includes(minute) || props.disabledSeconds(hour, minute, amPm).includes(second);
    });
  };
  function handleChange(value) {
    const newValue = isArray(value) ? sortRangeValue(value) : value;
    if (checkValueDisabled(newValue)) {
      return;
    }
    let oldValue = toRaw(accessor.value);
    oldValue = isArray(oldValue) ? oldValue.map((v) => convertToDate(dateConfig, v, formatRef.value)) : convertToDate(dateConfig, oldValue, formatRef.value);
    accessor.setValue(newValue);
    callEmit(props.onChange, newValue, oldValue);
  }
  function handleClear(evt) {
    handleChange(void 0);
    callEmit(props.onClear, evt);
  }
  function handleFocus(evt) {
    setFocused(true);
    callEmit(props.onFocus, evt);
  }
  function handleBlur(evt) {
    setFocused(false);
    accessor.markAsBlurred();
    callEmit(props.onBlur, evt);
  }
  return {
    accessor,
    mergedSize,
    mergedStatus,
    focused,
    handleChange,
    handleClear,
    handleFocus,
    handleBlur
  };
}
function useTriggerProps(context) {
  const {
    props,
    config,
    accessor,
    mergedSize,
    mergedStatus,
    mergedPrefixCls,
    focused,
    handleBlur,
    handleFocus,
    handleClear,
    handleKeyDown,
    overlayOpened,
    setOverlayOpened
  } = context;
  return computed(() => {
    var _a, _b, _c, _d, _e;
    return {
      autofocus: props.autofocus,
      borderless: props.borderless,
      value: accessor.value,
      clearable: (_a = props.clearable) != null ? _a : config.clearable,
      clearIcon: (_b = props.clearIcon) != null ? _b : config.clearIcon,
      disabled: accessor.disabled,
      focused: focused.value,
      open: overlayOpened.value,
      overlayContainer: (_c = props.overlayContainer) != null ? _c : config.overlayContainer,
      overlayContainerFallback: `.${mergedPrefixCls.value}-overlay-container`,
      overlayTabindex: (_d = props.overlayTabindex) != null ? _d : config.overlayTabindex,
      readonly: props.readonly,
      size: mergedSize.value,
      status: mergedStatus.value,
      suffix: (_e = props.suffix) != null ? _e : config.suffix,
      suffixRotate: false,
      "onUpdate:open": setOverlayOpened,
      onClear: handleClear,
      onFocus: handleFocus,
      onBlur: handleBlur,
      onKeydown: handleKeyDown
    };
  });
}
function useInputProps(context) {
  return computed(() => {
    var _a, _b;
    const { props, config, accessor } = context;
    return {
      borderless: false,
      clearable: (_a = props.clearable) != null ? _a : config.clearable,
      clearIcon: (_b = props.clearIcon) != null ? _b : config.clearIcon,
      disabled: accessor.disabled,
      size: "sm"
    };
  });
}
function useActiveValue(props, dateConfig, formatRef, valueRef) {
  const defaultOpenValue = computed(() => convertToDate(dateConfig, props.defaultOpenValue, formatRef.value));
  const [activeValue, setActiveValue] = useState(() => {
    var _a;
    return (_a = valueRef.value) != null ? _a : defaultOpenValue.value;
  });
  watch(valueRef, (value) => {
    setActiveValue(value != null ? value : defaultOpenValue.value);
  });
  return {
    activeValue,
    setActiveValue
  };
}
function useRangeActiveValue(props, dateConfig, formatRef, valuesRef) {
  const defaultOpenValue = computed(
    () => convertArray(props.defaultOpenValue).map((value) => convertToDate(dateConfig, value, formatRef.value))
  );
  const [fromActiveValue, setFromActiveValue] = useState(() => {
    var _a, _b;
    return (_b = (_a = valuesRef.value) == null ? void 0 : _a[0]) != null ? _b : defaultOpenValue.value[0];
  });
  const [toActiveValue, setToActiveValue] = useState(() => {
    var _a, _b;
    return (_b = (_a = valuesRef.value) == null ? void 0 : _a[1]) != null ? _b : defaultOpenValue.value[1];
  });
  watch(valuesRef, (values) => {
    var _a, _b;
    setFromActiveValue((_a = values == null ? void 0 : values[0]) != null ? _a : defaultOpenValue.value[0]);
    setToActiveValue((_b = values == null ? void 0 : values[1]) != null ? _b : defaultOpenValue.value[1]);
  });
  return {
    fromActiveValue,
    toActiveValue,
    setFromActiveValue,
    setToActiveValue
  };
}
function usePanelProps(props, formatRef) {
  return computed(() => {
    const { disabledHours, disabledMinutes, disabledSeconds, hideDisabledOptions, hourStep, minuteStep, secondStep } = props;
    const format = formatRef.value;
    return {
      disabledHours,
      disabledMinutes,
      disabledSeconds,
      hideDisabledOptions,
      hourStep,
      minuteStep,
      secondStep,
      hourEnabled: checkHourEnabled(format),
      minuteEnabled: checkMinuteEnabled(format),
      secondEnabled: checkSecondEnabled(format),
      use12Hours: checkUse12Hours(format),
      amPmCapital: checkAmPmCapital(format)
    };
  });
}
const timePickerContext = Symbol("timePickerContext");
const timeRangePickerContext = Symbol("timeRangePickerContext");
var Content = /* @__PURE__ */ defineComponent({
  setup() {
    const context = inject(timePickerContext);
    const {
      slots,
      props,
      locale,
      formatRef,
      dateConfig,
      inputRef,
      inputEnableStatus,
      mergedPrefixCls,
      overlayOpened,
      handleClear,
      handleKeyDown,
      controlContext: {
        inputValue,
        panelValue,
        inputFocused,
        handleInput,
        handleInputFocus,
        handleInputBlur,
        handlePanelChange
      }
    } = context;
    const inputInstance = ref();
    const setInputRef = () => {
      var _a;
      if (inputEnableStatus.value.enableInternalInput) {
        inputRef.value = (_a = inputInstance.value) == null ? void 0 : _a.getInputElement();
      }
    };
    onMounted(setInputRef);
    onUpdated(setInputRef);
    const inputProps = useInputProps(context);
    const panelProps = usePanelProps(props, formatRef);
    const {
      activeValue,
      setActiveValue
    } = useActiveValue(props, dateConfig, formatRef, panelValue);
    return () => {
      var _a;
      const prefixCls = `${mergedPrefixCls.value}-overlay`;
      const boardPrefixCls = `${mergedPrefixCls.value}-board`;
      const _panelProps = {
        ...panelProps.value,
        activeValue: activeValue.value,
        visible: overlayOpened.value,
        value: panelValue.value,
        onChange: handlePanelChange,
        "onUpdate:activeValue": setActiveValue
      };
      const children = [createVNode("div", {
        "class": `${prefixCls}-body ${boardPrefixCls}`
      }, [inputEnableStatus.value.enableInternalInput && createVNode(_Input, mergeProps({
        "ref": inputInstance,
        "class": `${boardPrefixCls}-input`
      }, inputProps.value, {
        "clearVisible": !!inputValue.value,
        "value": inputValue.value,
        "readonly": props.readonly,
        "focused": inputFocused.value,
        "placeholder": (_a = props.placeholder) != null ? _a : locale.timePicker.placeholder,
        "onInput": handleInput,
        "onClear": handleClear,
        "onFocus": handleInputFocus,
        "onBlur": handleInputBlur,
        "onKeydown": handleKeyDown
      }), slots), createVNode(_TimePanel, mergeProps(_panelProps, {
        "class": `${boardPrefixCls}-panel`,
        "tabindex": -1
      }), null)]), createVNode(_Footer, {
        "class": `${prefixCls}-footer`,
        "footer": props.footer
      }, slots)];
      return props.overlayRender ? props.overlayRender(children) : createVNode("div", null, [children]);
    };
  }
});
var Trigger = /* @__PURE__ */ defineComponent({
  inheritAttrs: false,
  setup(_, {
    expose
  }) {
    const context = inject(timePickerContext);
    const {
      accessor,
      props,
      locale,
      mergedPrefixCls,
      formatRef,
      inputRef,
      inputEnableStatus,
      controlContext: {
        inputValue,
        handleInput: _handleInput
      }
    } = context;
    const triggerInputRef = ref();
    const placeholder = computed(() => {
      var _a;
      return (_a = props.placeholder) != null ? _a : locale.timePicker.placeholder;
    });
    const inputSize = computed(() => Math.max(10, formatRef.value.length) + 2);
    const handleInput = (evt) => {
      _handleInput(evt);
      callEmit(props.onInput, evt);
    };
    const focus = () => {
      var _a;
      (_a = (inputEnableStatus.value.enableExternalInput ? inputRef : triggerInputRef).value) == null ? void 0 : _a.focus();
    };
    expose({
      focus
    });
    return () => {
      return createVNode("input", {
        "ref": inputEnableStatus.value.enableExternalInput ? inputRef : triggerInputRef,
        "class": `${mergedPrefixCls.value}-input`,
        "autocomplete": "off",
        "disabled": accessor.disabled,
        "placeholder": placeholder.value,
        "readonly": props.readonly || !inputEnableStatus.value.enableExternalInput,
        "size": inputSize.value,
        "value": inputValue.value,
        "onInput": handleInput
      }, null);
    };
  }
});
const timePickerCommonProps = {
  ..._baseTimePanelProps,
  open: {
    type: Boolean,
    default: void 0
  },
  control: {
    type: [String, Number, Object, Array],
    default: void 0
  },
  allowInput: {
    type: [Boolean, String],
    default: void 0
  },
  autofocus: {
    type: Boolean,
    default: void 0
  },
  borderless: {
    type: Boolean,
    default: void 0
  },
  clearable: {
    type: Boolean,
    default: void 0
  },
  clearIcon: String,
  clearText: String,
  disabled: {
    type: Boolean,
    default: false
  },
  format: String,
  overlayClassName: String,
  overlayContainer: {
    type: [String, HTMLElement, Function],
    default: void 0
  },
  overlayTabindex: { type: Number, default: void 0 },
  overlayRender: Function,
  readonly: {
    type: Boolean,
    default: false
  },
  size: String,
  status: String,
  suffix: String,
  "onUpdate:open": [Function, Array],
  onClear: [Function, Array],
  onFocus: [Function, Array],
  onBlur: [Function, Array]
};
const timePickerProps = {
  ...timePickerCommonProps,
  value: [String, Date, Number],
  defaultOpenValue: [String, Date, Number],
  footer: { type: [Boolean, Array, Object], default: false },
  placeholder: String,
  "onUpdate:value": [Function, Array],
  onChange: [Function, Array],
  onInput: [Function, Array]
};
const timeRangePickerProps = {
  ...timePickerCommonProps,
  value: Array,
  defaultOpenValue: Array,
  footer: { type: [Boolean, Array, Object], default: true },
  placeholder: Array,
  separator: [String, Object],
  "onUpdate:value": [Function, Array],
  onChange: [Function, Array],
  onInput: [Function, Array]
};
function getDefaultThemeTokens(tokens) {
  const {
    scrollbarThumbBg,
    scrollbarTrackBg,
    fontWeightXl,
    colorTextInfo,
    colorTextTitle,
    marginSizeXl,
    paddingSize2Xs,
    paddingSizeSm,
    paddingSizeLg,
    fontSizeSm
  } = tokens;
  const rangeSeparatorWidth = 32;
  const rangeSeparatorFontSize = fontSizeSm;
  return {
    separatorMarginHorizontal: marginSizeXl,
    overlayWidth: 200,
    overlayPadding: `${paddingSizeSm}px ${paddingSizeSm}px 0 ${paddingSizeSm}px`,
    overlayFooterPadding: `${paddingSizeSm}px ${paddingSizeLg}px`,
    rangeBoardWidth: 184,
    rangeBoardBorder: "none",
    rangeOverlayPadding: `${paddingSizeLg}px ${paddingSizeLg}px 0 ${paddingSizeLg}px`,
    rangeOverlayFooterPadding: `${paddingSizeSm}px 0`,
    rangeOverlaySeparatorPadding: `${paddingSize2Xs}px ${(rangeSeparatorWidth - fontSizeSm) / 2}px`,
    rangeOverlaySeparatorFontSize: rangeSeparatorFontSize,
    panelHeight: 224,
    panelPaddingHorizontal: paddingSizeSm,
    panelPaddingVertical: paddingSizeSm,
    panelFontSize: fontSizeSm,
    panelCellHeight: 32,
    panelCellPaddingHorizontal: paddingSizeSm,
    panelCellColor: colorTextInfo,
    panelCellColorActive: colorTextTitle,
    panelCellBgColorHover: "transparent",
    panelCellBgColorActive: "transparent",
    panelCellFontWeightActive: fontWeightXl,
    panelScrollbarWidth: 6,
    panelScrollbarThumbBgColor: scrollbarThumbBg,
    panelScrollbarThumbBorderRadius: 10,
    panelScrollbarTrackBgColor: scrollbarTrackBg
  };
}
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens) : getDefaultThemeTokens(tokens);
};
var TimePicker = /* @__PURE__ */ defineComponent({
  name: "IxTimePicker",
  inheritAttrs: false,
  props: timePickerProps,
  setup(props, {
    attrs,
    expose,
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("timePicker");
    registerToken(getThemeTokens);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-time-picker`);
    const locale = useGlobalConfig("locale");
    const config = useGlobalConfig("timePicker");
    const dateConfig = useDateConfig();
    const triggerRef = ref();
    const {
      elementRef: inputRef,
      focus,
      blur
    } = useFormElement();
    const formatRef = computed(() => {
      var _a;
      return (_a = props.format) != null ? _a : config.format;
    });
    const {
      overlayOpened,
      setOverlayOpened
    } = useOverlayState(props, focus);
    const pickerState = usePickerState(props, config, dateConfig, formatRef);
    const {
      accessor,
      focused,
      handleFocus,
      handleBlur,
      handleChange
    } = pickerState;
    const pickerControl = usePickerControl(dateConfig, formatRef, handleChange, toRef(accessor, "value"));
    const inputEnableStatus = useInputEnableStatus(props, config);
    const handleKeyDown = useKeyboardEvents(overlayOpened, setOverlayOpened);
    const context = {
      props,
      slots,
      dateConfig,
      common,
      locale,
      config,
      mergedPrefixCls,
      formatRef,
      handleKeyDown,
      inputRef,
      inputEnableStatus,
      overlayOpened,
      setOverlayOpened,
      controlContext: pickerControl,
      ...pickerState,
      handleFocus,
      handleBlur
    };
    provide(timePickerContext, context);
    const triggerProps = useTriggerProps(context);
    expose({
      focus,
      blur
    });
    watch(overlayOpened, (opened) => {
      var _a;
      if (opened) {
        setTimeout(() => {
          var _a2;
          (_a2 = inputRef.value) == null ? void 0 : _a2.focus();
        });
      } else {
        pickerControl.init(true);
        if (focused.value) {
          (_a = triggerRef.value) == null ? void 0 : _a.focus();
        }
      }
    });
    const renderTrigger = () => createVNode(Trigger, mergeProps({
      "ref": triggerRef
    }, attrs), null);
    const renderContent = () => createVNode(Content, null, null);
    const overlayClass = computed(() => normalizeClass([`${mergedPrefixCls.value}-overlay`, globalHashId.value, hashId.value, props.overlayClassName]));
    return () => createVNode(IxControlTrigger, mergeProps(triggerProps.value, {
      "class": `${mergedPrefixCls.value} ${globalHashId.value} ${hashId.value}`,
      "overlayClassName": overlayClass.value
    }, attrs), {
      default: renderTrigger,
      overlay: renderContent,
      suffix: slots.suffix,
      clearIcon: slots.clearIcon
    });
  }
});
function useRangePickerControl(dateConfig, formatRef, valueRef) {
  const [buffer, setBuffer] = useState(
    convertArray(valueRef.value).map((v) => convertToDate(dateConfig, v, formatRef.value))
  );
  const [bufferUpdated, setBufferUpdated] = useState(false);
  const handleBufferUpdate = (values) => {
    setBuffer(getRangeValue(dateConfig, values, formatRef.value));
    setBufferUpdated(true);
  };
  watch([valueRef, formatRef], ([value]) => {
    handleBufferUpdate(value);
  });
  const rangeValueRef = computed(() => convertArray(buffer.value));
  const fromValue = computed(() => rangeValueRef.value[0]);
  const toValue = computed(() => rangeValueRef.value[1]);
  const fromControl = usePickerControl(
    dateConfig,
    formatRef,
    (value) => {
      handleBufferUpdate([value, toValue.value]);
    },
    fromValue
  );
  const toControl = usePickerControl(
    dateConfig,
    formatRef,
    (value) => {
      handleBufferUpdate([fromValue.value, value]);
    },
    toValue
  );
  const init = (force = false) => {
    handleBufferUpdate(valueRef.value);
    setBufferUpdated(false);
    fromControl.init(force);
    toControl.init(force);
  };
  return {
    buffer,
    bufferUpdated,
    fromControl,
    toControl,
    init
  };
}
function getRangeValue(dateConfig, values, format) {
  return convertArray(values).map((v) => convertToDate(dateConfig, v, format));
}
var RangeOverlay = /* @__PURE__ */ defineComponent({
  setup() {
    const context = inject(timeRangePickerContext);
    const {
      props,
      slots,
      dateConfig,
      locale,
      mergedPrefixCls,
      formatRef,
      inputRef,
      inputEnableStatus,
      overlayOpened,
      handleChange,
      handleKeyDown,
      renderSeparator,
      setOverlayOpened,
      rangeControlContext: {
        buffer,
        fromControl,
        toControl
      }
    } = context;
    const inputInstance = ref();
    const setInputRef = () => {
      var _a;
      if (inputEnableStatus.value.enableInternalInput) {
        inputRef.value = (_a = inputInstance.value) == null ? void 0 : _a.getInputElement();
      }
    };
    onMounted(setInputRef);
    onUpdated(setInputRef);
    const inputProps = useInputProps(context);
    const panelProps = usePanelProps(props, formatRef);
    const {
      fromActiveValue,
      toActiveValue,
      setFromActiveValue,
      setToActiveValue
    } = useRangeActiveValue(props, dateConfig, formatRef, computed(() => [fromControl.panelValue.value, toControl.panelValue.value]));
    const handleConfirm = () => {
      handleChange(buffer.value);
      setOverlayOpened(false);
    };
    function renderBoard(isFrom) {
      var _a, _b;
      const {
        inputValue,
        inputFocused,
        panelValue,
        handleInput,
        handleInputClear,
        handleInputFocus,
        handleInputBlur,
        handlePanelChange
      } = isFrom ? fromControl : toControl;
      const idx = isFrom ? 0 : 1;
      const placeholder = (_b = (_a = props.placeholder) == null ? void 0 : _a[idx]) != null ? _b : locale.timeRangePicker.placeholder[idx];
      const prefixCls = `${mergedPrefixCls.value}-board`;
      const _panelProps = {
        ...panelProps.value,
        activeValue: isFrom ? fromActiveValue.value : toActiveValue.value,
        visible: overlayOpened.value,
        value: panelValue.value,
        onChange: handlePanelChange,
        "onUpdate:activeValue": isFrom ? setFromActiveValue : setToActiveValue
      };
      return createVNode("div", {
        "class": prefixCls
      }, [inputEnableStatus.value.enableInternalInput && createVNode(_Input, mergeProps({
        "ref": isFrom ? inputInstance : void 0,
        "class": `${prefixCls}-input`
      }, inputProps.value, {
        "clearVisible": !!inputValue.value,
        "value": inputValue.value,
        "readonly": props.readonly,
        "focused": inputFocused.value,
        "placeholder": placeholder,
        "onInput": handleInput,
        "onClear": handleInputClear,
        "onFocus": handleInputFocus,
        "onBlur": handleInputBlur,
        "onKeydown": handleKeyDown
      }), slots), createVNode(_TimePanel, mergeProps(_panelProps, {
        "class": `${prefixCls}-panel`,
        "tabindex": -1
      }), null)]);
    }
    return () => {
      const prefixCls = `${mergedPrefixCls.value}-overlay`;
      const separatorClasses = {
        [`${prefixCls}-separator`]: true,
        [`${prefixCls}-separator-line`]: !inputEnableStatus.value.enableInternalInput
      };
      const children = [createVNode("div", {
        "class": `${prefixCls}-body`
      }, [renderBoard(true), createVNode("div", {
        "class": separatorClasses
      }, [inputEnableStatus.value.enableInternalInput && renderSeparator()]), renderBoard(false)]), createVNode(_Footer, {
        "class": `${prefixCls}-footer`,
        "footer": props.footer,
        "okText": locale.timeRangePicker.okText,
        "okButton": {
          size: "xs",
          mode: "primary"
        },
        "cancelVisible": false,
        "ok": handleConfirm
      }, slots)];
      return props.overlayRender ? props.overlayRender(children) : createVNode("div", null, [children]);
    };
  }
});
var RangeTrigger = /* @__PURE__ */ defineComponent({
  inheritAttrs: false,
  setup(_, {
    expose
  }) {
    const context = inject(timeRangePickerContext);
    const {
      accessor,
      props,
      locale,
      mergedPrefixCls,
      inputRef,
      inputEnableStatus,
      renderSeparator,
      rangeControlContext: {
        fromControl,
        toControl
      }
    } = context;
    const triggerInputRef = ref();
    const placeholders = computed(() => {
      var _a, _b, _c, _d;
      return [(_b = (_a = props.placeholder) == null ? void 0 : _a[0]) != null ? _b : locale.timeRangePicker.placeholder[0], (_d = (_c = props.placeholder) == null ? void 0 : _c[1]) != null ? _d : locale.timeRangePicker.placeholder[1]];
    });
    const focus = () => {
      var _a;
      (_a = (inputEnableStatus.value.enableExternalInput ? inputRef : triggerInputRef).value) == null ? void 0 : _a.focus();
    };
    expose({
      focus
    });
    const handleFromInput = (evt) => {
      fromControl.handleInput(evt);
      callEmit(props.onInput, true, evt);
    };
    const handleToInput = (evt) => {
      toControl.handleInput(evt);
      callEmit(props.onInput, false, evt);
    };
    function renderSide(prefixCls, isFrom) {
      const {
        inputValue
      } = isFrom ? fromControl : toControl;
      const placeholder = placeholders.value[isFrom ? 0 : 1];
      const handleInput = isFrom ? handleFromInput : handleToInput;
      return createVNode("input", {
        "ref": isFrom ? inputEnableStatus.value.enableExternalInput ? inputRef : triggerInputRef : void 0,
        "class": `${prefixCls}-input-inner`,
        "autocomplete": "off",
        "disabled": accessor.disabled,
        "value": inputValue.value,
        "placeholder": placeholder,
        "readonly": props.readonly || !inputEnableStatus.value.enableExternalInput,
        "onInput": handleInput
      }, null);
    }
    return () => {
      const prefixCls = mergedPrefixCls.value;
      return createVNode("span", {
        "class": `${prefixCls}-input`
      }, [renderSide(prefixCls, true), createVNode("span", {
        "class": `${prefixCls}-input-separator`
      }, [renderSeparator()]), renderSide(prefixCls, false)]);
    };
  }
});
var TimeRangePicker = /* @__PURE__ */ defineComponent({
  name: "IxTimeRangePicker",
  inheritAttrs: false,
  props: timeRangePickerProps,
  setup(props, {
    attrs,
    expose,
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("timePicker");
    registerToken(getThemeTokens);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-time-range-picker`);
    const locale = useGlobalConfig("locale");
    const config = useGlobalConfig("timePicker");
    const dateConfig = useDateConfig();
    const triggerRef = ref();
    const {
      elementRef: inputRef,
      focus,
      blur
    } = useFormElement();
    const formatRef = computed(() => {
      var _a;
      return (_a = props.format) != null ? _a : config.format;
    });
    const {
      overlayOpened,
      setOverlayOpened
    } = useOverlayState(props, focus);
    const pickerState = usePickerState(props, config, dateConfig, formatRef);
    const {
      accessor,
      focused,
      handleFocus,
      handleBlur,
      handleChange
    } = pickerState;
    const rangePickerControl = useRangePickerControl(dateConfig, formatRef, toRef(accessor, "value"));
    const inputEnableStatus = useInputEnableStatus(props, config);
    const handleKeyDown = useRangeKeyboardEvents(rangePickerControl, overlayOpened, setOverlayOpened, handleChange);
    const renderSeparator = () => {
      var _a, _b, _c;
      return (_c = (_b = (_a = slots.separator) == null ? void 0 : _a.call(slots)) != null ? _b : props.separator) != null ? _c : locale.timeRangePicker.separator;
    };
    const context = {
      props,
      slots,
      dateConfig,
      common,
      locale,
      config,
      mergedPrefixCls,
      formatRef,
      handleKeyDown,
      inputRef,
      inputEnableStatus,
      overlayOpened,
      setOverlayOpened,
      renderSeparator,
      rangeControlContext: rangePickerControl,
      ...pickerState,
      handleFocus,
      handleBlur
    };
    const triggerProps = useTriggerProps(context);
    provide(timeRangePickerContext, context);
    expose({
      focus,
      blur
    });
    watch(overlayOpened, (opened) => {
      var _a;
      if (opened) {
        setTimeout(() => {
          var _a2;
          (_a2 = inputRef.value) == null ? void 0 : _a2.focus();
        });
      } else {
        rangePickerControl.init(true);
        if (focused.value) {
          (_a = triggerRef.value) == null ? void 0 : _a.focus();
        }
      }
    });
    const renderTrigger = () => createVNode(RangeTrigger, mergeProps({
      "ref": triggerRef
    }, attrs), null);
    const renderContent = () => createVNode(RangeOverlay, null, null);
    const overlayClass = computed(() => normalizeClass([`${mergedPrefixCls.value}-overlay`, globalHashId.value, hashId.value, props.overlayClassName]));
    return () => createVNode(IxControlTrigger, mergeProps(triggerProps.value, {
      "class": `${mergedPrefixCls.value} ${globalHashId.value} ${hashId.value}`,
      "overlayClassName": overlayClass.value
    }, attrs), {
      default: renderTrigger,
      overlay: renderContent,
      suffix: slots.suffix,
      clearIcon: slots.clearIcon
    });
  }
});
const IxTimePicker = TimePicker;
const IxTimeRangePicker = TimeRangePicker;
export { IxTimePicker, IxTimeRangePicker, getThemeTokens as getTimePickerThemeTokens };
