/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { ExtractInnerPropTypes, ExtractPublicPropTypes } from '@idux/cdk/utils';
import type { DefineComponent, HTMLAttributes, PropType } from 'vue';
export type TimelinePlacement = 'start' | 'alternate' | 'end';
export type TimelineLabelPlacement = 'top' | 'bottom';
export type TimelineItemPlacement = 'start' | 'end';
export declare const timelineProps: {
    readonly pending: {
        readonly type: PropType<string | boolean>;
        readonly default: false;
    };
    readonly pendingDot: StringConstructor;
    readonly reverse: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly placement: {
        readonly type: PropType<TimelinePlacement>;
        readonly default: "end";
    };
    readonly labelPlacement: {
        readonly type: PropType<TimelineLabelPlacement>;
        readonly default: "bottom";
    };
    readonly both: {
        readonly type: BooleanConstructor;
        readonly default: true;
    };
};
export type TimelineProps = ExtractInnerPropTypes<typeof timelineProps>;
export type TimelinePublicProps = ExtractPublicPropTypes<typeof timelineProps>;
export type TimelineComponent = DefineComponent<Omit<HTMLAttributes, keyof TimelinePublicProps> & TimelinePublicProps>;
export type TimelineInstance = InstanceType<DefineComponent<TimelineProps>>;
export declare const timelineItemProps: {
    readonly color: {
        readonly type: StringConstructor;
        readonly default: "primary";
    };
    readonly dot: StringConstructor;
    readonly label: StringConstructor;
    readonly placement: PropType<TimelineItemPlacement>;
};
export type TimelineItemProps = ExtractInnerPropTypes<typeof timelineItemProps>;
export type TimelineItemPublicProps = ExtractPublicPropTypes<typeof timelineItemProps>;
export type TimelineItemComponent = DefineComponent<Omit<HTMLAttributes, keyof TimelineItemPublicProps> & TimelineItemPublicProps>;
export type TimelineItemInstance = InstanceType<DefineComponent<TimelineItemProps>>;
