import { ref, onDeactivated, computed, defineComponent, createVNode, mergeProps } from "vue";
import { \u0275OverlayDelayDef as _OverlayDelayDef, \u0275OverlayPlacementDef as _OverlayPlacementDef, \u0275OverlayTriggerDef as _OverlayTriggerDef, \u0275Overlay as _Overlay } from "@idux/components/_private/overlay";
import { useGlobalConfig } from "@idux/components/config";
import { useThemeToken } from "@idux/components/theme";
import { useControlledProp } from "@idux/cdk/utils";
const tooltipProps = {
  visible: { type: Boolean, default: void 0 },
  autoAdjust: { type: Boolean, default: void 0 },
  closeOnDeactivated: { type: Boolean, default: true },
  destroyOnHide: { type: Boolean, default: void 0 },
  delay: _OverlayDelayDef,
  disabled: { type: Boolean, default: false },
  offset: Array,
  overlayContainer: {
    type: [String, HTMLElement, Function],
    default: void 0
  },
  placement: _OverlayPlacementDef,
  showArrow: { type: Boolean, default: void 0 },
  title: String,
  trigger: _OverlayTriggerDef,
  zIndex: Number,
  "onUpdate:visible": [Function, Array]
};
function useTooltipOverlay(props, config, mergedPrefixCls) {
  const overlayRef = ref();
  const updatePopper = () => {
    var _a;
    return (_a = overlayRef.value) == null ? void 0 : _a.updatePopper();
  };
  const [visible, setVisible] = useControlledProp(props, "visible", false);
  onDeactivated(() => {
    if (visible.value && props.closeOnDeactivated) {
      setVisible(false);
    }
  });
  const overlayProps = computed(() => {
    var _a, _b, _c, _d, _e, _f, _g, _h;
    const trigger = (_a = props.trigger) != null ? _a : config.trigger;
    return {
      visible: visible.value,
      ["onUpdate:visible"]: setVisible,
      autoAdjust: (_b = props.autoAdjust) != null ? _b : config.autoAdjust,
      clickOutside: trigger === "click" || trigger === "contextmenu",
      container: (_c = props.overlayContainer) != null ? _c : config.overlayContainer,
      containerFallback: `.${mergedPrefixCls.value}-overlay-container`,
      delay: (_d = props.delay) != null ? _d : config.delay,
      destroyOnHide: (_e = props.destroyOnHide) != null ? _e : config.destroyOnHide,
      disabled: props.disabled,
      offset: (_f = props.offset) != null ? _f : config.offset,
      showArrow: (_g = props.showArrow) != null ? _g : config.showArrow,
      placement: (_h = props.placement) != null ? _h : config.placement,
      trigger,
      zIndex: props.zIndex
    };
  });
  return { overlayRef, updatePopper, visible, setVisible, overlayProps };
}
function getDefaultThemeTokens(tokens) {
  const { fontSizeSm, colorTextInfo } = tokens;
  return {
    fontSize: fontSizeSm,
    color: colorTextInfo,
    minWidth: 32,
    maxWidth: 400,
    whiteSpace: "pre-wrap"
  };
}
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens) : getDefaultThemeTokens(tokens);
};
var Tooltip = /* @__PURE__ */ defineComponent({
  name: "IxTooltip",
  props: tooltipProps,
  setup(props, {
    slots,
    expose
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("tooltip");
    registerToken(getThemeTokens);
    const config = useGlobalConfig("tooltip");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-tooltip`);
    const {
      overlayRef,
      updatePopper,
      overlayProps
    } = useTooltipOverlay(props, config, mergedPrefixCls);
    expose({
      updatePopper
    });
    return () => {
      const prefixCls = mergedPrefixCls.value;
      return createVNode(_Overlay, mergeProps({
        "ref": overlayRef,
        "class": [prefixCls, globalHashId.value, hashId.value],
        "transitionName": `${common.prefixCls}-fade-fast`
      }, overlayProps.value), {
        default: slots.default,
        content: () => renderContent(props, slots, prefixCls)
      });
    };
  }
});
const renderContent = (props, slots, prefixCls) => {
  if (!(slots.title || props.title)) {
    return null;
  }
  return createVNode("div", {
    "class": `${prefixCls}-wrapper`
  }, [slots.title ? slots.title() : props.title]);
};
const IxTooltip = Tooltip;
export { IxTooltip, getThemeTokens as getTooltipThemeTokens, tooltipProps as \u0275TooltipProps, useTooltipOverlay as \u0275UseTooltipOverlay };
