import { defineComponent, inject, computed, normalizeClass, createVNode, mergeProps, onMounted, watch, normalizeStyle, onUnmounted, ref, shallowRef, toRef, provide, Fragment, Transition, withDirectives, vShow } from "vue";
import { CdkPortal } from "@idux/cdk/portal";
import { useState, isEmptyNode, callEmit, useControlledProp, convertElement, useEventListener, convertCssPixel, cancelRAF, rAF } from "@idux/cdk/utils";
import { \u0275OverlayPlacementDef as _OverlayPlacementDef, \u0275Overlay as _Overlay } from "@idux/components/_private/overlay";
import { useGlobalConfig } from "@idux/components/config";
import { useThemeToken } from "@idux/components/theme";
import { useZIndex } from "@idux/components/utils";
import { \u0275Header as _Header } from "@idux/components/_private/header";
import { IxButton } from "@idux/components/button";
import { merge, isNumber, isString, isFunction, isBoolean } from "lodash-es";
const tourToken = Symbol("tourToken");
var Mask = /* @__PURE__ */ defineComponent({
  setup() {
    const {
      globalHashId,
      hashId
    } = useThemeToken("tour");
    const {
      mergedPrefixCls,
      mergedProps,
      maskPath,
      maskOutlinePath,
      maskAttrs,
      maskClass,
      maskStyle,
      maskOutlineStyle,
      activeStep
    } = inject(tourToken);
    const classes = computed(() => {
      var _a;
      const prefixCls = `${mergedPrefixCls.value}-mask`;
      const {
        animatable
      } = mergedProps.value;
      return normalizeClass([prefixCls, globalHashId.value, hashId.value, maskClass.value, animatable ? `${prefixCls}-animatable` : void 0, ((_a = activeStep.value) == null ? void 0 : _a.targetDisabled) ? `${prefixCls}-target-disabled` : void 0]);
    });
    return () => {
      return createVNode("svg", mergeProps({
        "class": classes.value,
        "style": maskStyle.value
      }, maskAttrs.value), [createVNode("path", {
        "class": `${mergedPrefixCls.value}-mask-target`,
        "d": maskPath.value
      }, null), maskOutlinePath.value && createVNode("path", {
        "class": `${mergedPrefixCls.value}-mask-outline`,
        "style": maskOutlineStyle.value,
        "d": maskOutlinePath.value
      }, null)]);
    };
  }
});
var Panel = /* @__PURE__ */ defineComponent({
  setup(_, {
    slots
  }) {
    const {
      mergedPrefixCls,
      mergedProps,
      activeStep,
      activeIndex,
      goTo,
      finish,
      isCurrentMax,
      setVisible,
      onStepChange
    } = inject(tourToken);
    const [renderData, setRenderData] = useState(void 0);
    const updateRenderData = () => {
      setRenderData({
        step: activeStep.value,
        activeIndex: activeIndex.value,
        isCurrentMax: isCurrentMax.value
      });
    };
    onMounted(updateRenderData);
    onStepChange(updateRenderData);
    const handleClose = () => {
      setVisible(false);
      callEmit(mergedProps.value.onClose);
    };
    const renderHeader = () => {
      var _a;
      const {
        step: {
          title = void 0
        } = {},
        activeIndex: activeIndex2 = 0
      } = (_a = renderData.value) != null ? _a : {};
      const {
        closable
      } = mergedProps.value;
      if (!title && !slots.title && !slots.header && !closable) {
        return;
      }
      return createVNode(_Header, {
        "size": "sm",
        "closable": closable,
        "closeIcon": "close",
        "header": title,
        "onClose": handleClose
      }, {
        header: slots.header,
        title: slots.title ? () => slots.title({
          activeIndex: activeIndex2,
          title
        }) : void 0
      });
    };
    const renderDescription = (prefixCls) => {
      var _a;
      const {
        step: {
          description = void 0
        } = {},
        activeIndex: activeIndex2 = 0
      } = (_a = renderData.value) != null ? _a : {};
      const children = slots.description ? slots.description({
        activeIndex: activeIndex2,
        description
      }) : description;
      return !isEmptyNode(children) ? createVNode("div", {
        "class": `${prefixCls}-description`
      }, [children]) : void 0;
    };
    const renderIndicators = (prefixCls) => {
      var _a, _b, _c;
      const total = (_b = (_a = mergedProps.value.steps) == null ? void 0 : _a.length) != null ? _b : 0;
      const {
        activeIndex: activeIndex2 = 0
      } = (_c = renderData.value) != null ? _c : {};
      return createVNode("div", {
        "class": `${prefixCls}-indicators`
      }, [slots.indicators ? slots.indicators({
        activeIndex: activeIndex2,
        total
      }) : `${activeIndex2 + 1} / ${total}`]);
    };
    const _goTo = (offset) => {
      var _a;
      if (!((_a = renderData.value) == null ? void 0 : _a.step)) {
        return;
      }
      goTo(renderData.value.step.index + offset);
    };
    const next = () => {
      _goTo(1);
    };
    const prev = () => {
      _goTo(-1);
    };
    const renderButtons = (prefixCls) => {
      var _a;
      if (!((_a = renderData.value) == null ? void 0 : _a.step)) {
        return;
      }
      const {
        step: {
          nextButton,
          nextButtonText,
          prevButton,
          prevButtonText
        },
        activeIndex: activeIndex2,
        isCurrentMax: isCurrentMax2
      } = renderData.value;
      return createVNode("div", {
        "class": `${prefixCls}-buttons`
      }, [activeIndex2 > 0 && prevButton && createVNode(IxButton, mergeProps({
        "class": `${prefixCls}-button`
      }, prevButton, {
        "onClick": prev
      }), {
        default: () => [prevButtonText]
      }), nextButton && createVNode(IxButton, mergeProps({
        "class": `${prefixCls}-button`
      }, nextButton, {
        "onClick": isCurrentMax2 ? finish : next
      }), {
        default: () => [nextButtonText]
      })]);
    };
    return () => {
      const prefixCls = `${mergedPrefixCls.value}-panel`;
      const header = renderHeader();
      const innerCls = [`${prefixCls}-inner`, header ? void 0 : `${prefixCls}-inner-no-header`];
      return createVNode("div", {
        "class": prefixCls,
        "onClick": (evt) => evt.stopImmediatePropagation()
      }, [header, createVNode("div", {
        "class": innerCls
      }, [renderDescription(prefixCls), createVNode("div", {
        "class": `${prefixCls}-footer`
      }, [renderIndicators(prefixCls), renderButtons(prefixCls)])])]);
    };
  }
});
function useActiveState(props, visibleContext) {
  const { visible, setVisible } = visibleContext;
  const [activeIndex, _setActiveIndex] = useControlledProp(props, "activeIndex", 0);
  const maxIndex = computed(() => {
    var _a;
    if (!((_a = props.steps) == null ? void 0 : _a.length)) {
      return 0;
    }
    return props.steps.length - 1;
  });
  const isCurrentMax = computed(() => activeIndex.value >= maxIndex.value);
  const setActiveIndex = (index) => {
    if (index === activeIndex.value) {
      return;
    }
    const prevCurrent = activeIndex.value;
    _setActiveIndex(index);
    callEmit(props.onChange, index, prevCurrent);
  };
  const goTo = (step) => {
    const resolvedStep = Math.min(Math.max(step, 0), maxIndex.value);
    setActiveIndex(resolvedStep);
  };
  const finish = () => {
    setVisible(false);
    callEmit(props.onFinish);
  };
  watch(
    visible,
    (v) => {
      if (!v) {
        setActiveIndex(0);
      }
    },
    {
      flush: "post"
    }
  );
  return {
    isCurrentMax,
    activeIndex,
    setActiveIndex,
    goTo,
    finish
  };
}
function useActiveStep(mergedProps, activeIndex, isCurrentMax, locale) {
  const [activeStep, setActiveStep] = useState(void 0);
  let destructions = [];
  const destroySteps = () => {
    destructions.forEach((destory) => destory());
    destructions = [];
  };
  const getActiveStep = async (index) => {
    var _a, _b, _c, _d, _e, _f, _g, _h, _i;
    const props = mergedProps.value;
    const step = (_a = props.steps) == null ? void 0 : _a[index];
    if (!step) {
      return;
    }
    if (step.beforeEnter) {
      await step.beforeEnter();
    }
    const gap = (_b = step.gap) != null ? _b : props.gap;
    const mergedGap = merge({ ...props.gap }, isNumber(gap) ? { offset: gap } : gap);
    const target = async () => {
      var _a2, _b2;
      if (!step.target) {
        return null;
      }
      if (isString(step.target)) {
        return document.querySelector(step.target);
      }
      if (isFunction(step.target)) {
        return (_a2 = convertElement(await step.target())) != null ? _a2 : null;
      }
      return (_b2 = convertElement(step.target)) != null ? _b2 : null;
    };
    const nextButton = step.nextButton === false ? null : {
      size: "xs",
      mode: "primary",
      ...step.nextButton === true ? {} : step.nextButton
    };
    const prevButton = step.prevButton === false ? null : {
      size: "xs",
      ...step.prevButton === true ? {} : step.prevButton
    };
    return {
      ...step,
      index,
      target,
      targetDisabled: (_c = step.targetDisabled) != null ? _c : props.targetDisabled,
      gap: mergedGap,
      mask: (_d = step.mask) != null ? _d : props.mask,
      placement: (_e = step.placement) != null ? _e : props.placement,
      showArrow: (_f = step.showArrow) != null ? _f : props.showArrow,
      nextButton,
      prevButton,
      nextButtonText: (_g = step.nextButtonText) != null ? _g : isCurrentMax.value ? locale.finishText : locale.nextText,
      prevButtonText: (_h = step.prevButtonText) != null ? _h : locale.prevText,
      scrollIntoViewOptions: (_i = step.scrollIntoViewOptions) != null ? _i : props.scrollIntoViewOptions
    };
  };
  const pushCurrentUpdate = async (index) => {
    if (index < 0) {
      setActiveStep(void 0);
      return;
    }
    const promise = getActiveStep(index);
    destructions.push(async () => {
      var _a;
      const step2 = await promise;
      (_a = step2 == null ? void 0 : step2.afterLeave) == null ? void 0 : _a.call(step2);
    });
    const step = await promise;
    if (activeIndex.value === index) {
      setActiveStep(step);
    }
  };
  watch(
    [activeIndex, () => mergedProps.value.steps, mergedProps],
    ([current]) => {
      destroySteps();
      pushCurrentUpdate(current);
    },
    { immediate: true, flush: "post", deep: true }
  );
  return activeStep;
}
function isInViewPort(element) {
  const viewWidth = window.innerWidth || document.documentElement.clientWidth;
  const viewHeight = window.innerHeight || document.documentElement.clientHeight;
  const { top, right, bottom, left } = element.getBoundingClientRect();
  return top >= 0 && left >= 0 && right <= viewWidth && bottom <= viewHeight;
}
function isInBox(x, y, box) {
  return x > box.x && y > box.y && x < box.x + box.width && y < box.y + box.height;
}
function easeInOutQuad(elapsed, initialValue, amountOfChange, duration) {
  if ((elapsed /= duration / 2) < 1) {
    return amountOfChange / 2 * elapsed * elapsed + initialValue;
  }
  return -amountOfChange / 2 * (--elapsed * (elapsed - 2) - 1) + initialValue;
}
function useCloseTrigger(mergedProps, positionInfo, visible, setVisible) {
  let delayedVisible = visible.value;
  let delayTmr;
  watch(visible, (v) => {
    delayTmr && clearTimeout(delayTmr);
    delayTmr = setTimeout(() => {
      delayedVisible = v;
    }, 100);
  });
  useEventListener(window, "keydown", (evt) => {
    if (mergedProps.value.closeOnEsc && evt.code === "Escape") {
      setVisible(false);
      callEmit(mergedProps.value.onClose);
    }
  });
  useEventListener(window, "click", (evt) => {
    if (!positionInfo.value || !delayedVisible) {
      return;
    }
    if (mergedProps.value.closeOnClick && !isInBox(evt.clientX, evt.clientY, positionInfo.value)) {
      setVisible(false);
      callEmit(mergedProps.value.onClose);
    }
  });
}
const animateDuration = 200;
function useMask(mergedProps, activeStep, activeIndex, positionInfo, visible, currentZIndex) {
  const [maskPath, setMaskPath] = useState("");
  const [maskOutlinePath, setMaskOutlinePath] = useState(void 0);
  const updateMask = (positionInfo2) => {
    if (positionInfo2 === false) {
      setMaskPath("");
      setMaskOutlinePath(void 0);
    } else {
      setMaskPath(getMaskPath(positionInfo2));
      setMaskOutlinePath(getMaskOutlinePath(positionInfo2));
    }
  };
  const [isAnimating, setIsAnimating] = useState(false);
  const animateCbs = /* @__PURE__ */ new Set();
  const maskAttrs = computed(() => positionInfo.value ? getMaskAttrs() : {});
  const maskStyle = computed(() => {
    var _a;
    const { mask } = (_a = activeStep.value) != null ? _a : {};
    return normalizeStyle({
      fill: isBoolean(mask) ? void 0 : mask == null ? void 0 : mask.color,
      width: convertCssPixel(window.innerWidth),
      height: convertCssPixel(window.innerHeight),
      zIndex: currentZIndex.value
    });
  });
  const maskOutlineStyle = computed(() => {
    var _a;
    const { mask } = (_a = activeStep.value) != null ? _a : {};
    return normalizeStyle({
      fill: isBoolean(mask) ? void 0 : mask == null ? void 0 : mask.outlineColor
    });
  });
  const maskClass = computed(() => {
    var _a;
    const { mask } = (_a = activeStep.value) != null ? _a : {};
    return isBoolean(mask) ? void 0 : mask == null ? void 0 : mask.class;
  });
  const onAnimateEnd = (cb) => {
    animateCbs.add(cb);
  };
  const runAnimateCbs = () => {
    animateCbs.forEach((cb) => cb());
  };
  onUnmounted(() => {
    animateCbs.clear();
  });
  let rAFHandle;
  const cancelAnimate = () => {
    cancelRAF(rAFHandle);
    setIsAnimating(false);
  };
  const animate = (from, to) => {
    const start = Date.now();
    setIsAnimating(true);
    const tick = () => {
      const elapsed = Date.now() - start;
      const currentTickPos = {
        containerWidth: to.containerWidth,
        containerHeight: to.containerHeight,
        containerX: to.containerX,
        containerY: to.containerY,
        x: easeInOutQuad(elapsed, from.x, to.x - from.x, animateDuration),
        y: easeInOutQuad(elapsed, from.y, to.y - from.y, animateDuration),
        width: easeInOutQuad(elapsed, from.width, to.width - from.width, animateDuration),
        height: easeInOutQuad(elapsed, from.height, to.height - from.height, animateDuration),
        radius: easeInOutQuad(elapsed, from.radius, to.radius - from.radius, animateDuration),
        outline: to.outline
      };
      updateMask(currentTickPos);
      if (elapsed < animateDuration) {
        rAFHandle = rAF(tick);
      } else {
        updateMask(to);
        setIsAnimating(false);
        runAnimateCbs();
      }
    };
    rAFHandle = rAF(tick);
  };
  let _tempIndex = activeIndex.value;
  watch(
    positionInfo,
    (pos, prePos) => {
      var _a;
      if (!visible.value) {
        _tempIndex = activeIndex.value;
        return;
      }
      if (!((_a = activeStep.value) == null ? void 0 : _a.mask)) {
        cancelAnimate();
        updateMask(false);
      } else if (!mergedProps.value.animatable || (pos == null ? void 0 : pos.origin) !== "index" || !prePos || !pos) {
        updateMask(pos);
        if (mergedProps.value.animatable && (isAnimating.value || activeIndex.value !== _tempIndex)) {
          runAnimateCbs();
        }
        cancelAnimate();
      } else {
        cancelAnimate();
        animate(prePos, pos);
      }
      _tempIndex = activeIndex.value;
    },
    {
      flush: "post"
    }
  );
  return {
    maskPath,
    maskOutlinePath,
    maskAttrs,
    maskStyle,
    maskOutlineStyle,
    maskClass,
    isAnimating,
    onAnimateEnd
  };
}
function arch(h, v, radius) {
  return `a${radius},${radius} 0 0 1 ${h * radius},${v * radius}`;
}
function drawRect(x, y, width, height, radius = 0) {
  const limitedRadius = Math.min(radius, width / 2, height / 2);
  const normalizedRadius = Math.floor(Math.max(limitedRadius, 0));
  const boxX = x + normalizedRadius;
  const boxY = y;
  const boxWidth = width - normalizedRadius * 2;
  const boxHeight = height - normalizedRadius * 2;
  return `M${boxX},${boxY} h${boxWidth} ${arch(1, 1, normalizedRadius)} v${boxHeight} ${arch(-1, 1, normalizedRadius)} h-${boxWidth} ${arch(
    -1,
    -1,
    normalizedRadius
  )} v-${boxHeight} ${arch(1, -1, normalizedRadius)} z`;
}
function getMaskPath(positionInfo) {
  if (!positionInfo) {
    return drawRect(0, 0, window.innerWidth, window.innerHeight, 0);
  }
  const { containerX, containerY, containerWidth, containerHeight, x, y, width, height, radius } = positionInfo;
  return `${drawRect(containerX, containerY, containerWidth, containerHeight)}${drawRect(x, y, width, height, radius)}`;
}
function getMaskOutlinePath(positionInfo) {
  if (!positionInfo || !positionInfo.outline) {
    return;
  }
  const { x, y, width, height, radius, outline } = positionInfo;
  return `${drawRect(x - 1, y - 1, width + 2, height + 2, radius)}${drawRect(x + outline, y + outline, width - outline * 2, height - outline * 2, radius)}`;
}
function getMaskAttrs() {
  return {
    viewBox: `0 0 ${window.innerWidth} ${window.innerHeight}`,
    version: "1.1",
    preserveAspectRatio: "xMinYMin slice"
  };
}
function useMergedProps(props, config) {
  return computed(() => {
    var _a, _b, _c, _d, _e, _f, _g, _h, _i;
    const propGap = props.gap ? isNumber(props.gap) ? { offset: props.gap } : { ...props.gap } : void 0;
    return {
      ...props,
      animatable: (_a = props.animatable) != null ? _a : config.animatable,
      gap: propGap ? merge({ ...config.gap }, propGap) : config.gap,
      offset: (_b = props.offset) != null ? _b : config.offset,
      overlayContainer: (_c = props.overlayContainer) != null ? _c : config.overlayContainer,
      placement: (_d = props.placement) != null ? _d : config.placement,
      showArrow: (_e = props.showArrow) != null ? _e : config.showArrow,
      scrollIntoViewOptions: (_f = props.scrollIntoViewOptions) != null ? _f : config.scrollIntoViewOptions,
      closeOnClick: (_g = props.closeOnClick) != null ? _g : config.closeOnClick,
      closeOnEsc: (_h = props.closeOnEsc) != null ? _h : config.closeOnEsc,
      closable: (_i = props.closable) != null ? _i : config.closable
    };
  });
}
function useOverlayProps(componentCommonConfig, containerFallback, mergedProps, activeStep, isStepChanging, visible, currentZIndex, onStepChange) {
  const currentActiveStep = ref(activeStep.value);
  onStepChange(() => {
    currentActiveStep.value = activeStep.value;
  });
  return computed(() => {
    var _a;
    const { animatable, overlayContainer, offset } = mergedProps.value;
    const { placement = "bottomStart", showArrow } = (_a = currentActiveStep.value) != null ? _a : {};
    return {
      visible: visible.value && !!currentActiveStep.value && !isStepChanging.value,
      container: overlayContainer,
      containerFallback: containerFallback.value,
      trigger: "manual",
      placement,
      showArrow,
      delay: 0,
      transitionName: animatable ? `${componentCommonConfig.prefixCls}-fade-fast` : void 0,
      offset,
      zIndex: currentZIndex.value
    };
  });
}
const transitionDuration = 200;
function useStepChange(mergedProps, activeIndex, activeStep, visible, onAnimateEnd) {
  const stepChangeCbs = /* @__PURE__ */ new Set();
  const [isStepChanging, _setIsStepChanging] = useState(false);
  let transitionTmr;
  const locks = {
    stepChange: false,
    animate: false
  };
  const isLocked = () => locks.stepChange || locks.animate;
  const lock = () => {
    locks.stepChange = true;
    locks.animate = true;
    setIsStepChanging(true);
  };
  const unlock = (lock2) => {
    const locked = isLocked();
    locks[lock2] = false;
    if (locked && !locks.animate && !locks.stepChange) {
      setIsStepChanging(false);
    }
  };
  const unlockAll = () => {
    unlock("animate");
    unlock("stepChange");
  };
  const setIsStepChanging = (changing) => {
    if (isStepChanging.value === changing) {
      return;
    }
    _setIsStepChanging(changing);
    if (!changing) {
      runStepChangeCbs();
    }
  };
  const onStepChange = (cb) => {
    stepChangeCbs.add(cb);
  };
  const runStepChangeCbs = () => {
    stepChangeCbs.forEach((cb) => cb());
  };
  onAnimateEnd(() => {
    var _a;
    if (mergedProps.value.animatable && ((_a = activeStep.value) == null ? void 0 : _a.mask)) {
      unlock("animate");
    }
  });
  watch(
    activeIndex,
    (current, pre) => {
      if (current !== pre) {
        transitionTmr && clearTimeout(transitionTmr);
        lock();
      }
    },
    {
      flush: "pre"
    }
  );
  watch(
    activeStep,
    (step, preStep) => {
      if (!isLocked()) {
        runStepChangeCbs();
        return;
      }
      if (step && !preStep) {
        unlockAll();
        return;
      }
      if (!mergedProps.value.animatable || !(step == null ? void 0 : step.mask) || !visible.value) {
        unlock("animate");
      }
      transitionTmr && clearTimeout(transitionTmr);
      transitionTmr = setTimeout(
        () => {
          unlock("stepChange");
        },
        mergedProps.value.animatable ? transitionDuration : 0
      );
    },
    {
      immediate: true,
      flush: "post"
    }
  );
  onUnmounted(() => {
    stepChangeCbs.clear();
  });
  return {
    isStepChanging,
    onStepChange
  };
}
function useTarget(mergedProps, activeStep, visible) {
  const targetRef = shallowRef(null);
  const [positionInfo, setPositionInfo] = useState(null);
  let scrollIntoViewTarget = null;
  const updateTarget = async () => {
    var _a, _b;
    targetRef.value = (_b = await ((_a = activeStep.value) == null ? void 0 : _a.target())) != null ? _b : null;
  };
  const getContainerPos = () => {
    var _a;
    const mask = (_a = activeStep.value) == null ? void 0 : _a.mask;
    if (isBoolean(mask) || !mask || !mask.container || mask.container === "viewport") {
      return {
        containerX: 0,
        containerY: 0,
        containerWidth: window.innerWidth,
        containerHeight: window.innerHeight
      };
    }
    const { x, y, width, height } = mask.container;
    return {
      containerX: x,
      containerY: y,
      containerWidth: width,
      containerHeight: height
    };
  };
  const getTargetPositionInfo = () => {
    var _a, _b;
    const targetEl = targetRef.value;
    const { offset = 0, radius = 0, outline = 0 } = (_b = (_a = activeStep.value) == null ? void 0 : _a.gap) != null ? _b : {};
    if (!targetEl) {
      return {
        x: window.innerWidth / 2,
        y: window.innerHeight / 2,
        width: 0,
        height: 0,
        radius,
        outline
      };
    }
    let { x, y, width, height } = targetEl.getBoundingClientRect();
    const mergedOffset = offset + (outline ? outline + 1 : 0);
    if (mergedOffset) {
      x -= mergedOffset;
      y -= mergedOffset;
      width += mergedOffset * 2;
      height += mergedOffset * 2;
    }
    return {
      x,
      y,
      width,
      height,
      radius,
      outline
    };
  };
  const updatePopsition = (scrollIntoView = false, origin = "index") => {
    const targetEl = targetRef.value;
    const positionInfo2 = {
      ...getContainerPos(),
      ...getTargetPositionInfo(),
      origin
    };
    if (!targetEl) {
      setPositionInfo(positionInfo2);
      return;
    }
    if (isInViewPort(targetEl)) {
      scrollIntoViewTarget = null;
    } else if (scrollIntoView && targetEl !== scrollIntoViewTarget) {
      scrollIntoViewTarget = targetEl;
      targetEl.scrollIntoView(mergedProps.value.scrollIntoViewOptions);
      return;
    }
    setPositionInfo(positionInfo2);
  };
  let stopResizeLisiten;
  let stopScrollLisiten;
  onMounted(() => {
    watch([() => {
      var _a;
      return (_a = activeStep.value) == null ? void 0 : _a.target;
    }, visible], updateTarget, { immediate: true });
    watch(
      [targetRef, activeStep],
      ([target], [preTarget]) => updatePopsition(true, target !== preTarget ? "index" : "step-update"),
      { flush: "post", immediate: true }
    );
    watch(visible, () => updatePopsition(true, "visible"));
    watch(
      visible,
      (v) => {
        if (v) {
          stopResizeLisiten = useEventListener(window, "resize", () => updatePopsition(false, "resize"));
          stopScrollLisiten = useEventListener(window, "scroll", () => updatePopsition(false, "scroll"));
        } else {
          stopResizeLisiten == null ? void 0 : stopResizeLisiten();
          stopScrollLisiten == null ? void 0 : stopScrollLisiten();
        }
      },
      { immediate: true }
    );
  });
  onUnmounted(() => {
    stopResizeLisiten == null ? void 0 : stopResizeLisiten();
    stopScrollLisiten == null ? void 0 : stopScrollLisiten();
  });
  return {
    targetRef,
    positionInfo
  };
}
function useVisible(props) {
  const [visible, setVisible] = useControlledProp(props, "visible", false);
  return {
    visible,
    setVisible
  };
}
const tourProps = {
  activeIndex: {
    type: Number,
    default: void 0
  },
  visible: {
    type: Boolean,
    default: void 0
  },
  closable: {
    type: Boolean,
    default: void 0
  },
  closeOnClick: {
    type: Boolean,
    default: void 0
  },
  closeOnEsc: {
    type: Boolean,
    default: void 0
  },
  animatable: {
    type: Boolean,
    default: void 0
  },
  gap: [Number, Object],
  mask: {
    type: [Boolean, Object],
    default: true
  },
  offset: Array,
  overlayContainer: {
    type: [String, HTMLElement],
    default: void 0
  },
  placement: _OverlayPlacementDef,
  steps: Array,
  showArrow: {
    type: Boolean,
    default: void 0
  },
  scrollIntoViewOptions: {
    type: [Boolean, Object]
  },
  targetDisabled: {
    type: Boolean,
    default: false
  },
  zIndex: Number,
  onChange: [Function, Array],
  onClose: [Function, Array],
  onFinish: [Function, Array],
  "onUpdate:activeIndex": [Function, Array],
  "onUpdate:visible": [Function, Array]
};
function getDefaultThemeTokens(tokens) {
  const {
    colorPrimary,
    colorText,
    colorTextInfo,
    colorContainerBg,
    borderRadiusMd,
    fontSizeSm,
    marginSizeXs,
    paddingSizeLg
  } = tokens;
  return {
    bgColor: colorContainerBg,
    descriptionColor: colorText,
    indicatorsColor: colorTextInfo,
    outlineColor: colorPrimary,
    borderRadius: borderRadiusMd,
    minWidth: 250,
    paddingTop: paddingSizeLg,
    paddingLeft: paddingSizeLg,
    paddingRight: paddingSizeLg,
    paddingBottom: paddingSizeLg,
    descriptionFontSize: fontSizeSm,
    indicatorsFontSize: fontSizeSm,
    footerBtnGap: marginSizeXs
  };
}
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens) : getDefaultThemeTokens(tokens);
};
var Tour = /* @__PURE__ */ defineComponent({
  name: "IxTour",
  inheritAttrs: false,
  props: tourProps,
  setup(props, {
    slots,
    attrs
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("tour");
    registerToken(getThemeTokens);
    const config = useGlobalConfig("tour");
    const locale = useGlobalConfig("locale");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-tour`);
    const mergedProps = useMergedProps(props, config);
    const visibleContext = useVisible(props);
    const activeStateContext = useActiveState(props, visibleContext);
    const {
      visible,
      setVisible
    } = visibleContext;
    const {
      activeIndex,
      isCurrentMax
    } = activeStateContext;
    const activeStep = useActiveStep(mergedProps, activeIndex, isCurrentMax, locale.tour);
    const {
      positionInfo
    } = useTarget(mergedProps, activeStep, visible);
    const currentZIndex = useZIndex(toRef(props, "zIndex"), toRef(common, "overlayZIndex"), visible);
    const maskContext = useMask(mergedProps, activeStep, activeIndex, positionInfo, visible, currentZIndex);
    const {
      onAnimateEnd
    } = maskContext;
    useCloseTrigger(mergedProps, positionInfo, visible, setVisible);
    const stepChangeContext = useStepChange(mergedProps, activeIndex, activeStep, visible, onAnimateEnd);
    const {
      isStepChanging,
      onStepChange
    } = stepChangeContext;
    const mergedContainerFallback = computed(() => `.${mergedPrefixCls.value}-overlay-container`);
    const mergedContainer = computed(() => {
      var _a;
      return (_a = mergedProps.value.overlayContainer) != null ? _a : mergedContainerFallback.value;
    });
    const overlayProps = useOverlayProps(common, mergedContainerFallback, mergedProps, activeStep, isStepChanging, visible, currentZIndex, onStepChange);
    const placeholderStyle = computed(() => {
      if (!positionInfo.value) {
        return;
      }
      const {
        x,
        y,
        width,
        height
      } = positionInfo.value;
      return {
        top: convertCssPixel(y),
        left: convertCssPixel(x),
        width: convertCssPixel(width),
        height: convertCssPixel(height)
      };
    });
    provide(tourToken, {
      mergedPrefixCls,
      mergedProps,
      locale: locale.tour,
      activeStep,
      ...activeStateContext,
      ...maskContext,
      ...visibleContext,
      ...stepChangeContext
    });
    return () => {
      const prefixCls = mergedPrefixCls.value;
      const overlaySlots = {
        default: () => createVNode("div", {
          "class": `${prefixCls}-placeholder`,
          "style": placeholderStyle.value
        }, null),
        content: () => createVNode(Panel, null, slots)
      };
      return createVNode(Fragment, null, [createVNode(CdkPortal, {
        "target": mergedContainer.value,
        "load": visible.value
      }, {
        default: () => [createVNode(Transition, {
          "name": mergedProps.value.animatable ? `${common.prefixCls}-fade-fast` : void 0
        }, {
          default: () => {
            var _a;
            return [withDirectives(createVNode(Mask, null, null), [[vShow, visible.value && !!((_a = activeStep.value) == null ? void 0 : _a.mask)]])];
          }
        })]
      }), createVNode(_Overlay, mergeProps({
        "class": [`${prefixCls}-overlay`, globalHashId.value, hashId.value]
      }, overlayProps.value, attrs), overlaySlots)]);
    };
  }
});
const IxTour = Tour;
export { IxTour, getThemeTokens as getTourThemeTokens };
