/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { PopperPlacement } from '@idux/cdk/popper';
import type { ExtractInnerPropTypes, ExtractPublicPropTypes, MaybeArray, MaybeElement } from '@idux/cdk/utils';
import type { ButtonProps } from '@idux/components/button';
import type { DefineComponent, HTMLAttributes, PropType } from 'vue';
export type TargetPositionOrigin = 'resize' | 'scroll' | 'index' | 'visible' | 'step-update';
export interface TargetPositionInfo {
    containerX: number;
    containerY: number;
    containerHeight: number;
    containerWidth: number;
    x: number;
    y: number;
    width: number;
    height: number;
    radius: number;
    outline: number;
    origin: TargetPositionOrigin;
}
export interface TargetGap {
    offset?: number;
    radius?: number;
    outline?: number;
}
export interface TourMaskOptions {
    color?: string;
    class?: string;
    container?: 'viewport' | {
        x: number;
        y: number;
        width: number;
        height: number;
    };
    outlineColor?: string;
}
export type TargetGetter = () => MaybeElement | null | Promise<MaybeElement | null>;
export interface TourStep {
    title?: string;
    description?: string;
    gap?: number | TargetGap;
    mask?: boolean | TourMaskOptions;
    target?: MaybeElement | null | string | TargetGetter;
    targetDisabled?: boolean;
    placement?: PopperPlacement;
    showArrow?: boolean;
    nextButton?: ButtonProps | boolean;
    nextButtonText?: string;
    prevButton?: ButtonProps | boolean;
    prevButtonText?: string;
    scrollIntoViewOptions?: boolean | ScrollIntoViewOptions;
    beforeEnter?: () => void | Promise<void>;
    afterLeave?: () => void | Promise<void>;
}
export type ResolvedTourStep = Pick<TourStep, 'title' | 'description' | 'beforeEnter' | 'afterLeave'> & {
    target: () => HTMLElement | null | Promise<HTMLElement | null>;
    nextButton: ButtonProps | null;
    prevButton: ButtonProps | null;
    gap: TargetGap;
    index: number;
} & Required<Omit<TourStep, 'title' | 'description' | 'target' | 'gap' | 'beforeEnter' | 'afterLeave' | 'nextButton' | 'prevButton'>>;
export declare const tourProps: {
    readonly activeIndex: {
        readonly type: NumberConstructor;
        readonly default: undefined;
    };
    readonly visible: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly closable: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly closeOnClick: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly closeOnEsc: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly animatable: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly gap: PropType<number | TargetGap>;
    readonly mask: {
        readonly type: PropType<boolean | TourMaskOptions>;
        readonly default: true;
    };
    readonly offset: PropType<[number, number]>;
    readonly overlayContainer: {
        readonly type: PropType<string | HTMLElement>;
        readonly default: undefined;
    };
    readonly placement: PropType<PopperPlacement>;
    readonly steps: PropType<TourStep[]>;
    readonly showArrow: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly scrollIntoViewOptions: {
        readonly type: PropType<boolean | ScrollIntoViewOptions>;
    };
    readonly targetDisabled: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly zIndex: NumberConstructor;
    readonly onChange: PropType<MaybeArray<(current: number, pre: number) => void>>;
    readonly onClose: PropType<MaybeArray<() => void>>;
    readonly onFinish: PropType<MaybeArray<() => void>>;
    readonly 'onUpdate:activeIndex': PropType<MaybeArray<(activeIndex: number) => void>>;
    readonly 'onUpdate:visible': PropType<MaybeArray<(visible: boolean) => void>>;
};
export type TourProps = ExtractInnerPropTypes<typeof tourProps>;
export type TourPublicProps = ExtractPublicPropTypes<typeof tourProps>;
export type TourComponent = DefineComponent<Omit<HTMLAttributes, keyof TourPublicProps> & TourPublicProps>;
export type TourInstance = InstanceType<DefineComponent<TourProps>>;
