import { unref, defineComponent, inject, createVNode, withDirectives, resolveDirective, ref, computed, watchEffect, onUnmounted, watch, nextTick, normalizeClass, provide, normalizeStyle, mergeProps, onMounted, withKeys } from "vue";
import { useGlobalConfig } from "@idux/components/config";
import { useThemeToken } from "@idux/components/theme";
import { isString, isBoolean, isArray, isObject, isNil } from "lodash-es";
import { IxButton } from "@idux/components/button";
import { convertArray, callEmit, useControlledProp, convertCssPixel, useState, isEmptyNode } from "@idux/cdk/utils";
import { useGetKey, convertStringVNode } from "@idux/components/utils";
import { IxSpin } from "@idux/components/spin";
import { \u0275Empty as _Empty } from "@idux/components/_private/empty";
import { CdkVirtualScroll } from "@idux/cdk/scroll";
import { IxCheckbox } from "@idux/components/checkbox";
import { IxIcon } from "@idux/components/icon";
import { IxPagination } from "@idux/components/pagination";
import { \u0275Input as _Input } from "@idux/components/_private/input";
const transferContext = Symbol("transferContext");
const transferListContext = Symbol("transferListContext");
const TRANSFER_SOURCE_TOKEN = Symbol("TRANSFER_SOURCE_TOKEN");
const TRANSFER_TARGET_TOKEN = Symbol("TRANSFER_TARGET_TOKEN");
const TRANSFER_OPERATIONS_TOKEN = Symbol("TRANSFER_OPERATIONS_TOKEN");
function convertToSlotParams(obj) {
  const params = {};
  Object.keys(obj).forEach((key) => {
    params[key] = unref(obj[key]);
  });
  return params;
}
var TransferOperations = /* @__PURE__ */ defineComponent({
  setup() {
    const {
      slots,
      props: transferProps2,
      mergedPrefixCls
    } = inject(transferContext);
    const transferOperationsContext = inject(TRANSFER_OPERATIONS_TOKEN);
    const {
      appendDisabled,
      removeDisabled,
      triggerAppend,
      triggerRemove
    } = transferOperationsContext;
    const handleTransferRightClick = () => {
      triggerAppend();
    };
    const handleTransferLeftClick = () => {
      triggerRemove();
    };
    const renderBtn = (prefixCls, icon, disabled, onClick) => {
      if (isString(icon)) {
        return createVNode(IxButton, {
          "class": `${prefixCls}-btn`,
          "icon": icon,
          "disabled": disabled,
          "onClick": onClick
        }, null);
      }
      return withDirectives(createVNode(IxButton, {
        "class": `${prefixCls}-btn`,
        "disabled": disabled,
        "onClick": onClick
      }, null), [[resolveDirective("slos"), {
        icon: () => icon
      }]]);
    };
    const renderOperations = (prefixCls) => {
      if (slots.operations) {
        return slots.operations(convertToSlotParams(transferOperationsContext));
      }
      if (transferProps2.mode === "immediate") {
        return;
      }
      return createVNode("div", {
        "class": `${prefixCls}-inner`
      }, [renderBtn(prefixCls, "double-right", appendDisabled.value, handleTransferRightClick), renderBtn(prefixCls, "double-left", removeDisabled.value, handleTransferLeftClick)]);
    };
    return () => {
      const prefixCls = `${mergedPrefixCls.value}-operations`;
      return createVNode("div", {
        "class": prefixCls
      }, [renderOperations(prefixCls)]);
    };
  }
});
function usePagination(props) {
  const paginationConfig = useGlobalConfig("pagination");
  return {
    sourcePagination: createPagination(props, paginationConfig, true),
    targetPagination: createPagination(props, paginationConfig, false)
  };
}
function createPagination(props, paginationConfig, isSource) {
  const pageIndex = ref();
  const pageSize = ref();
  const tempPagination = computed(() => {
    const pagination = props.pagination;
    if (!pagination) {
      return null;
    }
    if (pagination === true) {
      return {};
    }
    const idx = isSource ? 0 : 1;
    const pageIndex2 = convertArray(pagination.pageIndex);
    const pageSize2 = convertArray(pagination.pageSize);
    const total = convertArray(pagination.total);
    return {
      pageIndex: pageIndex2 == null ? void 0 : pageIndex2[idx],
      pageSize: pageSize2 == null ? void 0 : pageSize2[idx],
      disabled: props.disabled || pagination.disabled,
      total: total == null ? void 0 : total[idx],
      onChange: (pageIndex3, pageSize3) => {
        var _a;
        return (_a = pagination.onChange) == null ? void 0 : _a.call(pagination, isSource, pageIndex3, pageSize3);
      }
    };
  });
  watchEffect(() => {
    var _a, _b;
    const pagination = tempPagination.value;
    pageIndex.value = (_a = pagination == null ? void 0 : pagination.pageIndex) != null ? _a : 1;
    pageSize.value = (_b = pagination == null ? void 0 : pagination.pageSize) != null ? _b : paginationConfig.pageSize;
  });
  const handlePageIndexChange = (index) => {
    pageIndex.value = index;
  };
  return computed(() => {
    const pagination = tempPagination.value;
    if (pagination === null) {
      return;
    }
    return {
      simple: true,
      showTotal: false,
      showQuickJumper: true,
      ...pagination,
      pageIndex: pageIndex.value,
      pageSize: pageSize.value,
      "onUpdate:pageIndex": handlePageIndexChange
    };
  });
}
function useSearchPlaceholder(props, locales) {
  return {
    source: computed(() => getPlaceholder(props, locales, true)),
    target: computed(() => getPlaceholder(props, locales, false))
  };
}
function getPlaceholder(props, locales, isSource) {
  var _a;
  const defaultPlaceholder = locales.searchPlaceholder[isSource ? 0 : 1];
  const searchPlaceholder = isString(props.searchPlaceholder) ? props.searchPlaceholder : (_a = props.searchPlaceholder) == null ? void 0 : _a[isSource ? 0 : 1];
  return searchPlaceholder != null ? searchPlaceholder : defaultPlaceholder;
}
function useSearchable(props, config) {
  return {
    source: computed(() => getSearchable(props, config, true)),
    target: computed(() => getSearchable(props, config, false))
  };
}
function getSearchable(props, config, isSource) {
  var _a;
  const searchable = (_a = props.searchable) != null ? _a : config.searchable;
  if (!searchable || isBoolean(searchable)) {
    return !!searchable;
  }
  return isSource ? searchable.source : searchable.target;
}
function useTransferBindings(props, transferDataContext, transferOperationsContext, transferSelectStateContext, transferPaginationContext, showSelectAll, sourceSearchable, sourceSearchPlaceholder, targetSearchable, targetSearchPlaceholder) {
  const {
    dataSource,
    sourceData,
    targetData,
    targetKeySet,
    dataKeyMap,
    disabledKeys,
    disabledSourceKeys,
    disabledTargetKeys,
    filteredDataSource,
    filteredSourceData,
    filteredTargetData,
    paginatedDataSource,
    paginatedSourceData,
    paginatedTargetData,
    sourceSearchValue,
    setSourceSearchValue,
    targetSearchValue,
    setTargetSearchValue,
    getKey
  } = transferDataContext;
  const {
    sourceSelectedKeys,
    targetSelectedKeys,
    sourceSelectedKeySet,
    targetSelectedKeySet,
    sourceSelectAllStatus,
    targetSelectAllStatus,
    sourceSelectAllDisabled,
    targetSelectAllDisabled,
    handleSourceSelectChange,
    handleTargetSelectChange,
    handleSourceSelectAll,
    handleTargetSelectAll
  } = transferSelectStateContext;
  const { triggerAppend, triggerRemove } = transferOperationsContext;
  const commonBindings = {
    dataSource,
    dataKeyMap,
    disabledDataSourceKeys: disabledKeys,
    filteredDataSource,
    paginatedDataSource,
    targetKeySet,
    showSelectAll,
    triggerAppend,
    triggerRemove,
    getKey
  };
  return {
    sourceBindings: {
      data: sourceData,
      filteredData: filteredSourceData,
      paginatedData: paginatedSourceData,
      pagination: transferPaginationContext.sourcePagination,
      disabledKeys: disabledSourceKeys,
      selectedKeys: sourceSelectedKeys,
      selectedKeySet: sourceSelectedKeySet,
      selectAllDisabled: sourceSelectAllDisabled,
      selectAllStatus: sourceSelectAllStatus,
      searchable: sourceSearchable,
      searchPlaceholder: sourceSearchPlaceholder,
      selectAll: handleSourceSelectAll,
      handleSelectChange: handleSourceSelectChange,
      searchValue: sourceSearchValue,
      handleSearchChange: (searchValue) => {
        setSourceSearchValue(searchValue);
        callEmit(props.onSearch, true, searchValue);
      },
      ...commonBindings
    },
    targetBindings: {
      data: targetData,
      filteredData: filteredTargetData,
      paginatedData: paginatedTargetData,
      pagination: transferPaginationContext.targetPagination,
      disabledKeys: disabledTargetKeys,
      selectedKeys: targetSelectedKeys,
      selectedKeySet: targetSelectedKeySet,
      selectAllDisabled: targetSelectAllDisabled,
      selectAllStatus: targetSelectAllStatus,
      searchable: targetSearchable,
      searchPlaceholder: targetSearchPlaceholder,
      selectAll: handleTargetSelectAll,
      handleSelectChange: handleTargetSelectChange,
      searchValue: targetSearchValue,
      handleSearchChange: (searchValue) => {
        setTargetSearchValue(searchValue);
        callEmit(props.onSearch, false, searchValue);
      },
      ...commonBindings
    }
  };
}
function useTransferData(props, config, transferDataStrategy, transferPaginationContext) {
  const _getKey = useGetKey(props, config, "transfer");
  const getKey = computed(() => {
    const getKeyFn = _getKey.value;
    return (data) => {
      var _a;
      return (_a = getKeyFn(data)) != null ? _a : data.key;
    };
  });
  const dataSource = computed(() => props.dataSource);
  const dataKeyMap = computed(() => transferDataStrategy.value.genDataKeyMap(dataSource.value, getKey.value));
  const [targetKeys, setTargetKeys] = useControlledProp(props, "value", () => []);
  const [sourceSearchValue, setSourceSearchValue] = useControlledProp(props, "sourceSearchValue", "");
  const [targetSearchValue, setTargetSearchValue] = useControlledProp(props, "targetSearchValue", "");
  const targetKeySet = computed(() => new Set(targetKeys.value));
  const handleChange = (keys) => {
    const oldKeys = targetKeys.value;
    setTargetKeys(keys);
    callEmit(props.onChange, keys, oldKeys);
  };
  const separatedData = computed(
    () => transferDataStrategy.value.separateDataSource(dataSource.value, dataKeyMap.value, targetKeySet.value, getKey.value)
  );
  const sourceData = computed(() => separatedData.value.sourceData);
  const targetData = computed(() => separatedData.value.targetData);
  const sourceDataKeys = computed(() => transferDataStrategy.value.genDataKeys(sourceData.value, getKey.value));
  const targetDataKeys = computed(() => transferDataStrategy.value.genDataKeys(targetData.value, getKey.value));
  const filteredDataSource = computed(
    () => transferDataStrategy.value.dataFilter(
      dataSource.value,
      sourceSearchValue.value,
      (item, searchValue) => !props.searchFn || props.searchFn(true, item, searchValue)
    )
  );
  const filteredSourceData = computed(
    () => transferDataStrategy.value.dataFilter(
      sourceData.value,
      sourceSearchValue.value,
      (item, searchValue) => !props.searchFn || props.searchFn(true, item, searchValue)
    )
  );
  const filteredTargetData = computed(
    () => transferDataStrategy.value.dataFilter(
      targetData.value,
      targetSearchValue.value,
      (item, searchValue) => !props.searchFn || props.searchFn(false, item, searchValue)
    )
  );
  const paginatedDataSource = computed(
    () => getPaginatedData(filteredDataSource.value, transferPaginationContext.sourcePagination.value)
  );
  const paginatedSourceData = computed(
    () => getPaginatedData(filteredSourceData.value, transferPaginationContext.sourcePagination.value)
  );
  const paginatedTargetData = computed(
    () => getPaginatedData(filteredTargetData.value, transferPaginationContext.targetPagination.value)
  );
  const changeTargetKeys = (removedKeys, appendedKeys) => {
    let newKeys = targetKeys.value;
    if (removedKeys.length > 0) {
      newKeys = transferDataStrategy.value.remove(removedKeys, newKeys, getKey.value);
    }
    if (appendedKeys.length > 0) {
      newKeys = transferDataStrategy.value.append(appendedKeys, newKeys, getKey.value);
    }
    handleChange(newKeys);
  };
  const clear = () => {
    handleChange(Array.from(disabledTargetKeys.value));
  };
  const disabledKeys = computed(() => transferDataStrategy.value.genDisabledKeys(dataSource.value, getKey.value));
  const disabledTargetKeys = computed(() => {
    const _disabledKeys = disabledKeys.value;
    const keys = transferDataStrategy.value.genDisabledKeys(targetData.value, getKey.value);
    targetKeySet.value.forEach((key) => {
      if (_disabledKeys.has(key)) {
        keys.add(key);
      }
    });
    return keys;
  });
  const disabledSourceKeys = computed(() => transferDataStrategy.value.genDisabledKeys(sourceData.value, getKey.value));
  return {
    dataSource,
    dataKeyMap,
    sourceData,
    targetData,
    sourceDataKeys,
    targetDataKeys,
    filteredDataSource,
    filteredSourceData,
    filteredTargetData,
    paginatedDataSource,
    paginatedSourceData,
    paginatedTargetData,
    targetKeys,
    targetKeySet,
    changeTargetKeys,
    clear,
    getKey,
    disabledKeys,
    disabledSourceKeys,
    disabledTargetKeys,
    sourceSearchValue,
    setSourceSearchValue,
    targetSearchValue,
    setTargetSearchValue
  };
}
function getPaginatedData(data, pagination) {
  if (!pagination || !pagination.pageSize) {
    return data;
  }
  const { total } = pagination;
  if (total && data.length < total) {
    return data;
  }
  const pageSize = pagination.pageSize;
  const startIndex = (pagination.pageIndex - 1) * pageSize;
  return data.slice(startIndex, startIndex + pageSize);
}
function createDefaultStrategies(defaultTargetData) {
  const cachedDataKeyMap = /* @__PURE__ */ new Map();
  onUnmounted(() => {
    cachedDataKeyMap.clear();
  });
  const genDataKeys = (data, getKey) => {
    return new Set(data.map((data2) => getKey(data2)));
  };
  return {
    genDataKeys,
    genDataKeyMap: (dataSource, getKey) => {
      const dataKeyMap = /* @__PURE__ */ new Map();
      dataSource.forEach((item) => {
        dataKeyMap.set(getKey(item), item);
      });
      return dataKeyMap;
    },
    genDisabledKeys: (data, getKey) => {
      const keys = /* @__PURE__ */ new Set();
      data.forEach((item) => {
        if (item.disabled) {
          keys.add(getKey(item));
        }
      });
      return keys;
    },
    getAllSelectedKeys: (selected, data, selectedKeySet, disabledkeySet, getKey) => {
      let tempKeys;
      if (!selected) {
        tempKeys = /* @__PURE__ */ new Set();
        disabledkeySet.forEach((key) => {
          selectedKeySet.has(key) && tempKeys.add(key);
        });
      } else {
        tempKeys = genDataKeys(data, getKey);
        disabledkeySet.forEach((key) => {
          !selectedKeySet.has(key) && tempKeys.delete(key);
        });
      }
      return Array.from(tempKeys);
    },
    separateDataSource: (dataSource, dataKeyMap, selectedKeySet, getKey) => {
      const sourceData = dataSource.filter((data) => !selectedKeySet.has(getKey(data)));
      const targetData = [];
      selectedKeySet.forEach((key) => {
        var _a, _b;
        const data = (_b = (_a = dataKeyMap.get(key)) != null ? _a : cachedDataKeyMap.get(key)) != null ? _b : defaultTargetData == null ? void 0 : defaultTargetData.find((data2) => getKey(data2) === key);
        if (data && !cachedDataKeyMap.has(key)) {
          cachedDataKeyMap.set(key, data);
        }
        targetData.push(data != null ? data : { key });
      });
      for (const cachedKey of cachedDataKeyMap.keys()) {
        if (!selectedKeySet.has(cachedKey)) {
          cachedDataKeyMap.delete(cachedKey);
        }
      }
      return {
        sourceData,
        targetData
      };
    },
    dataFilter: (data, searchValue, searchFn) => {
      if (!searchValue) {
        return data;
      }
      return data.filter((item) => searchFn(item, searchValue));
    },
    append: (keys, selectedKeySet) => {
      const newKeys = new Set(selectedKeySet);
      keys.forEach((key) => {
        newKeys.add(key);
      });
      return Array.from(newKeys);
    },
    remove: (keys, selectedKeySet) => {
      const newKeys = new Set(selectedKeySet);
      keys.forEach((key) => {
        newKeys.delete(key);
      });
      return Array.from(newKeys);
    }
  };
}
function useTransferDataStrategies(props) {
  var _a;
  const defaultStrategy = createDefaultStrategies((_a = props.defaultTargetData) != null ? _a : []);
  const strategy = computed(() => {
    return props.dataStrategy ? { ...defaultStrategy, ...props.dataStrategy } : defaultStrategy;
  });
  return strategy;
}
function useTransferOperations(props, transferDataContext, transferSelectStateContext) {
  const {
    dataKeyMap,
    sourceData,
    targetData,
    disabledSourceKeys,
    disabledTargetKeys,
    changeTargetKeys,
    clear,
    getKey
  } = transferDataContext;
  const { sourceSelectedKeys, targetSelectedKeys } = transferSelectStateContext;
  const appendDisabled = computed(() => props.disabled || sourceSelectedKeys.value.length <= 0);
  const removeDisabled = computed(() => props.disabled || targetSelectedKeys.value.length <= 0);
  const appendAllDisabled = computed(
    () => props.disabled || sourceData.value.every((item) => disabledSourceKeys.value.has(getKey.value(item)))
  );
  const clearDisabled = computed(
    () => props.disabled || targetData.value.every((item) => disabledTargetKeys.value.has(getKey.value(item)))
  );
  const triggerAppend = (keys) => {
    if (!keys && appendDisabled.value || keys && props.disabled) {
      return;
    }
    changeTargetKeys(
      [],
      (keys != null ? keys : Array.from(sourceSelectedKeys.value)).filter((key) => !disabledSourceKeys.value.has(key))
    );
  };
  const triggerRemove = (keys) => {
    if (!keys && removeDisabled.value || keys && props.disabled) {
      return;
    }
    changeTargetKeys(
      (keys != null ? keys : Array.from(targetSelectedKeys.value)).filter((key) => !disabledTargetKeys.value.has(key)),
      []
    );
  };
  const triggerAppendAll = () => {
    if (appendAllDisabled.value) {
      return;
    }
    changeTargetKeys(
      [],
      Array.from(dataKeyMap.value.keys()).filter((key) => !disabledSourceKeys.value.has(key))
    );
  };
  const triggerClear = () => {
    if (clearDisabled.value) {
      return;
    }
    clear();
    callEmit(props.onClear);
  };
  return {
    appendDisabled,
    removeDisabled,
    appendAllDisabled,
    clearDisabled,
    triggerAppend,
    triggerRemove,
    triggerAppendAll,
    triggerClear
  };
}
function useTransferSelectState(props, transferDataContext, transferDataStrategy) {
  const [sourceSelectedKeys, setSourceSelectedKeys] = useControlledProp(props, "sourceSelectedKeys", () => []);
  const [targetSelectedKeys, setTargetSelectedKeys] = useControlledProp(props, "targetSelectedKeys", () => []);
  const sourceSelectedKeySet = computed(() => new Set(sourceSelectedKeys.value));
  const targetSelectedKeySet = computed(() => new Set(targetSelectedKeys.value));
  const {
    dataKeyMap,
    dataSource,
    sourceData,
    targetData,
    sourceDataKeys,
    targetDataKeys,
    targetKeySet,
    disabledKeys,
    disabledSourceKeys,
    disabledTargetKeys,
    getKey
  } = transferDataContext;
  const sourceCheckableDataCount = computed(() => {
    const sourceDataCount = props.mode === "immediate" ? dataKeyMap.value.size : sourceDataKeys.value.size;
    const disabledCount = props.mode === "immediate" ? disabledKeys.value.size : disabledSourceKeys.value.size;
    return sourceDataCount - disabledCount;
  });
  const targetCheckableDataCount = computed(() => targetDataKeys.value.size - disabledTargetKeys.value.size);
  const getAllSelectedStatus = (isSource) => {
    const allSelectedKeys = transferDataStrategy.value.getAllSelectedKeys(
      true,
      isSource ? props.mode === "immediate" ? dataSource.value : sourceData.value : targetData.value,
      /* @__PURE__ */ new Set(),
      /* @__PURE__ */ new Set(),
      getKey.value
    );
    const selectedkeyCount = (isSource ? sourceSelectedKeys : targetSelectedKeys).value.length;
    const selectedKeySet = isSource ? sourceSelectedKeySet.value : targetSelectedKeySet.value;
    return {
      checked: selectedkeyCount > 0,
      indeterminate: (allSelectedKeys.length !== selectedkeyCount || allSelectedKeys.some((key) => !selectedKeySet.has(key))) && selectedkeyCount > 0
    };
  };
  const sourceSelectAllStatus = computed(() => getAllSelectedStatus(true));
  const targetSelectAllStatus = computed(() => getAllSelectedStatus(false));
  let transferBySelectionChangeLocked = false;
  watch(sourceSelectedKeySet, (currentCheckedKeys, originalCheckedKeys) => {
    if (props.mode === "immediate") {
      if (transferBySelectionChangeLocked) {
        return;
      }
      transferBySelectionChange(originalCheckedKeys, currentCheckedKeys, transferDataContext);
    }
  });
  watch(
    [sourceCheckableDataCount, dataKeyMap, disabledKeys, targetKeySet],
    (_, [, , , prevSelectedKeys]) => {
      const tempKeys = new Set(sourceSelectedKeys.value);
      const deletedKeys = /* @__PURE__ */ new Set();
      const addedKeys = /* @__PURE__ */ new Set();
      const addKey = (key) => {
        if (!tempKeys.has(key)) {
          tempKeys.add(key);
          addedKeys.add(key);
          deletedKeys.delete(key);
        }
      };
      const deleteKey = (key) => {
        if (tempKeys.has(key)) {
          tempKeys.delete(key);
          deletedKeys.add(key);
          addedKeys.delete(key);
        }
      };
      sourceSelectedKeys.value.forEach((key) => {
        if (!dataKeyMap.value.has(key) || disabledKeys.value.has(key)) {
          deleteKey(key);
          return;
        }
        if (props.mode === "default" && targetKeySet.value.has(key)) {
          deleteKey(key);
        }
      });
      if (props.mode === "immediate") {
        targetKeySet.value.forEach((key) => {
          if (dataKeyMap.value.has(key)) {
            addKey(key);
          }
        });
        prevSelectedKeys == null ? void 0 : prevSelectedKeys.forEach((key) => {
          if (!targetKeySet.value.has(key)) {
            deleteKey(key);
          }
        });
      }
      if (addedKeys.size > 0 || deletedKeys.size > 0) {
        setSourceSelectedKeys(Array.from(tempKeys));
        transferBySelectionChangeLocked = true;
        nextTick(() => {
          transferBySelectionChangeLocked = false;
        });
      }
    },
    { immediate: true, flush: "post" }
  );
  watch(
    [targetCheckableDataCount, targetDataKeys, disabledTargetKeys],
    () => {
      const tempKeys = new Set(targetSelectedKeys.value);
      targetSelectedKeys.value.forEach((key) => {
        if (!targetDataKeys.value.has(key) || disabledTargetKeys.value.has(key)) {
          tempKeys.delete(key);
        }
      });
      setTargetSelectedKeys(Array.from(tempKeys));
    },
    { immediate: true, flush: "post" }
  );
  const handleSourceSelectChange = (keys) => {
    if (props.disabled) {
      return;
    }
    setSourceSelectedKeys(isArray(keys) ? keys : Array.from(keys));
  };
  const handleTargetSelectChange = (keys) => {
    if (props.disabled) {
      return;
    }
    setTargetSelectedKeys(isArray(keys) ? keys : Array.from(keys));
  };
  const handleSelectAll = (selected = true, isSource = true) => {
    if (props.disabled) {
      return;
    }
    const data = isSource ? props.mode === "default" ? sourceData.value : dataSource.value : targetData.value;
    const _disabledKeys = isSource ? props.mode === "default" ? disabledSourceKeys : disabledKeys : disabledTargetKeys;
    const selectedKeySet = isSource ? sourceSelectedKeySet : targetSelectedKeySet;
    const setSelectedKeys = isSource ? setSourceSelectedKeys : setTargetSelectedKeys;
    setSelectedKeys(
      transferDataStrategy.value.getAllSelectedKeys(
        selected,
        data,
        selectedKeySet.value,
        _disabledKeys.value,
        getKey.value
      )
    );
    callEmit(props.onSelectAll, isSource, selected);
  };
  const sourceSelectAllDisabled = computed(() => props.disabled || sourceCheckableDataCount.value <= 0);
  const targetSelectAllDisabled = computed(() => props.disabled || targetCheckableDataCount.value <= 0);
  return {
    sourceSelectedKeys,
    targetSelectedKeys,
    sourceSelectedKeySet,
    targetSelectedKeySet,
    handleSourceSelectChange,
    handleTargetSelectChange,
    sourceSelectAllStatus,
    targetSelectAllStatus,
    handleSourceSelectAll: (selected) => handleSelectAll(selected, true),
    handleTargetSelectAll: (selected) => handleSelectAll(selected, false),
    sourceSelectAllDisabled,
    targetSelectAllDisabled,
    setSourceSelectedKeys,
    setTargetSelectedKeys
  };
}
function transferBySelectionChange(originalCheckedKeys, currentCheckedKeys, transferDataContext) {
  const { changeTargetKeys } = transferDataContext;
  const appendedKeys = [];
  const removedKeys = [];
  for (const key of originalCheckedKeys.values()) {
    if (!currentCheckedKeys.has(key)) {
      removedKeys.push(key);
    }
  }
  for (const key of currentCheckedKeys.values()) {
    if (!originalCheckedKeys.has(key)) {
      appendedKeys.push(key);
    }
  }
  changeTargetKeys(removedKeys, appendedKeys);
}
const transferProps = {
  value: Array,
  sourceSelectedKeys: Array,
  targetSelectedKeys: Array,
  sourceSearchValue: String,
  targetSearchValue: String,
  clearable: {
    type: Boolean,
    default: void 0
  },
  clearIcon: String,
  customAdditional: { type: Function, default: void 0 },
  dataSource: {
    type: Array,
    default: () => []
  },
  dataStrategy: Object,
  defaultTargetData: Array,
  disabled: {
    type: Boolean,
    default: false
  },
  empty: { type: [String, Object], default: "simple" },
  getKey: [String, Function],
  mode: {
    type: String,
    default: "default"
  },
  pagination: {
    type: [Boolean, Object],
    default: void 0
  },
  scroll: Object,
  searchable: {
    type: [Boolean, Object],
    default: void 0
  },
  searchFn: Function,
  searchPlaceholder: [String, Array],
  spin: {
    type: [Boolean, Object],
    default: void 0
  },
  showSelectAll: { type: Boolean, default: void 0 },
  virtual: { type: Boolean, default: false },
  virtualScrollMode: { type: String, default: void 0 },
  virtualItemHeight: { type: Number, default: void 0 },
  "onUpdate:value": [Function, Array],
  "onUpdate:sourceSelectedKeys": [Function, Array],
  "onUpdate:targetSelectedKeys": [Function, Array],
  "onUpdate:sourceSearchValue": [Function, Array],
  "onUpdate:targetSearchValue": [Function, Array],
  onChange: [Function, Array],
  onSearch: [Function, Array],
  onSelectAll: [Function, Array],
  onClear: [Function, Array],
  onScroll: [Function, Array],
  onScrolledChange: [Function, Array],
  onScrolledBottom: [Function, Array]
};
const transferContentProps = {
  isSource: {
    type: Boolean,
    required: true
  }
};
const transferBodyProps = transferContentProps;
const transferHeaderProps = transferContentProps;
const transferFooterProps = transferContentProps;
const transferListProps = {
  dataSource: Array,
  checkable: {
    type: Boolean,
    default: true
  },
  removable: {
    type: Boolean,
    default: false
  },
  checked: Function,
  customAdditional: { type: Function, default: void 0 },
  disabled: Function,
  getKey: Function,
  labelKey: String,
  scroll: Object,
  virtual: { type: Boolean, default: false },
  virtualScrollMode: { type: String, default: void 0 },
  virtualItemHeight: { type: Number, default: 32 },
  onCheckChange: [Function, Array],
  onRemove: [Function, Array],
  onScroll: [Function, Array],
  onScrolledChange: [Function, Array],
  onScrolledBottom: [Function, Array]
};
const transferListItemProps = {
  checked: {
    type: Boolean,
    default: false
  },
  checkable: {
    type: Boolean,
    default: true
  },
  removable: {
    type: Boolean,
    default: false
  },
  disabled: {
    type: Boolean,
    default: false
  },
  label: String,
  value: {
    type: [String, Number, Symbol],
    required: true
  },
  onCheckChange: [Function, Array],
  onRemove: [Function, Array]
};
var ListItem = /* @__PURE__ */ defineComponent({
  props: transferListItemProps,
  setup(props, {
    slots
  }) {
    const {
      mergedPrefixCls
    } = inject(transferListContext);
    const onCheckChange = (value) => {
      callEmit(props.onCheckChange, !!value);
    };
    const onRemove = () => {
      callEmit(props.onRemove);
    };
    const classes = computed(() => {
      const prefixCls = `${mergedPrefixCls.value}-item`;
      return normalizeClass({
        [prefixCls]: true,
        [`${prefixCls}-disabled`]: props.disabled
      });
    });
    return () => {
      const prefixCls = `${mergedPrefixCls.value}-item`;
      const children = [];
      if (props.checkable) {
        children.push(createVNode(IxCheckbox, {
          "class": `${prefixCls}-checkbox`,
          "label": props.label,
          "value": props.value,
          "checked": props.checked,
          "disabled": props.disabled,
          "onChange": onCheckChange
        }, slots));
      } else {
        const labelNode = convertStringVNode(slots.default, props.label);
        children.push(createVNode("label", {
          "class": `${prefixCls}-label`
        }, [labelNode]));
      }
      if (props.removable && !props.disabled) {
        children.push(createVNode("div", {
          "class": `${prefixCls}-close-icon`,
          "onClick": onRemove
        }, [createVNode(IxIcon, {
          "name": "close"
        }, null)]));
      }
      return createVNode("li", {
        "class": classes.value
      }, [children]);
    };
  }
});
var TransferList$1 = /* @__PURE__ */ defineComponent({
  name: "IxTransferList",
  props: transferListProps,
  setup(props, {
    slots,
    expose
  }) {
    const common = useGlobalConfig("common");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-transfer-list`);
    const labelKey = computed(() => {
      var _a;
      return (_a = props.labelKey) != null ? _a : "label";
    });
    const virtualScrollRef = ref();
    provide(transferListContext, {
      mergedPrefixCls
    });
    const transferListApi = {
      scrollTo: (option) => {
        var _a;
        return (_a = virtualScrollRef.value) == null ? void 0 : _a.scrollTo(option);
      }
    };
    expose(transferListApi);
    const getKey = (item) => {
      var _a, _b;
      return (_b = (_a = props.getKey) == null ? void 0 : _a.call(props, item)) != null ? _b : item.key;
    };
    const handleScroll = (evt) => {
      callEmit(props.onScroll, evt);
    };
    const handleScrolledBottom = () => {
      callEmit(props.onScrolledBottom);
    };
    const handleScrolledChange = (startIndex, endIndex, visibleData) => {
      callEmit(props.onScrolledChange, startIndex, endIndex, visibleData);
    };
    const renderListItem = ({
      item,
      index
    }) => {
      var _a, _b;
      const key = getKey(item);
      const onCheckChange = (checked) => {
        callEmit(props.onCheckChange, item, checked);
      };
      const onRemove = () => {
        callEmit(props.onRemove, item);
      };
      const customAdditional = props.customAdditional ? props.customAdditional({
        data: item,
        index
      }) : void 0;
      return createVNode(ListItem, mergeProps({
        "key": key,
        "value": key,
        "label": item[labelKey.value],
        "checked": !!((_a = props.checked) == null ? void 0 : _a.call(props, item)),
        "disabled": !!((_b = props.disabled) == null ? void 0 : _b.call(props, item)),
        "checkable": props.checkable,
        "removable": props.removable,
        "onCheckChange": onCheckChange,
        "onRemove": onRemove
      }, customAdditional), {
        default: slots.label && (() => {
          var _a2;
          return (_a2 = slots.label) == null ? void 0 : _a2.call(slots, item);
        })
      });
    };
    const renderBody = () => {
      var _a;
      const {
        dataSource,
        virtual,
        virtualItemHeight,
        virtualScrollMode,
        scroll
      } = props;
      const data = dataSource != null ? dataSource : [];
      if (data.length <= 0) {
        return;
      }
      if (virtual) {
        return createVNode(CdkVirtualScroll, {
          "class": mergedPrefixCls.value + "-content",
          "ref": virtualScrollRef,
          "dataSource": data,
          "fullHeight": !!(scroll == null ? void 0 : scroll.fullHeight),
          "getKey": getKey,
          "height": (_a = scroll == null ? void 0 : scroll.height) != null ? _a : "100%",
          "rowHeight": virtualItemHeight,
          "rowRender": renderListItem,
          "virtual": true,
          "scrollMode": virtualScrollMode,
          "onScroll": handleScroll,
          "onScrolledBottom": handleScrolledBottom,
          "onScrolledChange": handleScrolledChange
        }, null);
      }
      return createVNode("ul", {
        "class": `${mergedPrefixCls.value}-inner`
      }, [data.map((item, index) => renderListItem({
        item,
        index
      }))]);
    };
    const classes = computed(() => {
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [prefixCls]: true,
        [`${prefixCls}-virtual`]: !!props.virtual
      });
    });
    const style = computed(() => {
      const scroll = props.scroll;
      return scroll && normalizeStyle({
        [scroll.fullHeight ? "height" : "maxHeight"]: convertCssPixel(scroll.height)
      });
    });
    return () => {
      return createVNode("div", {
        "class": classes.value,
        "style": style.value,
        "onScroll": handleScroll
      }, [renderBody()]);
    };
  }
});
var Body = /* @__PURE__ */ defineComponent({
  props: transferBodyProps,
  setup(props) {
    const {
      props: transferProps2,
      slots,
      mergedPrefixCls,
      sourceTransferListRef,
      targetTransferListRef,
      getKey
    } = inject(transferContext);
    const transferBindings = props.isSource ? inject(TRANSFER_SOURCE_TOKEN) : inject(TRANSFER_TARGET_TOKEN);
    const {
      paginatedDataSource,
      paginatedData,
      selectedKeySet,
      disabledDataSourceKeys,
      disabledKeys,
      handleSelectChange
    } = transferBindings;
    const {
      triggerRemove
    } = inject(TRANSFER_OPERATIONS_TOKEN);
    const transferListRef = props.isSource ? sourceTransferListRef : targetTransferListRef;
    const handleScroll = (evt) => {
      callEmit(transferProps2.onScroll, props.isSource, evt);
    };
    const handleScrolledBottom = () => {
      callEmit(transferProps2.onScrolledBottom, props.isSource);
    };
    const handleScrolledChange = (startIndex, endIndex, visibleData) => {
      callEmit(transferProps2.onScrolledChange, props.isSource, startIndex, endIndex, visibleData);
    };
    const checkListData = computed(() => props.isSource && transferProps2.mode === "immediate" ? paginatedDataSource.value : paginatedData.value);
    const checkListDisabledKeys = computed(() => props.isSource && transferProps2.mode === "immediate" ? disabledDataSourceKeys.value : disabledKeys.value);
    const defaultBodyRenderer = (prefixCls) => {
      const data = props.isSource && transferProps2.mode === "immediate" ? paginatedDataSource.value : paginatedData.value;
      if (data.length <= 0) {
        return createVNode("div", {
          "class": `${prefixCls}-empty-wrapper`
        }, [createVNode(_Empty, {
          "empty": transferProps2.empty
        }, slots)]);
      }
      const onCheckChange = (item, checked) => {
        const key = getKey.value(item);
        const _checkedKeys = new Set(selectedKeySet.value);
        checked ? _checkedKeys.add(key) : _checkedKeys.delete(key);
        handleSelectChange(_checkedKeys);
      };
      const onRemove = (item) => {
        const key = getKey.value(item);
        triggerRemove([key]);
      };
      const listSlots = {
        label: slots.label && ((item) => {
          var _a;
          return (_a = slots.label) == null ? void 0 : _a.call(slots, {
            item,
            isSource: props.isSource
          });
        })
      };
      return createVNode(TransferList$1, {
        "ref": transferListRef,
        "dataSource": checkListData.value,
        "getKey": getKey.value,
        "checked": (item) => selectedKeySet.value.has(getKey.value(item)),
        "disabled": (item) => transferProps2.disabled || checkListDisabledKeys.value.has(getKey.value(item)),
        "checkable": props.isSource || transferProps2.mode === "default",
        "removable": !props.isSource && transferProps2.mode === "immediate",
        "virtual": transferProps2.virtual,
        "virtualItemHeight": transferProps2.virtualItemHeight,
        "virtualScrollMode": transferProps2.virtualScrollMode,
        "scroll": transferProps2.scroll,
        "onCheckChange": onCheckChange,
        "onRemove": onRemove,
        "onScroll": handleScroll,
        "onScrolledChange": handleScrolledChange,
        "onScrolledBottom": handleScrolledBottom
      }, listSlots);
    };
    const renderBody = (prefixCls) => {
      if (slots.default) {
        return slots.default({
          ...convertToSlotParams(transferBindings),
          isSource: props.isSource
        });
      }
      return defaultBodyRenderer(prefixCls);
    };
    return () => {
      const prefixCls = `${mergedPrefixCls.value}-body`;
      return createVNode("div", {
        "class": `${mergedPrefixCls.value}-body`
      }, [renderBody(prefixCls)]);
    };
  }
});
var Footer = /* @__PURE__ */ defineComponent({
  props: transferFooterProps,
  setup(props) {
    const {
      slots,
      props: transferProps2,
      mergedPrefixCls
    } = inject(transferContext);
    const transferBindings = props.isSource ? inject(TRANSFER_SOURCE_TOKEN) : inject(TRANSFER_TARGET_TOKEN);
    const {
      pagination,
      filteredData,
      filteredDataSource
    } = transferBindings;
    const renderFooter = (prefixCls) => {
      var _a, _b;
      if (slots.footer) {
        return slots.footer({
          ...convertToSlotParams(transferBindings),
          isSource: props.isSource
        });
      }
      if (!pagination.value) {
        return;
      }
      const total = (_b = (_a = pagination.value) == null ? void 0 : _a.total) != null ? _b : props.isSource && transferProps2.mode === "immediate" ? filteredDataSource.value.length : filteredData.value.length;
      return createVNode("div", {
        "class": `${prefixCls}-inner`
      }, [createVNode(IxPagination, mergeProps({
        "class": `${prefixCls}-pagination`
      }, pagination.value, {
        "total": total
      }), null)]);
    };
    return () => {
      const prefixCls = `${mergedPrefixCls.value}-footer`;
      const children = renderFooter(prefixCls);
      if (!children || isArray(children) && children.length <= 0) {
        return;
      }
      return createVNode("div", {
        "class": prefixCls
      }, [children]);
    };
  }
});
var Header = /* @__PURE__ */ defineComponent({
  props: transferHeaderProps,
  setup(props) {
    const {
      props: transferProps2,
      slots,
      config,
      locale,
      mergedPrefixCls
    } = inject(transferContext);
    const transferBindings = props.isSource ? inject(TRANSFER_SOURCE_TOKEN) : inject(TRANSFER_TARGET_TOKEN);
    const {
      clearDisabled,
      triggerClear
    } = inject(TRANSFER_OPERATIONS_TOKEN);
    const {
      showSelectAll,
      data,
      selectAllDisabled,
      selectAllStatus,
      searchable,
      searchPlaceholder,
      searchValue,
      handleSearchChange,
      selectAll
    } = transferBindings;
    const [searchInputValue, setSearchInputValue] = useState("");
    const inputRef = ref();
    const syncValue = () => {
      var _a;
      const element = (_a = inputRef.value) == null ? void 0 : _a.getInputElement();
      if (element && element.value !== searchValue.value) {
        element.value = searchValue.value;
      }
      if (searchInputValue.value !== searchValue.value) {
        setSearchInputValue(searchValue.value);
      }
    };
    onMounted(() => {
      syncValue();
    });
    watch(searchValue, () => {
      syncValue();
    });
    const triggerSearch = () => {
      handleSearchChange(searchInputValue.value);
    };
    const handleInput = (evt) => {
      const value = evt.target.value;
      setSearchInputValue(value);
      if (!value) {
        triggerSearch();
      }
    };
    const handleKeyDown = withKeys(() => {
      triggerSearch();
    }, ["enter"]);
    const handleCheckChange = (value) => {
      selectAll(!!value);
    };
    const renderCheckAll = (prefixCls) => {
      if (!showSelectAll.value || !props.isSource && transferProps2.mode === "immediate") {
        return;
      }
      return createVNode(IxCheckbox, {
        "class": `${prefixCls}-check-all`,
        "checked": selectAllStatus.value.checked,
        "indeterminate": selectAllStatus.value.indeterminate,
        "disabled": selectAllDisabled.value,
        "onChange": handleCheckChange
      }, null);
    };
    const renderLabel = (prefixCls) => {
      let children;
      if (slots.headerLabel) {
        children = slots.headerLabel({
          data: data.value,
          isSource: props.isSource
        });
      } else {
        children = `${props.isSource ? locale.toSelect : locale.selected} (${data.value.length})`;
      }
      return createVNode("span", {
        "class": `${prefixCls}-label`
      }, [children]);
    };
    const renderInput = (prefixCls) => {
      if (!searchable.value) {
        return;
      }
      const inputSlots = {
        suffix: () => createVNode(IxIcon, {
          "class": `${prefixCls}-search-icon`,
          "name": "search",
          "onClick": triggerSearch
        }, null)
      };
      return createVNode(_Input, {
        "ref": inputRef,
        "class": `${prefixCls}-search-input`,
        "disabled": transferProps2.disabled,
        "type": "text",
        "size": "sm",
        "placeholder": searchPlaceholder.value,
        "onInput": handleInput,
        "onKeydown": handleKeyDown
      }, inputSlots);
    };
    const renderClearIcon = (prefixCls) => {
      var _a, _b;
      if (props.isSource || !((_a = transferProps2.clearable) != null ? _a : config.clearable)) {
        return;
      }
      const classes = normalizeClass({
        [`${prefixCls}-clear-icon`]: true,
        [`${prefixCls}-clear-icon-disabled`]: clearDisabled.value
      });
      return createVNode("span", {
        "class": classes,
        "onClick": triggerClear
      }, [slots.clearIcon ? slots.clearIcon() : createVNode(IxIcon, {
        "name": (_b = transferProps2.clearIcon) != null ? _b : config.clearIcon
      }, null)]);
    };
    const renderSuffix = (prefixCls) => {
      var _a;
      const suffix = (_a = slots.headerSuffix) == null ? void 0 : _a.call(slots, {
        isSource: props.isSource
      });
      if (isEmptyNode(suffix)) {
        return;
      }
      return createVNode("span", {
        "class": `${prefixCls}-suffix`
      }, [suffix]);
    };
    const renderHeader = (prefixCls) => {
      if (slots.header) {
        return slots.header({
          ...convertToSlotParams(transferBindings),
          isSource: props.isSource
        });
      }
      return createVNode("div", {
        "class": `${prefixCls}-inner`
      }, [renderCheckAll(prefixCls), renderLabel(prefixCls), renderInput(prefixCls), renderClearIcon(prefixCls), renderSuffix(prefixCls)]);
    };
    return () => {
      const prefixCls = `${mergedPrefixCls.value}-header`;
      const children = renderHeader(prefixCls);
      if (isEmptyNode(children)) {
        return;
      }
      return createVNode("div", {
        "class": prefixCls
      }, [children]);
    };
  }
});
var TransferList = /* @__PURE__ */ defineComponent({
  props: transferContentProps,
  setup(props) {
    const {
      props: transferProps2,
      mergedPrefixCls
    } = inject(transferContext);
    const classes = computed(() => {
      const prefixCls = `${mergedPrefixCls.value}-content`;
      return normalizeClass({
        [prefixCls]: true,
        [`${prefixCls}-${props.isSource ? "source" : "target"}`]: true,
        [`${prefixCls}-disabled`]: transferProps2.disabled
      });
    });
    const style = computed(() => {
      var _a;
      const scrollWidth = (_a = transferProps2.scroll) == null ? void 0 : _a.width;
      let width;
      if (isObject(scrollWidth)) {
        width = props.isSource ? scrollWidth.source : scrollWidth.target;
      } else {
        width = scrollWidth;
      }
      if (!width) {
        return;
      }
      return normalizeStyle({
        width: convertCssPixel(width)
      });
    });
    const spinProps = computed(() => {
      if (isNil(transferProps2.spin)) {
        return;
      }
      if (isBoolean(transferProps2.spin)) {
        return {
          spinning: !!transferProps2.spin
        };
      }
      return {
        spinning: props.isSource ? transferProps2.spin.source : transferProps2.spin.target
      };
    });
    return () => {
      const children = [createVNode(Header, {
        "isSource": props.isSource
      }, null), createVNode(Body, {
        "isSource": props.isSource
      }, null), createVNode(Footer, {
        "isSource": props.isSource
      }, null)];
      return createVNode("div", {
        "class": classes.value,
        "style": style.value
      }, [spinProps.value ? createVNode(IxSpin, spinProps.value, {
        default: () => [children]
      }) : children]);
    };
  }
});
function getDefaultThemeTokens(tokens) {
  const { heightLg, heightMd, marginSizeMd, marginSizeSm, paddingSizeSm, paddingSizeMd } = tokens;
  return {
    height: 290,
    contentWidth: 260,
    headerHeight: heightLg,
    suffixMarginLeft: marginSizeMd,
    footerHeight: heightLg,
    contentPadding: `0 ${paddingSizeSm}px 0 ${paddingSizeMd}px`,
    oprMinWidth: 4,
    oprPadding: paddingSizeSm,
    oprButtonWidth: 24,
    oprButtonHeight: 32,
    oprButtonGap: marginSizeSm,
    searchWidth: 110,
    listItemMinHeight: heightMd,
    closeIconPadding: `0 ${paddingSizeMd}px`
  };
}
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens) : getDefaultThemeTokens(tokens);
};
var Transfer = /* @__PURE__ */ defineComponent({
  name: "IxTransfer",
  props: transferProps,
  setup(props, {
    slots,
    expose
  }) {
    const common = useGlobalConfig("common");
    const config = useGlobalConfig("transfer");
    const locale = useGlobalConfig("locale").transfer;
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("transfer");
    registerToken(getThemeTokens);
    const mergedPrefixCls = computed(() => `${common.prefixCls}-transfer`);
    const showSelectAll = computed(() => {
      var _a;
      return (_a = props.showSelectAll) != null ? _a : config.showSelectAll;
    });
    const {
      source: sourceSearchable,
      target: targetSearchable
    } = useSearchable(props, config);
    const {
      source: sourcePlaceholder,
      target: targetPlaceholder
    } = useSearchPlaceholder(props, locale);
    const transferDataStrategies = useTransferDataStrategies(props);
    const transferPaginationContext = usePagination(props);
    const transferDataContext = useTransferData(props, config, transferDataStrategies, transferPaginationContext);
    const transferSelectStateContext = useTransferSelectState(props, transferDataContext, transferDataStrategies);
    const transferOperationsContext = useTransferOperations(props, transferDataContext, transferSelectStateContext);
    const {
      sourceBindings,
      targetBindings
    } = useTransferBindings(props, transferDataContext, transferOperationsContext, transferSelectStateContext, transferPaginationContext, showSelectAll, sourceSearchable, sourcePlaceholder, targetSearchable, targetPlaceholder);
    const sourceTransferListRef = ref();
    const targetTransferListRef = ref();
    const transferApi = {
      scrollTo: (isSource, ...params) => {
        var _a;
        return (_a = (isSource ? sourceTransferListRef : targetTransferListRef).value) == null ? void 0 : _a.scrollTo(...params);
      }
    };
    expose(transferApi);
    provide(transferContext, {
      props,
      slots,
      locale,
      config,
      mergedPrefixCls,
      sourceTransferListRef,
      targetTransferListRef,
      showSelectAll,
      getKey: transferDataContext.getKey
    });
    provide(TRANSFER_SOURCE_TOKEN, sourceBindings);
    provide(TRANSFER_TARGET_TOKEN, targetBindings);
    provide(TRANSFER_OPERATIONS_TOKEN, transferOperationsContext);
    const classes = computed(() => {
      var _a;
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [prefixCls]: true,
        [`${prefixCls}-fit-content`]: !!((_a = props.scroll) == null ? void 0 : _a.height)
      });
    });
    return () => createVNode("div", {
      "class": classes.value
    }, [createVNode(TransferList, {
      "isSource": true
    }, null), createVNode(TransferOperations, null, null), createVNode(TransferList, {
      "isSource": false
    }, null)]);
  }
});
const IxTransfer = Transfer;
const IxTransferList = TransferList$1;
export { IxTransfer, IxTransferList, TRANSFER_OPERATIONS_TOKEN, TRANSFER_SOURCE_TOKEN, TRANSFER_TARGET_TOKEN, getThemeTokens as getTransferThemeTokens };
