/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { TransferPaginationContext } from './usePagination';
import type { TransferData, TransferDataStrategy, TransferProps } from '../types';
import type { TransferConfig } from '@idux/components/config';
import { type ComputedRef } from 'vue';
import { type VKey } from '@idux/cdk/utils';
import { type GetKeyFn } from '@idux/components/utils';
export interface TransferDataContext<T extends TransferData = TransferData> {
    dataSource: ComputedRef<T[]>;
    dataKeyMap: ComputedRef<Map<VKey, T>>;
    sourceData: ComputedRef<T[]>;
    targetData: ComputedRef<T[]>;
    sourceDataKeys: ComputedRef<Set<VKey>>;
    targetDataKeys: ComputedRef<Set<VKey>>;
    filteredDataSource: ComputedRef<T[]>;
    filteredSourceData: ComputedRef<T[]>;
    filteredTargetData: ComputedRef<T[]>;
    paginatedDataSource: ComputedRef<T[]>;
    paginatedSourceData: ComputedRef<T[]>;
    paginatedTargetData: ComputedRef<T[]>;
    targetKeys: ComputedRef<VKey[]>;
    targetKeySet: ComputedRef<Set<VKey>>;
    changeTargetKeys: (removedKeys: VKey[], appendedKeys: VKey[]) => void;
    clear: () => void;
    getKey: ComputedRef<GetKeyFn>;
    disabledKeys: ComputedRef<Set<VKey>>;
    disabledSourceKeys: ComputedRef<Set<VKey>>;
    disabledTargetKeys: ComputedRef<Set<VKey>>;
    sourceSearchValue: ComputedRef<string>;
    setSourceSearchValue: (value: string) => void;
    targetSearchValue: ComputedRef<string>;
    setTargetSearchValue: (value: string) => void;
}
export declare function useTransferData<T extends TransferData = TransferData>(props: TransferProps, config: TransferConfig, transferDataStrategy: ComputedRef<TransferDataStrategy<T>>, transferPaginationContext: TransferPaginationContext): TransferDataContext<T>;
