import { computed, toRaw, defineComponent, inject, ref, createVNode, watch, provide, normalizeClass, mergeProps } from "vue";
import { isBoolean, isFunction } from "lodash-es";
import { useAccessorAndControl } from "@idux/cdk/forms";
import { convertArray, callEmit, useControlledProp, NoopFunction, useState } from "@idux/cdk/utils";
import { useGlobalConfig } from "@idux/components/config";
import { IxControlTrigger } from "@idux/components/control-trigger";
import { useFormItemRegister, useFormSize, useFormStatus } from "@idux/components/form";
import { IxSelector } from "@idux/components/selector";
import { IxSpin } from "@idux/components/spin";
import { useThemeToken } from "@idux/components/theme";
import { useGetKey } from "@idux/components/utils";
import { \u0275Input as _Input } from "@idux/components/_private/input";
import { IxButton } from "@idux/components/button";
import { IxTree } from "@idux/components/tree";
function useMergeNodes(props, mergedChildrenKey, mergedGetKey, mergedLabelKey) {
  const mergedNodeMap = computed(() => {
    const map = /* @__PURE__ */ new Map();
    const nodes = convertMergeNodes(
      props,
      props.dataSource,
      mergedChildrenKey.value,
      mergedGetKey.value,
      mergedLabelKey.value
    );
    convertMergedNodeMap(nodes, map);
    return map;
  });
  return { mergedNodeMap };
}
function convertMergeNodes(props, nodes, childrenKey, getKey, labelKey, parentKey) {
  const { treeDisabled, loadChildren } = props;
  return nodes.map(
    (option) => convertMergeNode(option, childrenKey, getKey, labelKey, treeDisabled, !!loadChildren, parentKey)
  );
}
function convertMergeNode(rawData, childrenKey, getKey, labelKey, disabled, hasLoad, parentKey) {
  var _a;
  const key = getKey(rawData);
  const { check, drag, drop, select } = convertDisabled(rawData, disabled);
  const subNodes = rawData[childrenKey];
  const label = rawData[labelKey];
  const children = subNodes == null ? void 0 : subNodes.map(
    (subNode) => convertMergeNode(subNode, childrenKey, getKey, labelKey, disabled, hasLoad, key)
  );
  return {
    children,
    label,
    key,
    isLeaf: (_a = rawData.isLeaf) != null ? _a : !((children == null ? void 0 : children.length) || hasLoad),
    parentKey,
    rawData,
    checkDisabled: check,
    dragDisabled: drag,
    dropDisabled: drop,
    selectDisabled: select
  };
}
function convertDisabled(option, disabled) {
  const optionDisabled = option.disabled;
  if (isBoolean(optionDisabled)) {
    return { check: optionDisabled, drag: optionDisabled, drop: optionDisabled, select: optionDisabled };
  } else {
    let { drag, drop, select } = optionDisabled != null ? optionDisabled : {};
    let check;
    if (disabled) {
      const treeDisabled = disabled(option);
      if (isBoolean(treeDisabled)) {
        check != null ? check : check = treeDisabled;
        drag != null ? drag : drag = treeDisabled;
        drop != null ? drop : drop = treeDisabled;
        select != null ? select : select = treeDisabled;
      } else {
        drag != null ? drag : drag = treeDisabled.drag;
        drop != null ? drop : drop = treeDisabled.drop;
        select != null ? select : select = treeDisabled.select;
        check != null ? check : check = treeDisabled.select;
      }
    }
    return { check, drag, drop, select };
  }
}
function convertMergedNodeMap(MergedNodes, map) {
  MergedNodes.forEach((item) => {
    const { key, children } = item;
    map.set(key, item);
    if (children) {
      convertMergedNodeMap(children, map);
    }
  });
}
function useSelectedState(props, accessor, treeRef, mergedNodeMap) {
  const selectedValue = computed(() => convertArray(accessor.value));
  const selectedNodes = computed(() => {
    const nodesMap = mergedNodeMap.value;
    return selectedValue.value.map((value) => nodesMap.get(value)).filter(Boolean);
  });
  const setValue = (value) => {
    const currValue = props.multiple ? value : value[0];
    const oldValue = toRaw(accessor.value);
    if (currValue !== oldValue) {
      accessor.setValue(currValue);
      callEmit(props.onChange, currValue, oldValue);
    }
  };
  const changeSelected = (value) => {
    if (!props.multiple && !value.length) {
      return;
    }
    setValue(value);
  };
  const handleRemove = (key) => {
    var _a;
    (_a = treeRef.value) == null ? void 0 : _a._handleCheck(key);
  };
  const handleClear = (evt) => {
    setValue([]);
    callEmit(props.onClear, evt);
  };
  return {
    selectedValue,
    selectedNodes,
    changeSelected,
    handleRemove,
    handleClear
  };
}
const treeSelectToken = Symbol("treeSelectToken");
var Content = /* @__PURE__ */ defineComponent({
  setup() {
    const {
      config,
      props,
      slots: treeSelectSlots,
      locale,
      mergedPrefixCls,
      mergedChildrenKey,
      mergedGetKey,
      mergedLabelKey,
      mergedNodeMap,
      inputValue,
      setInputValue,
      treeRef,
      expandedKeys,
      setExpandedKeys,
      selectedValue,
      changeSelected,
      handleNodeClick
    } = inject(treeSelectToken);
    const [loadedKeys, setLoadedKeys] = useControlledProp(props, "loadedKeys", () => []);
    const expandAllBtnStatus = ref(false);
    const mergedCheckable = computed(() => props.multiple && props.checkable);
    const mergedCascaderStrategy = computed(() => {
      if (!mergedCheckable.value) {
        return "off";
      }
      return props.cascaderStrategy;
    });
    const handleScrolledChange = (startIndex, endIndex, visibleNodes) => {
      const {
        onScrolledChange
      } = props;
      callEmit(onScrolledChange, startIndex, endIndex, visibleNodes.map((item) => item.rawData));
    };
    const handleClick = (evt) => {
      evt.stopPropagation();
    };
    const handleCheck = (checked, node) => {
      const {
        onCheck
      } = props;
      callEmit(onCheck, checked, node);
      handleNodeClick();
    };
    const handleSelect = (selected, node) => {
      var _a;
      const {
        onSelect
      } = props;
      if (!props.multiple && ((_a = selectedValue.value) == null ? void 0 : _a[0]) !== node.key) {
        callEmit(onSelect, selected, node);
      }
      handleNodeClick();
    };
    const handleExpand = (expanded, node) => {
      const {
        onExpand
      } = props;
      callEmit(onExpand, expanded, node);
    };
    const handleExpandedChange = (expendedKeys, expendedNodes) => {
      const {
        onExpandedChange
      } = props;
      callEmit(onExpandedChange, expendedKeys, expendedNodes);
      setExpandedKeys(expendedKeys);
    };
    const handleExpandAll = (evt) => {
      var _a, _b;
      const currStatus = expandAllBtnStatus.value;
      currStatus ? (_a = treeRef.value) == null ? void 0 : _a.expandAll() : (_b = treeRef.value) == null ? void 0 : _b.collapseAll();
      expandAllBtnStatus.value = !currStatus;
      evt.stopPropagation();
    };
    const onLoaded = async (loadedKeys2, node) => {
      const childrenKey = mergedChildrenKey.value;
      const labelKey = mergedLabelKey.value;
      const getKey = mergedGetKey.value;
      const childrenNodes = node[childrenKey];
      const key = getKey(node);
      const nodeMap = mergedNodeMap.value;
      const currNode = nodeMap.get(key);
      if (childrenNodes.length && currNode) {
        const mergedChildren = convertMergeNodes(props, childrenNodes, childrenKey, getKey, labelKey, key);
        convertMergedNodeMap(mergedChildren, nodeMap);
        currNode.rawData[childrenKey] = childrenNodes;
        currNode.children = mergedChildren;
        setLoadedKeys(loadedKeys2);
        callEmit(props.onLoaded, loadedKeys2, node);
      }
    };
    const mergedClearIcon = computed(() => {
      var _a;
      return (_a = props.clearIcon) != null ? _a : config.clearIcon;
    });
    const handleInput = (evt) => {
      const {
        value
      } = evt.target;
      setInputValue(value);
      props.searchable && callEmit(props.onSearch, value);
    };
    const handleClear = () => {
      setInputValue("");
      props.searchable && callEmit(props.onSearch, "");
    };
    return () => {
      const {
        customAdditional,
        dataSource,
        draggable,
        draggableIcon,
        empty,
        expandIcon,
        multiple,
        leafLineIcon,
        virtual,
        virtualItemHeight,
        searchable,
        showLine,
        searchPlaceholder,
        onDragstart,
        onDragend,
        onDragenter,
        onDragleave,
        onDragover,
        onDrop,
        onNodeClick,
        onNodeContextmenu,
        onScroll,
        onScrolledBottom,
        droppable,
        treeDisabled,
        loadChildren,
        searchFn,
        overlayRender,
        overlayHeight
      } = props;
      const prefixCls = mergedPrefixCls.value;
      const treeSlots = {
        label: treeSelectSlots.treeLabel,
        prefix: treeSelectSlots.treePrefix,
        suffix: treeSelectSlots.treeSuffix,
        leafLineIcon: treeSelectSlots.leafLineIcon,
        empty: treeSelectSlots.empty,
        expandIcon: treeSelectSlots.expandIcon,
        draggableIcon: treeSelectSlots.draggableIcon
      };
      const children = [createVNode(IxTree, {
        "ref": treeRef,
        "blocked": true,
        "checkOnClick": true,
        "checkedKeys": selectedValue.value,
        "customAdditional": customAdditional,
        "expandedKeys": expandedKeys.value,
        "loadedKeys": loadedKeys.value,
        "labelKey": mergedLabelKey.value,
        "checkable": mergedCheckable.value,
        "cascaderStrategy": mergedCascaderStrategy.value,
        "childrenKey": mergedChildrenKey.value,
        "dataSource": dataSource,
        "draggable": draggable,
        "draggableIcon": draggableIcon,
        "droppable": droppable,
        "disabled": treeDisabled,
        "empty": empty,
        "expandIcon": expandIcon,
        "getKey": mergedGetKey.value,
        "height": overlayHeight,
        "loadChildren": loadChildren,
        "leafLineIcon": leafLineIcon,
        "virtual": virtual,
        "virtualItemHeight": virtualItemHeight,
        "selectable": multiple ? "multiple" : true,
        "selectedKeys": selectedValue.value,
        "searchValue": searchFn !== false ? inputValue.value : void 0,
        "searchFn": isFunction(searchFn) ? searchFn : void 0,
        "showLine": showLine,
        "onClick": handleClick,
        "onCheck": handleCheck,
        "onDragstart": onDragstart,
        "onDragend": onDragend,
        "onDragenter": onDragenter,
        "onDragleave": onDragleave,
        "onDragover": onDragover,
        "onDrop": onDrop,
        "onNodeClick": onNodeClick,
        "onNodeContextmenu": onNodeContextmenu,
        "onExpand": handleExpand,
        "onSelect": handleSelect,
        "onLoaded": onLoaded,
        "onCheckedChange": changeSelected,
        "onSelectedChange": !mergedCheckable.value ? changeSelected : NoopFunction,
        "onExpandedChange": handleExpandedChange,
        "onScroll": onScroll,
        "onScrolledBottom": onScrolledBottom,
        "onScrolledChange": handleScrolledChange
      }, treeSlots)];
      if (searchable === "overlay") {
        const value = inputValue.value;
        children.unshift(createVNode("div", {
          "class": `${prefixCls}-overlay-search-wrapper`
        }, [createVNode(IxButton, {
          "size": "xs",
          "shape": "square",
          "title": expandAllBtnStatus.value ? locale.treeSelect.expandAll : locale.treeSelect.collapseAll,
          "icon": expandAllBtnStatus.value ? "expand-all" : "collapse-all",
          "onClick": handleExpandAll
        }, null), createVNode(_Input, {
          "clearable": true,
          "clearIcon": mergedClearIcon.value,
          "clearVisible": !!value,
          "size": "sm",
          "suffix": "search",
          "value": value,
          "placeholder": searchPlaceholder,
          "onClear": handleClear,
          "onInput": handleInput
        }, null)]));
      }
      return createVNode("div", {
        "tabindex": -1
      }, [overlayRender ? overlayRender(children) : children]);
    };
  }
});
const treeSelectProps = {
  control: {
    type: [String, Number, Object, Array],
    default: void 0
  },
  value: { type: null, default: void 0 },
  open: { type: Boolean, default: void 0 },
  expandedKeys: { type: Array, default: void 0 },
  loadedKeys: { type: Array, default: void 0 },
  autocomplete: { type: String, default: "off" },
  autofocus: { type: Boolean, default: false },
  borderless: { type: Boolean, default: void 0 },
  childrenKey: { type: String, default: void 0 },
  cascaderStrategy: { type: String, default: "off" },
  checkable: { type: Boolean, default: false },
  clearable: { type: Boolean, default: false },
  clearIcon: { type: String, default: void 0 },
  customAdditional: { type: Function, default: void 0 },
  dataSource: { type: Array, default: () => [] },
  disabled: { type: Boolean, default: false },
  draggable: { type: Boolean, default: false },
  draggableIcon: { type: String, default: void 0 },
  droppable: { type: Function, default: void 0 },
  empty: { type: [String, Object], default: "simple" },
  expandIcon: {
    type: [String, Function, Array],
    default: void 0
  },
  getKey: { type: [String, Function], default: void 0 },
  labelKey: { type: String, default: void 0 },
  leafLineIcon: { type: String, default: void 0 },
  loadChildren: {
    type: Function,
    default: void 0
  },
  maxLabel: { type: [Number, String], default: Number.MAX_SAFE_INTEGER },
  multiple: { type: Boolean, default: false },
  offset: Array,
  overlayClassName: { type: String, default: void 0 },
  overlayContainer: {
    type: [String, HTMLElement, Function],
    default: void 0
  },
  overlayTabindex: { type: Number, default: void 0 },
  overlayMatchWidth: { type: [Boolean, String], default: void 0 },
  overlayRender: { type: Function, default: void 0 },
  placeholder: { type: String, default: void 0 },
  readonly: { type: Boolean, default: false },
  searchable: { type: [Boolean, String], default: false },
  searchFn: {
    type: [Boolean, Function],
    default: true
  },
  searchPlaceholder: { type: String, default: void 0 },
  size: { type: String, default: void 0 },
  spin: { type: [Boolean, Object], default: void 0 },
  showLine: { type: Boolean, default: void 0 },
  status: String,
  suffix: { type: String, default: void 0 },
  treeDisabled: {
    type: Function,
    default: void 0
  },
  virtual: { type: Boolean, default: false },
  virtualItemHeight: { type: Number, default: 32 },
  "onUpdate:value": [Function, Array],
  "onUpdate:open": [Function, Array],
  "onUpdate:expandedKeys": [Function, Array],
  "onUpdate:loadedKeys": [Function, Array],
  onCheck: [Function, Array],
  onChange: [Function, Array],
  onClear: [Function, Array],
  onDragstart: [Function, Array],
  onDragend: [Function, Array],
  onDragenter: [Function, Array],
  onDragleave: [Function, Array],
  onDragover: [Function, Array],
  onDrop: [Function, Array],
  onFocus: [Function, Array],
  onBlur: [Function, Array],
  onExpand: [Function, Array],
  onExpandedChange: [Function, Array],
  onLoaded: [Function, Array],
  onNodeClick: [Function, Array],
  onNodeContextmenu: [Function, Array],
  onSearch: [Function, Array],
  onSelect: [Function, Array],
  onScroll: [Function, Array],
  onScrolledChange: [Function, Array],
  onScrolledBottom: [Function, Array],
  overlayHeight: { type: Number, default: 256 }
};
function getDefaultThemeTokens(tokens) {
  const { fontSizeSm, heightMd, marginSizeSm, paddingSizeXs, paddingSizeSm, paddingSizeMd } = tokens;
  return {
    optionFontSize: fontSizeSm,
    optionHeight: heightMd,
    optionPadding: `${paddingSizeSm}px ${paddingSizeMd}px`,
    optionMarginLeft: marginSizeSm,
    optionContainerPadding: `${paddingSizeXs}px 0`,
    optionContainerTreeNodePadding: `0 0 0 ${paddingSizeMd}px`,
    overlaySearchWrapperPadding: `${paddingSizeXs}px ${paddingSizeMd}px ${paddingSizeSm}px`
  };
}
const getThemeTokens = (tokens, presetTheme) => {
  return presetTheme === "default" ? getDefaultThemeTokens(tokens) : getDefaultThemeTokens(tokens);
};
var TreeSelect = /* @__PURE__ */ defineComponent({
  name: "IxTreeSelect",
  inheritAttrs: false,
  props: treeSelectProps,
  setup(props, {
    attrs,
    expose,
    slots
  }) {
    const common = useGlobalConfig("common");
    const {
      globalHashId,
      hashId,
      registerToken
    } = useThemeToken("treeSelect");
    registerToken(getThemeTokens);
    const locale = useGlobalConfig("locale");
    const config = useGlobalConfig("treeSelect");
    const mergedPrefixCls = computed(() => `${common.prefixCls}-tree-select`);
    const mergedChildrenKey = computed(() => {
      var _a;
      return (_a = props.childrenKey) != null ? _a : config.childrenKey;
    });
    const mergedGetKey = useGetKey(props, config, "components/tree-select");
    const mergedLabelKey = computed(() => {
      var _a;
      return (_a = props.labelKey) != null ? _a : config.labelKey;
    });
    const triggerRef = ref();
    const treeRef = ref();
    const [inputValue, setInputValue] = useState("");
    const focus = () => {
      var _a;
      return (_a = triggerRef.value) == null ? void 0 : _a.focus();
    };
    const blur = () => {
      var _a;
      return (_a = triggerRef.value) == null ? void 0 : _a.blur();
    };
    const clearInput = () => {
      var _a;
      props.searchable === "overlay" ? setInputValue("") : (_a = triggerRef.value) == null ? void 0 : _a.clearInput();
    };
    const [expandedKeys, setExpandedKeys] = useControlledProp(props, "expandedKeys", () => []);
    const {
      accessor,
      control
    } = useAccessorAndControl();
    useFormItemRegister(control);
    const mergedSize = useFormSize(props, config);
    const mergedStatus = useFormStatus(props, control);
    const {
      mergedNodeMap
    } = useMergeNodes(props, mergedChildrenKey, mergedGetKey, mergedLabelKey);
    const {
      selectedValue,
      selectedNodes,
      changeSelected,
      handleRemove,
      handleClear
    } = useSelectedState(props, accessor, treeRef, mergedNodeMap);
    const [overlayOpened, setOverlayOpened] = useControlledProp(props, "open", false);
    const scrollTo = (options) => {
      var _a;
      (_a = treeRef.value) == null ? void 0 : _a.scrollTo(options);
    };
    expose({
      focus,
      blur,
      collapseAll: () => {
        var _a;
        return (_a = treeRef.value) == null ? void 0 : _a.collapseAll();
      },
      expandAll: () => {
        var _a;
        return (_a = treeRef.value) == null ? void 0 : _a.expandAll();
      },
      scrollTo,
      getNode: (key) => {
        var _a;
        return treeRef.value ? treeRef.value.getNode(key) : (_a = mergedNodeMap.value.get(key)) == null ? void 0 : _a.rawData;
      }
    });
    watch(overlayOpened, (opened) => {
      opened && focus();
      clearInput();
    });
    const handleNodeClick = () => {
      if (!props.multiple) {
        setOverlayOpened(false);
      }
    };
    const handleFocus = (evt) => {
      callEmit(props.onFocus, evt);
    };
    const handleBlur = (evt) => {
      accessor.markAsBlurred();
      callEmit(props.onBlur, evt);
    };
    const handleItemRemove = (key) => {
      focus();
      handleRemove(key);
    };
    provide(treeSelectToken, {
      props,
      slots,
      config,
      locale,
      mergedPrefixCls,
      mergedChildrenKey,
      mergedGetKey,
      mergedLabelKey,
      expandedKeys,
      mergedNodeMap,
      inputValue,
      setInputValue,
      treeRef,
      accessor,
      setExpandedKeys,
      overlayOpened,
      setOverlayOpened,
      handleNodeClick,
      selectedValue,
      changeSelected
    });
    const overlayClasses = computed(() => {
      const {
        overlayClassName,
        multiple
      } = props;
      const prefixCls = mergedPrefixCls.value;
      return normalizeClass({
        [globalHashId.value]: !!globalHashId.value,
        [hashId.value]: !!hashId.value,
        [`${prefixCls}-overlay`]: true,
        [`${prefixCls}-overlay-multiple`]: multiple,
        [overlayClassName || ""]: !!overlayClassName
      });
    });
    const renderTrigger = ({
      focused,
      disabled,
      clearable,
      clearIcon,
      opened
    }) => [createVNode(IxSelector, mergeProps({
      "ref": triggerRef,
      "class": mergedPrefixCls.value,
      "allowInput": props.searchable === true ? "searchable" : false,
      "autocomplete": props.autocomplete,
      "autofocus": props.autofocus,
      "borderless": props.borderless,
      "clearable": clearable,
      "clearIcon": clearIcon,
      "dataSource": selectedNodes.value,
      "disabled": disabled,
      "focused": focused,
      "maxLabel": props.maxLabel,
      "multiple": props.multiple,
      "opened": opened,
      "placeholder": props.placeholder,
      "readonly": props.readonly,
      "size": mergedSize.value,
      "status": mergedStatus.value,
      "suffix": props.suffix,
      "onClear": handleClear,
      "onInputValueChange": setInputValue,
      "onItemRemove": handleItemRemove
    }, attrs), slots)];
    const renderLoading = (children) => {
      const {
        spin
      } = props;
      const spinProps = isBoolean(spin) ? {
        spinning: spin
      } : spin;
      return spinProps ? createVNode(IxSpin, spinProps, {
        default: () => [children]
      }) : children;
    };
    const renderContent = () => renderLoading(createVNode(Content, null, null));
    return () => {
      var _a, _b, _c, _d;
      const triggerProps = {
        autofocus: props.autofocus,
        clearable: props.clearable,
        clearIcon: props.clearIcon,
        disabled: accessor.disabled || props.readonly,
        overlayLazy: false,
        readonly: props.readonly,
        value: selectedValue.value,
        open: overlayOpened.value,
        offset: (_a = props.offset) != null ? _a : config.offset,
        overlayClassName: overlayClasses.value,
        overlayContainer: (_b = props.overlayContainer) != null ? _b : config.overlayContainer,
        overlayContainerFallback: `.${mergedPrefixCls.value}-overlay-container`,
        overlayTabindex: (_c = props.overlayTabindex) != null ? _c : config.overlayTabindex,
        overlayMatchWidth: (_d = props.overlayMatchWidth) != null ? _d : config.overlayMatchWidth,
        "onUpdate:open": setOverlayOpened,
        onFocus: handleFocus,
        onBlur: handleBlur
      };
      return createVNode(IxControlTrigger, triggerProps, {
        trigger: renderTrigger,
        overlay: renderContent
      });
    };
  }
});
const IxTreeSelect = TreeSelect;
export { IxTreeSelect, getThemeTokens as getTreeSelectThemeTokens };
