/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { AbstractControl, ValidateStatus } from '@idux/cdk/forms';
import type { VirtualScrollToFn } from '@idux/cdk/scroll';
import type { ExtractInnerPropTypes, ExtractPublicPropTypes, MaybeArray, VKey } from '@idux/cdk/utils';
import type { CascaderStrategy } from '@idux/components/cascader';
import type { EmptyProps } from '@idux/components/empty';
import type { FormSize } from '@idux/components/form';
import type { SpinProps } from '@idux/components/spin';
import type { TreeCustomAdditional, TreeDragDropOptions, TreeDroppable, TreeExpandIconRenderer, TreeNode } from '@idux/components/tree';
import type { OverlayContainerType } from '@idux/components/utils';
import type { DefineComponent, HTMLAttributes, PropType, VNode, VNodeChild } from 'vue';
export declare const treeSelectProps: {
    readonly control: {
        readonly type: PropType<string | number | (string | number)[] | AbstractControl<any>>;
        readonly default: undefined;
    };
    readonly value: {
        readonly type: null;
        readonly default: undefined;
    };
    readonly open: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly expandedKeys: {
        readonly type: PropType<VKey[]>;
        readonly default: undefined;
    };
    readonly loadedKeys: {
        readonly type: PropType<VKey[]>;
        readonly default: undefined;
    };
    readonly autocomplete: {
        readonly type: StringConstructor;
        readonly default: "off";
    };
    readonly autofocus: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly borderless: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly childrenKey: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly cascaderStrategy: {
        readonly type: PropType<CascaderStrategy>;
        readonly default: "off";
    };
    readonly checkable: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly clearable: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly clearIcon: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly customAdditional: {
        readonly type: PropType<TreeCustomAdditional>;
        readonly default: undefined;
    };
    readonly dataSource: {
        readonly type: PropType<TreeSelectNode<VKey>[]>;
        readonly default: () => never[];
    };
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly draggable: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly draggableIcon: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly droppable: {
        readonly type: PropType<TreeDroppable>;
        readonly default: undefined;
    };
    readonly empty: {
        readonly type: PropType<"default" | "simple" | EmptyProps>;
        readonly default: "simple";
    };
    readonly expandIcon: {
        readonly type: PropType<string | TreeExpandIconRenderer | [string, string]>;
        readonly default: undefined;
    };
    readonly getKey: {
        readonly type: PropType<string | ((data: TreeSelectNode<any>) => any)>;
        readonly default: undefined;
    };
    readonly labelKey: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly leafLineIcon: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly loadChildren: {
        readonly type: PropType<(node: TreeSelectNode<any>) => Promise<TreeSelectNode<any>[]>>;
        readonly default: undefined;
    };
    readonly maxLabel: {
        readonly type: PropType<number | "responsive">;
        readonly default: number;
    };
    readonly multiple: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly offset: PropType<[number, number]>;
    readonly overlayClassName: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly overlayContainer: {
        readonly type: PropType<OverlayContainerType>;
        readonly default: undefined;
    };
    readonly overlayTabindex: {
        readonly type: NumberConstructor;
        readonly default: undefined;
    };
    readonly overlayMatchWidth: {
        readonly type: PropType<boolean | "minWidth">;
        readonly default: undefined;
    };
    readonly overlayRender: {
        readonly type: PropType<(children: VNode[]) => VNodeChild>;
        readonly default: undefined;
    };
    readonly placeholder: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly readonly: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly searchable: {
        readonly type: PropType<boolean | "overlay">;
        readonly default: false;
    };
    readonly searchFn: {
        readonly type: PropType<boolean | ((node: TreeSelectNode, searchValue?: string) => boolean)>;
        readonly default: true;
    };
    readonly searchPlaceholder: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly size: {
        readonly type: PropType<FormSize>;
        readonly default: undefined;
    };
    readonly spin: {
        readonly type: PropType<boolean | SpinProps>;
        readonly default: undefined;
    };
    readonly showLine: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly status: PropType<ValidateStatus>;
    readonly suffix: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly treeDisabled: {
        readonly type: PropType<(node: TreeSelectNode<any>) => boolean | TreeSelectNodeDisabled>;
        readonly default: undefined;
    };
    readonly virtual: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly virtualItemHeight: {
        readonly type: NumberConstructor;
        readonly default: 32;
    };
    readonly 'onUpdate:value': PropType<MaybeArray<(value: any) => void>>;
    readonly 'onUpdate:open': PropType<MaybeArray<(opened: boolean) => void>>;
    readonly 'onUpdate:expandedKeys': PropType<MaybeArray<(keys: any[]) => void>>;
    readonly 'onUpdate:loadedKeys': PropType<MaybeArray<(keys: any[]) => void>>;
    readonly onCheck: PropType<MaybeArray<(checked: boolean, node: TreeSelectNode<any>) => void>>;
    readonly onChange: PropType<MaybeArray<(value: any, oldValue: any) => void>>;
    readonly onClear: PropType<MaybeArray<(evt: Event) => void>>;
    readonly onDragstart: PropType<MaybeArray<(options: TreeDragDropOptions<any>) => void>>;
    readonly onDragend: PropType<MaybeArray<(options: TreeDragDropOptions<any>) => void>>;
    readonly onDragenter: PropType<MaybeArray<(options: TreeDragDropOptions<any>) => void>>;
    readonly onDragleave: PropType<MaybeArray<(options: TreeDragDropOptions<any>) => void>>;
    readonly onDragover: PropType<MaybeArray<(options: TreeDragDropOptions<any>) => void>>;
    readonly onDrop: PropType<MaybeArray<(options: TreeDragDropOptions<any>) => void>>;
    readonly onFocus: PropType<MaybeArray<(evt: FocusEvent) => void>>;
    readonly onBlur: PropType<MaybeArray<(evt: FocusEvent) => void>>;
    readonly onExpand: PropType<MaybeArray<(expanded: boolean, node: TreeSelectNode<any>) => void>>;
    readonly onExpandedChange: PropType<MaybeArray<(expendedKeys: any[], expendedNodes: TreeSelectNode<any>[]) => void>>;
    readonly onLoaded: PropType<MaybeArray<(loadedKeys: any[], node: TreeSelectNode<any>) => void>>;
    readonly onNodeClick: PropType<MaybeArray<(evt: Event, node: TreeSelectNode<any>) => void>>;
    readonly onNodeContextmenu: PropType<MaybeArray<(evt: Event, node: TreeSelectNode<any>) => void>>;
    readonly onSearch: PropType<MaybeArray<(value: string) => void>>;
    readonly onSelect: PropType<MaybeArray<(selected: boolean, node: TreeSelectNode<any>) => void>>;
    readonly onScroll: PropType<MaybeArray<(evt: Event) => void>>;
    readonly onScrolledChange: PropType<MaybeArray<(startIndex: number, endIndex: number, visibleData: TreeSelectNode<any>[]) => void>>;
    readonly onScrolledBottom: PropType<MaybeArray<() => void>>;
    readonly overlayHeight: {
        readonly type: NumberConstructor;
        readonly default: 256;
    };
};
export type TreeSelectProps = ExtractInnerPropTypes<typeof treeSelectProps>;
export type TreeSelectPublicProps = Omit<ExtractPublicPropTypes<typeof treeSelectProps>, 'overlayHeight'>;
export interface TreeSelectBindings<K = VKey> {
    focus: (options?: FocusOptions) => void;
    blur: () => void;
    collapseAll: () => void;
    expandAll: () => void;
    scrollTo: VirtualScrollToFn;
    /**
     * get node by it's key
     *
     * @param key
     * @returns node
     */
    getNode: (key: K) => TreeNode<K> | undefined;
}
export type TreeSelectComponent = DefineComponent<Omit<HTMLAttributes, keyof TreeSelectPublicProps> & TreeSelectPublicProps, TreeSelectBindings>;
export type TreeSelectInstance = InstanceType<DefineComponent<TreeSelectProps, TreeSelectBindings>>;
export type TreeSelectCustomAdditional = TreeCustomAdditional;
export type TreeSelectNode<K = VKey> = TreeNode<K>;
export interface TreeSelectNodeDisabled {
    select?: boolean;
    drag?: boolean;
    drop?: boolean;
}
export declare const treeSelectorProps: {
    readonly clearable: BooleanConstructor;
    readonly suffix: StringConstructor;
};
export type TreeSelectorProps = ExtractInnerPropTypes<typeof treeSelectorProps>;
