/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { ExpandableContext } from './useExpandable';
import type { TreeNode, TreeNodeDisabled, TreeProps } from '../types';
import type { VKey } from '@idux/cdk/utils';
import type { GetKeyFn } from '@idux/components/utils';
import type { ComputedRef } from 'vue';
export interface MergedNode {
    children?: MergedNode[];
    label: string;
    isFirst: boolean;
    isLeaf: boolean;
    isLast: boolean;
    key: VKey;
    expanded: boolean;
    hidden?: boolean;
    level: number;
    parentKey?: VKey;
    rawNode: TreeNode;
    checkDisabled?: boolean;
    dragDisabled?: boolean;
    dropDisabled?: boolean;
    selectDisabled?: boolean;
}
export declare function useMergeNodes(props: TreeProps, mergedChildrenKey: ComputedRef<string>, mergedGetKey: ComputedRef<GetKeyFn>, mergedLabelKey: ComputedRef<string>): {
    mergedNodes: ComputedRef<MergedNode[]>;
    mergedNodeMap: ComputedRef<Map<VKey, MergedNode>>;
    parentKeyMap: ComputedRef<Map<VKey, VKey | undefined>>;
    depthMap: ComputedRef<Map<VKey, number>>;
    setLoadedNodes: (key: VKey, nodes: TreeNode[]) => void;
};
export declare function useFlattedNodes(mergedNodes: ComputedRef<MergedNode[]>, { expandedKeys }: ExpandableContext, props: TreeProps, searchedKeys: ComputedRef<VKey[]>): {
    flattedNodes: ComputedRef<MergedNode[]>;
    flattenedNodeMap: ComputedRef<Map<VKey, MergedNode>>;
};
export declare function convertMergeNodes(nodes: TreeNode[], getLoadedNodes: (key: VKey) => TreeNode[] | undefined, options: {
    props: TreeProps;
    childrenKey: string;
    getKey: GetKeyFn;
    labelKey: string;
    parentKey: VKey | undefined;
    parentLevel: number;
    parentsDisabled: TreeNodeDisabled[];
}, maps: {
    dataKeyMap: Map<VKey, MergedNode>;
    parentKeyMap: Map<VKey, VKey | undefined>;
    depthMap: Map<VKey, number>;
}): MergedNode[];
export declare function convertMergedNodeMap(mergedNodes: MergedNode[], map: Map<VKey, MergedNode>): void;
