/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { MergedNode } from './composables/useDataSource';
import type { VirtualScrollMode, VirtualScrollToFn } from '@idux/cdk/scroll';
import type { ExtractInnerPropTypes, ExtractPublicPropTypes, MaybeArray, VKey } from '@idux/cdk/utils';
import type { CascaderStrategy } from '@idux/components/cascader';
import type { EmptyProps } from '@idux/components/empty';
import type { DefineComponent, HTMLAttributes, PropType, VNodeChild } from 'vue';
export type CheckStrategy = 'all' | 'parent' | 'child';
export type TreeExpandIconRenderer = (data: {
    key: VKey;
    expanded: boolean;
    node: TreeNode<any>;
}) => VNodeChild | string;
export declare const treeProps: {
    readonly autoHeight: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly checkedKeys: PropType<VKey[]>;
    readonly expandedKeys: PropType<VKey[]>;
    readonly indeterminateKeys: PropType<VKey[]>;
    readonly loadedKeys: PropType<VKey[]>;
    readonly selectedKeys: PropType<VKey[]>;
    readonly blocked: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly cascaderStrategy: {
        readonly type: PropType<CascaderStrategy>;
        readonly default: "off";
    };
    readonly checkable: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly childrenKey: StringConstructor;
    readonly checkOnClick: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly customAdditional: {
        readonly type: PropType<TreeCustomAdditional>;
        readonly default: undefined;
    };
    readonly dataSource: {
        readonly type: PropType<TreeNode<VKey>[]>;
        readonly default: () => TreeNode[];
    };
    readonly disabled: PropType<(node: TreeNode<any>) => boolean | TreeNodeDisabled>;
    readonly draggable: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly draggableIcon: {
        readonly type: StringConstructor;
        readonly default: undefined;
    };
    readonly droppable: PropType<TreeDroppable>;
    readonly empty: {
        readonly type: PropType<"default" | "simple" | EmptyProps>;
        readonly default: "simple";
    };
    readonly expandIcon: {
        readonly type: PropType<string | TreeExpandIconRenderer | [string, string]>;
        readonly default: undefined;
    };
    readonly getKey: {
        readonly type: PropType<string | ((data: TreeNode<any>) => any)>;
        readonly default: undefined;
    };
    readonly height: NumberConstructor;
    readonly labelKey: StringConstructor;
    readonly leafLineIcon: StringConstructor;
    readonly loadChildren: PropType<(node: TreeNode<any>) => Promise<TreeNode<any>[]>>;
    readonly searchFn: PropType<(node: TreeNode<any>, searchValue?: string) => boolean>;
    readonly searchValue: StringConstructor;
    readonly selectable: {
        readonly type: PropType<boolean | "multiple">;
        readonly default: true;
    };
    readonly showLine: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly virtual: {
        readonly type: BooleanConstructor;
        readonly default: false;
    };
    readonly virtualScrollMode: {
        readonly type: PropType<VirtualScrollMode>;
        readonly default: undefined;
    };
    readonly virtualItemHeight: {
        readonly type: NumberConstructor;
        readonly default: 32;
    };
    readonly 'onUpdate:checkedKeys': PropType<MaybeArray<(keys: any[]) => void>>;
    readonly 'onUpdate:expandedKeys': PropType<MaybeArray<(keys: any[]) => void>>;
    readonly 'onUpdate:loadedKeys': PropType<MaybeArray<(keys: any[]) => void>>;
    readonly 'onUpdate:selectedKeys': PropType<MaybeArray<(keys: any[]) => void>>;
    readonly onCheck: PropType<MaybeArray<(checked: boolean, node: TreeNode<any>) => void>>;
    readonly onCheckedChange: PropType<MaybeArray<(checkedKeys: any[], checkedNodes: TreeNode<any>[]) => void>>;
    readonly onDragstart: PropType<MaybeArray<(options: TreeDragDropOptions<any>) => void>>;
    readonly onDragend: PropType<MaybeArray<(options: TreeDragDropOptions<any>) => void>>;
    readonly onDragenter: PropType<MaybeArray<(options: TreeDragDropOptions<any>) => void>>;
    readonly onDragleave: PropType<MaybeArray<(options: TreeDragDropOptions<any>) => void>>;
    readonly onDragover: PropType<MaybeArray<(options: TreeDragDropOptions<any>) => void>>;
    readonly onDrop: PropType<MaybeArray<(options: TreeDragDropOptions<any>) => void>>;
    readonly onExpand: PropType<MaybeArray<(expanded: boolean, node: TreeNode<any>) => void>>;
    readonly onExpandedChange: PropType<MaybeArray<(expendedKeys: any[], expendedNodes: TreeNode<any>[]) => void>>;
    readonly onLoaded: PropType<MaybeArray<(loadedKeys: any[], node: TreeNode<any>) => void>>;
    readonly onSelect: PropType<MaybeArray<(selected: boolean, node: TreeNode<any>) => void>>;
    readonly onSelectedChange: PropType<MaybeArray<(selectedKeys: any[], selectedNodes: TreeNode<any>[]) => void>>;
    readonly onFocus: PropType<MaybeArray<(evt: FocusEvent) => void>>;
    readonly onBlur: PropType<MaybeArray<(evt: FocusEvent) => void>>;
    readonly onKeydown: PropType<MaybeArray<(evt: KeyboardEvent) => void>>;
    readonly onKeyup: PropType<MaybeArray<(evt: KeyboardEvent) => void>>;
    readonly onNodeClick: PropType<MaybeArray<(evt: Event, node: TreeNode<any>) => void>>;
    readonly onNodeContextmenu: PropType<MaybeArray<(evt: Event, node: TreeNode<any>) => void>>;
    readonly onScroll: PropType<MaybeArray<(evt: Event) => void>>;
    readonly onScrolledChange: PropType<MaybeArray<(startIndex: number, endIndex: number, visibleNodes: TreeNode<any>[]) => void>>;
    readonly onScrolledBottom: PropType<MaybeArray<() => void>>;
};
export type TreeProps = ExtractInnerPropTypes<typeof treeProps>;
export type TreePublicProps = ExtractPublicPropTypes<typeof treeProps>;
export interface TreeBindings<K = VKey> {
    focus: (options?: FocusOptions) => void;
    blur: () => void;
    expandAll: () => void;
    collapseAll: () => void;
    scrollTo: VirtualScrollToFn;
    /**
     * get node by it's key
     *
     * @param key
     * @returns node
     */
    getNode: (key: K) => TreeNode<K> | undefined;
    _getFlattedNodes: () => MergedNode[];
    _handleCheck: (key: VKey) => void;
}
export type TreeComponent = DefineComponent<Omit<HTMLAttributes, keyof TreePublicProps> & TreePublicProps, TreeBindings>;
export type TreeInstance = InstanceType<DefineComponent<TreeProps, TreeBindings>>;
export type TreeCustomAdditional = (options: {
    node: TreeNode<any>;
    level: number;
}) => Record<string, any> | undefined;
export interface TreeNode<K = VKey> {
    children?: TreeNode<K>[];
    disabled?: boolean | TreeNodeDisabled;
    isLeaf?: boolean;
    key?: K;
    label?: string;
    prefix?: string;
    suffix?: string;
    [key: string]: unknown;
}
export interface TreeNodeDisabled {
    check?: boolean;
    drag?: boolean;
    drop?: boolean;
    select?: boolean;
}
export type TreeDroppable = <K = VKey>(options: TreeDroppableOptions<K>) => TreeDropType | boolean | Promise<TreeDropType | boolean>;
export interface TreeDroppableOptions<K = VKey> {
    evt: DragEvent;
    dragNode: TreeNode<K>;
    dropNode: TreeNode<K>;
    isTopHalf: boolean;
}
export type TreeDropType = 'before' | 'inside' | 'after';
export interface TreeDragDropOptions<K = VKey> {
    evt: DragEvent;
    node: TreeNode<K>;
    dragNode?: TreeNode<K>;
    dropNode?: TreeNode<K>;
    dropType?: TreeDropType;
}
export declare const motionTreeNodeProps: {
    readonly expanded: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly expandedNodes: PropType<MergedNode[]>;
    readonly node: PropType<MergedNode>;
    readonly prefixCls: {
        readonly type: StringConstructor;
        readonly required: true;
    };
};
export declare const treeNodeProps: {
    readonly node: {
        readonly type: PropType<MergedNode>;
        readonly required: true;
    };
    readonly isLeaf: {
        readonly type: BooleanConstructor;
        readonly required: true;
    };
    readonly isFirst: {
        readonly type: BooleanConstructor;
        readonly required: true;
    };
    readonly isLast: {
        readonly type: BooleanConstructor;
        readonly required: true;
    };
    readonly label: StringConstructor;
    readonly level: {
        readonly type: NumberConstructor;
        readonly required: true;
    };
    readonly rawNode: {
        readonly type: PropType<TreeNode<VKey>>;
        readonly required: true;
    };
    readonly expanded: {
        readonly type: BooleanConstructor;
        readonly required: true;
    };
    readonly children: PropType<MergedNode[]>;
    readonly parentKey: PropType<VKey>;
    readonly checkDisabled: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly dragDisabled: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly dropDisabled: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly selectDisabled: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
};
export declare const treeNodeCheckboxProps: {
    readonly node: {
        readonly type: PropType<MergedNode>;
        readonly required: true;
    };
    readonly checkDisabled: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
};
export declare const treeNodeExpandProps: {
    readonly expanded: {
        readonly type: BooleanConstructor;
        readonly required: true;
    };
    readonly hasTopLine: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly hasBottomLine: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly isLeaf: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly nodeKey: {
        readonly type: PropType<VKey>;
        readonly required: true;
    };
    readonly rawNode: {
        readonly type: PropType<TreeNode<VKey>>;
        readonly required: true;
    };
};
export declare const treeNodeContentProps: {
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly node: {
        readonly type: PropType<MergedNode>;
        readonly required: true;
    };
    readonly nodeKey: {
        readonly type: PropType<VKey>;
        readonly required: true;
    };
    readonly label: StringConstructor;
    readonly selected: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
};
