/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { ExtractInnerPropTypes, ExtractPublicPropTypes, MaybeArray, VKey } from '@idux/cdk/utils';
import type { ProgressProps } from '@idux/components/progress';
import type { DefineComponent, HTMLAttributes, PropType, VNode } from 'vue';
type DataType = Record<string, unknown>;
export type UploadRequestHeader = Record<string, string>;
export type UploadRequestMethod = 'POST' | 'PUT' | 'PATCH' | 'post' | 'put' | 'patch';
export type UploadRequestStatus = 'loadstart' | 'progress' | 'abort' | 'error' | 'loadend';
export type UploadFileStatus = 'selected' | 'cancel' | 'uploading' | 'error' | 'success' | 'abort';
export type UploadFilesType = 'text' | 'image' | 'imageCard';
export type UploadIconType = 'file' | 'preview' | 'download' | 'remove' | 'retry';
export interface FilteredFile {
    file: File;
    reason: 'acceptNotMatch' | 'maxCountExceeded';
}
export interface UploadRequestHandler {
    abort?: () => void;
}
export interface UploadProgressEvent extends ProgressEvent {
    percent?: number;
}
export interface UploadRequestError extends Error {
    status?: number;
    method?: UploadRequestMethod;
    url?: string;
}
export interface UploadFile<K = VKey> {
    key: K;
    name: string;
    raw?: File;
    status?: UploadFileStatus;
    error?: UploadRequestError;
    errorTip?: string;
    thumbUrl?: string;
    percent?: number;
    response?: any;
}
export interface UploadRequestOption<T = unknown> {
    onProgress?: (event: UploadProgressEvent) => void;
    onError?: (error: UploadRequestError) => void;
    onSuccess?: (body: T) => void;
    name: string;
    file: File;
    withCredentials?: boolean;
    action: string;
    requestHeaders?: UploadRequestHeader;
    requestMethod: UploadRequestMethod;
    requestData?: DataType;
}
export interface UploadRequestChangeOption<K = VKey> {
    file: UploadFile<K>;
    status: UploadRequestStatus;
    response?: any;
    event?: UploadProgressEvent;
    error?: UploadRequestError;
}
export declare const uploadProps: {
    readonly files: {
        readonly type: PropType<UploadFile<VKey>[]>;
        readonly default: () => never[];
    };
    readonly accept: StringConstructor;
    readonly action: {
        readonly type: PropType<string | ((file: UploadFile<any>) => Promise<string>)>;
        readonly default: undefined;
    };
    readonly dragable: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly directory: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly disabled: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly maxCount: NumberConstructor;
    readonly multiple: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly progress: PropType<ProgressProps>;
    readonly name: StringConstructor;
    readonly customRequest: PropType<(option: UploadRequestOption<any>) => Promise<UploadRequestHandler | void> | UploadRequestHandler | void>;
    readonly withCredentials: {
        readonly type: BooleanConstructor;
        readonly default: undefined;
    };
    readonly requestData: PropType<DataType | ((file: UploadFile) => DataType | Promise<DataType>)>;
    readonly requestHeaders: PropType<UploadRequestHeader>;
    readonly requestMethod: PropType<"post" | "POST" | "PUT" | "PATCH" | "put" | "patch">;
    readonly 'onUpdate:files': PropType<MaybeArray<(fileList: UploadFile<any>[]) => void>>;
    readonly onSelect: PropType<MaybeArray<(files: File[], filteredFiles: FilteredFile[]) => boolean | File[] | Promise<boolean | File[]> | void>>;
    readonly onMaxCountExceeded: PropType<MaybeArray<() => void>>;
    readonly onBeforeUpload: PropType<MaybeArray<(file: UploadFile<any>) => boolean | UploadFile<any> | Promise<boolean | UploadFile<any>>>>;
    readonly onFileStatusChange: PropType<MaybeArray<(file: UploadFile<any>) => void>>;
    readonly onRequestChange: PropType<MaybeArray<(option: UploadRequestChangeOption<any>) => void>>;
};
export type UploadProps = ExtractInnerPropTypes<typeof uploadProps>;
export type UploadPublicProps = ExtractPublicPropTypes<typeof uploadProps>;
export type UploadComponent = DefineComponent<Omit<HTMLAttributes, keyof UploadPublicProps> & UploadPublicProps>;
export type UploadInstance = InstanceType<DefineComponent<UploadProps>>;
export declare const uploadFilesProps: {
    readonly type: PropType<UploadFilesType>;
    readonly icon: PropType<Partial<Record<UploadIconType, string | VNode<import("vue").RendererNode, import("vue").RendererElement, {
        [key: string]: any;
    }>>>>;
    readonly onDownload: PropType<MaybeArray<(file: UploadFile<any>) => void>>;
    readonly onPreview: PropType<MaybeArray<(file: UploadFile<any>) => void>>;
    readonly onRemove: PropType<MaybeArray<(file: UploadFile<any>) => boolean | Promise<boolean>>>;
    readonly onRetry: PropType<MaybeArray<(file: UploadFile<any>) => void>>;
};
export type UploadFilesProps = ExtractInnerPropTypes<typeof uploadFilesProps>;
export type UploadFilesPublicProps = ExtractPublicPropTypes<typeof uploadFilesProps>;
export type UploadFilesComponent = DefineComponent<Omit<HTMLAttributes, keyof UploadFilesPublicProps> & UploadFilesPublicProps>;
export type UploadFilesInstance = InstanceType<DefineComponent<UploadFilesProps>>;
export {};
