/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import type { CascaderStrategy } from '@idux/components/cascader';
import { type ComputedRef, type Ref } from 'vue';
import { type TreeTypeData, type VKey } from '@idux/cdk/utils';
import { GetKeyFn, type TreeCheckStateResolver, type TreeCheckStateResolverContext } from '@idux/components/utils';
interface GetAllCheckedKeys<V extends TreeTypeData<V, C>, C extends keyof V> {
    (data: V[], cached?: boolean): VKey[];
    (defaultUnCheckedKeys: VKey[]): VKey[];
    (data: V[], defaultUnCheckedKeys: VKey[], cached?: boolean): VKey[];
}
interface GetAllUncheckedKeys<V extends TreeTypeData<V, C>, C extends keyof V> {
    (data: V[], cached?: boolean): VKey[];
    (defaultCheckedKeys: VKey[]): VKey[];
    (data: V[], defaultCheckedKeys: VKey[], cached?: boolean): VKey[];
}
export interface TreeCheckStateContext<V extends TreeTypeData<V, C>, C extends keyof V> {
    allCheckedKeySet: Ref<Set<VKey>>;
    checkDisabledKeySet: ComputedRef<Set<VKey>>;
    checkStateResolver: TreeCheckStateResolver<V, C>;
    isChecked: (key: VKey) => boolean;
    isCheckDisabled: (key: VKey) => boolean;
    isIndeterminate: (key: VKey) => boolean;
    toggle: (data: V) => {
        checked: boolean;
        checkedKeys: VKey[];
    };
    getAllCheckedKeys: GetAllCheckedKeys<V, C>;
    getAllUncheckedKeys: GetAllUncheckedKeys<V, C>;
    getDataByKeys: (keys: VKey[]) => V[];
}
export declare function useTreeCheckState<V extends TreeTypeData<V, C>, C extends keyof V>(checkedKeys: Ref<VKey[]>, resolverContext: Ref<TreeCheckStateResolverContext<V, C>>, childrenKey: Ref<C>, getKey: Ref<GetKeyFn>, cascaderStrategy: Ref<CascaderStrategy>, isDisabled: Ref<((data: V) => boolean) | undefined>): TreeCheckStateContext<V, C>;
export {};
