/**
 * @license
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/IDuxFE/idux/blob/main/LICENSE
 */
import { type Ref } from 'vue';
import { type TreeTypeData, type VKey } from '@idux/cdk/utils';
interface GetAllCheckedKeys<V extends TreeTypeData<V, C>, C extends keyof V> {
    (data: V[]): VKey[];
    (defaultUnCheckedKeys: VKey[]): VKey[];
    (data: V[], defaultUnCheckedKeys: VKey[]): VKey[];
}
interface GetAllUncheckedKeys<V extends TreeTypeData<V, C>, C extends keyof V> {
    (data: V[]): VKey[];
    (defaultCheckedKeys: VKey[]): VKey[];
    (data: V[], defaultCheckedKeys: VKey[]): VKey[];
}
export interface TreeCheckStateResolverContext<V extends TreeTypeData<V, C>, C extends keyof V> {
    data: V[] | undefined;
    dataMap: Map<VKey, V>;
    parentKeyMap: Map<VKey, VKey | undefined>;
    depthMap: Map<VKey, number>;
}
export interface TreeCheckStateResolver<V extends TreeTypeData<V, C>, C extends keyof V> {
    appendKeys: (checkedKeys: VKey[] | Set<VKey>, appendedKeys: VKey[]) => VKey[];
    removeKeys: (checkedKeys: VKey[] | Set<VKey>, removedKeys: VKey[]) => VKey[];
    getAllCheckedKeys: GetAllCheckedKeys<V, C>;
    getAllUncheckedKeys: GetAllUncheckedKeys<V, C>;
}
export type TreeCascadeStrategy = 'all' | 'child' | 'parent' | 'off';
export declare function useTreeCheckStateResolver<V extends TreeTypeData<V, C>, C extends keyof V>(data: Ref<V[]>, childrenKey: Ref<C>, getKey: Ref<(item: V) => VKey>, cascadeStrategy?: Ref<TreeCascadeStrategy>): TreeCheckStateResolver<V, C>;
export declare function useTreeCheckStateResolver<V extends TreeTypeData<V, C>, C extends keyof V>(context: Ref<TreeCheckStateResolverContext<V, C>>, childrenKey: Ref<C>, getKey: Ref<(item: V) => VKey>, cascadeStrategy?: Ref<TreeCascadeStrategy>): TreeCheckStateResolver<V, C>;
export declare function getTreeCheckStateResolverContext<V extends TreeTypeData<V, C>, C extends keyof V>(data: V[], childrenKey: C, getKey: (item: V) => VKey): TreeCheckStateResolverContext<V, C>;
export {};
